'use strict';

exports.__esModule = true;

var _internalProperties = require('../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _eventEmitter = require('../utils/event-emitter');

var _eventEmitter2 = _interopRequireDefault(_eventEmitter);

var _baseDomAdapter = require('../../processing/dom/base-dom-adapter');

var _baseDomAdapter2 = _interopRequireDefault(_baseDomAdapter);

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _settings = require('../settings');

var _settings2 = _interopRequireDefault(_settings);

var _destinationLocation = require('../utils/destination-location');

var _url = require('../utils/url');

var _dom = require('../utils/dom');

var domUtils = _interopRequireWildcard(_dom);

var _fastApply = require('../utils/fast-apply');

var _fastApply2 = _interopRequireDefault(_fastApply);

var _writer = require('../sandbox/node/document/writer');

var _writer2 = _interopRequireDefault(_writer);

var _windowsStorage = require('../sandbox/windows-storage');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ClientDomAdapter = function (_BaseDomAdapter) {
    _inherits(ClientDomAdapter, _BaseDomAdapter);

    function ClientDomAdapter() {
        _classCallCheck(this, ClientDomAdapter);

        return _possibleConstructorReturn(this, _BaseDomAdapter.apply(this, arguments));
    }

    ClientDomAdapter.prototype.removeAttr = function removeAttr(el, attr) {
        return _nativeMethods2.default.removeAttribute.call(el, attr);
    };

    ClientDomAdapter.prototype.getAttr = function getAttr(el, attr) {
        return _nativeMethods2.default.getAttribute.call(el, attr);
    };

    ClientDomAdapter.prototype.hasAttr = function hasAttr(el, attr) {
        return el.hasAttribute(attr);
    };

    ClientDomAdapter.prototype.isSVGElement = function isSVGElement(el) {
        return domUtils.isSVGElement(el);
    };

    ClientDomAdapter.prototype.getClassName = function getClassName(el) {
        return el.className;
    };

    ClientDomAdapter.prototype.hasEventHandler = function hasEventHandler(el) {
        var attributes = _nativeMethods2.default.elementAttributesGetter.call(el);

        for (var _i2 = 0; _i2 < attributes.length; _i2++) {
            var attr = attributes[_i2];
            if (this.EVENTS.indexOf(attr.name) !== -1) return true;
        }

        return false;
    };

    ClientDomAdapter.prototype.getTagName = function getTagName(el) {
        return domUtils.getTagName(el);
    };

    ClientDomAdapter.prototype.setAttr = function setAttr(el, attr, value) {
        return _nativeMethods2.default.setAttribute.call(el, attr, value);
    };

    ClientDomAdapter.prototype.setScriptContent = function setScriptContent(script, content) {
        _nativeMethods2.default.scriptTextSetter.call(script, content);
    };

    ClientDomAdapter.prototype.getScriptContent = function getScriptContent(script) {
        return _nativeMethods2.default.scriptTextGetter.call(script);
    };

    ClientDomAdapter.prototype.getStyleContent = function getStyleContent(style) {
        return _nativeMethods2.default.elementInnerHTMLGetter.call(style);
    };

    ClientDomAdapter.prototype.setStyleContent = function setStyleContent(style, content) {
        _nativeMethods2.default.elementInnerHTMLSetter.call(style, content);
    };

    ClientDomAdapter.prototype.needToProcessContent = function needToProcessContent(el) {
        return !_writer2.default.hasUnclosedElementFlag(el);
    };

    ClientDomAdapter.prototype.needToProcessUrl = function needToProcessUrl() {
        return true;
    };

    ClientDomAdapter.prototype.hasIframeParent = function hasIframeParent(el) {
        try {
            if (el[_internalProperties2.default.processedContext]) return window.top !== el[_internalProperties2.default.processedContext];
            return window.top.document !== domUtils.findDocument(el);
        } catch (e) {
            return true;
        }
    };

    ClientDomAdapter.prototype.attachEventEmitter = function attachEventEmitter(domProcessor) {
        var eventEmitter = new _eventEmitter2.default();
        domProcessor.on = function (evt, listener) {
            return eventEmitter.on(evt, listener);
        };
        domProcessor.off = function (evt, listener) {
            return eventEmitter.off(evt, listener);
        };
        domProcessor.emit = function () {
            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
            }

            return (0, _fastApply2.default)(eventEmitter, 'emit', args);
        };
    };

    ClientDomAdapter.prototype.getCrossDomainPort = function getCrossDomainPort() {
        return _settings2.default.get().crossDomainProxyPort;
    };

    ClientDomAdapter.prototype.getProxyUrl = function getProxyUrl() {
        return _url.getProxyUrl.apply(null, arguments);
    };

    ClientDomAdapter.prototype.isTopParentIframe = function isTopParentIframe(el) {
        var elWindow = el[_internalProperties2.default.processedContext];
        return elWindow && window.top === elWindow.parent;
    };

    ClientDomAdapter.prototype.sameOriginCheck = function sameOriginCheck(location, checkedUrl) {
        return (0, _destinationLocation.sameOriginCheck)(location, checkedUrl);
    };

    ClientDomAdapter.prototype.isExistingTarget = function isExistingTarget(target) {
        return !!(0, _windowsStorage.findByName)(target);
    };

    return ClientDomAdapter;
}(_baseDomAdapter2.default);

exports.default = ClientDomAdapter;
module.exports = exports['default'];