'use strict';

exports.__esModule = true;

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _sandbox = require('./sandbox');

var _sandbox2 = _interopRequireDefault(_sandbox);

var _eventEmitter = require('./utils/event-emitter');

var _eventEmitter2 = _interopRequireDefault(_eventEmitter);

var _xhr = require('./sandbox/xhr');

var _xhr2 = _interopRequireDefault(_xhr);

var _settings = require('./settings');

var _settings2 = _interopRequireDefault(_settings);

var _transport = require('./transport');

var _transport2 = _interopRequireDefault(_transport);

var _jsonHammerhead = require('json-hammerhead');

var JSON = _interopRequireWildcard(_jsonHammerhead);

var _browser = require('./utils/browser');

var browserUtils = _interopRequireWildcard(_browser);

var _dom = require('./utils/dom');

var domUtils = _interopRequireWildcard(_dom);

var _event = require('./utils/event');

var eventUtils = _interopRequireWildcard(_event);

var _types = require('./utils/types');

var typeUtils = _interopRequireWildcard(_types);

var _position = require('./utils/position');

var positionUtils = _interopRequireWildcard(_position);

var _style = require('./utils/style');

var styleUtils = _interopRequireWildcard(_style);

var _stringTrim = require('../utils/string-trim');

var _stringTrim2 = _interopRequireDefault(_stringTrim);

var _url = require('./utils/url');

var urlUtils = _interopRequireWildcard(_url);

var _featureDetection = require('./utils/feature-detection');

var featureDetection = _interopRequireWildcard(_featureDetection);

var _html = require('./utils/html');

var htmlUtils = _interopRequireWildcard(_html);

var _nativeMethods = require('./sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _script = require('../processing/script');

var _header = require('../processing/script/header');

var _style2 = require('../processing/style');

var _style3 = _interopRequireDefault(_style2);

var _extend = require('./utils/extend');

var _extend2 = _interopRequireDefault(_extend);

var _internalProperties = require('../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _pageNavigationWatch = require('./page-navigation-watch');

var _pageNavigationWatch2 = _interopRequireDefault(_pageNavigationWatch);

var _domProcessor = require('./dom-processor');

var _domProcessor2 = _interopRequireDefault(_domProcessor);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Hammerhead = function () {
    function Hammerhead() {
        var _this = this;

        _classCallCheck(this, Hammerhead);

        this.win = null;
        this.sandbox = new _sandbox2.default();
        this.pageNavigationWatch = new _pageNavigationWatch2.default(this.sandbox.event, this.sandbox.codeInstrumentation, this.sandbox.node.element);
        this.EVENTS = {
            beforeFormSubmit: this.sandbox.node.element.BEFORE_FORM_SUBMIT_EVENT,
            beforeBeforeUnload: this.sandbox.event.unload.BEFORE_BEFORE_UNLOAD_EVENT,
            beforeUnload: this.sandbox.event.unload.BEFORE_UNLOAD_EVENT,
            unload: this.sandbox.event.unload.UNLOAD_EVENT,
            bodyCreated: this.sandbox.node.mutation.BODY_CREATED_EVENT,
            documentCleaned: this.sandbox.node.mutation.DOCUMENT_CLEANED_EVENT,
            uncaughtJsError: this.sandbox.node.win.UNCAUGHT_JS_ERROR_EVENT,
            unhandledRejection: this.sandbox.node.win.UNHANDLED_REJECTION_EVENT,
            startFileUploading: this.sandbox.upload.START_FILE_UPLOADING_EVENT,
            endFileUploading: this.sandbox.upload.END_FILE_UPLOADING_EVENT,
            evalIframeScript: this.sandbox.iframe.EVAL_EXTERNAL_SCRIPT_EVENT,
            xhrCompleted: this.sandbox.xhr.XHR_COMPLETED_EVENT,
            xhrError: this.sandbox.xhr.XHR_ERROR_EVENT,
            beforeXhrSend: this.sandbox.xhr.BEFORE_XHR_SEND_EVENT,
            fetchSent: this.sandbox.fetch.FETCH_REQUEST_SENT_EVENT,
            pageNavigationTriggered: this.pageNavigationWatch.PAGE_NAVIGATION_TRIGGERED_EVENT,
            scriptElementAdded: this.sandbox.node.element.SCRIPT_ELEMENT_ADDED_EVENT,
            consoleMethCalled: this.sandbox.console.CONSOLE_METH_CALLED_EVENT
        };
        this.PROCESSING_COMMENTS = {
            stylesheetStart: _style3.default.STYLESHEET_PROCESSING_START_COMMENT,
            stylesheetEnd: _style3.default.STYLESHEET_PROCESSING_END_COMMENT,
            scriptStart: _header.SCRIPT_PROCESSING_START_COMMENT,
            scriptEndHeader: _header.SCRIPT_PROCESSING_END_HEADER_COMMENT,
            scriptEnd: _header.SCRIPT_PROCESSING_END_COMMENT
        };
        this.EventEmitter = _eventEmitter2.default;
        // Methods
        this.doUpload = function (input, filePaths) {
            return _this.sandbox.upload.doUpload(input, filePaths);
        };
        this.createNativeXHR = _xhr2.default.createNativeXHR;
        this.processScript = _script.processScript;
        // NOTE: We should provide a function to retrieve modules, because hammerhead will be bundled into a single
        // file and we will not have access to the internal modules by default.
        // eslint-disable-next-line no-undef
        this.get = require;
        // Modules
        this.Promise = _pinkie2.default;
        this.json = JSON;
        this.transport = _transport2.default;
        this.nativeMethods = this.sandbox.nativeMethods;
        this.shadowUI = this.sandbox.shadowUI;
        this.storages = this.sandbox.storageSandbox;
        this.eventSandbox = {
            listeners: this.sandbox.event.listeners,
            hover: this.sandbox.event.hover,
            focusBlur: this.sandbox.event.focusBlur,
            elementEditingWatcher: this.sandbox.event.elementEditingWatcher,
            eventSimulator: this.sandbox.event.eventSimulator,
            selection: this.sandbox.event.selection,
            message: this.sandbox.event.message,
            timers: this.sandbox.event.timers,
            DataTransfer: this.sandbox.event.DataTransfer,
            DragDataStore: this.sandbox.event.DragDataStore
        };
        this.utils = {
            browser: browserUtils,
            dom: domUtils,
            event: eventUtils,
            position: positionUtils,
            style: styleUtils,
            types: typeUtils,
            trim: _stringTrim2.default,
            extend: _extend2.default,
            html: htmlUtils,
            url: urlUtils,
            featureDetection: featureDetection
        };
    }

    Hammerhead.prototype._getEventOwner = function _getEventOwner(evtName) {
        switch (evtName) {
            case this.EVENTS.pageNavigationTriggered:
                return this.pageNavigationWatch;
            case this.EVENTS.beforeUnload:
            case this.EVENTS.beforeBeforeUnload:
            case this.EVENTS.unload:
                return this.sandbox.event.unload;
            case this.EVENTS.bodyCreated:
            case this.EVENTS.documentCleaned:
                return this.sandbox.node.mutation;
            case this.EVENTS.uncaughtJsError:
            case this.EVENTS.unhandledRejection:
                return this.sandbox.node.win;
            case this.EVENTS.startFileUploading:
            case this.EVENTS.endFileUploading:
                return this.sandbox.upload;
            case this.EVENTS.evalIframeScript:
                return this.sandbox.iframe;
            case this.EVENTS.xhrCompleted:
            case this.EVENTS.xhrError:
            case this.EVENTS.beforeXhrSend:
                return this.sandbox.xhr;
            case this.EVENTS.beforeFormSubmit:
            case this.EVENTS.scriptElementAdded:
                return this.sandbox.node.element;
            case this.EVENTS.fetchSent:
                return this.sandbox.fetch;
            case this.EVENTS.consoleMethCalled:
                return this.sandbox.console;
            default:
                return null;
        }
    };

    Hammerhead._cleanLocalStorageServiceData = function _cleanLocalStorageServiceData(sessionId, window) {
        _nativeMethods2.default.winLocalStorageGetter.call(window).removeItem(sessionId);
    };

    Hammerhead.prototype.on = function on(evtName, handler) {
        var eventOwner = this._getEventOwner(evtName);
        if (eventOwner) eventOwner.on(evtName, handler);
    };

    Hammerhead.prototype.off = function off(evtName, handler) {
        var eventOwner = this._getEventOwner(evtName);
        if (eventOwner) eventOwner.off(evtName, handler);
    };

    Hammerhead.prototype.navigateTo = function navigateTo(url) {
        var navigationUrl = urlUtils.getNavigationUrl(url, this.win);
        if (!navigationUrl) return;
        this.win.location = navigationUrl;
    };

    Hammerhead.prototype.start = function start(initSettings, win) {
        this.win = win || window;
        if (initSettings) {
            _settings2.default.set(initSettings);
            if (initSettings.isFirstPageLoad) Hammerhead._cleanLocalStorageServiceData(initSettings.sessionId, this.win);
            _domProcessor2.default.forceProxySrcForImage = !!initSettings.forceProxySrcForImage;
        }
        this.sandbox.attach(this.win);
        this.pageNavigationWatch.start();
    };

    return Hammerhead;
}();

var hammerhead = new Hammerhead();
// NOTE: The 'load' event is raised after calling document.close for a same-domain iframe
// So, we need to define the '%hammerhead%' variable as 'configurable' so that it can be redefined.
_nativeMethods2.default.objectDefineProperty(window, _internalProperties2.default.hammerhead, {
    value: hammerhead,
    configurable: true
});
exports.default = hammerhead;
module.exports = exports['default'];