'use strict';

exports.__esModule = true;

var _eventEmitter = require('./utils/event-emitter');

var _eventEmitter2 = _interopRequireDefault(_eventEmitter);

var _url = require('../utils/url');

var _url2 = require('./utils/url');

var _dom = require('./utils/dom');

var _windowsStorage = require('./sandbox/windows-storage');

var windowsStorage = _interopRequireWildcard(_windowsStorage);

var _dom2 = require('../processing/dom');

var _dom3 = _interopRequireDefault(_dom2);

var _nextTick = require('./utils/next-tick');

var _nextTick2 = _interopRequireDefault(_nextTick);

var _nativeMethods = require('./sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _internalProperties = require('../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PageNavigationWatch = function (_EventEmiter) {
    _inherits(PageNavigationWatch, _EventEmiter);

    function PageNavigationWatch(eventSandbox, codeInstrumentation, elementSandbox) {
        _classCallCheck(this, PageNavigationWatch);

        var _this = _possibleConstructorReturn(this, _EventEmiter.call(this));

        _this.PAGE_NAVIGATION_TRIGGERED_EVENT = 'hammerhead|event|page-navigation-triggered';
        _this.PAGE_NAVIGATION_TRIGGERED_EVENT = 'hammerhead|event|page-navigation-triggered';
        _this.lastLocationValue = window.location.toString();
        _this.codeInstrumentation = codeInstrumentation;
        _this.eventSandbox = eventSandbox;
        _this.elementSandbox = elementSandbox;
        return _this;
    }

    PageNavigationWatch.prototype._formWatch = function _formWatch(elementSandbox, eventSandbox) {
        var onFormSubmit = function onFormSubmit(form) {
            var targetWindow = PageNavigationWatch._getTargetWindow(form);
            PageNavigationWatch._onNavigationTriggeredInWindow(targetWindow, _nativeMethods2.default.formActionGetter.call(form));
        };
        // NOTE: fires when form.submit() is called
        elementSandbox.on(elementSandbox.BEFORE_FORM_SUBMIT_EVENT, function (e) {
            return onFormSubmit(e.form);
        });
        // NOTE: fires when the form is submitted by clicking the submit button
        eventSandbox.listeners.initElementListening(window, ['submit']);
        eventSandbox.listeners.addInternalEventListener(window, ['submit'], function (e) {
            var prevented = false;
            if (!(0, _dom.isFormElement)(e.target)) return;
            var onPreventDefault = function onPreventDefault(preventedEvent) {
                prevented = prevented || preventedEvent === e;
            };
            eventSandbox.on(eventSandbox.EVENT_PREVENTED_EVENT, onPreventDefault);
            (0, _nextTick2.default)().then(function () {
                eventSandbox.off(eventSandbox.EVENT_PREVENTED_EVENT, onPreventDefault);
                // NOTE: the defaultPrevented flag is saved between event raises in all browsers
                // except IE. In IE, it is reset to false before the next handler is executed.
                if (!e.defaultPrevented && !prevented) onFormSubmit(e.target);
            });
        });
    };

    PageNavigationWatch._getTargetWindow = function _getTargetWindow(el) {
        var target = _nativeMethods2.default.getAttribute.call(el, _dom3.default.getStoredAttrName('target')) || _nativeMethods2.default.getAttribute.call(el, 'target') || '_self';
        switch (target) {
            case '_top':
                return window.top;
            case '_parent':
                return window.parent;
            case '_self':
                return window;
            default:
                return windowsStorage.findByName(target);
        }
    };

    PageNavigationWatch.prototype._linkWatch = function _linkWatch(eventSandbox) {
        eventSandbox.listeners.initElementListening(window, ['click']);
        eventSandbox.listeners.addInternalEventListener(window, ['click'], function (e) {
            var link = (0, _dom.isAnchorElement)(e.target) ? e.target : (0, _dom.closest)(e.target, 'a');
            if (link && !(0, _dom.isShadowUIElement)(link)) {
                var prevented = false;
                var targetWindow = PageNavigationWatch._getTargetWindow(link);
                var href = _nativeMethods2.default.anchorHrefGetter.call(link);
                var onPreventDefault = function onPreventDefault(preventedEvent) {
                    prevented = prevented || preventedEvent === e;
                };
                eventSandbox.on(eventSandbox.EVENT_PREVENTED_EVENT, onPreventDefault);
                (0, _nextTick2.default)().then(function () {
                    eventSandbox.off(eventSandbox.EVENT_PREVENTED_EVENT, onPreventDefault);
                    // NOTE: the defaultPrevented flag is saved between event raises in all browsers
                    // except IE. In IE, it is reset to false before the next handler is executed.
                    if (!e.defaultPrevented && !prevented) PageNavigationWatch._onNavigationTriggeredInWindow(targetWindow, href);
                });
            }
        });
    };

    PageNavigationWatch.prototype._locationWatch = function _locationWatch(codeInstrumentation) {
        var _this2 = this;

        var locationAccessorsInstrumentation = codeInstrumentation.locationAccessorsInstrumentation;
        var locationChangedHandler = function locationChangedHandler(newLocation) {
            return _this2.onNavigationTriggered(newLocation);
        };
        locationAccessorsInstrumentation.on(locationAccessorsInstrumentation.LOCATION_CHANGED_EVENT, locationChangedHandler);
    };

    PageNavigationWatch._onNavigationTriggeredInWindow = function _onNavigationTriggeredInWindow(win, url) {
        try {
            win[_internalProperties2.default.hammerhead].pageNavigationWatch.onNavigationTriggered(url);
        }
        // eslint-disable-next-line no-empty
        catch (e) {}
    };

    PageNavigationWatch.prototype.onNavigationTriggered = function onNavigationTriggered(url) {
        var currentLocation = this.lastLocationValue;
        this.lastLocationValue = window.location.toString();
        if (url !== currentLocation && (url.charAt(0) === '#' || (0, _url2.isChangedOnlyHash)(currentLocation, url)) || _dom3.default.isJsProtocol(url)) return;
        this.emit(this.PAGE_NAVIGATION_TRIGGERED_EVENT, (0, _url.parseProxyUrl)(url).destUrl);
    };

    PageNavigationWatch.prototype.start = function start() {
        this._locationWatch(this.codeInstrumentation);
        this._linkWatch(this.eventSandbox);
        this._formWatch(this.elementSandbox, this.eventSandbox);
    };

    return PageNavigationWatch;
}(_eventEmitter2.default);

exports.default = PageNavigationWatch;
module.exports = exports['default'];