'use strict';

exports.__esModule = true;

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _properties = require('./properties');

var _properties2 = _interopRequireDefault(_properties);

var _location = require('./location');

var _location2 = _interopRequireDefault(_location);

var _methods = require('./methods');

var _methods2 = _interopRequireDefault(_methods);

var _script = require('../../../processing/script');

var _instruction = require('../../../processing/script/instruction');

var _instruction2 = _interopRequireDefault(_instruction);

var _nativeMethods = require('../../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _html = require('../../utils/html');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CodeInstrumentation = function (_SandboxBase) {
    _inherits(CodeInstrumentation, _SandboxBase);

    function CodeInstrumentation(eventSandbox, windowSandbox, messageSandbox) {
        _classCallCheck(this, CodeInstrumentation);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.methodCallInstrumentation = new _methods2.default(eventSandbox.message);
        _this.locationAccessorsInstrumentation = new _location2.default(messageSandbox);
        _this.propertyAccessorsInstrumentation = new _properties2.default(windowSandbox);
        return _this;
    }

    CodeInstrumentation.prototype.attach = function attach(window) {
        _SandboxBase.prototype.attach.call(this, window);
        this.methodCallInstrumentation.attach(window);
        this.locationAccessorsInstrumentation.attach(window);
        this.elementPropertyAccessors = this.propertyAccessorsInstrumentation.attach(window);
        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
        // NOTE: GH-260
        _nativeMethods2.default.objectDefineProperty(window, _instruction2.default.getEval, {
            value: function value(evalFn) {
                // @ts-ignore
                if (evalFn !== window.eval) return evalFn;
                return function (script) {
                    if (typeof script === 'string') script = (0, _script.processScript)(script);
                    return evalFn(script);
                };
            },
            configurable: true
        });
        _nativeMethods2.default.objectDefineProperty(window, _instruction2.default.processScript, {
            value: function value(script, isApply) {
                if (isApply) {
                    if (script && script.length && typeof script[0] === 'string') {
                        var args = [(0, _script.processScript)(script[0], false)];
                        // NOTE: shallow-copy the remaining args. Don't use arr.slice(),
                        // since it may leak the arguments object.
                        // See: https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Functions/arguments
                        for (var i = 1; i < script.length; i++) {
                            args.push(script[i]);
                        }return args;
                    }
                } else if (typeof script === 'string') return (0, _script.processScript)(script, false);
                return script;
            },
            configurable: true
        });
        _nativeMethods2.default.objectDefineProperty(window, _instruction2.default.processHtml, {
            value: function value(win, html) {
                if (typeof html === 'string') html = (0, _html.processHtml)('<html><body>' + html + '</body></html>', { processedContext: win });
                return html;
            },
            configurable: true
        });
    };

    return CodeInstrumentation;
}(_base2.default);

exports.default = CodeInstrumentation;
module.exports = exports['default'];