'use strict';

exports.__esModule = true;
exports.default = DOMStringListWrapper;

var _nativeMethods = require('../../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

var _propertyOverriding = require('../../../utils/property-overriding');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var lengthWeakMap = new WeakMap();
function DOMStringListWrapper(window, getCrossDomainOrigin) {
    var _this = this;

    var nativeOrigins = window.location.ancestorOrigins;
    var length = nativeOrigins.length;
    var parentWindow = window.parent;
    lengthWeakMap.set(this, length);

    var _loop = function _loop(i) {
        var parentLocationWrapper = _index2.default.getLocationWrapper(parentWindow);
        var isCrossDomainParent = parentLocationWrapper === parentWindow.location;
        // eslint-disable-next-line no-restricted-properties
        updateOrigin(nativeOrigins, _this, i, isCrossDomainParent ? '' : parentLocationWrapper.origin);
        if (isCrossDomainParent) getCrossDomainOrigin(parentWindow, function (origin) {
            return updateOrigin(nativeOrigins, _this, i, origin);
        });
        parentWindow = parentWindow.parent;
    };

    for (var i = 0; i < length; i++) {
        _loop(i);
    }
}
DOMStringListWrapper.prototype = _nativeMethods2.default.objectCreate(DOMStringList.prototype);
DOMStringListWrapper.prototype.item = function (index) {
    return this[index];
};
DOMStringListWrapper.prototype.contains = function (origin) {
    if (typeof origin !== 'string') origin = String(origin);
    var length = lengthWeakMap.get(this);
    for (var i = 0; i < length; i++) {
        if (this[i] === origin) return true;
    }
    return false;
};
var lengthDescriptor = (0, _propertyOverriding.createOverriddenDescriptor)(DOMStringList.prototype, 'length', {
    getter: function getter() {
        return lengthWeakMap.get(this);
    }
});
_nativeMethods2.default.objectDefineProperty(DOMStringListWrapper.prototype, 'length', lengthDescriptor);
function updateOrigin(ancestorOrigins, wrapper, index, origin) {
    var descriptor = (0, _propertyOverriding.createOverriddenDescriptor)(ancestorOrigins, index, { value: origin });
    _nativeMethods2.default.objectDefineProperty(wrapper, index, descriptor);
}
module.exports = exports['default'];