'use strict';

exports.__esModule = true;

var _wrapper = require('./wrapper');

var _wrapper2 = _interopRequireDefault(_wrapper);

var _base = require('../../base');

var _base2 = _interopRequireDefault(_base);

var _dom = require('../../../utils/dom');

var _instruction = require('../../../../processing/script/instruction');

var _instruction2 = _interopRequireDefault(_instruction);

var _nativeMethods = require('../../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var LOCATION_WRAPPER = 'hammerhead|location-wrapper';

var LocationAccessorsInstrumentation = function (_SandboxBase) {
    _inherits(LocationAccessorsInstrumentation, _SandboxBase);

    function LocationAccessorsInstrumentation(messageSandbox) {
        _classCallCheck(this, LocationAccessorsInstrumentation);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.LOCATION_CHANGED_EVENT = 'hammerhead|event|location-changed';
        _this.messageSandbox = messageSandbox;
        _this.locationChangedEventCallback = function (e) {
            return _this.emit(_this.LOCATION_CHANGED_EVENT, e);
        };
        return _this;
    }

    LocationAccessorsInstrumentation.isLocationWrapper = function isLocationWrapper(obj) {
        return obj instanceof _wrapper2.default;
    };

    LocationAccessorsInstrumentation.getLocationWrapper = function getLocationWrapper(owner) {
        // NOTE: When the owner is cross-domain, we cannot get its location wrapper, so we return the original
        // location, which cannot be accessed but behaves like a real one. Cross-domain location retains the 'replace'
        // and 'assign' methods, so we intercept calls to them through MethodCallInstrumentation.
        try {
            return owner[LOCATION_WRAPPER];
        } catch (e) {
            return owner.location;
        }
    };

    LocationAccessorsInstrumentation.prototype.attach = function attach(window) {
        _SandboxBase.prototype.attach.call(this, window);
        var document = window.document;
        var locationWrapper = new _wrapper2.default(window, this.messageSandbox, this.locationChangedEventCallback);
        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
        _nativeMethods2.default.objectDefineProperty(window, LOCATION_WRAPPER, { value: locationWrapper, configurable: true });
        _nativeMethods2.default.objectDefineProperty(document, LOCATION_WRAPPER, { value: locationWrapper, configurable: true });
        _nativeMethods2.default.objectDefineProperty(window, _instruction2.default.getLocation, {
            value: function value(location) {
                return (0, _dom.isLocation)(location) ? locationWrapper : location;
            },
            configurable: true
        });
        _nativeMethods2.default.objectDefineProperty(window, _instruction2.default.setLocation, {
            value: function value(location, _value) {
                if ((0, _dom.isLocation)(location) && typeof _value === 'string') {
                    // @ts-ignore
                    locationWrapper.href = _value; // eslint-disable-line no-restricted-properties
                    return _value;
                }
                return null;
            },
            configurable: true
        });
    };

    return LocationAccessorsInstrumentation;
}(_base2.default);

exports.default = LocationAccessorsInstrumentation;
module.exports = exports['default'];