'use strict';

exports.__esModule = true;

var _destinationLocation = require('../../../utils/destination-location');

var _url = require('../../../utils/url');

var _url2 = require('../../../../utils/url');

var _nativeMethods = require('../../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _urlResolver = require('../../../utils/url-resolver');

var _urlResolver2 = _interopRequireDefault(_urlResolver);

var _index = require('../../../../processing/dom/index');

var _index2 = _interopRequireDefault(_index);

var _ancestorOriginsWrapper = require('./ancestor-origins-wrapper');

var _ancestorOriginsWrapper2 = _interopRequireDefault(_ancestorOriginsWrapper);

var _integerIdGenerator = require('../../../utils/integer-id-generator');

var _integerIdGenerator2 = _interopRequireDefault(_integerIdGenerator);

var _propertyOverriding = require('../../../utils/property-overriding');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var GET_ORIGIN_CMD = 'hammerhead|command|get-origin';
var ORIGIN_RECEIVED_CMD = 'hammerhead|command|origin-received';
function getLocationUrl(window) {
    try {
        return window.location.toString();
    } catch (e) {
        return void 0;
    }
}

var LocationWrapper = function LocationWrapper(window, messageSandbox, onChanged) {
    var _this = this;

    _classCallCheck(this, LocationWrapper);

    var parsedLocation = (0, _url.parseProxyUrl)(getLocationUrl(window));
    var locationResourceType = parsedLocation ? parsedLocation.resourceType : '';
    var parsedResourceType = (0, _url.parseResourceType)(locationResourceType);
    // @ts-ignore
    var isLocationPropsInProto = _nativeMethods2.default.objectHasOwnProperty.call(window.Location.prototype, 'href');
    // @ts-ignore
    var locationPropsOwner = isLocationPropsInProto ? window.Location.prototype : window.location;
    var locationProps = {};
    parsedResourceType.isIframe = parsedResourceType.isIframe || window !== window.top;
    var resourceType = (0, _url2.getResourceTypeString)({
        isIframe: parsedResourceType.isIframe,
        isForm: parsedResourceType.isForm
    });
    var getHref = function getHref() {
        // eslint-disable-next-line no-restricted-properties
        if (window !== window.top && window.location.href === 'about:blank') return 'about:blank';
        var locationUrl = (0, _destinationLocation.get)();
        var resolveElement = _urlResolver2.default.getResolverElement(window.document);
        _nativeMethods2.default.anchorHrefSetter.call(resolveElement, locationUrl);
        var href = _nativeMethods2.default.anchorHrefGetter.call(resolveElement);
        return (0, _url2.ensureTrailingSlash)(href, locationUrl);
    };
    var getProxiedHref = function getProxiedHref(href) {
        if (typeof href !== 'string') href = String(href);
        href = (0, _url2.prepareUrl)(href);
        if (_index2.default.isJsProtocol(href)) return _index2.default.processJsAttrValue(href, { isJsProtocol: true, isEventAttr: false });
        var locationUrl = getLocationUrl(window);
        var proxyPort = null;
        if (window !== window.parent) {
            var parentLocationUrl = getLocationUrl(window.parent);
            var parsedParentLocationUrl = (0, _url.parseProxyUrl)(parentLocationUrl);
            if (parsedParentLocationUrl && parsedParentLocationUrl.proxy) {
                // eslint-disable-next-line no-restricted-properties
                var parentProxyPort = parsedParentLocationUrl.proxy.port;
                proxyPort = (0, _url2.sameOriginCheck)(parentLocationUrl, href) ? parentProxyPort : (0, _url.getCrossDomainProxyPort)(parentProxyPort);
            }
        }
        var changedOnlyHash = locationUrl && (0, _url.isChangedOnlyHash)(locationUrl, href);
        var currentResourceType = changedOnlyHash ? locationResourceType : resourceType;
        return (0, _url.getProxyUrl)(href, { resourceType: currentResourceType, proxyPort: proxyPort });
    };
    // eslint-disable-next-line no-restricted-properties
    locationProps.href = (0, _propertyOverriding.createOverriddenDescriptor)(locationPropsOwner, 'href', {
        getter: getHref,
        setter: function setter(href) {
            var proxiedHref = getProxiedHref(href);
            // eslint-disable-next-line no-restricted-properties
            window.location.href = proxiedHref;
            onChanged(proxiedHref);
            return href;
        }
    });
    // eslint-disable-next-line no-restricted-properties
    locationProps.search = (0, _propertyOverriding.createOverriddenDescriptor)(locationPropsOwner, 'search', {
        // eslint-disable-next-line no-restricted-properties
        getter: function getter() {
            return window.location.search;
        },
        setter: function setter(search) {
            var newLocation = (0, _url.changeDestUrlPart)(window.location.toString(), _nativeMethods2.default.anchorSearchSetter, search, resourceType);
            window.location = newLocation;
            onChanged(newLocation);
            return search;
        }
    });
    // eslint-disable-next-line no-restricted-properties
    locationProps.origin = (0, _propertyOverriding.createOverriddenDescriptor)(locationPropsOwner, 'origin', {
        getter: function getter() {
            return (0, _url2.getDomain)((0, _destinationLocation.getParsed)());
        },
        setter: function setter(origin) {
            return origin;
        }
    });
    locationProps.hash = (0, _propertyOverriding.createOverriddenDescriptor)(locationPropsOwner, 'hash', {
        getter: function getter() {
            return window.location.hash;
        },
        setter: function setter(hash) {
            window.location.hash = hash;
            return hash;
        }
    });
    if (window.location.ancestorOrigins) {
        var callbacks = _nativeMethods2.default.objectCreate(null);
        var idGenerator = (0, _integerIdGenerator2.default)();
        var getCrossDomainOrigin = function getCrossDomainOrigin(win, callback) {
            var id = idGenerator.increment();
            callbacks[id] = callback;
            messageSandbox.sendServiceMsg({ id: id, cmd: GET_ORIGIN_CMD }, win);
        };
        messageSandbox.on(messageSandbox.SERVICE_MSG_RECEIVED_EVENT, function (_ref) {
            var message = _ref.message,
                source = _ref.source;

            if (message.cmd === GET_ORIGIN_CMD) {
                // @ts-ignore
                messageSandbox.sendServiceMsg({ id: message.id, cmd: ORIGIN_RECEIVED_CMD, origin: _this.origin }, source); // eslint-disable-line no-restricted-properties
            } else if (message.cmd === ORIGIN_RECEIVED_CMD) {
                var callback = callbacks[message.id];
                if (callback) callback(message.origin); // eslint-disable-line no-restricted-properties
            }
        });
        var ancestorOrigins = new _ancestorOriginsWrapper2.default(window, getCrossDomainOrigin);
        locationProps.ancestorOrigins = (0, _propertyOverriding.createOverriddenDescriptor)(locationPropsOwner, 'ancestorOrigins', {
            getter: function getter() {
                return ancestorOrigins;
            }
        });
    }
    var createLocationPropertyDesc = function createLocationPropertyDesc(property, nativePropSetter) {
        locationProps[property] = (0, _propertyOverriding.createOverriddenDescriptor)(locationPropsOwner, property, {
            getter: function getter() {
                return (0, _destinationLocation.getParsed)()[property];
            },
            setter: function setter(value) {
                var newLocation = (0, _url.changeDestUrlPart)(window.location.toString(), nativePropSetter, value, resourceType);
                window.location = newLocation;
                onChanged(newLocation);
                return value;
            }
        });
    };
    createLocationPropertyDesc('port', _nativeMethods2.default.anchorPortSetter);
    createLocationPropertyDesc('host', _nativeMethods2.default.anchorHostSetter);
    createLocationPropertyDesc('hostname', _nativeMethods2.default.anchorHostnameSetter);
    createLocationPropertyDesc('pathname', _nativeMethods2.default.anchorPathnameSetter);
    createLocationPropertyDesc('protocol', _nativeMethods2.default.anchorProtocolSetter);
    locationProps.assign = (0, _propertyOverriding.createOverriddenDescriptor)(locationPropsOwner, 'assign', {
        value: function value(url) {
            var proxiedHref = getProxiedHref(url);
            var result = window.location.assign(proxiedHref);
            onChanged(proxiedHref);
            return result;
        }
    });
    locationProps.replace = (0, _propertyOverriding.createOverriddenDescriptor)(locationPropsOwner, 'replace', {
        value: function value(url) {
            var proxiedHref = getProxiedHref(url);
            var result = window.location.replace(proxiedHref);
            onChanged(proxiedHref);
            return result;
        }
    });
    locationProps.reload = (0, _propertyOverriding.createOverriddenDescriptor)(locationPropsOwner, 'reload', {
        value: function value(forcedReload) {
            var result = window.location.reload(forcedReload);
            onChanged(window.location.toString());
            return result;
        }
    });
    locationProps.toString = (0, _propertyOverriding.createOverriddenDescriptor)(locationPropsOwner, 'toString', { value: getHref });
    if (!isLocationPropsInProto && _nativeMethods2.default.objectHasOwnProperty.call(window.location, 'valueOf')) locationProps.valueOf = (0, _propertyOverriding.createOverriddenDescriptor)(locationPropsOwner, 'valueOf', { value: function value() {
            return _this;
        } });
    _nativeMethods2.default.objectDefineProperties(this, locationProps);
};

exports.default = LocationWrapper;
module.exports = exports['default'];