'use strict';

exports.__esModule = true;

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _instruction = require('../../../processing/script/instruction');

var _instruction2 = _interopRequireDefault(_instruction);

var _instrumented = require('../../../processing/script/instrumented');

var _dom = require('../../utils/dom');

var _fastApply = require('../../utils/fast-apply');

var _fastApply2 = _interopRequireDefault(_fastApply);

var _types = require('../../utils/types');

var typeUtils = _interopRequireWildcard(_types);

var _url = require('../../utils/url');

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var MethodCallInstrumentation = function (_SandboxBase) {
    _inherits(MethodCallInstrumentation, _SandboxBase);

    function MethodCallInstrumentation(messageSandbox) {
        _classCallCheck(this, MethodCallInstrumentation);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.messageSandbox = messageSandbox;
        _this.methodWrappers = {
            postMessage: {
                condition: _dom.isWindow,
                method: function method(contentWindow, args) {
                    return messageSandbox.postMessage(contentWindow, args);
                }
            },
            // NOTE: We cannot get the location wrapper for a cross-domain window. Therefore, we need to
            // intercept calls to the native 'replace' method.
            replace: {
                condition: _dom.isLocation,
                method: function method(location, args) {
                    return location.replace((0, _url.getProxyUrl)(args[0], {
                        resourceType: MethodCallInstrumentation._getLocationResourceType(location)
                    }));
                }
            },
            // NOTE: We cannot get the location wrapper for a cross-domain window. Therefore, we need to
            // intercept calls to the native 'assign' method.
            assign: {
                condition: _dom.isLocation,
                method: function method(location, args) {
                    return location.assign((0, _url.getProxyUrl)(args[0], {
                        resourceType: MethodCallInstrumentation._getLocationResourceType(location)
                    }));
                }
            }
        };
        return _this;
    }
    // NOTE: Isolate throw statement into a separate function because JS engine doesn't optimize such functions.


    MethodCallInstrumentation._error = function _error(msg) {
        throw new Error(msg);
    };

    MethodCallInstrumentation._getLocationResourceType = function _getLocationResourceType(location) {
        return window.top.location === location ? null : (0, _url.stringifyResourceType)({ isIframe: true });
    };

    MethodCallInstrumentation._isPostMessageFn = function _isPostMessageFn(win, fn) {
        // NOTE: in iOS Safari 9.3 win.postMessage === win.postMessage equals false
        if (win.postMessage === win.postMessage) return win.postMessage === fn;
        return fn && typeof fn.toString === 'function' && fn.toString() === win.postMessage.toString();
    };

    MethodCallInstrumentation.prototype.attach = function attach(window) {
        var _this2 = this;

        _SandboxBase.prototype.attach.call(this, window);
        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
        _nativeMethods2.default.objectDefineProperty(window, _instruction2.default.callMethod, {
            value: function value(owner, methName, args) {
                if (typeUtils.isNullOrUndefined(owner)) MethodCallInstrumentation._error('Cannot call method \'' + methName + '\' of ' + typeUtils.inaccessibleTypeToStr(owner));
                if (typeof owner[methName] !== 'function') MethodCallInstrumentation._error('\'' + methName + '\' is not a function');
                // OPTIMIZATION: previously we've performed the
                // `this.methodWrappers.hasOwnProperty(methName)`
                // check which is quite slow. Now we use the
                // fast RegExp check instead.
                if (typeof methName === 'string' && (0, _instrumented.shouldInstrumentMethod)(methName) && _this2.methodWrappers[methName].condition(owner)) return _this2.methodWrappers[methName].method(owner, args);
                return (0, _fastApply2.default)(owner, methName, args);
            },
            configurable: true
        });
        var methodCallInstrumentation = this;
        _nativeMethods2.default.objectDefineProperty(window, _instruction2.default.getPostMessage, {
            value: function value(win, postMessageFn) {
                if (arguments.length === 1 && !(0, _dom.isWindow)(win)) return win.postMessage;
                if (arguments.length === 2 && !MethodCallInstrumentation._isPostMessageFn(this, postMessageFn)) return postMessageFn;
                return function () {
                    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                        args[_key] = arguments[_key];
                    }

                    return methodCallInstrumentation.messageSandbox.postMessage(this, args);
                };
            },
            configurable: true
        });
    };

    return MethodCallInstrumentation;
}(_base2.default);

exports.default = MethodCallInstrumentation;
module.exports = exports['default'];