'use strict';

exports.__esModule = true;

var _internalProperties = require('../../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _location = require('../location');

var _location2 = _interopRequireDefault(_location);

var _wrapper = require('../location/wrapper');

var _wrapper2 = _interopRequireDefault(_wrapper);

var _base = require('../../base');

var _base2 = _interopRequireDefault(_base);

var _dom = require('../../../utils/dom');

var domUtils = _interopRequireWildcard(_dom);

var _types = require('../../../utils/types');

var typeUtils = _interopRequireWildcard(_types);

var _url = require('../../../utils/url');

var urlUtils = _interopRequireWildcard(_url);

var _url2 = require('../../../../utils/url');

var _instruction = require('../../../../processing/script/instruction');

var _instruction2 = _interopRequireDefault(_instruction);

var _instrumented = require('../../../../processing/script/instrumented');

var _nativeMethods = require('../../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dom2 = require('../../../../processing/dom');

var _dom3 = _interopRequireDefault(_dom2);

var _settings = require('../../../settings');

var _settings2 = _interopRequireDefault(_settings);

var _browser = require('../../../utils/browser');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PropertyAccessorsInstrumentation = function (_SandboxBase) {
    _inherits(PropertyAccessorsInstrumentation, _SandboxBase);

    function PropertyAccessorsInstrumentation(windowSandbox) {
        _classCallCheck(this, PropertyAccessorsInstrumentation);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.windowSandbox = windowSandbox;
        return _this;
    }
    // NOTE: Isolate throw statements into a separate function because the
    // JS engine doesn't optimize such functions.


    PropertyAccessorsInstrumentation._error = function _error(msg) {
        throw new Error(msg);
    };

    PropertyAccessorsInstrumentation._safeIsShadowUIElement = function _safeIsShadowUIElement(el) {
        try {
            return domUtils.isShadowUIElement(el);
        } catch (e) {
            return false;
        }
    };

    PropertyAccessorsInstrumentation._setCrossDomainLocation = function _setCrossDomainLocation(location, value) {
        var proxyUrl = '';
        if (typeof value !== 'string') value = String(value);
        if (!_dom3.default.isJsProtocol(value)) {
            var resourceType = urlUtils.stringifyResourceType({ isIframe: true });
            value = (0, _url2.prepareUrl)(value);
            proxyUrl = location !== window.top.location ? urlUtils.getProxyUrl(value, { resourceType: resourceType }) : urlUtils.getProxyUrl(value, { proxyPort: _settings2.default.get().crossDomainProxyPort });
        } else proxyUrl = _dom3.default.processJsAttrValue(value, { isJsProtocol: true, isEventAttr: false });
        location.href = proxyUrl; // eslint-disable-line no-restricted-properties
        return value;
    };

    PropertyAccessorsInstrumentation.prototype._createPropertyAccessors = function _createPropertyAccessors() {
        return {
            href: {
                condition: domUtils.isLocation,
                // eslint-disable-next-line no-restricted-properties
                get: function get(crossDomainLocation) {
                    return crossDomainLocation.href;
                },
                set: PropertyAccessorsInstrumentation._setCrossDomainLocation
            },
            location: {
                condition: function condition(owner) {
                    return domUtils.isDocument(owner) || domUtils.isWindow(owner);
                },
                get: function get(owner) {
                    var locationWrapper = _location2.default.getLocationWrapper(owner);
                    if (locationWrapper) return locationWrapper;
                    var wnd = domUtils.isWindow(owner) ? owner : owner.defaultView;
                    return new _wrapper2.default(wnd);
                },
                set: function set(owner, location) {
                    var ownerWindow = domUtils.isWindow(owner) ? owner : owner.defaultView;
                    var locationWrapper = _location2.default.getLocationWrapper(ownerWindow);
                    if (!locationWrapper || locationWrapper === owner.location || _browser.isIE && domUtils.isCrossDomainWindows(window, ownerWindow)) PropertyAccessorsInstrumentation._setCrossDomainLocation(owner.location, location);else if (locationWrapper) locationWrapper.href = location; // eslint-disable-line no-restricted-properties
                    return location;
                }
            }
        };
    };

    PropertyAccessorsInstrumentation._getSetPropertyInstructionByOwner = function _getSetPropertyInstructionByOwner(owner, window) {
        try {
            return owner && owner[_internalProperties2.default.processedContext] && owner[_internalProperties2.default.processedContext] !== window && owner[_internalProperties2.default.processedContext][_instruction2.default.setProperty];
        } catch (e) {
            return null;
        }
    };

    PropertyAccessorsInstrumentation.prototype.attach = function attach(window) {
        _SandboxBase.prototype.attach.call(this, window);
        var accessors = this._createPropertyAccessors();
        var windowSandbox = this.windowSandbox;
        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
        _nativeMethods2.default.objectDefineProperty(window, _instruction2.default.getProperty, {
            value: function value(owner, propName) {
                if (typeUtils.isNullOrUndefined(owner)) PropertyAccessorsInstrumentation._error('Cannot read property \'' + propName + '\' of ' + typeUtils.inaccessibleTypeToStr(owner));
                if (typeof propName === 'string' && (0, _instrumented.shouldInstrumentProperty)(propName) && accessors[propName].condition(owner)) return accessors[propName].get(owner);
                var propertyValue = owner[propName];
                windowSandbox.isInternalGetter = true;
                if (propertyValue && PropertyAccessorsInstrumentation._safeIsShadowUIElement(propertyValue)) return void 0;
                windowSandbox.isInternalGetter = false;
                return propertyValue;
            },
            configurable: true
        });
        _nativeMethods2.default.objectDefineProperty(window, _instruction2.default.setProperty, {
            value: function value(owner, propName, _value) {
                if (typeUtils.isNullOrUndefined(owner)) PropertyAccessorsInstrumentation._error('Cannot set property \'' + propName + '\' of ' + typeUtils.inaccessibleTypeToStr(owner));
                var ownerSetPropertyInstruction = PropertyAccessorsInstrumentation._getSetPropertyInstructionByOwner(owner, window);
                if (ownerSetPropertyInstruction) return ownerSetPropertyInstruction(owner, propName, _value);
                if (typeof propName === 'string' && (0, _instrumented.shouldInstrumentProperty)(propName) && accessors[propName].condition(owner)) return accessors[propName].set(owner, _value);
                // eslint-disable-next-line no-return-assign
                return owner[propName] = _value;
            },
            configurable: true
        });
        return accessors;
    };

    return PropertyAccessorsInstrumentation;
}(_base2.default);

exports.default = PropertyAccessorsInstrumentation;
module.exports = exports['default'];