'use strict';

exports.__esModule = true;

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _settings = require('../../settings');

var _settings2 = _interopRequireDefault(_settings);

var _windowSync = require('./window-sync');

var _windowSync2 = _interopRequireDefault(_windowSync);

var _destinationLocation = require('../../utils/destination-location');

var destLocation = _interopRequireWildcard(_destinationLocation);

var _cookie = require('../../utils/cookie');

var cookieUtils = _interopRequireWildcard(_cookie);

var _stringTrim = require('../../../utils/string-trim');

var _stringTrim2 = _interopRequireDefault(_stringTrim);

var _cookieLimit = require('../../../session/cookie-limit');

var _cookieLimit2 = _interopRequireDefault(_cookieLimit);

var _nativeMethods = require('../../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _cookie2 = require('../../../utils/cookie');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var MIN_DATE_VALUE = new _nativeMethods2.default.date(0).toUTCString(); // eslint-disable-line new-cap

var CookieSandbox = function (_SandboxBase) {
    _inherits(CookieSandbox, _SandboxBase);

    function CookieSandbox(messageSandbox) {
        _classCallCheck(this, CookieSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.messageSandbox = messageSandbox;
        _this.windowSync = null;
        _this.pendingWindowSync = [];
        return _this;
    }

    CookieSandbox.prototype._canSetCookie = function _canSetCookie(cookie, setByClient) {
        // eslint-disable-next-line no-restricted-properties
        if (setByClient && (cookie.length > _cookieLimit2.default || destLocation.getParsed().protocol === 'file:')) return false;
        var clientCookie = 'key' + _nativeMethods2.default.mathRandom.call(_nativeMethods2.default.math) + '=value';
        _nativeMethods2.default.documentCookieSetter.call(this.document, clientCookie);
        var documentCookieIsEmpty = !_nativeMethods2.default.documentCookieGetter.call(this.document);
        if (!documentCookieIsEmpty) _nativeMethods2.default.documentCookieSetter.call(this.document, clientCookie + ';expires=' + MIN_DATE_VALUE);
        return !documentCookieIsEmpty;
    };

    CookieSandbox._updateClientCookieStr = function _updateClientCookieStr(cookieKey, newCookieStr) {
        var cookieStr = _settings2.default.get().cookie; // eslint-disable-line no-restricted-properties
        var cookies = cookieStr ? cookieStr.split(';') : [];
        var changedCookies = [];
        var replaced = false;
        var searchStr = cookieKey === '' ? null : cookieKey + '=';
        // NOTE: Replace a cookie if it already exists.

        for (var _i2 = 0; _i2 < cookies.length; _i2++) {
            var cookie = cookies[_i2];
            cookie = (0, _stringTrim2.default)(cookie);
            var isCookieExists = searchStr ? cookie.indexOf(searchStr) === 0 : cookie.indexOf('=') === -1;
            if (!isCookieExists) changedCookies.push(cookie);else if (newCookieStr !== null) {
                changedCookies.push(newCookieStr);
                replaced = true;
            }
        }

        if (!replaced && newCookieStr !== null) changedCookies.push(newCookieStr);
        _settings2.default.get().cookie = changedCookies.join('; '); // eslint-disable-line no-restricted-properties
    };

    CookieSandbox.prototype.getCookie = function getCookie() {
        this.syncCookie();
        // eslint-disable-next-line no-restricted-properties
        return _settings2.default.get().cookie || '';
    };

    CookieSandbox.prototype.setCookie = function setCookie(_document, cookie) {
        var setByClient = typeof cookie === 'string';
        // NOTE: Cookie cannot be set in iframe without src in IE
        // Also cookie cannot be set on a page with 'file:' protocol
        // or if the length of cookie higher than limit
        if (!this._canSetCookie(cookie, setByClient)) return;
        var parsedCookie = setByClient ? cookieUtils.parse(cookie) : cookie;
        if (!parsedCookie || parsedCookie.httpOnly) return;
        var parsedDestLocation = destLocation.getParsed();
        // NOTE: All Hammerhad sessions have the same domain, so we need to validate the Domain attribute manually
        // according to a test url.
        // eslint-disable-next-line no-restricted-properties
        if (!cookieUtils.domainMatch(parsedDestLocation.hostname, parsedCookie.domain)) return;
        // eslint-disable-next-line no-restricted-properties
        if ((!parsedCookie.secure || parsedDestLocation.protocol === 'https:') &&
        // eslint-disable-next-line no-restricted-properties
        cookieUtils.pathMatch(parsedDestLocation.pathname, parsedCookie.path)) {
            var currentDate = cookieUtils.getUTCDate();
            var clientCookieStr = null;
            if (!parsedCookie.expires || parsedCookie.expires === 'Infinity' || parsedCookie.expires > currentDate) clientCookieStr = cookieUtils.formatClientString(parsedCookie);
            CookieSandbox._updateClientCookieStr(parsedCookie.key, clientCookieStr);
        }
        if (setByClient) {
            cookieUtils.setDefaultValues(parsedCookie, parsedDestLocation);
            this._syncClientCookie(parsedCookie);
            this.syncCookie();
        }
    };

    CookieSandbox.prototype.syncCookie = function syncCookie() {
        var cookies = _nativeMethods2.default.documentCookieGetter.call(this.document);
        var parsedCookies = (0, _cookie2.parseClientSyncCookieStr)(cookies);
        var sessionId = _settings2.default.get().sessionId;
        var serverSyncCookies = [];

        for (var _i4 = 0, _parsedCookies$outdat2 = parsedCookies.outdated; _i4 < _parsedCookies$outdat2.length; _i4++) {
            var outdatedCookie = _parsedCookies$outdat2[_i4];
            _nativeMethods2.default.documentCookieSetter.call(this.document, (0, _cookie2.generateDeleteSyncCookieStr)(outdatedCookie));
        }

        for (var _i6 = 0, _parsedCookies$actual2 = parsedCookies.actual; _i6 < _parsedCookies$actual2.length; _i6++) {
            var parsedCookie = _parsedCookies$actual2[_i6];
            if (parsedCookie.sid !== sessionId) continue;
            if (parsedCookie.isServerSync) serverSyncCookies.push(parsedCookie);else if (parsedCookie.isWindowSync) this.setCookie(this.document, parsedCookie);
        }

        this._syncServerCookie(serverSyncCookies);
    };

    CookieSandbox.prototype._syncServerCookie = function _syncServerCookie(parsedCookies) {
        var _this2 = this;

        for (var _i8 = 0; _i8 < parsedCookies.length; _i8++) {
            var parsedCookie = parsedCookies[_i8];
            this.setCookie(this.document, parsedCookie);
            _nativeMethods2.default.documentCookieSetter.call(this.document, (0, _cookie2.generateDeleteSyncCookieStr)(parsedCookie));
            (0, _cookie2.changeSyncType)(parsedCookie, { server: false, window: true });
            _nativeMethods2.default.documentCookieSetter.call(this.document, (0, _cookie2.formatSyncCookie)(parsedCookie));
        }

        this.windowSync.syncBetweenWindows(parsedCookies, null, function () {
            for (var _i10 = 0; _i10 < parsedCookies.length; _i10++) {
                var _parsedCookie = parsedCookies[_i10];
                _nativeMethods2.default.documentCookieSetter.call(_this2.document, (0, _cookie2.generateDeleteSyncCookieStr)(_parsedCookie));
            }
        });
    };

    CookieSandbox.prototype._syncClientCookie = function _syncClientCookie(parsedCookie) {
        var _this3 = this;

        parsedCookie.isClientSync = true;
        parsedCookie.isWindowSync = true;
        parsedCookie.sid = _settings2.default.get().sessionId;
        parsedCookie.lastAccessed = new _nativeMethods2.default.date(); // eslint-disable-line new-cap
        (0, _cookie2.prepareSyncCookieProperties)(parsedCookie);
        _nativeMethods2.default.documentCookieSetter.call(this.document, (0, _cookie2.formatSyncCookie)(parsedCookie));
        this.windowSync.syncBetweenWindows([parsedCookie], null, function () {
            _nativeMethods2.default.documentCookieSetter.call(_this3.document, (0, _cookie2.generateDeleteSyncCookieStr)(parsedCookie));
            (0, _cookie2.changeSyncType)(parsedCookie, { window: false });
            _nativeMethods2.default.documentCookieSetter.call(_this3.document, (0, _cookie2.formatSyncCookie)(parsedCookie));
        });
    };

    CookieSandbox.prototype._processPendingWindowSync = function _processPendingWindowSync() {
        for (var _i12 = 0, _pendingWindowSync2 = this.pendingWindowSync; _i12 < _pendingWindowSync2.length; _i12++) {
            var _pendingWindowSync2$_ = _pendingWindowSync2[_i12],
                parsedCookies = _pendingWindowSync2$_.parsedCookies,
                win = _pendingWindowSync2$_.win,
                resolve = _pendingWindowSync2$_.resolve;

            var syncResultPromise = this.syncWindowCookie(parsedCookies, win);
            if (syncResultPromise) syncResultPromise.then(resolve);else resolve();
        }

        this.pendingWindowSync = [];
    };

    CookieSandbox.prototype.syncWindowCookie = function syncWindowCookie(parsedCookies, win) {
        var _this4 = this;

        // NOTE: This function can be called before the 'attach' call.
        if (!this.document) return new _pinkie2.default(function (resolve) {
            return _this4.pendingWindowSync.push({ parsedCookies: parsedCookies, win: win, resolve: resolve });
        });
        var clientCookie = _nativeMethods2.default.documentCookieGetter.call(this.document);
        var actualCookies = [];

        for (var _i14 = 0; _i14 < parsedCookies.length; _i14++) {
            var parsedCookie = parsedCookies[_i14];
            var startIndex = clientCookie.indexOf(parsedCookie.cookieStr);
            var endIndex = startIndex + parsedCookie.cookieStr.length;
            if (startIndex > -1 && (clientCookie.length === endIndex || clientCookie.charAt(endIndex) === ';')) {
                this.setCookie(this.document, parsedCookie);
                actualCookies.push(parsedCookie);
            }
        }

        return this.windowSync.syncBetweenWindows(actualCookies, win);
    };

    CookieSandbox.prototype.attach = function attach(window) {
        _SandboxBase.prototype.attach.call(this, window);
        this.windowSync = new _windowSync2.default(window, this, this.messageSandbox);
        this._processPendingWindowSync();
    };

    return CookieSandbox;
}(_base2.default);

exports.default = CookieSandbox;
module.exports = exports['default'];