'use strict';

exports.__esModule = true;

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _internalProperties = require('../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _integerIdGenerator = require('../../utils/integer-id-generator');

var _integerIdGenerator2 = _interopRequireDefault(_integerIdGenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var SYNC_COOKIE_START_CMD = 'hammerhead|command|sync-cookie-start';
var SYNC_COOKIE_DONE_CMD = 'hammerhead|command|sync-cookie-done';

var WindowSync = function () {
    function WindowSync(win, cookieSandbox, messageSandbox) {
        var _this = this;

        _classCallCheck(this, WindowSync);

        this.win = win;
        this.cookieSandbox = cookieSandbox;
        this.messageSandbox = messageSandbox;
        this.messageIdGenerator = (0, _integerIdGenerator2.default)();
        this.resolversMap = {};
        messageSandbox.on(messageSandbox.SERVICE_MSG_RECEIVED_EVENT, function (_ref) {
            var message = _ref.message,
                source = _ref.source;

            if (message.cmd === SYNC_COOKIE_START_CMD) {
                var syncResultPromise = _this.cookieSandbox.syncWindowCookie(message.cookies, source);
                var callback = function callback() {
                    return _this.messageSandbox.sendServiceMsg({
                        id: message.id,
                        cmd: SYNC_COOKIE_DONE_CMD
                    }, source);
                };
                if (syncResultPromise) syncResultPromise.then(callback);else callback();
            }
            // NOTE: We need to remove the second part of the condition after a fix of GH-1715
            else if (message.cmd === SYNC_COOKIE_DONE_CMD && _this.resolversMap[message.id]) {
                    _this.resolversMap[message.id]();
                    delete _this.resolversMap[message.id];
                }
        });
    }

    WindowSync._getCookieSandbox = function _getCookieSandbox(win) {
        try {
            // eslint-disable-next-line no-restricted-properties
            return win[_internalProperties2.default.hammerhead].sandbox.cookie;
        } catch (e) {
            return null;
        }
    };

    WindowSync.prototype._getWindowsForSync = function _getWindowsForSync(initiator) {
        var windows = [];
        if (this.win !== this.win.parent && this.win.parent !== initiator) windows.push(this.win.parent);

        for (var _i2 = 0, _win$window2 = this.win.window; _i2 < _win$window2.length; _i2++) {
            var frameWin = _win$window2[_i2];
            if (frameWin !== initiator) windows.push(frameWin);
        }

        return windows;
    };

    WindowSync.prototype.syncBetweenWindows = function syncBetweenWindows(cookies, initiator, callback) {
        if (!cookies.length) {
            if (callback) callback();
            return null;
        }
        var windowsForSync = this._getWindowsForSync(initiator);
        var syncMessages = [];

        for (var _i4 = 0; _i4 < windowsForSync.length; _i4++) {
            var win = windowsForSync[_i4];
            var cookieSandbox = WindowSync._getCookieSandbox(win);
            if (cookieSandbox) {
                var syncResultPromise = cookieSandbox.syncWindowCookie(cookies, this.win);
                if (syncResultPromise) syncMessages.push(syncResultPromise);
            } else syncMessages.push(this.sendSyncMessage(win, SYNC_COOKIE_START_CMD, cookies));
        }

        if (syncMessages.length) {
            var promiseAllMessages = _pinkie2.default.all(syncMessages);
            return callback ? promiseAllMessages.then(callback) : promiseAllMessages;
        }
        if (callback) callback();
        return null;
    };

    WindowSync.prototype.sendSyncMessage = function sendSyncMessage(win, cmd, cookies) {
        var _this2 = this;

        var id = this.messageIdGenerator.increment();
        return new _pinkie2.default(function (resolve) {
            _this2.resolversMap[id] = resolve;
            _this2.messageSandbox.sendServiceMsg({ id: id, cmd: cmd, cookies: cookies }, win);
        });
    };

    return WindowSync;
}();

exports.default = WindowSync;
module.exports = exports['default'];