'use strict';

exports.__esModule = true;

var _nativeMethods = require('../../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dataTransferItem = require('./data-transfer-item');

var _dataTransferItem2 = _interopRequireDefault(_dataTransferItem);

var _dataStoreMode = require('./data-store-mode');

var _dataStoreMode2 = _interopRequireDefault(_dataStoreMode);

var _dataTransferItemKind = require('./data-transfer-item-kind');

var _dataTransferItemKind2 = _interopRequireDefault(_dataTransferItemKind);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function parseTextUriList(textUriList) {
    textUriList = textUriList.replace(/\r\n$/, '');
    var res = [];
    if (textUriList === '') return res;
    textUriList = textUriList.split(/\r\n/);

    for (var _i2 = 0, _textUriList2 = textUriList; _i2 < _textUriList2.length; _i2++) {
        var textUri = _textUriList2[_i2];
        if (textUri !== '#') res.push(textUri);
    }

    return res;
}
function processFormat(format) {
    if (format === 'text') return 'text/plain';
    if (format === 'url') return 'text/uri-list';
    return format;
}
// https://html.spec.whatwg.org/multipage/interaction.html#datatransferitemlist

var DataTransferItemList = function DataTransferItemList(dataStore) {
    var _this = this;

    _classCallCheck(this, DataTransferItemList);

    // Internals
    var items = [];
    var itemsData = [];
    var getTypes = function getTypes() {
        var res = [];

        for (var _i4 = 0, _items2 = items; _i4 < _items2.length; _i4++) {
            var item = _items2[_i4];
            res.push(item.type);
        }

        return res;
    };
    var updateIndexes = function updateIndexes() {
        var idx = 0;
        while (items[idx] !== void 0 || _this[idx] !== void 0) {
            var item = items[idx];
            _nativeMethods2.default.objectDefineProperty(_this, idx, {
                enumerable: item !== void 0,
                configurable: true,
                value: item
            });
            idx++;
        }
    };
    var getItemData = function getItemData(format) {
        var convertToUrl = false;
        format = processFormat(format);
        if (format === 'url') convertToUrl = true;
        var item = '';
        for (var i = 0; i < items.length; i++) {
            if (items[i].type === format) item = itemsData[i];
        }
        if (convertToUrl && item) item = parseTextUriList(item)[0];
        return item;
    };
    var removeItem = function removeItem(format) {
        format = processFormat(format);
        for (var i = 0; i < items.length; i++) {
            if (items[i].type === format) {
                items.splice(i, 1);
                itemsData.splice(i, 1);
                break;
            }
        }
        updateIndexes();
    };
    var addItem = function addItem(data, type, allowReplace) {
        var newItem = null;
        if (typeof data === 'string') {
            var typeLowerCase = type.toString().toLowerCase();
            var itemData = getItemData(typeLowerCase);
            if (!allowReplace && itemData) throw new Error('Failed to execute \'add\' on \'DataTransferItemList\': An item already exists for type \'' + typeLowerCase + '\'.');
            if (itemData) removeItem(typeLowerCase);
            newItem = new _dataTransferItem2.default(_dataTransferItemKind2.default.string, processFormat(type), data);
        } else newItem = new _dataTransferItem2.default(_dataTransferItemKind2.default.file, null, data);
        items.push(newItem);
        itemsData.push(data);
        updateIndexes();
        return newItem;
    };
    // Internal API
    this.getAndHideInternalMethods = function () {
        var res = { getTypes: getTypes, getItemData: getItemData, removeItem: removeItem, addItem: addItem };
        delete _this.getAndHideInternalMethods;
        return res;
    };
    // API
    _nativeMethods2.default.objectDefineProperty(this, 'length', {
        enumerable: true,
        get: function get() {
            return items.length;
        }
    });
    _nativeMethods2.default.objectDefineProperty(this, 'remove', {
        configurable: true,
        enumerable: true,
        get: function get() {
            return function (idx) {
                if (dataStore.mode !== _dataStoreMode2.default.readwrite) return;
                items.splice(idx, 1);
                itemsData.splice(idx, 1);
                updateIndexes();
            };
        }
    });
    _nativeMethods2.default.objectDefineProperty(this, 'clear', {
        configurable: true,
        enumerable: true,
        get: function get() {
            return function () {
                if (dataStore.mode !== _dataStoreMode2.default.readwrite) return;
                items = [];
                itemsData = [];
                updateIndexes();
            };
        }
    });
    _nativeMethods2.default.objectDefineProperty(this, 'add', {
        configurable: true,
        enumerable: true,
        get: function get() {
            return function (data, type) {
                if (!arguments.length) throw new Error("Failed to execute 'add' on 'DataTransferItemList': 1 argument required, but only 0 present.");
                if (arguments.length === 1 && typeof data === 'string') throw new Error("Failed to execute 'add' on 'DataTransferItemList': parameter 1 is not of type 'File'.");
                if (dataStore.mode !== _dataStoreMode2.default.readwrite) return void 0;
                return addItem(data, type, false);
            };
        }
    });
};

exports.default = DataTransferItemList;

if (_nativeMethods2.default.DataTransferItemList) DataTransferItemList.prototype = _nativeMethods2.default.DataTransferItemList.prototype;
module.exports = exports['default'];