'use strict';

exports.__esModule = true;

var _browser = require('../../../utils/browser');

var _featureDetection = require('../../../utils/feature-detection');

var _nativeMethods = require('../../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dataTransferItemList = require('./data-transfer-item-list');

var _dataTransferItemList2 = _interopRequireDefault(_dataTransferItemList);

var _fileList = require('./file-list');

var _fileList2 = _interopRequireDefault(_fileList);

var _dataStoreMode = require('./data-store-mode');

var _dataStoreMode2 = _interopRequireDefault(_dataStoreMode);

var _dropEffect = require('./drop-effect');

var _dropEffect2 = _interopRequireDefault(_dropEffect);

var _effectAllowed = require('./effect-allowed');

var _effectAllowed2 = _interopRequireDefault(_effectAllowed);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } } // NOTE: Although DataTransfer interface has a constructor, it is not possible to
// create a useful DataTransfer object from script, since DataTransfer objects have a
// processing and security model that is coordinated by the browser during drag-and-drops.
// So we have to create a mock for it to use it in drag-and-drop events


// https://html.spec.whatwg.org/multipage/interaction.html#datatransfer
var DataTransfer = function DataTransfer(dataStore) {
    _classCallCheck(this, DataTransfer);

    var dropEffect = _dropEffect2.default.none;
    var effectAllowed = _effectAllowed2.default.uninitialized;
    var itemList = new _dataTransferItemList2.default(dataStore);
    var itemListInternals = itemList.getAndHideInternalMethods();
    var fileList = new _fileList2.default();
    var emptyItemList = new _dataTransferItemList2.default(dataStore);
    var emptyListInternals = emptyItemList.getAndHideInternalMethods();
    var getActualItemList = function getActualItemList() {
        return dataStore.mode === _dataStoreMode2.default.protected ? emptyItemList : itemList;
    };
    var getActualItemListInternals = function getActualItemListInternals() {
        return dataStore.mode === _dataStoreMode2.default.protected ? emptyListInternals : itemListInternals;
    };
    _nativeMethods2.default.objectDefineProperty(this, 'dropEffect', {
        configurable: true,
        enumerable: true,
        get: function get() {
            return dropEffect;
        },
        set: function set(value) {
            if (_dropEffect2.default[value]) dropEffect = _dropEffect2.default[value];
            return value;
        }
    });
    _nativeMethods2.default.objectDefineProperty(this, 'effectAllowed', {
        configurable: true,
        enumerable: true,
        get: function get() {
            return effectAllowed;
        },
        set: function set(value) {
            if (_effectAllowed2.default[value]) effectAllowed = _effectAllowed2.default[value];
            return value;
        }
    });
    if (!_browser.isIE11) {
        _nativeMethods2.default.objectDefineProperty(this, 'items', {
            configurable: true,
            enumerable: true,
            get: getActualItemList
        });
    }
    _nativeMethods2.default.objectDefineProperty(this, 'types', {
        configurable: true,
        enumerable: true,
        get: function get() {
            return getActualItemListInternals().getTypes();
        }
    });
    _nativeMethods2.default.objectDefineProperty(this, 'files', {
        configurable: true,
        enumerable: true,
        get: function get() {
            return fileList;
        }
    });
    if (!_browser.isIE11) {
        this.setDragImage = function () {
            // do nothing
        };
    }
    this.getData = function (format) {
        if (!arguments.length) throw new Error("Failed to execute 'getData' on 'DataTransfer': 1 argument required, but only 0 present.");
        format = format.toString().toLowerCase();
        return getActualItemListInternals().getItemData(format);
    };
    this.setData = function (format, data) {
        if (arguments.length < 2) throw new Error('Failed to execute \'setData\' on \'DataTransfer\': 2 argument required, but only ' + arguments.length + ' present.');
        if (dataStore.mode !== _dataStoreMode2.default.readwrite) return;
        format = format.toString().toLowerCase();
        itemListInternals.addItem(data, format, true);
    };
    this.clearData = function (format) {
        if (dataStore.mode !== _dataStoreMode2.default.readwrite) return;
        if (format === void 0)
            // @ts-ignore
            itemList.clear();else itemListInternals.removeItem(format);
    };
};

exports.default = DataTransfer;

if (_featureDetection.hasDataTransfer) DataTransfer.prototype = _nativeMethods2.default.DataTransfer.prototype;
module.exports = exports['default'];