'use strict';

exports.__esModule = true;

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dom = require('../../utils/dom');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ELEMENT_EDITING_OBSERVED_FLAG = 'hammerhead|element-editing-observed';
var OLD_VALUE_PROPERTY = 'hammerhead|old-value';

var ElementEditingWatcher = function () {
    function ElementEditingWatcher(eventSimulator) {
        _classCallCheck(this, ElementEditingWatcher);

        this.eventSimulator = eventSimulator;
    }

    ElementEditingWatcher.prototype._onBlur = function _onBlur(e) {
        if (!this.processElementChanging(e.target)) this.stopWatching(e.target);
    };

    ElementEditingWatcher.prototype._onChange = function _onChange(e) {
        this.stopWatching(e.target);
    };

    ElementEditingWatcher._getValue = function _getValue(el) {
        if ((0, _dom.isInputElement)(el)) return _nativeMethods2.default.inputValueGetter.call(el);else if ((0, _dom.isTextAreaElement)(el)) return _nativeMethods2.default.textAreaValueGetter.call(el);
        // eslint-disable-next-line no-restricted-properties
        return el.value;
    };

    ElementEditingWatcher.prototype.stopWatching = function stopWatching(el) {
        var _this = this;

        if (el) {
            _nativeMethods2.default.removeEventListener.call(el, 'blur', function (e) {
                return _this._onBlur(e);
            });
            _nativeMethods2.default.removeEventListener.call(el, 'change', function (e) {
                return _this._onChange(e);
            });
            if (el[ELEMENT_EDITING_OBSERVED_FLAG]) delete el[ELEMENT_EDITING_OBSERVED_FLAG];
            if (el[OLD_VALUE_PROPERTY]) delete el[OLD_VALUE_PROPERTY];
        }
    };

    ElementEditingWatcher.prototype.watchElementEditing = function watchElementEditing(el) {
        var _this2 = this;

        if (el && !el[ELEMENT_EDITING_OBSERVED_FLAG] && (0, _dom.isTextEditableElementAndEditingAllowed)(el) && !(0, _dom.isShadowUIElement)(el)) {
            el[ELEMENT_EDITING_OBSERVED_FLAG] = true;
            el[OLD_VALUE_PROPERTY] = ElementEditingWatcher._getValue(el);
            _nativeMethods2.default.addEventListener.call(el, 'blur', function (e) {
                return _this2._onBlur(e);
            });
            _nativeMethods2.default.addEventListener.call(el, 'change', function (e) {
                return _this2._onChange(e);
            });
        }
    };

    ElementEditingWatcher.prototype.restartWatchingElementEditing = function restartWatchingElementEditing(el) {
        if (el && el[ELEMENT_EDITING_OBSERVED_FLAG]) el[OLD_VALUE_PROPERTY] = ElementEditingWatcher._getValue(el);
    };

    ElementEditingWatcher.prototype.processElementChanging = function processElementChanging(el) {
        if (el && el[ELEMENT_EDITING_OBSERVED_FLAG] && ElementEditingWatcher._getValue(el) !== el[OLD_VALUE_PROPERTY]) {
            this.eventSimulator.change(el);
            this.restartWatchingElementEditing(el);
            return true;
        }
        return false;
    };

    return ElementEditingWatcher;
}();

exports.default = ElementEditingWatcher;
module.exports = exports['default'];