'use strict';

exports.__esModule = true;

var _internalAttributes = require('../../../processing/dom/internal-attributes');

var _internalAttributes2 = _interopRequireDefault(_internalAttributes);

var _internalProperties = require('../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _activeWindowTracker = require('../event/active-window-tracker');

var _activeWindowTracker2 = _interopRequireDefault(_activeWindowTracker);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _browser = require('../../utils/browser');

var browserUtils = _interopRequireWildcard(_browser);

var _dom = require('../../utils/dom');

var domUtils = _interopRequireWildcard(_dom);

var _style = require('../../utils/style');

var styleUtils = _interopRequireWildcard(_style);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var INTERNAL_FOCUS_BLUR_FLAG_PREFIX = 'hammerhead|event|internal-';
var PREVENT_FOCUS_ON_CHANGE = browserUtils.isChrome;
var eventsMap = {
    bubbles: {
        'focus': 'focusin',
        'blur': 'focusout'
    },
    nonBubbles: {
        'focusin': 'focus',
        'focusout': 'blur'
    }
};

var FocusBlurSandbox = function (_SandboxBase) {
    _inherits(FocusBlurSandbox, _SandboxBase);

    function FocusBlurSandbox(listeners, eventSimulator, messageSandbox, shadowUI, timersSandbox, elementEditingWatcher) {
        _classCallCheck(this, FocusBlurSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.topWindow = null;
        _this.lastFocusedElement = null;
        _this.scrollState = {};
        _this.eventSimulator = eventSimulator;
        _this.activeWindowTracker = new _activeWindowTracker2.default(messageSandbox);
        _this.shadowUI = shadowUI;
        _this.listeners = listeners;
        _this.elementEditingWatcher = elementEditingWatcher;
        _this.timersSandbox = timersSandbox;
        return _this;
    }

    FocusBlurSandbox._getNativeMeth = function _getNativeMeth(el, event) {
        if (domUtils.isSVGElement(el)) {
            if (event === 'focus') return _nativeMethods2.default.svgFocus;else if (event === 'blur') return _nativeMethods2.default.svgBlur;
        }
        return _nativeMethods2.default[event];
    };

    FocusBlurSandbox._restoreElementScroll = function _restoreElementScroll(el, scroll) {
        var newScroll = styleUtils.getElementScroll(el);
        if (newScroll.left !== scroll.left) styleUtils.setScrollLeft(el, scroll.left);
        if (newScroll.top !== scroll.top) styleUtils.setScrollTop(el, scroll.top);
    };

    FocusBlurSandbox.prototype._onChangeActiveElement = function _onChangeActiveElement(activeElement) {
        if (this.lastFocusedElement === activeElement) return;
        if (this.lastFocusedElement && _nativeMethods2.default.getAttribute.call(this.lastFocusedElement, _internalAttributes2.default.focusPseudoClass)) _nativeMethods2.default.removeAttribute.call(this.lastFocusedElement, _internalAttributes2.default.focusPseudoClass);
        if (domUtils.isElementFocusable(activeElement) && !(domUtils.isBodyElement(activeElement) && domUtils.getTabIndex(activeElement) === null)) {
            this.lastFocusedElement = activeElement;
            _nativeMethods2.default.setAttribute.call(activeElement, _internalAttributes2.default.focusPseudoClass, true);
        } else this.lastFocusedElement = null;
    };

    FocusBlurSandbox.prototype._shouldUseLabelHtmlForElement = function _shouldUseLabelHtmlForElement(el, type) {
        return type === 'focus' && domUtils.isLabelElement(el) && el.htmlFor;
    };

    FocusBlurSandbox.prototype._getElementNonScrollableParentsScrollState = function _getElementNonScrollableParentsScrollState(el) {
        var scrollState = [];
        var elementParents = domUtils.getParents(el);

        for (var _i2 = 0; _i2 < elementParents.length; _i2++) {
            var elementParent = elementParents[_i2];
            if (styleUtils.get(elementParent, 'overflow') === 'hidden') {
                scrollState.push({
                    element: elementParent,
                    state: styleUtils.getElementScroll(elementParent)
                });
            }
        }

        return scrollState;
    };

    FocusBlurSandbox.prototype._restoreElementNonScrollableParentsScrollState = function _restoreElementNonScrollableParentsScrollState(scrollState) {
        for (var _i4 = 0; _i4 < scrollState.length; _i4++) {
            var scrollStateEntry = scrollState[_i4];
            FocusBlurSandbox._restoreElementScroll(scrollStateEntry.element, scrollStateEntry.state);
        }
    };

    FocusBlurSandbox.prototype._saveScrollStateIfNecessary = function _saveScrollStateIfNecessary(el, preventScrolling) {
        if (preventScrolling) this.scrollState.windowScroll = styleUtils.getElementScroll(this.window);
        if (browserUtils.isIE) this.scrollState.elementNonScrollableParentsScrollState = this._getElementNonScrollableParentsScrollState(el);
    };

    FocusBlurSandbox.prototype._restoreScrollStateIfNecessary = function _restoreScrollStateIfNecessary(preventScrolling) {
        if (preventScrolling) FocusBlurSandbox._restoreElementScroll(this.window, this.scrollState.windowScroll);
        if (browserUtils.isIE) this._restoreElementNonScrollableParentsScrollState(this.scrollState.elementNonScrollableParentsScrollState);
    };

    FocusBlurSandbox.prototype._raiseEvent = function _raiseEvent(el, type, callback, _ref) {
        var _this2 = this;

        var withoutHandlers = _ref.withoutHandlers,
            isAsync = _ref.isAsync,
            forMouseEvent = _ref.forMouseEvent,
            preventScrolling = _ref.preventScrolling,
            relatedTarget = _ref.relatedTarget,
            focusedOnChange = _ref.focusedOnChange;

        // NOTE: We cannot use Promise because 'resolve' will be called async, but we need to resolve
        // immediately in IE9 and IE10.
        var simulateEvent = function simulateEvent() {
            // NOTE: The focus and blur events should be raised after activeElement is changed (B237489)
            // in MSEdge, the focus/blur events are executed  synchronously.
            if (browserUtils.isIE && browserUtils.version < 12) {
                _this2.window.setTimeout(function () {
                    _this2.window.setTimeout(function () {
                        delete el[FocusBlurSandbox.getInternalEventFlag(type)];
                    }, 0);
                }, 0);
            } else delete el[FocusBlurSandbox.getInternalEventFlag(type)];
            if (!withoutHandlers) {
                var bubblesEventType = eventsMap.bubbles[type];
                var bubblesEventShouldRaiseFirstly = browserUtils.isIE11 || browserUtils.isMSEdge && browserUtils.version < 17 && type === 'blur';
                if (isAsync) {
                    // NOTE: focusin, focusout events are synchronously
                    _this2.eventSimulator[bubblesEventType](el, relatedTarget);
                    _this2.timersSandbox.deferFunction(function () {
                        return _this2.eventSimulator[type](el, relatedTarget);
                    });
                } else if (bubblesEventShouldRaiseFirstly) {
                    _this2.eventSimulator[bubblesEventType](el, relatedTarget);
                    _this2.eventSimulator[type](el, relatedTarget);
                } else {
                    _this2.eventSimulator[type](el, relatedTarget);
                    _this2.eventSimulator[bubblesEventType](el, relatedTarget);
                }
            } else if (type === 'focus' && PREVENT_FOCUS_ON_CHANGE) {
                var preventFocus = function preventFocus(_e, _dispatched, _preventEvent, cancelHandlers, stopEventPropagation) {
                    cancelHandlers();
                    stopEventPropagation();
                };
                _this2.listeners.addInternalEventListener(window, ['focus'], preventFocus);
                _this2.eventSimulator['focus'](el, relatedTarget);
                _this2.listeners.removeInternalEventListener(window, ['focus'], preventFocus);
            }
            callback();
        };
        if (el[type]) {
            // NOTE: To guarantee that all focus/blur events are raised, we need to raise them manually.
            this._saveScrollStateIfNecessary(el, preventScrolling);
            if (this._shouldUseLabelHtmlForElement(el, type)) {
                var htmlForElement = _nativeMethods2.default.getElementById.call(domUtils.findDocument(el), el.htmlFor);
                if (htmlForElement) el = htmlForElement;else {
                    callback();
                    return;
                }
            }
            el[FocusBlurSandbox.getInternalEventFlag(type)] = true;
            // NOTE: We should guarantee that activeElement will be changed, therefore we need to call the native
            // focus/blur event.
            if (!focusedOnChange) FocusBlurSandbox._getNativeMeth(el, type).call(el);
            this._restoreScrollStateIfNecessary(preventScrolling);
            var curDocument = domUtils.findDocument(el);
            var activeElement = domUtils.getActiveElement(curDocument);
            // NOTE: If the element was not focused and has a parent with tabindex, we focus this parent.
            var parent = el.parentNode;
            var parentWithTabIndex = parent === document ? null : domUtils.closest(parent, '[tabindex]');
            if (type === 'focus' && activeElement !== el && parentWithTabIndex && forMouseEvent) {
                // NOTE: In WebKit, Safari and MSEdge, calling the native focus event for a parent element
                // raises page scrolling. We can't prevent it. Therefore, we need to restore a page scrolling value.
                var needPreventScrolling = browserUtils.isWebKit || browserUtils.isSafari || browserUtils.isIE;
                this._raiseEvent(parentWithTabIndex, 'focus', simulateEvent, {
                    preventScrolling: needPreventScrolling,
                    forMouseEvent: forMouseEvent
                });
            }
            // NOTE: Some browsers don't change document.activeElement after calling element.blur() if a browser
            // window is in the background. That's why we call body.focus() without handlers. It should be called
            // synchronously because client scripts may expect that document.activeElement will be changed immediately
            // after element.blur() is called.
            else if (type === 'blur' && activeElement === el && el !== curDocument.body) this._raiseEvent(curDocument.body, 'focus', simulateEvent, { withoutHandlers: true });else if (!el.disabled) simulateEvent();else callback();
        } else simulateEvent();
    };

    FocusBlurSandbox.getInternalEventFlag = function getInternalEventFlag(type) {
        return INTERNAL_FOCUS_BLUR_FLAG_PREFIX + type;
    };

    FocusBlurSandbox.getNonBubblesEventType = function getNonBubblesEventType(bubblesEventType) {
        return eventsMap.nonBubbles[bubblesEventType];
    };

    FocusBlurSandbox.prototype.attach = function attach(window) {
        var _this3 = this;

        _SandboxBase.prototype.attach.call(this, window);
        this.activeWindowTracker.attach(window);
        this.topWindow = domUtils.isCrossDomainWindows(window, window.top) ? window : window.top;
        this.listeners.addInternalEventListener(window, ['focus', 'blur'], function () {
            var activeElement = domUtils.getActiveElement(_this3.document);
            _this3._onChangeActiveElement(activeElement);
        });
    };

    FocusBlurSandbox.prototype._raiseSelectionChange = function _raiseSelectionChange(callback, el) {
        // NOTE: In MSEdge, the 'selectionchange' event doesn't occur immediately (it occurs with a some delay)
        // so we should raise it right after the 'focus' event is raised.
        if (browserUtils.isMSEdge && el && domUtils.isTextEditableElement(el)) this.eventSimulator.selectionchange(el);
        if (typeof callback === 'function') callback();
    };

    FocusBlurSandbox.prototype.focus = function focus(el, callback, silent, forMouseEvent, isNativeFocus, preventScrolling) {
        var _this4 = this;

        // NOTE: el.focus() does not raise the event if the element is invisible. If the element is located
        // within an invisible iframe, all browsers except Chrome do not raise the event (GH-442)
        var raiseEventInIframe = !isNativeFocus || browserUtils.isWebKit || !styleUtils.isElementInInvisibleIframe(el);
        var elDocument = (el[_internalProperties2.default.processedContext] || this.window).document;
        if (!raiseEventInIframe || isNativeFocus && !styleUtils.isElementVisible(el, elDocument)) return null;
        var isElementInIframe = domUtils.isElementInIframe(el);
        var iframeElement = isElementInIframe ? domUtils.getIframeByElement(el) : null;
        var curDocument = domUtils.findDocument(el);
        var isBodyElement = domUtils.isBodyElement(el);
        var activeElement = domUtils.getActiveElement();
        var activeElementDocument = domUtils.findDocument(activeElement);
        var withoutHandlers = false;
        var needBlur = false;
        var needBlurIframe = false;
        var isContentEditable = domUtils.isContentEditableElement(el);
        var isCurrentWindowActive = this.activeWindowTracker.isCurrentWindowActive();
        if (activeElement === el) withoutHandlers = !(isBodyElement && isContentEditable && !isCurrentWindowActive);else withoutHandlers = isBodyElement && !(isContentEditable || browserUtils.isIE);
        // NOTE: In IE, if you call focus() or blur() methods from script, an active element is changed immediately,
        // but events are raised asynchronously after some timeout.
        var isAsync = false;
        var raiseFocusEvent = function raiseFocusEvent() {
            if (!isCurrentWindowActive && !domUtils.isShadowUIElement(el)) _this4.activeWindowTracker.makeCurrentWindowActive();
            var raiseEventArgs = {
                withoutHandlers: withoutHandlers || silent,
                isAsync: isAsync,
                forMouseEvent: forMouseEvent,
                preventScrolling: preventScrolling,
                relatedTarget: activeElement
            };
            _this4._raiseEvent(el, 'focus', function () {
                if (!silent) _this4.elementEditingWatcher.watchElementEditing(el);
                // NOTE: If we call focus for an unfocusable element (like 'div' or 'image') in iframe, we should
                // specify document.active for this iframe manually, so we call focus without handlers.
                if (isElementInIframe && iframeElement && domUtils.getActiveElement(_this4.topWindow.document) !== iframeElement) _this4._raiseEvent(iframeElement, 'focus', function () {
                    return _this4._raiseSelectionChange(callback, el);
                }, { withoutHandlers: true, isAsync: isAsync });else _this4._raiseSelectionChange(callback, el);
            }, raiseEventArgs);
        };
        if (isNativeFocus && browserUtils.isIE) {
            // NOTE: In IE, the focus() method does not have any effect if it is called in the focus event handler
            // during the  second event phase.
            if ((this.eventSimulator.isSavedWindowsEventsExists() || browserUtils.version > 10) && this.window.event && this.window.event.type === 'focus' && this.window.event.srcElement === el) {
                this._raiseSelectionChange(callback, el);
                return null;
            }
            // NOTE: In MSEdge, the focus/blur events are executed synchronously.
            if (browserUtils.version < 12) isAsync = true;
        }
        if (activeElement && activeElement.tagName) {
            if (activeElement !== el) {
                // NOTE: B253685
                if (curDocument !== activeElementDocument && activeElement === activeElementDocument.body) needBlur = false;else if (activeElement === curDocument.body) {
                    // NOTE: The Blur event is raised for the body only in IE. In addition, we must not call the
                    // blur function for the body because this moves the browser window into the background.
                    if (!silent && browserUtils.isIE) {
                        if (isAsync) this.timersSandbox.setTimeout.call(this.window, function () {
                            return _this4.eventSimulator.blur(activeElement);
                        }, 0);else this.eventSimulator.blur(activeElement);
                    }
                } else if (!el.disabled) needBlur = true;
            }
            // NOTE: B254260
            needBlurIframe = curDocument !== activeElementDocument && domUtils.isElementInIframe(activeElement, activeElementDocument);
        }
        // NOTE: We always call blur for iframe manually without handlers (B254260).
        if (needBlurIframe && !needBlur) {
            if (browserUtils.isIE) {
                // NOTE: We should call blur for iframe with handlers in IE but we can't call the method 'blur'
                // because activeElement !== element and handlers will not be called.
                this.eventSimulator.blur(domUtils.getIframeByElement(activeElement));
                raiseFocusEvent();
            } else this.blur(domUtils.getIframeByElement(activeElement), raiseFocusEvent, true, isNativeFocus);
        } else if (needBlur) {
            this.blur(activeElement, function (focusOnChange) {
                if (needBlurIframe) _this4.blur(domUtils.getIframeByElement(activeElement), raiseFocusEvent, true, isNativeFocus);else if (!focusOnChange) raiseFocusEvent();else if (typeof callback === 'function') callback();
            }, silent, isNativeFocus, el);
        } else raiseFocusEvent();
        return null;
    };

    FocusBlurSandbox.prototype.blur = function blur(el, callback, withoutHandlers, isNativeBlur, relatedTarget) {
        var curDocument = domUtils.findDocument(el);
        var activeElement = domUtils.getActiveElement(curDocument);
        // NOTE: In IE, if you call the focus() or blur() method from script, an active element is changed
        // immediately but events are raised asynchronously after some timeout (in MSEdgethe focus/blur methods
        // are executed synchronously).
        var isAsync = isNativeBlur && browserUtils.isIE && browserUtils.version < 12;
        if (activeElement !== el) withoutHandlers = true;
        var focusedOnChange = false;
        if (!withoutHandlers) {
            var focusOnChangeHandler = function focusOnChangeHandler(e) {
                focusedOnChange = e.target === el;
            };
            if (PREVENT_FOCUS_ON_CHANGE) this.listeners.addInternalEventListener(window, ['focus'], focusOnChangeHandler);
            this.elementEditingWatcher.processElementChanging(el);
            if (PREVENT_FOCUS_ON_CHANGE) this.listeners.removeInternalEventListener(window, ['focus'], focusOnChangeHandler);
            this.elementEditingWatcher.stopWatching(el);
        }
        var raiseEventParameters = {
            withoutHandlers: withoutHandlers,
            isAsync: isAsync,
            relatedTarget: relatedTarget,
            focusedOnChange: focusedOnChange
        };
        this._raiseEvent(el, 'blur', function () {
            if (typeof callback === 'function') callback(focusedOnChange);
        }, raiseEventParameters);
    };

    FocusBlurSandbox._processFocusPseudoClassSelector = function _processFocusPseudoClassSelector(selector) {
        // NOTE: When a selector that contains the ':focus' pseudo-class is used in the querySelector and
        // querySelectorAll functions, these functions return an empty result if the browser is not focused.
        // This replaces ':focus' with a custom CSS class to return the current active element in that case.
        // IE returns a valid element, so there is no need to replace the selector for it.
        if (!browserUtils.isIE) return selector.replace(/\s*:focus\b/gi, '[' + _internalAttributes2.default.focusPseudoClass + ']');
        return selector;
    };

    return FocusBlurSandbox;
}(_base2.default);

exports.default = FocusBlurSandbox;
module.exports = exports['default'];