'use strict';

exports.__esModule = true;

var _internalAttributes = require('../../../processing/dom/internal-attributes');

var _internalAttributes2 = _interopRequireDefault(_internalAttributes);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dom = require('../../utils/dom');

var domUtils = _interopRequireWildcard(_dom);

var _position = require('../../utils/position');

var positionUtils = _interopRequireWildcard(_position);

var _browser = require('../../utils/browser');

var browserUtils = _interopRequireWildcard(_browser);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var HoverSandbox = function (_SandboxBase) {
    _inherits(HoverSandbox, _SandboxBase);

    function HoverSandbox(listeners) {
        _classCallCheck(this, HoverSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.listeners = listeners;
        _this.hoverElementFixed = false;
        _this.lastHoveredElement = null;
        return _this;
    }

    HoverSandbox._setHoverMarker = function _setHoverMarker(newHoveredElement, jointParent) {
        if (jointParent) _nativeMethods2.default.setAttribute.call(jointParent, _internalAttributes2.default.hoverPseudoClass, '');
        while (newHoveredElement && newHoveredElement.tagName) {
            // NOTE: Assign a pseudo-class marker to the elements until the joint parent is found.
            if (newHoveredElement !== jointParent) {
                _nativeMethods2.default.setAttribute.call(newHoveredElement, _internalAttributes2.default.hoverPseudoClass, '');
                newHoveredElement = newHoveredElement.parentNode;
            } else break;
        }
    };
    // NOTE: In this method, we go up to the tree of elements and look for a joint parent for the
    // previous and new hovered elements. Processing is needed only until  that parent is found.
    // In this case, we'll reduce the number of dom calls.


    HoverSandbox.prototype._clearHoverMarkerUntilJointParent = function _clearHoverMarkerUntilJointParent(newHoveredElement) {
        var jointParent = null;
        if (this.lastHoveredElement) {
            var el = this.lastHoveredElement;
            while (el && el.tagName && el.contains) {
                // NOTE: Check that the current element is a joint parent for the hovered elements.
                if (!el.contains(newHoveredElement)) {
                    _nativeMethods2.default.removeAttribute.call(el, _internalAttributes2.default.hoverPseudoClass);
                    el = el.parentNode;
                } else {
                    jointParent = el;
                    break;
                }
            }
            if (jointParent) _nativeMethods2.default.removeAttribute.call(jointParent, _internalAttributes2.default.hoverPseudoClass);
        }
        return jointParent;
    };

    HoverSandbox.prototype._onHover = function _onHover(_ref) {
        var target = _ref.target,
            clientX = _ref.clientX,
            clientY = _ref.clientY;

        var hoverIsDisabled = browserUtils.isIE && positionUtils.shouldIgnoreMouseEventInsideIframe(target, clientX, clientY);
        if (!hoverIsDisabled) this._hover(target);
    };

    HoverSandbox.prototype._hover = function _hover(el) {
        if (!this.hoverElementFixed && !domUtils.isShadowUIElement(el)) {
            var jointParent = this._clearHoverMarkerUntilJointParent(el);
            HoverSandbox._setHoverMarker(el, jointParent);
            this.lastHoveredElement = el;
        }
    };

    HoverSandbox.prototype.fixHoveredElement = function fixHoveredElement() {
        this.hoverElementFixed = true;
    };

    HoverSandbox.prototype.freeHoveredElement = function freeHoveredElement() {
        this.hoverElementFixed = false;
    };

    HoverSandbox.prototype.attach = function attach(window) {
        var _this2 = this;

        _SandboxBase.prototype.attach.call(this, window);
        this.listeners.addInternalEventListener(window, ['mouseover', 'touchstart'], function (e) {
            return _this2._onHover(e);
        });
    };

    return HoverSandbox;
}(_base2.default);

exports.default = HoverSandbox;
module.exports = exports['default'];