'use strict';

exports.__esModule = true;

var _focusBlur = require('./focus-blur');

var _focusBlur2 = _interopRequireDefault(_focusBlur);

var _hover = require('./hover');

var _hover2 = _interopRequireDefault(_hover);

var _listeners = require('./listeners');

var _listeners2 = _interopRequireDefault(_listeners);

var _selection = require('./selection');

var _selection2 = _interopRequireDefault(_selection);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dom = require('../../utils/dom');

var domUtils = _interopRequireWildcard(_dom);

var _event = require('../../utils/event');

var _dataTransfer = require('./drag-and-drop/data-transfer');

var _dataTransfer2 = _interopRequireDefault(_dataTransfer);

var _dragDataStore = require('./drag-and-drop/drag-data-store');

var _dragDataStore2 = _interopRequireDefault(_dragDataStore);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var EventSandbox = function (_SandboxBase) {
    _inherits(EventSandbox, _SandboxBase);

    function EventSandbox(listeners, eventSimulator, elementEditingWatcher, unloadSandbox, messageSandbox, shadowUI, timerSandbox) {
        _classCallCheck(this, EventSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.EVENT_PREVENTED_EVENT = 'hammerhead|event|event-prevented';
        _this.listeners = listeners;
        _this.elementEditingWatcher = elementEditingWatcher;
        _this.unload = unloadSandbox;
        _this.timers = timerSandbox;
        _this.eventSimulator = eventSimulator;
        _this.focusBlur = new _focusBlur2.default(listeners, eventSimulator, messageSandbox, shadowUI, timerSandbox, elementEditingWatcher);
        _this.selection = new _selection2.default(_this);
        _this.hover = new _hover2.default(listeners);
        _this.shadowUI = shadowUI;
        _this.message = messageSandbox;
        _this.DataTransfer = _dataTransfer2.default;
        _this.DragDataStore = _dragDataStore2.default;
        _this.overriddenMethods = null;
        _this.onFocus = null;
        _this.cancelInternalEvents = null;
        _this._createOverridedMethods();
        _this._createInternalHandlers();
        return _this;
    }

    EventSandbox.prototype._createOverridedMethods = function _createOverridedMethods() {
        var selection = this.selection;
        var focusBlurSandbox = this.focusBlur;
        var eventSimulator = this.eventSimulator;
        var sandbox = this;
        this.overriddenMethods = {
            dispatchEvent: function dispatchEvent() {
                _listeners2.default.beforeDispatchEvent(this);
                var isWindow = domUtils.isWindow(this);
                var res = isWindow ? _nativeMethods2.default.windowDispatchEvent.apply(this, arguments) : _nativeMethods2.default.dispatchEvent.apply(this, arguments);
                _listeners2.default.afterDispatchEvent(this);
                return res;
            },
            click: function click() {
                _listeners2.default.beforeDispatchEvent(this);
                var res = eventSimulator.nativeClick(this, _nativeMethods2.default.click);
                _listeners2.default.afterDispatchEvent(this);
                return res;
            },
            setSelectionRange: function setSelectionRange() {
                return selection.setSelectionRangeWrapper.apply(this, arguments);
            },
            select: function select() {
                return selection.selectWrapper.call(this);
            },
            focus: function focus() {
                return focusBlurSandbox.focus(this, null, false, false, true);
            },
            blur: function blur() {
                return focusBlurSandbox.blur(this, null, false, true);
            },
            preventDefault: function preventDefault() {
                sandbox.emit(sandbox.EVENT_PREVENTED_EVENT, this);
                return _nativeMethods2.default.preventDefault.call(this);
            }
        };
    };

    EventSandbox.prototype._createInternalHandlers = function _createInternalHandlers() {
        var shadowUI = this.shadowUI;
        var document = this.document;
        var eventSimulator = this.eventSimulator;
        this.onFocus = function (e) {
            var focusedEl = e.target;
            var activeEl = domUtils.getActiveElement(document);
            if (!domUtils.isShadowUIElement(focusedEl) && !domUtils.isShadowUIElement(activeEl)) shadowUI.setLastActiveElement(activeEl);
        };
        this.cancelInternalEvents = function (e, _dispatched, _preventEvent, _cancelHandlers, stopPropagation) {
            // NOTE: We should cancel events raised by calling the native function (focus, blur) only if the
            // element has a flag. If an event is dispatched, we shouldn't cancel it.
            // After calling a native function two events were raised
            // .focus() -> focus, focusin
            // .blur() -> blur, focusout
            // So we should prevent both events
            var eventType = _focusBlur2.default.getNonBubblesEventType(e.type) || e.type;
            var internalEventFlag = _focusBlur2.default.getInternalEventFlag(eventType);
            if (e.target[internalEventFlag] && !e[eventSimulator.DISPATCHED_EVENT_FLAG]) stopPropagation();
        };
    };

    EventSandbox.prototype.attach = function attach(window) {
        _SandboxBase.prototype.attach.call(this, window);
        window.HTMLInputElement.prototype.setSelectionRange = this.overriddenMethods.setSelectionRange;
        window.HTMLTextAreaElement.prototype.setSelectionRange = this.overriddenMethods.setSelectionRange;
        window.Window.prototype.dispatchEvent = this.overriddenMethods.dispatchEvent;
        window.Document.prototype.dispatchEvent = this.overriddenMethods.dispatchEvent;
        window.HTMLElement.prototype.dispatchEvent = this.overriddenMethods.dispatchEvent;
        window.SVGElement.prototype.dispatchEvent = this.overriddenMethods.dispatchEvent;
        window.HTMLElement.prototype.focus = this.overriddenMethods.focus;
        window.HTMLElement.prototype.blur = this.overriddenMethods.blur;
        window.HTMLElement.prototype.click = this.overriddenMethods.click;
        window.Window.focus = this.overriddenMethods.focus;
        window.Window.blur = this.overriddenMethods.blur;
        window.Event.prototype.preventDefault = this.overriddenMethods.preventDefault;
        if (window.TextRange && window.TextRange.prototype.select) window.TextRange.prototype.select = this.overriddenMethods.select;
        this.initDocumentListening(window.document);
        this.listeners.initElementListening(window, _event.DOM_EVENTS.concat(['load', 'beforeunload', 'pagehide', 'unload', 'message']));
        this.listeners.addInternalEventListener(window, ['focus'], this.onFocus);
        this.listeners.addInternalEventListener(window, ['focus', 'blur', 'change', 'focusin', 'focusout'], this.cancelInternalEvents);
        this.unload.attach(window);
        this.message.attach(window);
        this.timers.attach(window);
        this.focusBlur.attach(window);
        this.hover.attach(window);
    };

    EventSandbox.prototype.initDocumentListening = function initDocumentListening(document) {
        this.listeners.initElementListening(document, _event.DOM_EVENTS);
    };

    return EventSandbox;
}(_base2.default);

exports.default = EventSandbox;
module.exports = exports['default'];