'use strict';

exports.__esModule = true;

var _internalProperties = require('../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _eventEmitter = require('../../utils/event-emitter');

var _eventEmitter2 = _interopRequireDefault(_eventEmitter);

var _listeningContext = require('./listening-context');

var listeningCtx = _interopRequireWildcard(_listeningContext);

var _event = require('../../utils/event');

var _dom = require('../../utils/dom');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var LISTENED_EVENTS = ['click', 'mousedown', 'mouseup', 'dblclick', 'contextmenu', 'mousemove', 'mouseover', 'mouseout', 'pointerdown', 'pointermove', 'pointerover', 'pointerout', 'pointerup', 'MSPointerDown', 'MSPointerMove', 'MSPointerOver', 'MSPointerOut', 'MSPointerUp', 'touchstart', 'touchmove', 'touchend', 'keydown', 'keypress', 'keyup', 'change', 'focus', 'blur', 'focusin', 'focusout'];
var EVENT_SANDBOX_DISPATCH_EVENT_FLAG = 'hammerhead|event-sandbox-dispatch-event-flag';

var Listeners = function (_EventEmitter) {
    _inherits(Listeners, _EventEmitter);

    function Listeners() {
        _classCallCheck(this, Listeners);

        var _this = _possibleConstructorReturn(this, _EventEmitter.call(this));

        _this.EVENT_LISTENER_ATTACHED_EVENT = 'hammerhead|event|event-listener-attached';
        _this.EVENT_LISTENER_DETACHED_EVENT = 'hammerhead|event|event-listener-detached';
        _this.listeningCtx = listeningCtx;
        _this.addInternalEventListener = _this.listeningCtx.addInternalHandler;
        _this.addFirstInternalHandler = _this.listeningCtx.addFirstInternalHandler;
        _this.removeInternalEventListener = _this.listeningCtx.removeInternalHandler;
        return _this;
    }

    Listeners._getBodyEventListenerWrapper = function _getBodyEventListenerWrapper(documentEventCtx, listener) {
        return function (e) {
            if (documentEventCtx.cancelOuterHandlers) return null;
            return listener.call(this, e);
        };
    };

    Listeners._getNativeAddEventListener = function _getNativeAddEventListener(el) {
        if ((0, _dom.isWindow)(el)) return _nativeMethods2.default.windowAddEventListener;
        return el.body !== void 0 ? _nativeMethods2.default.documentAddEventListener : _nativeMethods2.default.addEventListener;
    };

    Listeners._getNativeRemoveEventListener = function _getNativeRemoveEventListener(el) {
        if ((0, _dom.isWindow)(el)) return _nativeMethods2.default.windowRemoveEventListener;
        return el.body !== void 0 ? _nativeMethods2.default.documentRemoveEventListener : _nativeMethods2.default.removeEventListener;
    };

    Listeners._getEventListenerWrapper = function _getEventListenerWrapper(eventCtx, listener) {
        return function (e) {
            var isIEServiceHandler = listener.toString() === '[object FunctionWrapper]';
            // NOTE: Ignore IE11's and Edge's service handlers (GH-379)
            if (isIEServiceHandler) return null;
            if (eventCtx.cancelOuterHandlers) return null;
            if (typeof eventCtx.outerHandlersWrapper === 'function') return eventCtx.outerHandlersWrapper.call(this, e, listener);
            return (0, _event.callEventListener)(this, listener, e);
        };
    };

    Listeners._isDifferentHandler = function _isDifferentHandler(outerHandlers, listener, useCapture) {
        for (var _i2 = 0; _i2 < outerHandlers.length; _i2++) {
            var outerHandler = outerHandlers[_i2];
            if (outerHandler.fn === listener && outerHandler.useCapture === useCapture) return false;
        }

        return true;
    };

    Listeners._getUseCaptureParam = function _getUseCaptureParam(optionalParam) {
        if (optionalParam && typeof optionalParam === 'boolean') return optionalParam;else if (optionalParam && typeof optionalParam === 'object') return !!optionalParam.capture;
        return false;
    };

    Listeners.prototype._createEventHandler = function _createEventHandler() {
        var listeners = this;
        return function (e) {
            var el = this;
            var elWindow = el[_internalProperties2.default.processedContext] || window;
            var eventPrevented = false;
            var handlersCancelled = false;
            var stopPropagationCalled = false;
            var eventCtx = listeners.listeningCtx.getEventCtx(el, e.type);
            if (!eventCtx) return;
            var internalHandlers = eventCtx.internalHandlers;
            eventCtx.cancelOuterHandlers = false;
            var preventEvent = function preventEvent() {
                eventPrevented = true;
                (0, _event.preventDefault)(e);
            };
            var cancelHandlers = function cancelHandlers() {
                if (!handlersCancelled) eventCtx.cancelOuterHandlers = true;
                handlersCancelled = true;
            };
            var stopEventPropagation = function stopEventPropagation() {
                stopPropagationCalled = true;
                (0, _event.stopPropagation)(e);
            };

            for (var _i4 = 0; _i4 < internalHandlers.length; _i4++) {
                var internalHandler = internalHandlers[_i4];
                internalHandler.call(el, e, elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG], preventEvent, cancelHandlers, stopEventPropagation);
                if (eventPrevented || stopPropagationCalled) break;
            }
        };
    };

    Listeners.prototype._createElementOverridedMethods = function _createElementOverridedMethods(el) {
        var listeners = this;
        return {
            addEventListener: function addEventListener() {
                for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                    args[_key] = arguments[_key];
                }

                var type = args[0];
                var listener = args[1];
                var useCapture = Listeners._getUseCaptureParam(args[2]);
                var eventListeningInfo = listeningCtx.getEventCtx(el, type);
                var nativeAddEventListener = Listeners._getNativeAddEventListener(el);
                if (!eventListeningInfo || !(0, _event.isValidEventListener)(listener)) return nativeAddEventListener.apply(el, args);
                // NOTE: T233158
                var isDifferentHandler = Listeners._isDifferentHandler(eventListeningInfo.outerHandlers, listener, useCapture);
                if (!isDifferentHandler) return null;
                var wrapper = Listeners._getEventListenerWrapper(eventListeningInfo, listener);
                listeningCtx.wrapEventListener(eventListeningInfo, listener, wrapper, useCapture);
                args[1] = wrapper;
                var res = nativeAddEventListener.apply(el, args);
                listeners.emit(listeners.EVENT_LISTENER_ATTACHED_EVENT, { el: el, listener: listener, eventType: type });
                return res;
            },
            removeEventListener: function removeEventListener() {
                for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                    args[_key2] = arguments[_key2];
                }

                var type = args[0];
                var listener = args[1];
                var useCapture = Listeners._getUseCaptureParam(args[2]);
                var nativeRemoveEventListener = Listeners._getNativeRemoveEventListener(el);
                var eventCtx = listeningCtx.getEventCtx(el, type);
                if (!eventCtx || !(0, _event.isValidEventListener)(listener)) return nativeRemoveEventListener.apply(el, args);
                args[1] = listeningCtx.getWrapper(eventCtx, listener, useCapture);
                var res = nativeRemoveEventListener.apply(el, args);
                listeners.emit(listeners.EVENT_LISTENER_DETACHED_EVENT, { el: el, listener: listener, eventType: type });
                return res;
            }
        };
    };

    Listeners.prototype._createDocumentBodyOverridedMethods = function _createDocumentBodyOverridedMethods(doc) {
        var listeners = this;
        var nativeAddEventListener = function () {
            return doc.body.addEventListener;
        }();
        var nativeRemoveEventListener = function () {
            return doc.body.removeEventListener;
        }();
        return {
            addEventListener: function addEventListener() {
                for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                    args[_key3] = arguments[_key3];
                }

                var type = args[0];
                var listener = args[1];
                var useCapture = Listeners._getUseCaptureParam(args[2]);
                var docEventListeningInfo = listeningCtx.getEventCtx(doc, type);
                var eventListeningInfo = listeningCtx.getEventCtx(this, type);
                if (!docEventListeningInfo || !(0, _event.isValidEventListener)(listener)) return nativeAddEventListener.apply(this, args);
                // NOTE: T233158
                var isDifferentHandler = Listeners._isDifferentHandler(eventListeningInfo.outerHandlers, listener, useCapture);
                if (!isDifferentHandler) return null;
                var wrapper = Listeners._getBodyEventListenerWrapper(docEventListeningInfo, listener);
                listeningCtx.wrapEventListener(eventListeningInfo, listener, wrapper, useCapture);
                var res = nativeAddEventListener.call(this, type, wrapper, useCapture);
                listeners.emit(listeners.EVENT_LISTENER_ATTACHED_EVENT, {
                    el: this,
                    eventType: type,
                    listener: listener
                });
                return res;
            },
            removeEventListener: function removeEventListener() {
                var type = arguments.length <= 0 ? undefined : arguments[0];
                var listener = arguments.length <= 1 ? undefined : arguments[1];
                var useCapture = Listeners._getUseCaptureParam(arguments.length <= 2 ? undefined : arguments[2]);
                var eventListeningInfo = listeningCtx.getEventCtx(this, type);
                if (!eventListeningInfo || !(0, _event.isValidEventListener)(listener)) return nativeRemoveEventListener.call(this, type, listener, useCapture);
                return nativeRemoveEventListener.call(this, type, listeningCtx.getWrapper(eventListeningInfo, listener, useCapture), useCapture);
            }
        };
    };

    Listeners.prototype.initElementListening = function initElementListening(el, events) {
        var nativeAddEventListener = Listeners._getNativeAddEventListener(el);
        events = events || LISTENED_EVENTS;
        this.listeningCtx.addListeningElement(el, events);

        for (var _i6 = 0, _events2 = events; _i6 < _events2.length; _i6++) {
            var event = _events2[_i6];
            nativeAddEventListener.call(el, event, this._createEventHandler(), true);
        }

        var overridedMethods = this._createElementOverridedMethods(el);
        el.addEventListener = overridedMethods.addEventListener;
        el.removeEventListener = overridedMethods.removeEventListener;
    };

    Listeners.prototype.initDocumentBodyListening = function initDocumentBodyListening(doc) {
        listeningCtx.addListeningElement(doc.body, _event.DOM_EVENTS);
        var overridedMethods = this._createDocumentBodyOverridedMethods(doc);
        doc.body.addEventListener = overridedMethods.addEventListener;
        doc.body.removeEventListener = overridedMethods.removeEventListener;
    };

    Listeners.prototype.restartElementListening = function restartElementListening(el) {
        var nativeAddEventListener = Listeners._getNativeAddEventListener(el);
        var elementCtx = this.listeningCtx.getElementCtx(el);
        if (elementCtx) {
            var eventNames = _nativeMethods2.default.objectKeys(elementCtx);

            for (var _i8 = 0; _i8 < eventNames.length; _i8++) {
                var eventName = eventNames[_i8];
                nativeAddEventListener.call(el, eventName, this._createEventHandler(), true);
            }
        }
    };

    Listeners.prototype.cancelElementListening = function cancelElementListening(el) {
        this.listeningCtx.removeListeningElement(el);
        if (el.body) this.listeningCtx.removeListeningElement(el.body);
    };

    Listeners.beforeDispatchEvent = function beforeDispatchEvent(el) {
        var elWindow = el[_internalProperties2.default.processedContext] || window;
        elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG] = (elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG] || 0) + 1;
    };

    Listeners.afterDispatchEvent = function afterDispatchEvent(el) {
        var elWindow = el[_internalProperties2.default.processedContext] || window;
        elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG]--;
        if (!elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG]) delete elWindow[EVENT_SANDBOX_DISPATCH_EVENT_FLAG];
    };

    Listeners.prototype.setEventListenerWrapper = function setEventListenerWrapper(el, events, wrapper) {
        if (!this.listeningCtx.isElementListening(el)) this.initElementListening(el, events);

        for (var _i10 = 0; _i10 < events.length; _i10++) {
            var event = events[_i10];
            var eventListeningInfo = this.listeningCtx.getEventCtx(el, event);
            eventListeningInfo.outerHandlersWrapper = wrapper;
        }
    };

    Listeners.prototype.getEventListeners = function getEventListeners(el, event) {
        var eventCtx = this.listeningCtx.getEventCtx(el, event);
        if (!eventCtx) return null;
        return _nativeMethods2.default.arrayMap.call(eventCtx.outerHandlers, function (handler) {
            return handler.fn;
        });
    };

    return Listeners;
}(_eventEmitter2.default);

exports.default = Listeners;
module.exports = exports['default'];