'use strict';

exports.__esModule = true;
exports.getElementCtx = getElementCtx;
exports.getEventCtx = getEventCtx;
exports.isElementListening = isElementListening;
exports.addListeningElement = addListeningElement;
exports.removeListeningElement = removeListeningElement;
exports.addFirstInternalHandler = addFirstInternalHandler;
exports.addInternalHandler = addInternalHandler;
exports.removeInternalHandler = removeInternalHandler;
exports.wrapEventListener = wrapEventListener;
exports.getWrapper = getWrapper;

var _browser = require('../../utils/browser');

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// NOTE: For internal usage of Listeners.
var ELEMENT_LISTENING_EVENTS_STORAGE_PROP = 'hammerhead|element-listening-events-storage-prop';
function getElementCtx(el) {
    return el[ELEMENT_LISTENING_EVENTS_STORAGE_PROP];
}
function getEventCtx(el, event) {
    event = _browser.isIE && _browser.version > 10 && /MSPointer/.test(event) ? event.replace('MS', '').toLowerCase() : event;
    var elementCtx = getElementCtx(el);
    return elementCtx && elementCtx[event];
}
function isElementListening(el) {
    return !!el[ELEMENT_LISTENING_EVENTS_STORAGE_PROP];
}
function addListeningElement(el, events) {
    var elementCtx = getElementCtx(el) || {};
    for (var i = 0; i < events.length; i++) {
        if (!elementCtx[events[i]]) {
            elementCtx[events[i]] = {
                internalHandlers: [],
                outerHandlers: [],
                outerHandlersWrapper: null,
                wrappers: [],
                cancelOuterHandlers: false
            };
        }
    }
    if (!isElementListening(el)) {
        _nativeMethods2.default.objectDefineProperty(el, ELEMENT_LISTENING_EVENTS_STORAGE_PROP, {
            value: elementCtx,
            writable: true
        });
    }
}
function removeListeningElement(el) {
    delete el[ELEMENT_LISTENING_EVENTS_STORAGE_PROP];
}
function addFirstInternalHandler(el, events, handler) {
    var elementCtx = getElementCtx(el);

    for (var _i2 = 0; _i2 < events.length; _i2++) {
        var event = events[_i2];
        elementCtx[event].internalHandlers.unshift(handler);
    }
}
function addInternalHandler(el, events, handler) {
    var elementCtx = getElementCtx(el);

    for (var _i4 = 0; _i4 < events.length; _i4++) {
        var event = events[_i4];
        elementCtx[event].internalHandlers.push(handler);
    }
}
function removeInternalHandler(el, events, handler) {
    var elementCtx = getElementCtx(el);

    for (var _i6 = 0; _i6 < events.length; _i6++) {
        var event = events[_i6];
        var internalHandlers = elementCtx[event].internalHandlers;
        var handlerIndex = internalHandlers.indexOf(handler);
        if (handlerIndex > -1) internalHandlers.splice(handlerIndex, 1);
    }
}
function wrapEventListener(eventCtx, listener, wrapper, useCapture) {
    eventCtx.outerHandlers.push({
        fn: listener,
        useCapture: useCapture || false
    });
    eventCtx.wrappers.push(wrapper);
}
function getWrapper(eventCtx, listener, useCapture) {
    var originListeners = eventCtx.outerHandlers;
    var wrappers = eventCtx.wrappers;
    var wrapper = null;
    for (var i = 0; i < originListeners.length; i++) {
        var curListener = originListeners[i];
        if (curListener.fn === listener && (curListener.useCapture || false) === (useCapture || false)) {
            wrapper = wrappers[i];
            wrappers.splice(i, 1);
            originListeners.splice(i, 1);
            return wrapper;
        }
    }
    return null;
}