'use strict';

exports.__esModule = true;

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _destinationLocation = require('../../utils/destination-location');

var destLocation = _interopRequireWildcard(_destinationLocation);

var _url = require('../../utils/url');

var _jsonHammerhead = require('json-hammerhead');

var _dom = require('../../utils/dom');

var _event = require('../../utils/event');

var _fastApply = require('../../utils/fast-apply');

var _fastApply2 = _interopRequireDefault(_fastApply);

var _propertyOverriding = require('../../utils/property-overriding');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var MESSAGE_TYPE = {
    service: 'hammerhead|service-msg',
    user: 'hammerhead|user-msg'
};

var MessageSandbox = function (_SandboxBase) {
    _inherits(MessageSandbox, _SandboxBase);

    function MessageSandbox(listeners, unloadSandbox) {
        _classCallCheck(this, MessageSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.PING_DELAY = 200;
        _this.PING_IFRAME_TIMEOUT = 7000;
        _this.PING_IFRAME_MIN_TIMEOUT = 100;
        _this.SERVICE_MSG_RECEIVED_EVENT = 'hammerhead|event|service-msg-received';
        _this.RECEIVE_MSG_FN = 'hammerhead|receive-msg-function';
        _this.pingCallback = null;
        _this.pingCmd = null;
        // NOTE: The window.top property may be changed after an iframe is removed from DOM in IE, so we save it.
        _this.topWindow = null;
        _this.window = null;
        _this.listeners = listeners;
        _this.unloadSandbox = unloadSandbox;
        _this.storedOnMessageHandler = null;
        _this.isWindowUnloaded = false;
        _this.iframeInternalMsgQueue = [];
        return _this;
    }

    MessageSandbox._getMessageData = function _getMessageData(e) {
        var rawData = (0, _dom.isMessageEvent)(e) ? _nativeMethods2.default.messageEventDataGetter.call(e) : e.data;
        return typeof rawData === 'string' ? (0, _jsonHammerhead.parse)(rawData) : rawData;
    };

    MessageSandbox.prototype._onMessage = function _onMessage(e) {
        var data = MessageSandbox._getMessageData(e);
        if (data.type === MESSAGE_TYPE.service && e.source) {
            if (this.pingCmd && data.message.cmd === this.pingCmd && data.message.isPingResponse) {
                this.pingCallback();
                this.pingCallback = null;
                this.pingCmd = null;
            } else this.emit(this.SERVICE_MSG_RECEIVED_EVENT, { message: data.message, source: e.source });
        }
    };

    MessageSandbox.prototype._onWindowMessage = function _onWindowMessage(e, originListener) {
        var data = MessageSandbox._getMessageData(e);
        if (data.type !== MESSAGE_TYPE.service) {
            var originUrl = destLocation.get();
            if (data.targetUrl === '*' || destLocation.sameOriginCheck(originUrl, data.targetUrl)) return (0, _event.callEventListener)(this.window, originListener, e);
        }
        return null;
    };

    MessageSandbox._wrapMessage = function _wrapMessage(type, message, targetUrl) {
        var parsedDest = destLocation.getParsed();
        var originUrl = (0, _url.formatUrl)({
            /*eslint-disable no-restricted-properties*/
            protocol: parsedDest.protocol,
            host: parsedDest.host
            /*eslint-enable no-restricted-properties*/
        });
        return { message: message, originUrl: originUrl, targetUrl: targetUrl, type: type };
    };

    MessageSandbox.prototype._removeInternalMsgFromQueue = function _removeInternalMsgFromQueue(sendFunc) {
        for (var index = 0, length = this.iframeInternalMsgQueue.length; index < length; index++) {
            if (this.iframeInternalMsgQueue[index].sendFunc === sendFunc) {
                this.iframeInternalMsgQueue.splice(index, 1);
                return true;
            }
        }
        return false;
    };

    MessageSandbox.prototype.attach = function attach(window) {
        var _this2 = this;

        _SandboxBase.prototype.attach.call(this, window);
        // NOTE: The window.top property may be changed after an iframe is removed from DOM in IE, so we save it.
        this.topWindow = window.top;
        this.isWindowUnloaded = false;
        this.unloadSandbox.on(this.unloadSandbox.UNLOAD_EVENT, function () {
            _this2.isWindowUnloaded = true;
            while (_this2.iframeInternalMsgQueue.length) {
                var msgInfo = _this2.iframeInternalMsgQueue[0];
                _nativeMethods2.default.clearTimeout.call(_this2.window, msgInfo.timeoutId);
                msgInfo.sendFunc();
            }
        });
        var onMessageHandler = function onMessageHandler() {
            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
            }

            return (0, _fastApply2.default)(_this2, '_onMessage', args);
        };
        var onWindowMessageHandler = function onWindowMessageHandler() {
            for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                args[_key2] = arguments[_key2];
            }

            return (0, _fastApply2.default)(_this2, '_onWindowMessage', args);
        };
        this.listeners.addInternalEventListener(window, ['message'], onMessageHandler);
        this.listeners.setEventListenerWrapper(window, ['message'], onWindowMessageHandler);
        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
        _nativeMethods2.default.objectDefineProperty(window, this.RECEIVE_MSG_FN, {
            value: onMessageHandler,
            configurable: true
        });
        // @ts-ignore
        (0, _propertyOverriding.overrideDescriptor)(window.MessageEvent.prototype, 'data', {
            getter: function getter() {
                var target = this.target;
                var data = _nativeMethods2.default.messageEventDataGetter.call(this);
                if (data && data.type !== MESSAGE_TYPE.service && (0, _dom.isWindow)(target)) return data.message;
                return data;
            }
        });
        // @ts-ignore
        var eventPropsOwner = _nativeMethods2.default.isEventPropsLocatedInProto ? window.Window.prototype : window;
        (0, _propertyOverriding.overrideDescriptor)(eventPropsOwner, 'onmessage', {
            getter: function getter() {
                return _this2.storedOnMessageHandler;
            },
            setter: function setter(handler) {
                _this2.storedOnMessageHandler = typeof handler === 'function' ? handler : null;
                _nativeMethods2.default.winOnMessageSetter.call(window, _this2.storedOnMessageHandler ? function (e) {
                    return _this2._onWindowMessage(e, handler);
                } : null);
            }
        });
    };

    MessageSandbox.prototype.postMessage = function postMessage(contentWindow, args) {
        var targetUrl = args[1] || destLocation.getOriginHeader();
        if ((0, _dom.isCrossDomainWindows)(this.window, contentWindow)) args[1] = (0, _url.getCrossDomainProxyUrl)();else if (!(0, _url.isSupportedProtocol)(contentWindow.location.toString()) || !(0, _url.isSupportedProtocol)(this.window.location.toString())) args[1] = '*';else {
            args[1] = (0, _url.formatUrl)({
                /*eslint-disable no-restricted-properties*/
                protocol: this.window.location.protocol,
                host: this.window.location.host
                /*eslint-enable no-restricted-properties*/
            });
        }
        args[0] = MessageSandbox._wrapMessage(MESSAGE_TYPE.user, args[0], targetUrl);
        return (0, _fastApply2.default)(contentWindow, 'postMessage', args);
    };

    MessageSandbox.prototype.sendServiceMsg = function sendServiceMsg(msg, targetWindow) {
        var _this3 = this;

        var message = MessageSandbox._wrapMessage(MESSAGE_TYPE.service, msg);
        var canSendDirectly = !(0, _dom.isCrossDomainWindows)(targetWindow, this.window) && !!targetWindow[this.RECEIVE_MSG_FN];
        if (canSendDirectly) {
            var sendFunc = function sendFunc(force) {
                // NOTE: In IE, this function is called on the timeout despite the fact that the timer has been cleared
                // in the unload event handler, so we check whether the function is in the queue
                if (force || _this3._removeInternalMsgFromQueue(sendFunc)) {
                    // NOTE: The 'sendFunc' function may be called on timeout, so we must call 'canSendDirectly' again,
                    // because the iframe could become cross-domain in the meantime. Unfortunately, Chrome hangs when
                    // trying to call the 'isCrossDomainWindows' function, so we have to wrap it in 'try/catch'.
                    try {
                        targetWindow[_this3.RECEIVE_MSG_FN]({
                            // NOTE: Cloning a message to prevent this modification.
                            data: (0, _jsonHammerhead.parse)((0, _jsonHammerhead.stringify)(message)),
                            source: _this3.window
                        });
                    }
                    // eslint-disable-next-line no-empty
                    catch (e) {}
                }
            };
            if (!this.isWindowUnloaded) {
                // NOTE: Imitation of a delay for the postMessage method.
                // We use the same-domain top window
                // so that the function called by setTimeout is executed after removing the iframe
                var topSameDomainWindow = (0, _dom.getTopSameDomainWindow)(this.window);
                var timeoutId = _nativeMethods2.default.setTimeout.call(topSameDomainWindow, sendFunc, 10);
                this.iframeInternalMsgQueue.push({ timeoutId: timeoutId, sendFunc: sendFunc });
            } else sendFunc(true);
            return null;
        }
        return targetWindow.postMessage(message, '*');
    };

    MessageSandbox.prototype.pingIframe = function pingIframe(targetIframe, pingMessageCommand, shortWaiting) {
        var _this4 = this;

        return new _pinkie2.default(function (resolve, reject) {
            var pingInterval = null;
            var pingTimeout = null;
            var targetWindow = null;
            var sendPingRequest = function sendPingRequest() {
                targetWindow = _nativeMethods2.default.contentWindowGetter.call(targetIframe);
                if (targetWindow) {
                    _this4.sendServiceMsg({
                        cmd: _this4.pingCmd,
                        isPingRequest: true
                    }, targetWindow);
                }
            };
            var cleanTimeouts = function cleanTimeouts() {
                _nativeMethods2.default.clearInterval.call(_this4.window, pingInterval);
                _nativeMethods2.default.clearTimeout.call(_this4.window, pingTimeout);
                _this4.pingCallback = null;
                _this4.pingCmd = null;
                pingInterval = null;
                pingTimeout = null;
            };
            pingTimeout = _nativeMethods2.default.setTimeout.call(_this4.window, function () {
                cleanTimeouts();
                reject();
            }, shortWaiting ? _this4.PING_IFRAME_MIN_TIMEOUT : _this4.PING_IFRAME_TIMEOUT);
            _this4.pingCallback = function () {
                cleanTimeouts();
                resolve();
            };
            _this4.pingCmd = pingMessageCommand;
            sendPingRequest();
            pingInterval = _nativeMethods2.default.setInterval.call(_this4.window, sendPingRequest, _this4.PING_DELAY);
        });
    };

    return MessageSandbox;
}(_base2.default);

exports.default = MessageSandbox;
module.exports = exports['default'];