'use strict';

exports.__esModule = true;

var _focusBlur = require('./focus-blur');

var _focusBlur2 = _interopRequireDefault(_focusBlur);

var _listeners = require('./listeners');

var _listeners2 = _interopRequireDefault(_listeners);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _browser = require('../../utils/browser');

var browserUtils = _interopRequireWildcard(_browser);

var _dom = require('../../utils/dom');

var domUtils = _interopRequireWildcard(_dom);

var _internalProperties = require('../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var browserResetInputSelection = browserUtils.isFirefox && browserUtils.version > 50;

var Selection = function () {
    function Selection(eventSandbox) {
        _classCallCheck(this, Selection);

        this.focusBlurSandbox = eventSandbox.focusBlur;
        this.timersSandbox = eventSandbox.timers;
        this.listeners = eventSandbox.listeners;
        this.eventSimulator = eventSandbox.eventSimulator;
        var selection = this;
        var eventSimulator = this.eventSimulator;
        var listeners = this.listeners;
        var timersSandbox = this.timersSandbox;
        this.setSelectionRangeWrapper = function () {
            var selectionStart = arguments[0];
            var selectionEnd = arguments[1];
            var selectionDirection = arguments[2] || 'none';
            var el = this;
            var fn = domUtils.isTextAreaElement(el) ? _nativeMethods2.default.textAreaSetSelectionRange : _nativeMethods2.default.setSelectionRange;
            var activeElement = domUtils.getActiveElement(domUtils.findDocument(el));
            var curDocument = domUtils.findDocument(el);
            var isElementActive = false;
            var selectionSetter = function selectionSetter() {
                var changeType = Selection._needChangeInputType(el);
                var useInternalSelection = Selection._needForInternalSelection(el);
                var savedType = el.type;
                var res = void 0;
                if (changeType) el.type = 'text';
                // NOTE: In MSEdge, an error occurs when the setSelectionRange method is called for an input with
                // 'display = none' and selectionStart !== selectionEnd in other IEs, the error doesn't occur, but
                // as a result selectionStart === selectionEnd === 0.
                try {
                    res = fn.call(el, selectionStart, selectionEnd, selectionDirection);
                } catch (e) {
                    res = fn.call(el, 0, 0, selectionDirection);
                }
                if (useInternalSelection) {
                    el[_internalProperties2.default.selection] = {
                        selectionStart: el.selectionStart,
                        selectionEnd: el.selectionEnd,
                        selectionDirection: el.selectionDirection
                    };
                }
                if (changeType) {
                    el.type = savedType;
                    // HACK: (A problem with input type = 'number' after Chrome is updated to v.33.0.1750.117 and
                    // in Firefox 29.0.  T101195) To set right selection: if the input type is 'number' or 'email',
                    // we need to change the type to text, and then restore it after setting selection.(B254340).
                    // However, the type is changed asynchronously in this case. To force type changing,we need to
                    // call blur, Then raise the focus event to make the element active.
                    if (isElementActive) {
                        selection.focusBlurSandbox.blur(el, null, true);
                        selection.focusBlurSandbox.focus(el, null, true);
                        // HACK: we should call focus for previous active element again because
                        // in Firefox 55.0.3 after first focus active element isn't changed
                        if (domUtils.getActiveElement(domUtils.findDocument(el)) !== el) selection.focusBlurSandbox.focus(el, null, true);
                    }
                }
                // NOTE: In MSEdge, the 'selectionchange' event doesn't occur immediately (it occurs with a delay)
                // So, we should raise it right after the 'setSelectionRange' method.
                if (browserUtils.isMSEdge) eventSimulator.selectionchange(el);
                return res;
            };
            if (activeElement === el) {
                isElementActive = true;
                return selectionSetter();
            }
            var needFocus = browserUtils.isIE11 || browserUtils.isMSEdge && (browserUtils.version === 17 && !curDocument.hasFocus() || browserUtils.version > 17);
            return selection.wrapSetterSelection(el, selectionSetter, needFocus);
        };
        this.selectWrapper = function () {
            var element = this.parentElement();
            if (!element || domUtils.getActiveElement(domUtils.findDocument(element)) === element) return _nativeMethods2.default.select.call(this);
            var result = null;
            var focusRaised = false;
            var focusHandler = function focusHandler(e) {
                if (e.target === element || element.style.display === 'none') focusRaised = true;
            };
            listeners.addInternalEventListener(document, ['focus'], focusHandler);
            result = _nativeMethods2.default.select.call(this);
            timersSandbox.setTimeout.call(window, function () {
                timersSandbox.setTimeout.call(window, function () {
                    listeners.removeInternalEventListener(document, ['focus'], focusHandler);
                    if (!focusRaised) eventSimulator.focus(element);
                }, 0);
            }, 0);
            return result;
        };
    }

    Selection._isNumberOrEmailInput = function _isNumberOrEmailInput(el) {
        return domUtils.isInputElement(el) && /^(number|email)$/.test(el.type);
    };

    Selection._needChangeInputType = function _needChangeInputType(el) {
        return (browserUtils.isWebKit || browserResetInputSelection) && Selection._isNumberOrEmailInput(el);
    };
    // NOTE: We need to store the state of element's selection
    // because it is cleared when element's type is changed


    Selection._needForInternalSelection = function _needForInternalSelection(el) {
        return Selection._isNumberOrEmailInput(el) && browserResetInputSelection;
    };

    Selection.prototype.setSelection = function setSelection(el, start, end, direction) {
        if (el.setSelectionRange) el.setSelectionRange(start, end, direction);else {
            el.selectionStart = start;
            el.selectionEnd = end;
        }
    };

    Selection.prototype.getSelection = function getSelection(el) {
        var changeType = Selection._needChangeInputType(el);
        var activeElement = domUtils.getActiveElement(domUtils.findDocument(el));
        var isElementActive = activeElement === el;
        var savedType = el.type;
        var selection = null;
        // HACK: (A problem with input type = ‘number’ after Chrome is updated to v.33.0.1750.117 and in
        // Firefox 29.0. T101195) To get selection, if the input type is  'number' or 'email', we need to change
        // the type to text (B254340). However, the type is changed asynchronously in this case. To force type changing,
        // we need to call blur.Then call focus to make the element active.
        if (changeType) {
            // NOTE: We shouldn't call blur while changing element's type in Firefox, cause
            // sometimes it can't be focused after. The reason of this behavior is hard to
            // be determinated, this was found during execution testcafe client tests.
            if (!browserResetInputSelection && isElementActive) this.focusBlurSandbox.blur(el, null, true);
            el.type = 'text';
        }
        var internalSelection = el[_internalProperties2.default.selection];
        selection = {
            start: internalSelection ? internalSelection.selectionStart : el.selectionStart,
            end: internalSelection ? internalSelection.selectionEnd : el.selectionEnd,
            direction: internalSelection ? internalSelection.selectionDirection : el.selectionDirection
        };
        if (changeType) {
            el.type = savedType;
            if (isElementActive) this.focusBlurSandbox.focus(el, null, true);
        }
        return selection;
    };

    Selection.prototype.wrapSetterSelection = function wrapSetterSelection(el, selectionSetter, needFocus, isContentEditable) {
        var _this = this;

        var curDocument = domUtils.findDocument(el);
        var activeElement = domUtils.getActiveElement(curDocument);
        var result = null;
        var focusRaised = false;
        // NOTE: we should not call focus during selection setting
        // if element has been focused already (TestCafe GH-2301)
        needFocus = needFocus && activeElement !== el;
        var focusHandler = function focusHandler(e) {
            if (e.target === el || el.style.display === 'none') focusRaised = true;
        };
        if (needFocus) this.listeners.addInternalEventListener(document, ['focus'], focusHandler);
        // The focus and blur events
        _listeners2.default.beforeDispatchEvent(el);
        _listeners2.default.beforeDispatchEvent(el);
        result = selectionSetter();
        // The focus and blur events
        _listeners2.default.afterDispatchEvent(el);
        _listeners2.default.afterDispatchEvent(el);
        if (needFocus) {
            activeElement = domUtils.getActiveElement(curDocument);
            if (activeElement !== el && (browserUtils.isWebKit || browserUtils.isMSEdge && browserUtils.version > 17)) {
                if (focusRaised) el[_focusBlur2.default.getInternalEventFlag('focus')] = true;
                el.focus();
            }
            // NOTE: In MSEdge, focus and blur are sync.
            if (browserUtils.isIE11) {
                this.timersSandbox.setTimeout.call(window, function () {
                    _this.timersSandbox.setTimeout.call(window, function () {
                        _this.listeners.removeInternalEventListener(document, ['focus'], focusHandler);
                        if (!focusRaised) _this.eventSimulator.focus(el);
                    }, 0);
                }, 0);
            } else {
                this.listeners.removeInternalEventListener(document, ['focus'], focusHandler);
                if (!focusRaised) {
                    // NOTE: In Firefox, raising the dispatchEvent 'focus' doesn’t activate an element.
                    // We should call the native focus method.
                    if (isContentEditable && browserUtils.isFirefox) this.focusBlurSandbox.focus(el, null, true, false, true);else this.eventSimulator.focus(el);
                }
            }
        }
        return result;
    };

    return Selection;
}();

exports.default = Selection;
module.exports = exports['default'];