'use strict';

exports.__esModule = true;

var _extend = require('../../utils/extend');

var _extend2 = _interopRequireDefault(_extend);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _browser = require('../../utils/browser');

var browserUtils = _interopRequireWildcard(_browser);

var _dom = require('../../utils/dom');

var domUtils = _interopRequireWildcard(_dom);

var _event = require('../../utils/event');

var eventUtils = _interopRequireWildcard(_event);

var _position = require('../../utils/position');

var _style = require('../../utils/style');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var TOUCH_EVENT_RADIUS = 25;
var TOUCH_EVENT_FORCE = 0.5;
var POINTER_EVENT_BUTTON = {
    noButton: -1,
    leftButton: 0,
    rightButton: 2
};
// NOTE: (IE11 only) 'MouseEvent.detail' value always equals 0 for 'click' and 'dblclick' events.
// Otherwise, MouseEvent.detail behave as described in specification - https://www.w3.org/TR/uievents/#events-mouseevents
var DEFAULT_MOUSE_EVENT_DETAIL_PROP_VALUE = {
    click: browserUtils.isIE ? 0 : 1,
    dblclick: browserUtils.isIE ? 0 : 2,
    mousedown: 1,
    mouseup: 1
};
var KEY_EVENT_NAME_RE = /^key\w+$/;
var MOUSE_EVENT_NAME_RE = /^((mouse\w+)|((dbl)?click)|(contextmenu)|(drag\w*)|(drop))$/;
var TOUCH_EVENT_NAME_RE = /^touch\w+$/;
var FOCUS_IN_OUT_EVENT_NAME_RE = /^focus(in|out)$/;
// NOTE: initTextEvent method required INPUT_METHOD param in IE
var DOM_INPUT_METHOD_KEYBOARD = 1;

var EventSimulator = function () {
    function EventSimulator() {
        _classCallCheck(this, EventSimulator);

        this.DISPATCHED_EVENT_FLAG = 'hammerhead|dispatched-event';
        this.touchIdentifier = _nativeMethods2.default.dateNow();
        // NOTE: (IE only) If event dispatching calls a native click function, we should clear the window.event
        // property (which was set in the raiseDispatchEvent function). Otherwise, the window.event property will
        // contain the dispatched event, not the native click event. We should restore the window.event value after
        // the click is handled. (B237144)
        this.savedWindowEvents = [];
        this.savedNativeClickCount = 0;
        this.browserWithNewEventsStyle = !browserUtils.isIE11;
    }

    EventSimulator._dispatchStorageEvent = function _dispatchStorageEvent(el, args) {
        var ev = _nativeMethods2.default.documentCreateEvent.call(document, 'StorageEvent');
        ev.initStorageEvent('storage', args.canBubble, args.cancelable, args.key, args.oldValue, args.newValue, args.url, null);
        _nativeMethods2.default.objectDefineProperty(ev, 'storageArea', {
            get: function get() {
                return args.storageArea;
            },
            configurable: true
        });
        if (args.key === null) {
            _nativeMethods2.default.objectDefineProperty(ev, 'key', {
                get: function get() {
                    return null;
                },
                configurable: true
            });
        }
        return el.dispatchEvent(ev);
    };

    EventSimulator._dispatchTouchEvent = function _dispatchTouchEvent(el, args) {
        var ev = _nativeMethods2.default.documentCreateEvent.call(document, 'TouchEvent');
        // HACK: A test for iOS by using initTouchEvent arguments.
        // TODO: Replace it with a user agent analysis later.
        if (ev.initTouchEvent) {
            if (browserUtils.isIOS) {
                ev.initTouchEvent(args.type, args.canBubble, args.cancelable, args.view, args.detail, args.screenX, args.screenY, args.pageX, args.pageY, args.ctrlKey, args.altKey, args.shiftKey, args.metaKey, args.touches, args.targetTouches, args.changedTouches, args.scale === void 0 ? 1.0 : args.scale,
                // NOTE: B237995
                args.rotation === void 0 ? 0.0 : args.rotation);
            } else if (ev.initTouchEvent.length === 12) {
                // NOTE: The Firefox.
                ev.initTouchEvent(args.type, args.canBubble, args.cancelable, args.view, args.detail, args.ctrlKey, args.altKey, args.shiftKey, args.metaKey, args.touches, args.targetTouches, args.changedTouches);
            } else {
                // NOTE: The default Android browser, Dolphin.
                ev.initTouchEvent(args.touches, args.targetTouches, args.changedTouches, args.type, args.view, args.screenX, args.screenY, args.pageX - args.view.pageXOffset, args.pageY - args.view.pageYOffset, args.ctrlKey, args.altKey, args.shiftKey, args.metaKey);
            }
        } else {
            ev = new _nativeMethods2.default.WindowTouchEvent(args.type, {
                touches: args.touches,
                targetTouches: args.targetTouches,
                changedTouches: args.changedTouches,
                ctrlKey: args.ctrlKey,
                altKey: args.altKey,
                shiftKey: args.shiftKey,
                metaKey: args.metaKey,
                bubbles: args.canBubble,
                cancelable: args.cancelable,
                cancelBubble: false,
                defaultPrevented: false,
                detail: args.detail,
                view: args.view
            });
        }
        return el.dispatchEvent(ev);
    };

    EventSimulator._getUIEventArgs = function _getUIEventArgs(type) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var detail = 'detail' in options ? options.detail : DEFAULT_MOUSE_EVENT_DETAIL_PROP_VALUE[type];
        return {
            type: type,
            canBubble: options.canBubble !== false,
            cancelable: options.cancelable !== false,
            view: options.view || window,
            detail: detail || 0,
            ctrlKey: options.ctrlKey || false,
            altKey: options.altKey || false,
            shiftKey: options.shiftKey || false,
            metaKey: options.metaKey || false
        };
    };

    EventSimulator._getStorageEventArgs = function _getStorageEventArgs() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

        return (0, _extend2.default)(options, {
            canBubble: options.canBubble !== false,
            cancelable: options.cancelable !== false
        });
    };

    EventSimulator._getMouseEventArgs = function _getMouseEventArgs(type) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        return (0, _extend2.default)(EventSimulator._getUIEventArgs(type, options), {
            screenX: options.screenX || 0,
            screenY: options.screenY || 0,
            clientX: options.clientX || 0,
            clientY: options.clientY || 0,
            button: options.button === void 0 ? eventUtils.BUTTON.left : options.button,
            buttons: options.buttons === void 0 ? eventUtils.BUTTONS_PARAMETER.leftButton : options.buttons,
            relatedTarget: options.relatedTarget || null,
            which: options.which
        });
    };

    EventSimulator._getKeyEventArgs = function _getKeyEventArgs(type, options) {
        var keyOptions = {
            keyCode: options.keyCode || 0,
            charCode: options.charCode || 0,
            which: type === 'press' ? options.charCode : options.keyCode
        };
        if ('keyIdentifier' in options) keyOptions.keyIdentifier = options.keyIdentifier;
        if ('key' in options) keyOptions.key = options.key;
        return (0, _extend2.default)(EventSimulator._getUIEventArgs(type, options), keyOptions);
    };

    EventSimulator._getModifiersAsString = function _getModifiersAsString(args) {
        var modifiersString = '';
        for (var modifier in eventUtils.KEYBOARD_MODIFIERS_PARAMETER) {
            if (_nativeMethods2.default.objectHasOwnProperty.call(eventUtils.KEYBOARD_MODIFIERS_PARAMETER, modifier) && args[modifier]) modifiersString += eventUtils.KEYBOARD_MODIFIERS_PARAMETER[modifier] + ' ';
        }
        return modifiersString;
    };

    EventSimulator._prepareMouseEventOptions = function _prepareMouseEventOptions() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

        var buttons = options.buttons === void 0 ? eventUtils.BUTTONS_PARAMETER.noButton : options.buttons;
        var button = eventUtils.BUTTON.left;
        options.buttons = buttons;
        options.button = options.button || button;
        if (browserUtils.isWebKit) {
            options.which = eventUtils.WHICH_PARAMETER.leftButton;
            if (options.buttons === eventUtils.BUTTONS_PARAMETER.noButton) options.which = eventUtils.WHICH_PARAMETER.noButton;
            if (options.buttons === eventUtils.BUTTONS_PARAMETER.rightButton) options.which = eventUtils.WHICH_PARAMETER.rightButton;
        }
        return options;
    };

    EventSimulator.prototype._simulateEvent = function _simulateEvent(el, event, userOptions) {
        var _this = this;

        var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

        var args = null;
        var dispatch = null;
        // NOTE: We don't simulate a click on links with modifiers (ctrl, shift, ctrl+shift, alt),
        // because it causes the opening of a browser window or additional tabs in it or loading files.
        var isClickOnLink = event === 'click' && domUtils.isAnchorElement(el);
        var opts = (0, _extend2.default)(userOptions ? {
            clientX: userOptions.clientX,
            clientY: userOptions.clientY,
            screenX: userOptions.screenX,
            screenY: userOptions.screenY,
            altKey: isClickOnLink ? false : userOptions.alt,
            shiftKey: isClickOnLink ? false : userOptions.shift,
            ctrlKey: isClickOnLink ? false : userOptions.ctrl,
            metaKey: userOptions.meta,
            button: userOptions.button,
            which: userOptions.which,
            buttons: userOptions.buttons,
            relatedTarget: userOptions.relatedTarget
        } : {}, options);
        if (!opts.relatedTarget) opts.relatedTarget = document.body;
        if (event === 'storage') {
            opts = (0, _extend2.default)(opts, userOptions);
            args = EventSimulator._getStorageEventArgs(opts);
            dispatch = EventSimulator._dispatchStorageEvent;
        } else if (MOUSE_EVENT_NAME_RE.test(event)) {
            if (userOptions && userOptions.button !== void 0) opts = (0, _extend2.default)(opts, { button: userOptions.button });
            args = EventSimulator._getMouseEventArgs(event, opts);
            // eslint-disable-next-line no-shadow
            dispatch = function dispatch(el, args) {
                return _this._dispatchMouseRelatedEvents(el, args, userOptions ? userOptions.dataTransfer : void 0);
            };
        } else if (KEY_EVENT_NAME_RE.test(event)) {
            if (userOptions && (userOptions.keyCode !== void 0 || userOptions.charCode !== void 0)) {
                opts = (0, _extend2.default)(opts, {
                    key: userOptions.key || void 0,
                    keyCode: userOptions.keyCode || 0,
                    charCode: userOptions.charCode || 0
                });
                if ('keyIdentifier' in userOptions) opts.keyIdentifier = userOptions.keyIdentifier;
                if ('key' in userOptions) opts.key = userOptions.key;
            }
            args = EventSimulator._getKeyEventArgs(event, opts);
            // eslint-disable-next-line no-shadow
            dispatch = function dispatch(el, args) {
                return _this._dispatchKeyEvent(el, args);
            };
        } else if (TOUCH_EVENT_NAME_RE.test(event)) {
            args = this._getTouchEventArgs(event, (0, _extend2.default)(opts, { target: el }));
            dispatch = EventSimulator._dispatchTouchEvent;
        }
        return dispatch(el, args);
    };

    EventSimulator.prototype._getTouchEventArgs = function _getTouchEventArgs(type) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var args = (0, _extend2.default)(EventSimulator._getUIEventArgs(type, options), {
            screenX: options.screenX || 0,
            screenY: options.screenY || 0,
            clientX: options.clientX || 0,
            clientY: options.clientY || 0,
            pageX: options.clientX || 0,
            pageY: options.clientY || 0,
            identifier: this._getTouchIdentifier(type)
        });
        if (_nativeMethods2.default.documentCreateTouch) {
            if (browserUtils.isIOS) args.touch = _nativeMethods2.default.documentCreateTouch.call(document, args.view, options.target, args.identifier, args.clientX, args.clientY, 0, 0);else {
                // NOTE: B237995
                args.touch = _nativeMethods2.default.documentCreateTouch.call(document, args.view, options.target, args.identifier, args.pageX, args.pageY, args.screenX, args.screenY, args.clientX, args.clientY, null, null, args.rotation === void 0 ? 0 : args.rotation);
            }
        } else {
            args.touch = new _nativeMethods2.default.WindowTouch({
                identifier: args.identifier,
                target: options.target,
                clientX: args.clientX,
                clientY: args.clientY,
                pageX: args.pageX,
                pageY: args.pageY,
                screenX: args.screenX,
                screenY: args.screenY,
                rotationAngle: 0,
                radiusX: TOUCH_EVENT_RADIUS,
                radiusY: TOUCH_EVENT_RADIUS,
                force: TOUCH_EVENT_FORCE
            });
        }
        args.changedTouches = [args.touch];
        // NOTE: T170088
        args.touches = args.type === 'touchend' ? [] : args.changedTouches;
        if (_nativeMethods2.default.documentCreateTouchList) {
            var _nativeMethods$docume, _nativeMethods$docume2;

            args.changedTouches = (_nativeMethods$docume = _nativeMethods2.default.documentCreateTouchList).call.apply(_nativeMethods$docume, [document].concat(args.changedTouches));
            args.touches = (_nativeMethods$docume2 = _nativeMethods2.default.documentCreateTouchList).call.apply(_nativeMethods$docume2, [document].concat(args.touches));
        }
        args.targetTouches = args.touches;
        return args;
    };

    EventSimulator.prototype._getTouchIdentifier = function _getTouchIdentifier(type) {
        // NOTE: A touch point is created when the 'touchstart' event occurs. When the point' is moved,
        // its id must not be changed (T112153).
        if (type === 'touchstart') this.touchIdentifier++;
        return this.touchIdentifier;
    };

    EventSimulator.prototype._raiseNativeClick = function _raiseNativeClick(el, originClick) {
        // NOTE: B254199
        var iframe = domUtils.isElementInIframe(el) && domUtils.getIframeByElement(el);
        var curWindow = iframe ? _nativeMethods2.default.contentWindowGetter.call(iframe) : window;
        var prevWindowEvent = curWindow.event;
        if (browserUtils.isIE11) delete curWindow.event;
        originClick.call(el);
        // NOTE: Window.event becomes empty when the click event handler
        // triggers the click event for a different element in IE11.(GH-226).
        if (browserUtils.isIE11 && prevWindowEvent) {
            _nativeMethods2.default.objectDefineProperty(curWindow, 'event', {
                get: function get() {
                    return prevWindowEvent;
                },
                configurable: true
            });
        }
    };

    EventSimulator.prototype._dispatchKeyEvent = function _dispatchKeyEvent(el, args) {
        var ev = null;
        if (this.browserWithNewEventsStyle && _nativeMethods2.default.WindowKeyboardEvent) {
            var eventArgs = {
                bubbles: args.canBubble,
                cancelable: args.cancelable,
                cancelBubble: false,
                defaultPrevented: false,
                view: args.view,
                detail: args.detail,
                ctrlKey: args.ctrlKey,
                altKey: args.altKey,
                shiftKey: args.shiftKey,
                metaKey: args.metaKey,
                keyCode: args.keyCode,
                charCode: args.charCode,
                which: args.which
            };
            if ('keyIdentifier' in args) eventArgs.keyIdentifier = args.keyIdentifier;
            if ('key' in args) eventArgs.key = args.key;
            ev = new _nativeMethods2.default.WindowKeyboardEvent(args.type, eventArgs);
        } else if (_nativeMethods2.default.documentCreateEvent) {
            ev = _nativeMethods2.default.documentCreateEvent.call(document, 'KeyboardEvent');
            ev.initKeyboardEvent(args.type, args.canBubble, args.cancelable, args.view, '', 0, EventSimulator._getModifiersAsString(args), false, '');
        }
        if (ev) {
            // NOTE: the window.event.keyCode, window.event.charCode, window.event.which and
            // window.event.key properties are not assigned after KeyboardEvent is created
            _nativeMethods2.default.objectDefineProperty(ev, 'keyCode', {
                configurable: true,
                enumerable: true,
                get: function get() {
                    return args.keyCode;
                }
            });
            _nativeMethods2.default.objectDefineProperty(ev, 'charCode', {
                configurable: true,
                enumerable: true,
                get: function get() {
                    return args.charCode;
                }
            });
            _nativeMethods2.default.objectDefineProperty(ev, 'which', {
                configurable: true,
                enumerable: true,
                get: function get() {
                    return args.which;
                }
            });
            if ('key' in args) {
                _nativeMethods2.default.objectDefineProperty(ev, 'key', {
                    configurable: true,
                    enumerable: true,
                    get: function get() {
                        return args.key;
                    }
                });
            }
            if ('keyIdentifier' in args) {
                _nativeMethods2.default.objectDefineProperty(ev, 'keyIdentifier', {
                    configurable: true,
                    enumerable: true,
                    get: function get() {
                        return args.keyIdentifier;
                    }
                });
            }
            var defaultPrevented = false;
            var returnValue = true;
            // NOTE: the dispatchEvent method does not return false in the case when preventDefault method
            // was called for events that were created with the KeyboardEvent constructor
            if (this.browserWithNewEventsStyle) {
                ev.preventDefault = function () {
                    defaultPrevented = true;
                    _nativeMethods2.default.preventDefault.call(ev);
                    return false;
                };
            }
            // NOTE: the dispatchEvent method does not return false when returnValue was set to false (only in MSEdge)
            if (browserUtils.isMSEdge) {
                _nativeMethods2.default.objectDefineProperty(ev, 'returnValue', {
                    get: function get() {
                        return returnValue;
                    },
                    set: function set(value) {
                        if (value === false) ev.preventDefault();
                        returnValue = value;
                    }
                });
            }
            var res = this._raiseDispatchEvent(el, ev);
            if (browserUtils.isMSEdge) return returnValue && !defaultPrevented;else if (browserUtils.isIE) return res;
            return !defaultPrevented;
        }
        return null;
    };

    EventSimulator.prototype._dispatchPointerEvent = function _dispatchPointerEvent(el, args) {
        var pointEvent = null;
        var elPosition = (0, _position.getOffsetPosition)(el);
        var elBorders = (0, _style.getBordersWidth)(el);
        var elClientPosition = (0, _position.offsetToClientCoords)({
            x: elPosition.left + elBorders.left,
            y: elPosition.top + elBorders.top
        });
        var eventShortType = args.type.replace('mouse', '');
        var pointerEventType = 'pointer' + eventShortType;
        var pointerArgs = (0, _extend2.default)({
            width: 1,
            height: 1,
            pressure: 0,
            tiltX: 0,
            tiltY: 0,
            // NOTE: This parameter must be "1" for “mouse”.
            pointerId: 1,
            pointerType: 'mouse',
            timeStamp: _nativeMethods2.default.dateNow(),
            isPrimary: true
        }, args);
        pointerArgs.type = pointerEventType;
        pointerArgs.offsetX = args.clientX - elClientPosition.x;
        pointerArgs.offsetY = args.clientY - elClientPosition.y;
        if (eventShortType === 'move' || eventShortType === 'over' || eventShortType === 'out') pointerArgs.button = args.buttons === eventUtils.BUTTONS_PARAMETER.noButton ? POINTER_EVENT_BUTTON.noButton : pointerArgs.button;
        if (browserUtils.isIE11) {
            pointerArgs.rotation = 0;
            pointEvent = _nativeMethods2.default.documentCreateEvent.call(document, 'PointerEvent');
            // NOTE: We set the relatedTarget argument to null because IE has a memory leak.
            pointEvent.initPointerEvent(pointerArgs.type, pointerArgs.canBubble, pointerArgs.cancelable, window, pointerArgs.detail, pointerArgs.screenX, pointerArgs.screenY, pointerArgs.clientX, pointerArgs.clientY, pointerArgs.ctrlKey, pointerArgs.altKey, pointerArgs.shiftKey, pointerArgs.metaKey, pointerArgs.button, null, pointerArgs.offsetX, pointerArgs.offsetY, pointerArgs.width, pointerArgs.height, pointerArgs.pressure, pointerArgs.rotation, pointerArgs.tiltX, pointerArgs.tiltY, pointerArgs.pointerId, pointerArgs.pointerType, pointerArgs.timeStamp, pointerArgs.isPrimary);
            // NOTE: After dispatching the pointer event, it doesn't contain the 'target' and 'relatedTarget' properties.
            _nativeMethods2.default.objectDefineProperty(pointEvent, 'target', {
                get: function get() {
                    return el;
                },
                configurable: true
            });
            _nativeMethods2.default.objectDefineProperty(pointEvent, 'relatedTarget', {
                get: function get() {
                    return args.relatedTarget;
                },
                configurable: true
            });
            _nativeMethods2.default.objectDefineProperty(pointEvent, 'buttons', {
                get: function get() {
                    return args.buttons;
                }
            });
        } else {
            pointerArgs.bubbles = true;
            pointerArgs.cancelable = true;
            pointEvent = new _nativeMethods2.default.WindowPointerEvent(pointerEventType, pointerArgs);
        }
        this._raiseDispatchEvent(el, pointEvent);
    };

    EventSimulator.prototype._dispatchMouseRelatedEvents = function _dispatchMouseRelatedEvents(el, args, dataTransfer) {
        if (args.type !== 'mouseover' && args.type !== 'mouseenter' && (0, _position.shouldIgnoreMouseEventInsideIframe)(el, args.clientX, args.clientY)) return true;
        var pointerRegExp = /mouse(down|up|move|over|out)/;
        // NOTE: In IE, submit doesn't work if a click is simulated for some submit button's children (for example,
        // img, B236676). In addition, if a test is being recorded in IE, the target of a click event is always a
        // button, not a child, so the child does not receive the click event.
        if (browserUtils.isIE) {
            if (args.type === 'click' || args.type === 'mouseup' || args.type === 'mousedown') {
                var closestButton = domUtils.closest(el.parentNode, 'button');
                if (el.parentNode && closestButton) {
                    if (_nativeMethods2.default.getAttribute.call(closestButton, 'type') === 'submit') el = closestButton;
                }
            }
        }
        if (eventUtils.hasPointerEvents && pointerRegExp.test(args.type)) this._dispatchPointerEvent(el, args);
        return this._dispatchMouseEvent(el, args, dataTransfer);
    };

    EventSimulator.prototype._dispatchMouseEvent = function _dispatchMouseEvent(el, args, dataTransfer) {
        var disabledParent = domUtils.findParent(el, true, function (node) {
            return node.disabled;
        });
        if (disabledParent) return null;
        var event = null;
        if (this.browserWithNewEventsStyle && _nativeMethods2.default.WindowMouseEvent) {
            event = new _nativeMethods2.default.WindowMouseEvent(args.type, {
                bubbles: args.canBubble,
                cancelable: args.cancelable,
                view: window,
                detail: args.detail,
                screenX: args.screenX,
                screenY: args.screenY,
                clientX: args.clientX,
                clientY: args.clientY,
                ctrlKey: args.ctrlKey,
                altKey: args.altKey,
                shiftKey: args.shiftKey,
                metaKey: args.metaKey,
                button: args.button,
                buttons: args.buttons,
                relatedTarget: args.relatedTarget
            });
        } else {
            event = _nativeMethods2.default.documentCreateEvent.call(document, 'MouseEvents');
            event.initMouseEvent(args.type, args.canBubble, args.cancelable, window, args.detail, args.screenX, args.screenY, args.clientX, args.clientY, args.ctrlKey, args.altKey, args.shiftKey, args.metaKey, args.button, args.relatedTarget);
            _nativeMethods2.default.objectDefineProperty(event, 'buttons', {
                get: function get() {
                    return args.buttons;
                }
            });
        }
        // NOTE: T188166 (act.hover triggers the mouseenter event with the "which" parameter set to 1).
        if (args.which !== void 0 && browserUtils.isWebKit) {
            _nativeMethods2.default.objectDefineProperty(event, 'which', {
                get: function get() {
                    return args.which;
                }
            });
        }
        if (dataTransfer) {
            _nativeMethods2.default.objectDefineProperty(event, 'dataTransfer', {
                configurable: true,
                enumerable: true,
                get: function get() {
                    return dataTransfer;
                }
            });
        }
        return this._raiseDispatchEvent(el, event);
    };

    EventSimulator.prototype._dispatchFocusEvent = function _dispatchFocusEvent(el, name) {
        var relatedTarget = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

        var event = null;
        var bubbles = FOCUS_IN_OUT_EVENT_NAME_RE.test(name);
        if (this.browserWithNewEventsStyle && _nativeMethods2.default.WindowFocusEvent) {
            event = new _nativeMethods2.default.WindowFocusEvent(name, {
                bubbles: bubbles,
                cancelable: false,
                cancelBubble: false,
                relatedTarget: relatedTarget,
                defaultPrevented: false
            });
        } else if (_nativeMethods2.default.documentCreateEvent) {
            event = _nativeMethods2.default.documentCreateEvent.call(document, 'FocusEvent');
            event.initFocusEvent(name, bubbles, true, null, 0, bubbles ? relatedTarget : null);
        }
        if (event) {
            event[this.DISPATCHED_EVENT_FLAG] = true;
            return this._raiseDispatchEvent(el, event);
        }
        return null;
    };

    EventSimulator.prototype._dispatchTextEvent = function _dispatchTextEvent(el, text) {
        if (_nativeMethods2.default.WindowTextEvent && _nativeMethods2.default.documentCreateEvent) {
            var event = _nativeMethods2.default.documentCreateEvent.call(document, 'TextEvent');
            var args = {
                eventType: browserUtils.isIE11 ? 'textinput' : 'textInput',
                bubbles: true,
                cancelable: true,
                view: window,
                data: text,
                inputMethod: DOM_INPUT_METHOD_KEYBOARD,
                locale: navigator.language
            };
            event.initTextEvent(args.eventType, args.bubbles, args.cancelable, args.view, args.data, args.inputMethod, args.locale);
            return this._raiseDispatchEvent(el, event);
        }
        return null;
    };

    EventSimulator.prototype._dispatchEvent = function _dispatchEvent(el, name, shouldBubble, flag) {
        var ev = null;
        if (_nativeMethods2.default.documentCreateEvent) {
            ev = _nativeMethods2.default.documentCreateEvent.call(document, 'Events');
            ev.initEvent(name, shouldBubble, true);
        }
        if (ev) {
            if (flag) ev[flag] = true;
            return this._raiseDispatchEvent(el, ev);
        }
        return null;
    };

    EventSimulator.prototype._raiseDispatchEvent = function _raiseDispatchEvent(el, ev) {
        var iframe = domUtils.isElementInIframe(el) && domUtils.getIframeByElement(el);
        var curWindow = iframe ? _nativeMethods2.default.contentWindowGetter.call(iframe) : window;
        if (browserUtils.isIE11 && iframe && curWindow) {
            // NOTE: In IE, when we raise an event by using the dispatchEvent function, the window.event object is null.
            // If a real event happens, there is a window.event object, but it is not identical with the first argument
            // of the event handler. The window.Event object is identical with the object that is created when we raise
            // the event by using  the fireEvent function. So, when we raise the event by using the dispatchEvent function,
            // we need to set the window.event object manually. An exception for IE11: The window.event object is not null
            // and it’s the same as in the event handler (only in window.top.event). Also, in iE11, window.event doesn’t
            // have the returnValue property, so it’s impossible to prevent the event by assigning window.event.returnValue
            // to false.
            // NOTE: In IE11, iframe's window.event object is null. We need to set
            // iframe's window.event object manually by using window.event (B254199).
            _nativeMethods2.default.objectDefineProperty(curWindow, 'event', {
                get: function get() {
                    return window.event;
                },
                configurable: true
            });
        }
        var res = el.dispatchEvent(ev);
        // NOTE: GH-226
        if (browserUtils.isIE11 && curWindow) delete curWindow.event;
        return res;
    };
    /* NOTE: options = {
     [clientX: integer,]
     [clientY: integer,]
     [alt: true|false,]
     [ctrl: true|false,]
     [shift: true|false,]
     [meta: true|false,]
     [button: Util.BUTTON]
    } */


    EventSimulator.prototype.click = function click(el, options) {
        return this._simulateEvent(el, 'click', options, {
            button: eventUtils.BUTTON.left,
            buttons: eventUtils.BUTTONS_PARAMETER.noButton
        });
    };

    EventSimulator.prototype.nativeClick = function nativeClick(el, originClick) {
        this._raiseNativeClick(el, originClick);
    };

    EventSimulator.prototype.dblclick = function dblclick(el, options) {
        return this._simulateEvent(el, 'dblclick', options, {
            button: eventUtils.BUTTON.left,
            buttons: eventUtils.BUTTONS_PARAMETER.noButton
        });
    };

    EventSimulator.prototype.rightclick = function rightclick(el, options) {
        return this._simulateEvent(el, 'click', options, {
            button: eventUtils.BUTTON.right,
            buttons: eventUtils.BUTTONS_PARAMETER.rightButton
        });
    };

    EventSimulator.prototype.contextmenu = function contextmenu(el, options) {
        return this._simulateEvent(el, 'contextmenu', options, {
            button: eventUtils.BUTTON.right,
            buttons: eventUtils.BUTTONS_PARAMETER.noButton
        });
    };

    EventSimulator.prototype.mousedown = function mousedown(el) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var button = options.button === void 0 ? eventUtils.BUTTON.left : options.button;
        var buttons = button === eventUtils.BUTTON.left ? eventUtils.BUTTONS_PARAMETER.leftButton : eventUtils.BUTTONS_PARAMETER.rightButton;
        options.button = button;
        options.buttons = options.buttons === void 0 ? buttons : options.buttons;
        return this._simulateEvent(el, 'mousedown', options);
    };

    EventSimulator.prototype.mouseup = function mouseup(el) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var button = options.button === void 0 ? eventUtils.BUTTON.left : options.button;
        return this._simulateEvent(el, 'mouseup', options, {
            button: button,
            buttons: eventUtils.BUTTONS_PARAMETER.noButton
        });
    };

    EventSimulator.prototype.mouseover = function mouseover(el, options) {
        options = EventSimulator._prepareMouseEventOptions(options);
        return this._simulateEvent(el, 'mouseover', options);
    };

    EventSimulator.prototype.mousemove = function mousemove(el, options) {
        options = EventSimulator._prepareMouseEventOptions(options);
        return this._simulateEvent(el, 'mousemove', options, { cancelable: false });
    };

    EventSimulator.prototype.mouseout = function mouseout(el, options) {
        options = EventSimulator._prepareMouseEventOptions(options);
        return this._simulateEvent(el, 'mouseout', options);
    };

    EventSimulator.prototype.mouseenter = function mouseenter(el, options) {
        options = EventSimulator._prepareMouseEventOptions(options);
        return this._simulateEvent(el, 'mouseenter', options, { canBubble: false });
    };

    EventSimulator.prototype.mouseleave = function mouseleave(el, options) {
        options = EventSimulator._prepareMouseEventOptions(options);
        return this._simulateEvent(el, 'mouseleave', options, { canBubble: false });
    };
    // NOTE: Keyboard events.


    EventSimulator.prototype.keypress = function keypress(el, options) {
        return this._simulateEvent(el, 'keypress', options);
    };

    EventSimulator.prototype.keyup = function keyup(el, options) {
        return this._simulateEvent(el, 'keyup', options);
    };

    EventSimulator.prototype.keydown = function keydown(el, options) {
        return this._simulateEvent(el, 'keydown', options);
    };
    // NOTE: Control events.
    // NOTE: "focus", "blur" and "selectionchange" shouldn't bubble (T229732),
    // but "input", "change" and "submit" should do it (GH-318).


    EventSimulator.prototype.blur = function blur(el, relatedTarget) {
        return this._dispatchFocusEvent(el, 'blur', relatedTarget);
    };

    EventSimulator.prototype.focus = function focus(el, relatedTarget) {
        return this._dispatchFocusEvent(el, 'focus', relatedTarget);
    };

    EventSimulator.prototype.focusin = function focusin(el, relatedTarget) {
        return this._dispatchFocusEvent(el, 'focusin', relatedTarget);
    };

    EventSimulator.prototype.focusout = function focusout(el, relatedTarget) {
        return this._dispatchFocusEvent(el, 'focusout', relatedTarget);
    };

    EventSimulator.prototype.storage = function storage(window, options) {
        return this._simulateEvent(window, 'storage', options);
    };

    EventSimulator.prototype.change = function change(el) {
        return this._dispatchEvent(el, 'change', true, this.DISPATCHED_EVENT_FLAG);
    };

    EventSimulator.prototype.textInput = function textInput(el, text) {
        return this._dispatchTextEvent(el, text);
    };

    EventSimulator.prototype.input = function input(el) {
        return this._dispatchEvent(el, 'input', true);
    };

    EventSimulator.prototype.submit = function submit(el) {
        return this._dispatchEvent(el, 'submit', true);
    };

    EventSimulator.prototype.selectionchange = function selectionchange(el) {
        return this._dispatchEvent(el, 'selectionchange', false);
    };
    // NOTE: Touch events.


    EventSimulator.prototype.touchstart = function touchstart(el, options) {
        return this._simulateEvent(el, 'touchstart', options);
    };

    EventSimulator.prototype.touchend = function touchend(el, options) {
        return this._simulateEvent(el, 'touchend', options);
    };

    EventSimulator.prototype.touchmove = function touchmove(el, options) {
        return this._simulateEvent(el, 'touchmove', options);
    };
    // NOTE: drag and drop


    EventSimulator.prototype.dragstart = function dragstart(el, options) {
        return this._simulateEvent(el, 'dragstart', options);
    };

    EventSimulator.prototype.drag = function drag(el, options) {
        return this._simulateEvent(el, 'drag', options);
    };

    EventSimulator.prototype.dragenter = function dragenter(el, options) {
        return this._simulateEvent(el, 'dragenter', options);
    };

    EventSimulator.prototype.dragover = function dragover(el, options) {
        return this._simulateEvent(el, 'dragover', options);
    };

    EventSimulator.prototype.dragleave = function dragleave(el, options) {
        return this._simulateEvent(el, 'dragleave', options);
    };

    EventSimulator.prototype.drop = function drop(el, options) {
        return this._simulateEvent(el, 'drop', options);
    };

    EventSimulator.prototype.dragend = function dragend(el, options) {
        return this._simulateEvent(el, 'dragend', options);
    };

    EventSimulator.prototype.isSavedWindowsEventsExists = function isSavedWindowsEventsExists() {
        return this.savedWindowEvents && this.savedWindowEvents.length;
    };

    return EventSimulator;
}();

exports.default = EventSimulator;
module.exports = exports['default'];