'use strict';

exports.__esModule = true;

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _createPropertyDesc = require('../../utils/create-property-desc.js');

var _createPropertyDesc2 = _interopRequireDefault(_createPropertyDesc);

var _browser = require('../../utils/browser');

var _propertyOverriding = require('../../utils/property-overriding');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var UnloadSandbox = function (_SandboxBase) {
    _inherits(UnloadSandbox, _SandboxBase);

    function UnloadSandbox(listeners) {
        _classCallCheck(this, UnloadSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.BEFORE_UNLOAD_EVENT = 'hammerhead|event|before-unload';
        _this.BEFORE_BEFORE_UNLOAD_EVENT = 'hammerhead|event|before-before-unload';
        _this.UNLOAD_EVENT = 'hammerhead|event|unload';
        _this.listeners = listeners;
        _this.storedBeforeUnloadReturnValue = '';
        _this.prevented = false;
        _this.storedBeforeUnloadHandler = null;
        // NOTE: the ios devices do not support beforeunload event
        // https://developer.apple.com/library/ios/documentation/AppleApplications/Reference/SafariWebContent/HandlingEvents/HandlingEvents.html#//apple_ref/doc/uid/TP40006511-SW5
        _this.beforeUnloadEventName = UnloadSandbox._getBeforeUnloadEventName();
        return _this;
    }

    UnloadSandbox._getBeforeUnloadEventName = function _getBeforeUnloadEventName() {
        // NOTE: the ios devices do not support beforeunload event
        // https://developer.apple.com/library/ios/documentation/AppleApplications/Reference/SafariWebContent/HandlingEvents/HandlingEvents.html#//apple_ref/doc/uid/TP40006511-SW5
        return _browser.isIOS ? 'pagehide' : 'beforeunload';
    };
    // NOTE: This handler has to be called after others.


    UnloadSandbox.prototype._emitBeforeUnloadEvent = function _emitBeforeUnloadEvent() {
        this.emit(this.BEFORE_UNLOAD_EVENT, {
            returnValue: this.storedBeforeUnloadReturnValue,
            prevented: this.prevented
        });
    };

    UnloadSandbox.prototype._onBeforeUnloadHandler = function _onBeforeUnloadHandler(e, originListener) {
        var _this2 = this;

        var eventObj = e;
        // NOTE: 'window.event' required for safari 9.0 because it calls the handler without 'e' (GH-698)
        if (!e && window.event && window.event.type === this.beforeUnloadEventName) eventObj = window.event;
        if (eventObj) {
            // NOTE: Overriding the returnValue property to prevent a native dialog.
            _nativeMethods2.default.objectDefineProperty(eventObj, 'returnValue', (0, _createPropertyDesc2.default)({
                get: function get() {
                    return _this2.storedBeforeUnloadReturnValue;
                },
                set: function set(value) {
                    // NOTE: In all browsers, if the property is set to any value, unload is prevented. In FireFox,
                    // only if a value is set to an empty string, the unload operation is prevented.
                    _this2.storedBeforeUnloadReturnValue = value;
                    _this2.prevented = _browser.isFirefox ? value !== '' : true;
                }
            }));
            _nativeMethods2.default.objectDefineProperty(eventObj, 'preventDefault', (0, _createPropertyDesc2.default)({
                get: function get() {
                    return function () {
                        _this2.prevented = true;
                        return true;
                    };
                },
                set: function set() {
                    return void 0;
                }
            }));
        }
        var res = e ? originListener(e) : originListener();
        if (res !== void 0) {
            this.storedBeforeUnloadReturnValue = res;
            this.prevented = true;
        }
    };

    UnloadSandbox.prototype._reattachBeforeUnloadListener = function _reattachBeforeUnloadListener() {
        // NOTE: reattach the Listener, it'll be the last in the queue.
        _nativeMethods2.default.windowRemoveEventListener.call(this.window, this.beforeUnloadEventName, this);
        _nativeMethods2.default.windowAddEventListener.call(this.window, this.beforeUnloadEventName, this);
    };

    UnloadSandbox.prototype.attach = function attach(window) {
        var _this3 = this;

        _SandboxBase.prototype.attach.call(this, window);
        this.listeners.setEventListenerWrapper(window, [this.beforeUnloadEventName], function (e, listener) {
            return _this3._onBeforeUnloadHandler(e, listener);
        });
        this.listeners.addInternalEventListener(window, ['unload'], function () {
            return _this3.emit(_this3.UNLOAD_EVENT);
        });
        _nativeMethods2.default.windowAddEventListener.call(window, this.beforeUnloadEventName, this);
        this.listeners.addInternalEventListener(window, [this.beforeUnloadEventName], function () {
            return _this3.emit(_this3.BEFORE_BEFORE_UNLOAD_EVENT);
        });
        this.listeners.on(this.listeners.EVENT_LISTENER_ATTACHED_EVENT, function (e) {
            if (e.el === window && e.eventType === _this3.beforeUnloadEventName) _this3._reattachBeforeUnloadListener();
        });
        // @ts-ignore
        var eventPropsOwner = _nativeMethods2.default.isEventPropsLocatedInProto ? window.Window.prototype : window;
        (0, _propertyOverriding.overrideDescriptor)(eventPropsOwner, 'on' + this.beforeUnloadEventName, {
            getter: function getter() {
                return _this3.storedBeforeUnloadHandler;
            },
            setter: function setter(handler) {
                return _this3.setOnBeforeUnload(window, handler);
            }
        });
    };

    UnloadSandbox.prototype.setOnBeforeUnload = function setOnBeforeUnload(window, handler) {
        var _this4 = this;

        var beforeUnloadEventPropSetter = _browser.isIOS ? _nativeMethods2.default.winOnPageHideSetter : _nativeMethods2.default.winOnBeforeUnloadSetter;
        if (typeof handler === 'function') {
            this.storedBeforeUnloadHandler = handler;
            beforeUnloadEventPropSetter.call(window, function (e) {
                return _this4._onBeforeUnloadHandler(e, handler);
            });
            this._reattachBeforeUnloadListener();
        } else {
            this.storedBeforeUnloadHandler = null;
            beforeUnloadEventPropSetter.call(window, null);
        }
    };

    UnloadSandbox.prototype.handleEvent = function handleEvent() {
        this._emitBeforeUnloadEvent();
    };

    return UnloadSandbox;
}(_base2.default);

exports.default = UnloadSandbox;
module.exports = exports['default'];