'use strict';

exports.__esModule = true;

var _base = require('./base');

var _base2 = _interopRequireDefault(_base);

var _nativeMethods = require('./native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _headers = require('../../request-pipeline/xhr/headers');

var _headers2 = _interopRequireDefault(_headers);

var _url = require('../utils/url');

var _destinationLocation = require('../utils/destination-location');

var _dom = require('../utils/dom');

var _sameOriginCheckFailedStatusCode = require('../../request-pipeline/xhr/same-origin-check-failed-status-code');

var _sameOriginCheckFailedStatusCode2 = _interopRequireDefault(_sameOriginCheckFailedStatusCode);

var _propertyOverriding = require('../utils/property-overriding');

var _browser = require('../utils/browser');

var browserUtils = _interopRequireWildcard(_browser);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var DEFAULT_REQUEST_CREDENTIALS = _nativeMethods2.default.Request ? new _nativeMethods2.default.Request(window.location.toString()).credentials : void 0;

var FetchSandbox = function (_SandboxBase) {
    _inherits(FetchSandbox, _SandboxBase);

    function FetchSandbox(cookieSandbox) {
        _classCallCheck(this, FetchSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.FETCH_REQUEST_SENT_EVENT = 'hammerhead|event|fetch-request-sent-event';
        _this.cookieSandbox = cookieSandbox;
        return _this;
    }

    FetchSandbox._addSpecialHeadersToRequestInit = function _addSpecialHeadersToRequestInit(init) {
        var credentials = init.credentials || DEFAULT_REQUEST_CREDENTIALS;
        var headers = init.headers;
        if (!(0, _dom.isFetchHeaders)(headers)) headers = init.headers = headers ? new _nativeMethods2.default.Headers(headers) : new _nativeMethods2.default.Headers();
        // eslint-disable-next-line no-restricted-properties
        _nativeMethods2.default.headersSet.call(headers, _headers2.default.origin, (0, _destinationLocation.getOriginHeader)());
        _nativeMethods2.default.headersSet.call(headers, _headers2.default.fetchRequestCredentials, credentials);
        return init;
    };

    FetchSandbox._processArguments = function _processArguments(args) {
        var input = args[0];
        var inputIsString = typeof input === 'string';
        var inputIsFetchRequest = (0, _dom.isFetchRequest)(input);
        var init = args[1];
        if (!inputIsFetchRequest) {
            args[0] = (0, _url.getProxyUrl)(inputIsString ? input : String(input));
            init = init || {};
            args[1] = FetchSandbox._addSpecialHeadersToRequestInit(init);
        } else if (init) args[1] = FetchSandbox._addSpecialHeadersToRequestInit(init);
    };

    FetchSandbox._sameOriginCheck = function _sameOriginCheck(args) {
        var url = null;
        var requestMode = null;
        if ((0, _dom.isFetchRequest)(args[0])) {
            url = (0, _url.parseProxyUrl)(args[0].url).destUrl;
            requestMode = args[0].mode;
        } else {
            var parsedProxyUrl = (0, _url.parseProxyUrl)(args[0]);
            url = parsedProxyUrl ? parsedProxyUrl.destUrl : args[0];
            requestMode = (args[1] || {}).mode;
        }
        if (requestMode === 'same-origin') return (0, _destinationLocation.sameOriginCheck)((0, _destinationLocation.get)(), url);
        return true;
    };

    FetchSandbox._getResponseType = function _getResponseType(response) {
        var responseUrl = _nativeMethods2.default.responseUrlGetter.call(response);
        var parsedResponseUrl = (0, _url.parseProxyUrl)(responseUrl);
        var destUrl = parsedResponseUrl && parsedResponseUrl.destUrl;
        var isSameOrigin = (0, _destinationLocation.sameOriginCheck)((0, _destinationLocation.get)(), destUrl);
        if (isSameOrigin) return 'basic';
        return response.status === 0 ? 'opaque' : 'cors';
    };

    FetchSandbox._entriesFilteredNext = function _entriesFilteredNext(iterator, nativeNext) {
        var entry = nativeNext.apply(iterator);
        if (entry.done) return entry;
        // eslint-disable-next-line no-restricted-properties
        if (entry.value[0] === _headers2.default.origin || entry.value[0] === _headers2.default.fetchRequestCredentials) return FetchSandbox._entriesFilteredNext(iterator, nativeNext);
        return entry;
    };

    FetchSandbox._entriesWrapper = function _entriesWrapper() {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
        }

        var iterator = _nativeMethods2.default.headersEntries.apply(this, args);
        var nativeNext = iterator.next;
        iterator.next = function () {
            return FetchSandbox._entriesFilteredNext(iterator, nativeNext);
        };
        return iterator;
    };

    FetchSandbox._valuesWrapper = function _valuesWrapper() {
        for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            args[_key2] = arguments[_key2];
        }

        var iterator = _nativeMethods2.default.headersEntries.apply(this, args);
        var nativeNext = iterator.next;
        iterator.next = function () {
            var filteredEntry = FetchSandbox._entriesFilteredNext(iterator, nativeNext);
            if (!filteredEntry.done) filteredEntry.value = filteredEntry.value[1]; // eslint-disable-line no-restricted-properties
            return filteredEntry;
        };
        return iterator;
    };

    FetchSandbox.prototype.attach = function attach(window) {
        _SandboxBase.prototype.attach.call(this, window, window.document);
        if (!_nativeMethods2.default.fetch) return;
        var sandbox = this;
        window.Request = function () {
            for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                args[_key3] = arguments[_key3];
            }

            FetchSandbox._processArguments(args);
            window.Headers.prototype.entries = window.Headers.prototype[Symbol.iterator] = _nativeMethods2.default.headersEntries;
            var request = args.length === 1 ? new _nativeMethods2.default.Request(args[0]) : new _nativeMethods2.default.Request(args[0], args[1]);
            window.Headers.prototype.entries = window.Headers.prototype[Symbol.iterator] = FetchSandbox._entriesWrapper;
            return request;
        };
        window.Request.prototype = _nativeMethods2.default.Request.prototype;
        window.fetch = function () {
            for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
                args[_key4] = arguments[_key4];
            }

            // NOTE: Safari processed the empty `fetch()` request without `Promise` rejection (GH-1613)
            if (!args.length && !browserUtils.isSafari) return _nativeMethods2.default.fetch.apply(this);
            try {
                FetchSandbox._processArguments(args);
            } catch (e) {
                // @ts-ignore
                return _nativeMethods2.default.promiseReject.call(sandbox.window.Promise, e);
            }
            if (!FetchSandbox._sameOriginCheck(args))
                // @ts-ignore
                return _nativeMethods2.default.promiseReject.call(sandbox.window.Promise, new TypeError());
            window.Headers.prototype.entries = window.Headers.prototype[Symbol.iterator] = _nativeMethods2.default.headersEntries;
            var fetchPromise = _nativeMethods2.default.fetch.apply(this, args);
            window.Headers.prototype.entries = window.Headers.prototype[Symbol.iterator] = FetchSandbox._entriesWrapper;
            sandbox.emit(sandbox.FETCH_REQUEST_SENT_EVENT, fetchPromise);
            return _nativeMethods2.default.promiseThen.call(fetchPromise, function (response) {
                sandbox.cookieSandbox.syncCookie();
                return response;
            });
        };
        var fetchToString = _nativeMethods2.default.fetch.toString();
        window.fetch.toString = function () {
            return fetchToString;
        };
        (0, _propertyOverriding.overrideDescriptor)(window.Response.prototype, 'type', {
            getter: function getter() {
                return FetchSandbox._getResponseType(this);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.Response.prototype, 'status', {
            getter: function getter() {
                var responseStatus = _nativeMethods2.default.responseStatusGetter.call(this);
                return responseStatus === _sameOriginCheckFailedStatusCode2.default ? 0 : responseStatus;
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.Response.prototype, 'url', {
            getter: function getter() {
                var responseUrl = _nativeMethods2.default.responseUrlGetter.call(this);
                var parsedResponseUrl = responseUrl && (0, _url.parseProxyUrl)(responseUrl);
                return parsedResponseUrl ? parsedResponseUrl.destUrl : responseUrl;
            }
        });
        window.Headers.prototype.entries = window.Headers.prototype[Symbol.iterator] = FetchSandbox._entriesWrapper;
        window.Headers.prototype.values = FetchSandbox._valuesWrapper;
        window.Headers.prototype.forEach = function () {
            for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
                args[_key5] = arguments[_key5];
            }

            var callback = args[0];
            if (typeof callback === 'function') {
                args[0] = function (_value, name) {
                    // eslint-disable-next-line no-restricted-properties
                    if (name !== _headers2.default.origin && name !== _headers2.default.fetchRequestCredentials) callback.apply(this, arguments);
                };
            }
            return _nativeMethods2.default.headersForEach.apply(this, args);
        };
    };

    return FetchSandbox;
}(_base2.default);

exports.default = FetchSandbox;
module.exports = exports['default'];