'use strict';

exports.__esModule = true;

var _internalProperties = require('../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _base = require('./base');

var _base2 = _interopRequireDefault(_base);

var _settings = require('../settings');

var _settings2 = _interopRequireDefault(_settings);

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dom = require('../../processing/dom');

var _dom2 = _interopRequireDefault(_dom);

var _dom3 = require('../utils/dom');

var _browser = require('../utils/browser');

var _jsonHammerhead = require('json-hammerhead');

var JSON = _interopRequireWildcard(_jsonHammerhead);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var IFRAME_WINDOW_INITED = 'hammerhead|iframe-window-inited';

var IframeSandbox = function (_SandboxBase) {
    _inherits(IframeSandbox, _SandboxBase);

    function IframeSandbox(nodeMutation, cookieSandbox) {
        _classCallCheck(this, IframeSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.RUN_TASK_SCRIPT_EVENT = 'hammerhead|event|run-task-script';
        _this.EVAL_HAMMERHEAD_SCRIPT_EVENT = 'hammerhead|event|eval-hammerhead-script';
        _this.EVAL_EXTERNAL_SCRIPT_EVENT = 'hammerhead|event|eval-external-script';
        _this.IFRAME_DOCUMENT_CREATED_EVENT = 'hammerhead|event|iframe-document-created';
        _this.cookieSandbox = cookieSandbox;
        _this.on(_this.RUN_TASK_SCRIPT_EVENT, _this.iframeReadyToInitHandler);
        nodeMutation.on(nodeMutation.IFRAME_ADDED_TO_DOM_EVENT, function (e) {
            return _this.processIframe(e.iframe);
        });
        _this.iframeNativeMethodsBackup = null;
        return _this;
    }

    IframeSandbox.prototype._shouldSaveIframeNativeMethods = function _shouldSaveIframeNativeMethods(iframe) {
        if (!_browser.isWebKit) return false;
        var iframeSrc = this.nativeMethods.getAttribute.call(iframe, 'src');
        return _dom2.default.isJsProtocol(iframeSrc);
    };

    IframeSandbox.prototype._ensureIframeNativeMethodsForChrome = function _ensureIframeNativeMethodsForChrome(iframe) {
        var contentWindow = _nativeMethods2.default.contentWindowGetter.call(iframe);
        var contentDocument = _nativeMethods2.default.contentDocumentGetter.call(iframe);
        if (!this.iframeNativeMethodsBackup && this._shouldSaveIframeNativeMethods(iframe)) this.iframeNativeMethodsBackup = new this.nativeMethods.constructor(contentDocument, contentWindow);else if (this.iframeNativeMethodsBackup) {
            this.iframeNativeMethodsBackup.restoreDocumentMeths(contentWindow, contentDocument);
            this.iframeNativeMethodsBackup = null;
        }
    };

    IframeSandbox.prototype._ensureIframeNativeMethodsForIE = function _ensureIframeNativeMethodsForIE(iframe) {
        var contentWindow = _nativeMethods2.default.contentWindowGetter.call(iframe);
        var contentDocument = _nativeMethods2.default.contentDocumentGetter.call(iframe);
        var iframeNativeMethods = contentWindow[_internalProperties2.default.iframeNativeMethods];
        if (iframeNativeMethods) {
            iframeNativeMethods.restoreDocumentMeths(contentWindow, contentDocument);
            delete contentWindow[_internalProperties2.default.iframeNativeMethods];
        }
    };

    IframeSandbox.prototype._ensureIframeNativeMethods = function _ensureIframeNativeMethods(iframe) {
        // NOTE: In Chrome, iframe with javascript protocol src raises the load event twice.
        // As a result, when the second load event is raised, we write the overridden methods to the native methods.
        // So, we need to save the native methods when the first load event is raised.
        // https://code.google.com/p/chromium/issues/detail?id=578812
        this._ensureIframeNativeMethodsForChrome(iframe);
        // NOTE: Restore native document methods for the iframe's document if it overrided earlier (IE9, IE10 only)
        // https://github.com/DevExpress/testcafe-hammerhead/issues/279
        this._ensureIframeNativeMethodsForIE(iframe);
    };

    IframeSandbox.prototype._emitEvents = function _emitEvents(iframe) {
        // NOTE: Raise this internal event to eval the Hammerhead code script.
        this.emit(this.EVAL_HAMMERHEAD_SCRIPT_EVENT, { iframe: iframe });
        // NOTE: Raise this event to eval external code script.
        this.emit(this.EVAL_EXTERNAL_SCRIPT_EVENT, { iframe: iframe });
        // NOTE: Raise this event to eval the "task" script and to call the Hammerhead initialization method
        // and external script initialization code.
        this.emit(this.RUN_TASK_SCRIPT_EVENT, { iframe: iframe });
    };

    IframeSandbox.prototype._raiseReadyToInitEvent = function _raiseReadyToInitEvent(iframe) {
        if (!(0, _dom3.isIframeWithoutSrc)(iframe)) return;
        var contentWindow = _nativeMethods2.default.contentWindowGetter.call(iframe);
        var contentDocument = _nativeMethods2.default.contentDocumentGetter.call(iframe);
        if (!IframeSandbox.isIframeInitialized(iframe)) {
            // NOTE: Even if iframe is not loaded (iframe.contentDocument.documentElement does not exist), we
            // still need to override the document.write method without initializing Hammerhead. This method can
            // be called before iframe is fully loaded, we should override it now.
            if (contentDocument.write.toString() === this.nativeMethods.documentWrite.toString()) this.emit(this.IFRAME_DOCUMENT_CREATED_EVENT, { iframe: iframe });
        } else if (!contentWindow[IFRAME_WINDOW_INITED] && !contentWindow[_internalProperties2.default.hammerhead]) {
            this._ensureIframeNativeMethods(iframe);
            // NOTE: Ok, the iframe is fully loaded now, but Hammerhead is not injected.
            _nativeMethods2.default.objectDefineProperty(contentWindow, IFRAME_WINDOW_INITED, { value: true });
            this._emitEvents(iframe);
            contentWindow[_internalProperties2.default.processDomMethodName]();
        }
    };

    IframeSandbox.isIframeInitialized = function isIframeInitialized(iframe) {
        var contentWindow = _nativeMethods2.default.contentWindowGetter.call(iframe);
        var contentDocument = _nativeMethods2.default.contentDocumentGetter.call(iframe);
        var isFFIframeUninitialized = _browser.isFirefox && contentWindow.document.readyState === 'uninitialized';
        return !isFFIframeUninitialized && !!contentDocument.documentElement || _browser.isIE && contentWindow[_internalProperties2.default.documentWasCleaned];
    };

    IframeSandbox.isWindowInited = function isWindowInited(window) {
        return window[IFRAME_WINDOW_INITED];
    };

    IframeSandbox.prototype.iframeReadyToInitHandler = function iframeReadyToInitHandler(e) {
        // NOTE: We are using String.replace in order to avoid adding Mustache scripts on the client side.
        // If it is needed elsewhere in a certain place, we should consider using Mustache.
        var taskScriptTemplate = _settings2.default.get().iframeTaskScriptTemplate;
        var escapeStringPatterns = function escapeStringPatterns(str) {
            return str.replace(/\$/g, '$$$$');
        };
        var cookie = JSON.stringify(this.cookieSandbox.getCookie());
        var referer = _settings2.default.get().referer || this.window.location.toString();
        var iframeTaskScriptTemplate = JSON.stringify(taskScriptTemplate);
        var taskScript = taskScriptTemplate.replace('{{{cookie}}}', escapeStringPatterns(cookie)).replace('{{{referer}}}', escapeStringPatterns(referer)).replace('{{{iframeTaskScriptTemplate}}}', escapeStringPatterns(iframeTaskScriptTemplate));
        var contentWindow = _nativeMethods2.default.contentWindowGetter.call(e.iframe);
        contentWindow.eval.call(contentWindow, taskScript);
    };

    IframeSandbox.prototype.onIframeBeganToRun = function onIframeBeganToRun(iframe) {
        this._raiseReadyToInitEvent(iframe);
    };

    IframeSandbox.prototype.processIframe = function processIframe(el) {
        var _this2 = this;

        if ((0, _dom3.isShadowUIElement)(el)) return;
        var tagName = (0, _dom3.getTagName)(el);
        if (tagName === 'iframe' && _nativeMethods2.default.contentWindowGetter.call(el) || tagName === 'frame' && _nativeMethods2.default.frameContentWindowGetter.call(el)) this._raiseReadyToInitEvent(el);
        // NOTE: This handler exists for iframes without the src attribute. In some the browsers (e.g. Chrome)
        // the load event is triggering immediately after an iframe added to DOM. In other browsers,
        // the _raiseReadyToInitEvent function is calling in our function wrapper after an iframe added to DOM.
        this.nativeMethods.addEventListener.call(el, 'load', function () {
            return _this2._raiseReadyToInitEvent(el);
        });
    };

    return IframeSandbox;
}(_base2.default);

exports.default = IframeSandbox;
module.exports = exports['default'];