'use strict';

exports.__esModule = true;

var _internalProperties = require('../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _codeInstrumentation = require('./code-instrumentation');

var _codeInstrumentation2 = _interopRequireDefault(_codeInstrumentation);

var _cookie = require('./cookie');

var _cookie2 = _interopRequireDefault(_cookie);

var _elementEditingWatcher = require('./event/element-editing-watcher');

var _elementEditingWatcher2 = _interopRequireDefault(_elementEditingWatcher);

var _event = require('./event');

var _event2 = _interopRequireDefault(_event);

var _simulator = require('./event/simulator');

var _simulator2 = _interopRequireDefault(_simulator);

var _iframe = require('./iframe');

var _iframe2 = _interopRequireDefault(_iframe);

var _listeners = require('./event/listeners');

var _listeners2 = _interopRequireDefault(_listeners);

var _message = require('./event/message');

var _message2 = _interopRequireDefault(_message);

var _mutation = require('./node/mutation');

var _mutation2 = _interopRequireDefault(_mutation);

var _node = require('./node');

var _node2 = _interopRequireDefault(_node);

var _base = require('./base');

var _base2 = _interopRequireDefault(_base);

var _shadowUi = require('./shadow-ui');

var _shadowUi2 = _interopRequireDefault(_shadowUi);

var _timers = require('./timers');

var _timers2 = _interopRequireDefault(_timers);

var _unload = require('./event/unload');

var _unload2 = _interopRequireDefault(_unload);

var _upload = require('./upload');

var _upload2 = _interopRequireDefault(_upload);

var _xhr = require('./xhr');

var _xhr2 = _interopRequireDefault(_xhr);

var _fetch = require('./fetch');

var _fetch2 = _interopRequireDefault(_fetch);

var _storages = require('./storages');

var _storages2 = _interopRequireDefault(_storages);

var _electron = require('./electron');

var _electron2 = _interopRequireDefault(_electron);

var _console = require('./console');

var _console2 = _interopRequireDefault(_console);

var _style = require('./style');

var _style2 = _interopRequireDefault(_style);

var _browser = require('../utils/browser');

var _html = require('../utils/html');

var _anchor = require('./code-instrumentation/properties/anchor');

var _backup = require('./backup');

var _urlResolver = require('../utils/url-resolver');

var _urlResolver2 = _interopRequireDefault(_urlResolver);

var _windowsStorage = require('./windows-storage');

var windowStorage = _interopRequireWildcard(_windowsStorage);

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _ieDebug = require('./ie-debug');

var _ieDebug2 = _interopRequireDefault(_ieDebug);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Sandbox = function (_SandboxBase) {
    _inherits(Sandbox, _SandboxBase);

    function Sandbox() {
        _classCallCheck(this, Sandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        (0, _backup.create)(window, _this);
        windowStorage.add(window);
        var ieDebugSandbox = new _ieDebug2.default();
        var listeners = new _listeners2.default();
        var nodeMutation = new _mutation2.default();
        var unloadSandbox = new _unload2.default(listeners);
        var messageSandbox = new _message2.default(listeners, unloadSandbox);
        var eventSimulator = new _simulator2.default();
        var elementEditingWatcher = new _elementEditingWatcher2.default(eventSimulator);
        var timersSandbox = new _timers2.default();
        var cookieSandbox = new _cookie2.default(messageSandbox);
        // API
        _this.ieDebug = ieDebugSandbox;
        _this.cookie = cookieSandbox; // eslint-disable-line no-restricted-properties
        _this.storageSandbox = new _storages2.default(listeners, unloadSandbox, eventSimulator);
        _this.xhr = new _xhr2.default(cookieSandbox);
        _this.fetch = new _fetch2.default(cookieSandbox);
        _this.iframe = new _iframe2.default(nodeMutation, cookieSandbox);
        _this.shadowUI = new _shadowUi2.default(nodeMutation, messageSandbox, _this.iframe, ieDebugSandbox);
        _this.upload = new _upload2.default(listeners, eventSimulator, _this.shadowUI);
        _this.event = new _event2.default(listeners, eventSimulator, elementEditingWatcher, unloadSandbox, messageSandbox, _this.shadowUI, timersSandbox);
        _this.node = new _node2.default(nodeMutation, _this.iframe, _this.event, _this.upload, _this.shadowUI, cookieSandbox);
        _this.codeInstrumentation = new _codeInstrumentation2.default(_this.event, _this.node.win, messageSandbox);
        _this.console = new _console2.default(messageSandbox);
        _this.style = new _style2.default();
        _this.unload = unloadSandbox;
        if (_browser.isElectron) _this.electron = new _electron2.default();
        _this.windowStorage = windowStorage;
        return _this;
    }
    // NOTE: In some cases, IE raises the "Can't execute code from a freed script" exception,
    // so that we cannot use a sandbox created earlier and we have to create a new one.


    Sandbox._canUseSandbox = function _canUseSandbox(sandbox) {
        try {
            sandbox.off();
        } catch (e) {
            return false;
        }
        return true;
    };

    Sandbox.prototype.onIframeDocumentRecreated = function onIframeDocumentRecreated(iframe) {
        if (iframe) {
            var contentWindow = _nativeMethods2.default.contentWindowGetter.call(iframe);
            var contentDocument = _nativeMethods2.default.contentDocumentGetter.call(iframe);
            // NOTE: Try to find an existing iframe sandbox.
            var sandbox = (0, _backup.get)(contentWindow);
            if (sandbox && Sandbox._canUseSandbox(sandbox))
                // NOTE: Inform the sandbox so that it restores communication with the recreated document.
                sandbox.reattach(contentWindow, contentDocument);else {
                // NOTE: Remove saved native methods for iframe
                if (contentWindow[_internalProperties2.default.iframeNativeMethods]) delete contentWindow[_internalProperties2.default.iframeNativeMethods];
                // NOTE: If the iframe sandbox is not found, this means that iframe is not initialized.
                // In this case, we need to inject Hammerhead.
                // HACK: IE10 cleans up overridden methods after the document.write method call.
                this.nativeMethods.restoreDocumentMeths(contentWindow, contentDocument);
                // NOTE: A sandbox for this iframe is not found (iframe is not yet initialized).
                // Inform IFrameSandbox about this, and it injects Hammerhead.
                this.iframe.onIframeBeganToRun(iframe);
            }
        }
    };

    Sandbox.prototype.reattach = function reattach(window, document) {
        // NOTE: Assign the existing sandbox to the cleared document.
        if (_browser.isIE) this.nativeMethods.refreshIfNecessary(document, window);
        _urlResolver2.default.init(document);
        this.event.initDocumentListening(document);
        this.shadowUI.attach(window);
        // NOTE: T182337
        this.codeInstrumentation.attach(window);
        this.node.doc.attach(window, document);
        this.console.attach(window);
    };

    Sandbox.prototype.attach = function attach(window) {
        var _this2 = this;

        _SandboxBase.prototype.attach.call(this, window);
        _urlResolver2.default.init(this.document);
        // NOTE: Eval Hammerhead code script.
        this.iframe.on(this.iframe.EVAL_HAMMERHEAD_SCRIPT_EVENT, function (e) {
            // @ts-ignore
            _nativeMethods2.default.contentWindowGetter.call(e.iframe).eval('(' + initHammerheadClient.toString() + ')();//# sourceURL=hammerhead.js');
        });
        // NOTE: We need to reattach a sandbox to the recreated iframe document.
        this.node.mutation.on(this.node.mutation.DOCUMENT_CLEANED_EVENT, function (e) {
            return _this2.reattach(e.window, e.document);
        });
        this.ieDebug.attach(window);
        this.iframe.attach(window);
        this.xhr.attach(window);
        this.fetch.attach(window);
        this.storageSandbox.attach(window);
        this.codeInstrumentation.attach(window);
        this.shadowUI.attach(window);
        this.event.attach(window);
        this.node.attach(window);
        this.upload.attach(window);
        this.cookie.attach(window); // eslint-disable-line no-restricted-properties
        this.console.attach(window);
        this.style.attach(window);
        if (this.electron) this.electron.attach(window);
        this.unload.on(this.unload.UNLOAD_EVENT, function () {
            return _this2.dispose();
        });
    };

    Sandbox.prototype._removeInternalProperties = function _removeInternalProperties() {
        var removeListeningElement = this.event.listeners.listeningCtx.removeListeningElement;
        removeListeningElement(this.window);
        removeListeningElement(this.document);
        var childNodes = _nativeMethods2.default.querySelectorAll.call(this.document, '*');
        var length = _nativeMethods2.default.nodeListLengthGetter.call(childNodes);
        for (var i = 0; i < length; i++) {
            var childNode = childNodes[i];
            delete childNode[_internalProperties2.default.processedContext];
            removeListeningElement(childNode);
        }
    };

    Sandbox.prototype.dispose = function dispose() {
        this.event.hover.lastHoveredElement = null;
        this.event.focusBlur.lastFocusedElement = null;
        (0, _html.dispose)();
        (0, _anchor.dispose)();
        _urlResolver2.default.dispose(this.document);
        this.storageSandbox.dispose();
        this._removeInternalProperties();
    };

    return Sandbox;
}(_base2.default);

exports.default = Sandbox;
module.exports = exports['default'];