'use strict';

exports.__esModule = true;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*global Document, Window */
var NativeMethods = function () {
    function NativeMethods(doc, win) {
        _classCallCheck(this, NativeMethods);

        win = win || window;
        this.refreshDocumentMeths(doc, win);
        this.refreshElementMeths(doc, win);
        this.refreshWindowMeths(win);
    }

    NativeMethods._getDocumentPropOwnerName = function _getDocumentPropOwnerName(docPrototype, propName) {
        return docPrototype.hasOwnProperty(propName) ? 'Document' : 'HTMLDocument';
    };

    NativeMethods.prototype.getStoragesPropsOwner = function getStoragesPropsOwner(win) {
        // @ts-ignore
        return this.isStoragePropsLocatedInProto ? win.Window.prototype : win;
    };

    NativeMethods.prototype.refreshDocumentMeths = function refreshDocumentMeths(doc, win) {
        doc = doc || document;
        win = win || window;
        // @ts-ignore
        var docPrototype = win.Document.prototype;
        // Dom
        this.createDocumentFragment = docPrototype.createDocumentFragment;
        this.createElement = docPrototype.createElement;
        this.createElementNS = docPrototype.createElementNS;
        this.documentOpenPropOwnerName = NativeMethods._getDocumentPropOwnerName(docPrototype, 'open');
        this.documentClosePropOwnerName = NativeMethods._getDocumentPropOwnerName(docPrototype, 'close');
        this.documentWritePropOwnerName = NativeMethods._getDocumentPropOwnerName(docPrototype, 'write');
        this.documentWriteLnPropOwnerName = NativeMethods._getDocumentPropOwnerName(docPrototype, 'writeln');
        this.documentOpen = win[this.documentOpenPropOwnerName].prototype.open;
        this.documentClose = win[this.documentClosePropOwnerName].prototype.close;
        this.documentWrite = win[this.documentWritePropOwnerName].prototype.write;
        this.documentWriteLn = win[this.documentWriteLnPropOwnerName].prototype.writeln;
        this.elementFromPoint = docPrototype.elementFromPoint;
        this.caretRangeFromPoint = docPrototype.caretRangeFromPoint;
        this.caretPositionFromPoint = docPrototype.caretPositionFromPoint;
        this.getElementById = docPrototype.getElementById;
        this.getElementsByClassName = docPrototype.getElementsByClassName;
        this.getElementsByName = docPrototype.getElementsByName;
        this.getElementsByTagName = docPrototype.getElementsByTagName;
        this.querySelector = docPrototype.querySelector;
        this.querySelectorAll = docPrototype.querySelectorAll;
        // @ts-ignore
        this.createHTMLDocument = win.DOMImplementation.prototype.createHTMLDocument;
        // @ts-ignore
        if (doc.registerElement) this.registerElement = docPrototype.registerElement;
        // Event
        this.documentAddEventListener = docPrototype.addEventListener;
        this.documentRemoveEventListener = docPrototype.removeEventListener;
        this.documentCreateEvent = docPrototype.createEvent;
        this.documentCreateTouch = docPrototype.createTouch;
        this.documentCreateTouchList = docPrototype.createTouchList;
        // getters/setters
        this.documentCookiePropOwnerName = NativeMethods._getDocumentPropOwnerName(docPrototype, 'cookie');
        this.documentScriptsPropOwnerName = NativeMethods._getDocumentPropOwnerName(docPrototype, 'scripts');
        // @ts-ignore
        var documentCookieDescriptor = win.Object.getOwnPropertyDescriptor(win[this.documentCookiePropOwnerName].prototype, 'cookie');
        // TODO: remove this condition after the GH-1649 fix
        if (documentCookieDescriptor.get.toString().indexOf('native code') === -1 || documentCookieDescriptor.get.toString.toString().indexOf('native code') === -1) {
            try {
                var parentNativeMethods = win.parent['%hammerhead%'].nativeMethods;
                documentCookieDescriptor.get = parentNativeMethods.documentCookieGetter;
                documentCookieDescriptor.set = parentNativeMethods.documentCookieSetter;
            }
            // eslint-disable-next-line no-empty
            catch (e) {}
        }
        // @ts-ignore
        this.documentReferrerGetter = win.Object.getOwnPropertyDescriptor(docPrototype, 'referrer').get;
        // @ts-ignore
        this.documentStyleSheetsGetter = win.Object.getOwnPropertyDescriptor(docPrototype, 'styleSheets').get;
        // @ts-ignore
        this.documentActiveElementGetter = win.Object.getOwnPropertyDescriptor(docPrototype, 'activeElement').get;
        // @ts-ignore
        this.documentScriptsGetter = win.Object.getOwnPropertyDescriptor(win[this.documentScriptsPropOwnerName].prototype, 'scripts').get;
        this.documentCookieGetter = documentCookieDescriptor.get;
        this.documentCookieSetter = documentCookieDescriptor.set;
        // @ts-ignore
        var documentDocumentURIDescriptor = win.Object.getOwnPropertyDescriptor(docPrototype, 'documentURI');
        if (documentDocumentURIDescriptor) this.documentDocumentURIGetter = documentDocumentURIDescriptor.get;
    };

    NativeMethods.prototype.refreshElementMeths = function refreshElementMeths(doc, win) {
        var _this = this;

        win = win || window;
        var createElement = function createElement(tagName) {
            return _this.createElement.call(doc || document, tagName);
        };
        var nativeElement = createElement('div');
        // Dom
        this.appendChild = nativeElement.appendChild;
        this.replaceChild = nativeElement.replaceChild;
        this.cloneNode = nativeElement.cloneNode;
        this.elementGetElementsByClassName = nativeElement.getElementsByClassName;
        this.elementGetElementsByTagName = nativeElement.getElementsByTagName;
        this.elementQuerySelector = nativeElement.querySelector;
        this.elementQuerySelectorAll = nativeElement.querySelectorAll;
        this.getAttribute = nativeElement.getAttribute;
        this.getAttributeNS = nativeElement.getAttributeNS;
        this.insertAdjacentHTML = nativeElement.insertAdjacentHTML;
        this.insertBefore = nativeElement.insertBefore;
        this.insertCell = createElement('tr').insertCell;
        this.insertTableRow = createElement('table').insertRow;
        this.insertTBodyRow = createElement('tbody').insertRow;
        this.removeAttribute = nativeElement.removeAttribute;
        this.removeAttributeNS = nativeElement.removeAttributeNS;
        this.removeChild = nativeElement.removeChild;
        this.setAttribute = nativeElement.setAttribute;
        this.setAttributeNS = nativeElement.setAttributeNS;
        this.hasAttribute = nativeElement.hasAttribute;
        this.hasAttributeNS = nativeElement.hasAttributeNS;
        this.hasAttributes = nativeElement.hasAttributes;
        this.anchorToString = win.HTMLAnchorElement.prototype.toString;
        this.matches = nativeElement.matches || nativeElement.msMatchesSelector;
        this.closest = nativeElement.closest;
        // Event
        this.addEventListener = nativeElement.addEventListener;
        this.removeEventListener = nativeElement.removeEventListener;
        this.blur = nativeElement.blur;
        this.click = nativeElement.click;
        this.dispatchEvent = nativeElement.dispatchEvent;
        this.focus = nativeElement.focus;
        // @ts-ignore
        this.select = window.TextRange ? createElement('body').createTextRange().select : null;
        this.setSelectionRange = createElement('input').setSelectionRange;
        this.textAreaSetSelectionRange = createElement('textarea').setSelectionRange;
        this.svgFocus = win.SVGElement ? win.SVGElement.prototype.focus : this.focus;
        this.svgBlur = win.SVGElement ? win.SVGElement.prototype.blur : this.blur;
        // Style
        // NOTE: The 'style' descriptor is located in the Element.prototype in the Safari on IOS
        this.htmlElementStylePropOwnerName = win.Element.prototype.hasOwnProperty('style') ? 'Element' : 'HTMLElement';
        var htmlElementStyleDescriptor = win.Object.getOwnPropertyDescriptor(win[this.htmlElementStylePropOwnerName].prototype, 'style');
        this.htmlElementStyleGetter = htmlElementStyleDescriptor.get;
        // NOTE: IE does not allow to set a style property
        if (htmlElementStyleDescriptor.set) this.htmlElementStyleSetter = htmlElementStyleDescriptor.set;
        var styleCssTextDescriptor = win.Object.getOwnPropertyDescriptor(win.CSSStyleDeclaration.prototype, 'cssText');
        this.styleCssTextGetter = styleCssTextDescriptor.get;
        this.styleCssTextSetter = styleCssTextDescriptor.set;
    };

    NativeMethods.prototype.refreshWindowMeths = function refreshWindowMeths(win) {
        win = win || window;
        var winProto = win.constructor.prototype;
        // Dom
        this.eval = win.eval;
        this.formSubmit = win.HTMLFormElement.prototype.submit;
        this.documentFragmentQuerySelector = win.DocumentFragment.prototype.querySelector;
        this.documentFragmentQuerySelectorAll = win.DocumentFragment.prototype.querySelectorAll;
        this.preventDefault = win.Event.prototype.preventDefault;
        this.historyPushState = win.history.pushState;
        this.historyReplaceState = win.history.replaceState;
        this.windowDispatchEvent = win.dispatchEvent;
        this.postMessage = win.postMessage || winProto.postMessage;
        this.windowOpen = win.open || winProto.open;
        this.setTimeout = win.setTimeout || winProto.setTimeout;
        this.setInterval = win.setInterval || winProto.setInterval;
        this.clearTimeout = win.clearTimeout || winProto.clearTimeout;
        this.clearInterval = win.clearInterval || winProto.clearInterval;
        if (win.navigator.registerProtocolHandler) this.registerProtocolHandler = win.navigator.registerProtocolHandler;
        if (win.navigator.sendBeacon) this.sendBeacon = win.navigator.sendBeacon;
        // XHR
        this.xhrAbort = win.XMLHttpRequest.prototype.abort;
        this.xhrOpen = win.XMLHttpRequest.prototype.open;
        this.xhrSend = win.XMLHttpRequest.prototype.send;
        this.xhrAddEventListener = win.XMLHttpRequest.prototype.addEventListener;
        this.xhrRemoveEventListener = win.XMLHttpRequest.prototype.removeEventListener;
        this.xhrGetResponseHeader = win.XMLHttpRequest.prototype.getResponseHeader;
        this.xhrGetAllResponseHeaders = win.XMLHttpRequest.prototype.getAllResponseHeaders;
        this.xhrSetRequestHeader = win.XMLHttpRequest.prototype.setRequestHeader;
        this.xhrOverrideMimeType = win.XMLHttpRequest.prototype.overrideMimeType;
        this.xhrDispatchEvent = win.XMLHttpRequest.prototype.dispatchEvent;
        try {
            this.registerServiceWorker = win.navigator.serviceWorker.register;
            this.getRegistrationServiceWorker = win.navigator.serviceWorker.getRegistration;
        } catch (e) {
            this.registerServiceWorker = null;
            this.getRegistrationServiceWorker = null;
        }
        this.createContextualFragment = win.Range.prototype.createContextualFragment;
        var nativePerformance = win.performance;
        var nativePerformanceNow = win.performance.now || win.Performance.prototype.now;
        this.performanceNow = function () {
            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
            }

            return nativePerformanceNow.apply(nativePerformance, args);
        };
        // Fetch
        this.fetch = win.fetch;
        this.Request = win.Request;
        if (win.Headers) {
            this.Headers = win.Headers;
            this.headersSet = win.Headers.prototype.set;
            this.headersEntries = win.Headers.prototype.entries;
            this.headersForEach = win.Headers.prototype.forEach;
        }
        // Event
        this.windowAddEventListener = win.addEventListener || winProto.addEventListener;
        this.windowRemoveEventListener = win.removeEventListener || winProto.removeEventListener;
        this.WindowPointerEvent = win.PointerEvent || winProto.PointerEvent;
        this.WindowMSPointerEvent = win.MSPointerEvent || winProto.MSPointerEvent;
        this.WindowTouch = win.Touch || winProto.Touch;
        this.WindowTouchEvent = win.TouchEvent || winProto.TouchEvent;
        this.WindowKeyboardEvent = win.KeyboardEvent || winProto.KeyboardEvent;
        this.WindowFocusEvent = win.FocusEvent || winProto.FocusEvent;
        this.WindowTextEvent = win.TextEvent || winProto.TextEvent;
        this.WindowMouseEvent = win.MouseEvent || winProto.MouseEvent;
        this.canvasContextDrawImage = win.CanvasRenderingContext2D.prototype.drawImage;
        this.formDataAppend = win.FormData.prototype.append;
        // DateTime
        this.date = win.Date;
        this.dateNow = win.Date.now;
        // Math
        this.math = win.Math;
        this.mathRandom = win.Math.random;
        // Object
        this.objectToString = win.Object.prototype.toString;
        this.objectAssign = win.Object.assign;
        this.objectKeys = win.Object.keys;
        this.objectDefineProperty = win.Object.defineProperty;
        this.objectDefineProperties = win.Object.defineProperties;
        this.objectCreate = win.Object.create;
        this.objectIsExtensible = win.Object.isExtensible;
        this.objectIsFrozen = win.Object.isFrozen;
        this.objectGetOwnPropertyDescriptor = win.Object.getOwnPropertyDescriptor;
        this.objectHasOwnProperty = win.Object.hasOwnProperty;
        this.objectGetOwnPropertyNames = win.Object.getOwnPropertyNames;
        this.objectGetPrototypeOf = win.Object.getPrototypeOf;
        // Array
        this.arraySlice = win.Array.prototype.slice;
        this.arrayConcat = win.Array.prototype.concat;
        this.arrayFilter = win.Array.prototype.filter;
        this.arrayMap = win.Array.prototype.map;
        this.arrayJoin = win.Array.prototype.join;
        this.arraySplice = win.Array.prototype.splice;
        this.DOMParserParseFromString = win.DOMParser.prototype.parseFromString;
        this.arrayBufferIsView = win.ArrayBuffer.prototype.constructor.isView;
        var objectDataDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLObjectElement.prototype, 'data');
        var inputTypeDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'type');
        var inputValueDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'value');
        var inputRequiredDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'required');
        var textAreaValueDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLTextAreaElement.prototype, 'value');
        var imageSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLImageElement.prototype, 'src');
        var scriptSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLScriptElement.prototype, 'src');
        var scriptIntegrityDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLScriptElement.prototype, 'integrity');
        var embedSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLEmbedElement.prototype, 'src');
        var sourceSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLSourceElement.prototype, 'src');
        var mediaSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLMediaElement.prototype, 'src');
        var inputSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'src');
        var frameSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLFrameElement.prototype, 'src');
        var iframeSrcDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLIFrameElement.prototype, 'src');
        var anchorHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'href');
        var linkHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLLinkElement.prototype, 'href');
        var linkIntegrityDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLLinkElement.prototype, 'integrity');
        var linkRelDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLLinkElement.prototype, 'rel');
        var areaHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAreaElement.prototype, 'href');
        var baseHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLBaseElement.prototype, 'href');
        var anchorHostDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'host');
        var anchorHostnameDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'hostname');
        var anchorPathnameDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'pathname');
        var anchorPortDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'port');
        var anchorProtocolDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'protocol');
        var anchorSearchDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'search');
        var anchorTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'target');
        var formTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLFormElement.prototype, 'target');
        var areaTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAreaElement.prototype, 'target');
        var baseTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLBaseElement.prototype, 'target');
        var inputFormTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'formTarget');
        var buttonFormTargetDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLButtonElement.prototype, 'formTarget');
        var svgImageHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.SVGImageElement.prototype, 'href');
        var svgAnimStrAnimValDescriptor = win.Object.getOwnPropertyDescriptor(win.SVGAnimatedString.prototype, 'animVal');
        var svgAnimStrBaseValDescriptor = win.Object.getOwnPropertyDescriptor(win.SVGAnimatedString.prototype, 'baseVal');
        var inputAutocompleteDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'autocomplete');
        var formActionDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLFormElement.prototype, 'action');
        var inputFormActionDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'formAction');
        var buttonFormActionDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLButtonElement.prototype, 'formAction');
        var nodeTextContentDescriptor = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'textContent');
        var htmlElementInnerTextDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLElement.prototype, 'innerText');
        var scriptTextDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLScriptElement.prototype, 'text');
        var anchorTextDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'text');
        var iframeSandboxDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLIFrameElement.prototype, 'sandbox');
        // NOTE: Html properties is located in HTMLElement prototype in IE11 only
        this.elementHTMLPropOwnerName = win.Element.prototype.hasOwnProperty('innerHTML') ? 'Element' : 'HTMLElement';
        var elementInnerHTMLDescriptor = win.Object.getOwnPropertyDescriptor(win[this.elementHTMLPropOwnerName].prototype, 'innerHTML');
        var elementOuterHTMLDescriptor = win.Object.getOwnPropertyDescriptor(win[this.elementHTMLPropOwnerName].prototype, 'outerHTML');
        // Setters
        this.objectDataSetter = objectDataDescriptor.set;
        this.inputTypeSetter = inputTypeDescriptor.set;
        this.inputValueSetter = inputValueDescriptor.set;
        this.inputRequiredSetter = inputRequiredDescriptor.set;
        this.textAreaValueSetter = textAreaValueDescriptor.set;
        this.imageSrcSetter = imageSrcDescriptor.set;
        this.scriptSrcSetter = scriptSrcDescriptor.set;
        this.embedSrcSetter = embedSrcDescriptor.set;
        this.sourceSrcSetter = sourceSrcDescriptor.set;
        this.mediaSrcSetter = mediaSrcDescriptor.set;
        this.inputSrcSetter = inputSrcDescriptor.set;
        this.frameSrcSetter = frameSrcDescriptor.set;
        this.iframeSrcSetter = iframeSrcDescriptor.set;
        this.anchorHrefSetter = anchorHrefDescriptor.set;
        this.linkHrefSetter = linkHrefDescriptor.set;
        this.linkRelSetter = linkRelDescriptor.set;
        this.areaHrefSetter = areaHrefDescriptor.set;
        this.baseHrefSetter = baseHrefDescriptor.set;
        this.anchorHostSetter = anchorHostDescriptor.set;
        this.anchorHostnameSetter = anchorHostnameDescriptor.set;
        this.anchorPathnameSetter = anchorPathnameDescriptor.set;
        this.anchorPortSetter = anchorPortDescriptor.set;
        this.anchorProtocolSetter = anchorProtocolDescriptor.set;
        this.anchorSearchSetter = anchorSearchDescriptor.set;
        this.anchorTargetSetter = anchorTargetDescriptor.set;
        this.formTargetSetter = formTargetDescriptor.set;
        this.areaTargetSetter = areaTargetDescriptor.set;
        this.baseTargetSetter = baseTargetDescriptor.set;
        this.inputFormTargetSetter = inputFormTargetDescriptor.set;
        this.buttonFormTargetSetter = buttonFormTargetDescriptor.set;
        this.svgAnimStrBaseValSetter = svgAnimStrBaseValDescriptor.set;
        this.inputAutocompleteSetter = inputAutocompleteDescriptor.set;
        this.formActionSetter = formActionDescriptor.set;
        this.inputFormActionSetter = inputFormActionDescriptor.set;
        this.buttonFormActionSetter = buttonFormActionDescriptor.set;
        this.iframeSandboxSetter = iframeSandboxDescriptor.set;
        this.htmlElementOnloadSetter = win.Object.getOwnPropertyDescriptor(win.HTMLElement.prototype, 'onload').set;
        this.nodeTextContentSetter = nodeTextContentDescriptor.set;
        this.htmlElementInnerTextSetter = htmlElementInnerTextDescriptor.set;
        this.scriptTextSetter = scriptTextDescriptor.set;
        this.anchorTextSetter = anchorTextDescriptor.set;
        this.elementInnerHTMLSetter = elementInnerHTMLDescriptor.set;
        this.elementOuterHTMLSetter = elementOuterHTMLDescriptor.set;
        // NOTE: Some browsers (for example, Edge, Internet Explorer 11, Safari) don't support the 'integrity' property.
        if (scriptIntegrityDescriptor && linkIntegrityDescriptor) {
            this.scriptIntegritySetter = scriptIntegrityDescriptor.set;
            this.linkIntegritySetter = linkIntegrityDescriptor.set;
        }
        // NOTE: Event properties is located in window prototype only in IE11
        this.isEventPropsLocatedInProto = win.Window.prototype.hasOwnProperty('onerror');
        var eventPropsOwner = this.isEventPropsLocatedInProto ? win.Window.prototype : win;
        this.winOnBeforeUnloadSetter = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onbeforeunload').set;
        this.winOnPageHideSetter = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onpagehide').set;
        this.winOnMessageSetter = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onmessage').set;
        this.winOnErrorSetter = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onerror').set;
        var winOnUnhandledRejectionDescriptor = win.Object.getOwnPropertyDescriptor(eventPropsOwner, 'onunhandledrejection');
        if (winOnUnhandledRejectionDescriptor) this.winOnUnhandledRejectionSetter = winOnUnhandledRejectionDescriptor.set;
        // Getters
        if (win.WebSocket) {
            var urlPropDescriptor = win.Object.getOwnPropertyDescriptor(win.WebSocket.prototype, 'url');
            if (urlPropDescriptor && urlPropDescriptor.get && urlPropDescriptor.configurable) this.webSocketUrlGetter = urlPropDescriptor.get;
        }
        // NOTE: the classList property is located in HTMLElement prototype in IE11
        this.elementClassListPropOwnerName = win.Element.prototype.hasOwnProperty('classList') ? 'Element' : 'HTMLElement';
        this.elementClassListGetter = win.Object.getOwnPropertyDescriptor(win[this.elementClassListPropOwnerName].prototype, 'classList').get;
        this.messageEventOriginGetter = win.Object.getOwnPropertyDescriptor(win.MessageEvent.prototype, 'origin').get;
        this.htmlCollectionLengthGetter = win.Object.getOwnPropertyDescriptor(win.HTMLCollection.prototype, 'length').get;
        this.nodeListLengthGetter = win.Object.getOwnPropertyDescriptor(win.NodeList.prototype, 'length').get;
        this.elementChildElementCountGetter = win.Object.getOwnPropertyDescriptor(win.Element.prototype, 'childElementCount').get;
        this.inputFilesGetter = win.Object.getOwnPropertyDescriptor(win.HTMLInputElement.prototype, 'files').get;
        this.styleSheetHrefGetter = win.Object.getOwnPropertyDescriptor(win.StyleSheet.prototype, 'href').get;
        this.xhrStatusGetter = win.Object.getOwnPropertyDescriptor(win.XMLHttpRequest.prototype, 'status').get;
        this.objectDataGetter = objectDataDescriptor.get;
        this.inputTypeGetter = inputTypeDescriptor.get;
        this.inputValueGetter = inputValueDescriptor.get;
        this.inputRequiredGetter = inputRequiredDescriptor.get;
        this.textAreaValueGetter = textAreaValueDescriptor.get;
        this.imageSrcGetter = imageSrcDescriptor.get;
        this.scriptSrcGetter = scriptSrcDescriptor.get;
        this.embedSrcGetter = embedSrcDescriptor.get;
        this.sourceSrcGetter = sourceSrcDescriptor.get;
        this.mediaSrcGetter = mediaSrcDescriptor.get;
        this.inputSrcGetter = inputSrcDescriptor.get;
        this.frameSrcGetter = frameSrcDescriptor.get;
        this.iframeSrcGetter = iframeSrcDescriptor.get;
        this.anchorHrefGetter = anchorHrefDescriptor.get;
        this.linkHrefGetter = linkHrefDescriptor.get;
        this.linkRelGetter = linkRelDescriptor.get;
        this.areaHrefGetter = areaHrefDescriptor.get;
        this.baseHrefGetter = baseHrefDescriptor.get;
        this.anchorHostGetter = anchorHostDescriptor.get;
        this.anchorHostnameGetter = anchorHostnameDescriptor.get;
        this.anchorPathnameGetter = anchorPathnameDescriptor.get;
        this.anchorPortGetter = anchorPortDescriptor.get;
        this.anchorProtocolGetter = anchorProtocolDescriptor.get;
        this.anchorSearchGetter = anchorSearchDescriptor.get;
        this.anchorTargetGetter = anchorTargetDescriptor.get;
        this.formTargetGetter = formTargetDescriptor.get;
        this.areaTargetGetter = areaTargetDescriptor.get;
        this.baseTargetGetter = baseTargetDescriptor.get;
        this.inputFormTargetGetter = inputFormTargetDescriptor.get;
        this.buttonFormTargetGetter = buttonFormTargetDescriptor.get;
        this.svgImageHrefGetter = svgImageHrefDescriptor.get;
        this.svgAnimStrAnimValGetter = svgAnimStrAnimValDescriptor.get;
        this.svgAnimStrBaseValGetter = svgAnimStrBaseValDescriptor.get;
        this.inputAutocompleteGetter = inputAutocompleteDescriptor.get;
        this.formActionGetter = formActionDescriptor.get;
        this.inputFormActionGetter = inputFormActionDescriptor.get;
        this.buttonFormActionGetter = buttonFormActionDescriptor.get;
        this.iframeSandboxGetter = iframeSandboxDescriptor.get;
        this.contentWindowGetter = win.Object.getOwnPropertyDescriptor(win.HTMLIFrameElement.prototype, 'contentWindow').get;
        this.contentDocumentGetter = win.Object.getOwnPropertyDescriptor(win.HTMLIFrameElement.prototype, 'contentDocument').get;
        this.frameContentWindowGetter = win.Object.getOwnPropertyDescriptor(win.HTMLFrameElement.prototype, 'contentWindow').get;
        this.nodeTextContentGetter = nodeTextContentDescriptor.get;
        this.htmlElementInnerTextGetter = htmlElementInnerTextDescriptor.get;
        this.scriptTextGetter = scriptTextDescriptor.get;
        this.anchorTextGetter = anchorTextDescriptor.get;
        this.elementInnerHTMLGetter = elementInnerHTMLDescriptor.get;
        this.elementOuterHTMLGetter = elementOuterHTMLDescriptor.get;
        this.nodeFirstChildGetter = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'firstChild').get;
        this.nodeLastChildGetter = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'lastChild').get;
        this.nodeNextSiblingGetter = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'nextSibling').get;
        this.nodePrevSiblingGetter = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'previousSibling').get;
        this.elementFirstElementChildGetter = win.Object.getOwnPropertyDescriptor(win.Element.prototype, 'firstElementChild').get;
        this.elementLastElementChildGetter = win.Object.getOwnPropertyDescriptor(win.Element.prototype, 'lastElementChild').get;
        this.elementNextElementSiblingGetter = win.Object.getOwnPropertyDescriptor(win.Element.prototype, 'nextElementSibling').get;
        this.elementPrevElementSiblingGetter = win.Object.getOwnPropertyDescriptor(win.Element.prototype, 'previousElementSibling').get;
        var anchorOriginDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLAnchorElement.prototype, 'origin');
        // NOTE: Some browsers (for example, Edge, Internet Explorer 11, Safari) don't support the 'integrity' property.
        if (scriptIntegrityDescriptor && linkIntegrityDescriptor) {
            this.scriptIntegrityGetter = scriptIntegrityDescriptor.get;
            this.linkIntegrityGetter = linkIntegrityDescriptor.get;
        }
        // NOTE: IE and Edge don't support origin property
        if (anchorOriginDescriptor) this.anchorOriginGetter = anchorOriginDescriptor.get;
        var cssStyleSheetHrefDescriptor = win.Object.getOwnPropertyDescriptor(win.CSSStyleSheet.prototype, 'href');
        // NOTE: IE11 doesn't support the 'href' property
        if (cssStyleSheetHrefDescriptor) this.cssStyleSheetHrefGetter = cssStyleSheetHrefDescriptor.get;
        var nodeBaseURIDescriptor = win.Object.getOwnPropertyDescriptor(win.Node.prototype, 'baseURI');
        // NOTE: IE11 doesn't support the 'baseURI' property
        if (nodeBaseURIDescriptor) this.nodeBaseURIGetter = nodeBaseURIDescriptor.get;
        // NOTE: The 'attributes' property is located in Node prototype in IE11 only
        this.elementAttributesPropOwnerName = win.Element.prototype.hasOwnProperty('attributes') ? 'Element' : 'Node';
        this.elementAttributesGetter = win.Object.getOwnPropertyDescriptor(win[this.elementAttributesPropOwnerName].prototype, 'attributes').get;
        // NOTE: At present we proxy only the PerformanceNavigationTiming.
        // Another types of the PerformanceEntry will be fixed later
        // https://developer.mozilla.org/en-US/docs/Web/API/PerformanceEntry
        if (win.PerformanceNavigationTiming) this.performanceEntryNameGetter = win.Object.getOwnPropertyDescriptor(win.PerformanceEntry.prototype, 'name').get;
        var dataPropDescriptor = win.Object.getOwnPropertyDescriptor(win.MessageEvent.prototype, 'data');
        // NOTE: This condition is used for the Android 6.0 browser
        if (dataPropDescriptor) this.messageEventDataGetter = dataPropDescriptor.get;
        var htmlManifestDescriptor = win.Object.getOwnPropertyDescriptor(win.HTMLHtmlElement.prototype, 'manifest');
        // NOTE: Only the Safari browser supports the 'manifest' property
        if (htmlManifestDescriptor) {
            this.htmlManifestGetter = htmlManifestDescriptor.get;
            this.htmlManifestSetter = htmlManifestDescriptor.set;
        }
        if (win.fetch) {
            this.responseStatusGetter = win.Object.getOwnPropertyDescriptor(win.Response.prototype, 'status').get;
            this.responseTypeGetter = win.Object.getOwnPropertyDescriptor(win.Response.prototype, 'type').get;
            this.responseUrlGetter = win.Object.getOwnPropertyDescriptor(win.Response.prototype, 'url').get;
        }
        if (win.Promise) {
            this.promiseThen = win.Promise.prototype.then;
            this.promiseReject = win.Promise.reject;
        }
        var xhrResponseURLDescriptor = win.Object.getOwnPropertyDescriptor(win.XMLHttpRequest.prototype, 'responseURL');
        // NOTE: IE doesn't support the 'responseURL' property
        if (xhrResponseURLDescriptor) this.xhrResponseURLGetter = xhrResponseURLDescriptor.get;
        // NOTE: The 'localStorage' and 'sessionStorage' properties is located in window prototype only in IE11
        this.isStoragePropsLocatedInProto = win.Window.prototype.hasOwnProperty('localStorage');
        var storagesPropsOwner = this.getStoragesPropsOwner(win);
        this.winLocalStorageGetter = win.Object.getOwnPropertyDescriptor(storagesPropsOwner, 'localStorage').get;
        this.winSessionStorageGetter = win.Object.getOwnPropertyDescriptor(storagesPropsOwner, 'sessionStorage').get;
        // MutationRecord
        this.mutationRecordNextSiblingGetter = win.Object.getOwnPropertyDescriptor(win.MutationRecord.prototype, 'nextSibling').get;
        this.mutationRecordPrevSiblingGetter = win.Object.getOwnPropertyDescriptor(win.MutationRecord.prototype, 'previousSibling').get;
        // Stylesheets
        this.styleGetPropertyValue = win.CSSStyleDeclaration.prototype.getPropertyValue;
        this.styleSetProperty = win.CSSStyleDeclaration.prototype.setProperty;
        this.styleRemoveProperty = win.CSSStyleDeclaration.prototype.removeProperty;
        this.styleInsertRule = win.CSSStyleSheet.prototype.insertRule;
        // Console
        this.console = win.console;
        if (this.console) {
            this.consoleMeths = {
                log: win.console.log,
                warn: win.console.warn,
                error: win.console.error,
                info: win.console.info
            };
        }
        // DOMTokenList
        this.tokenListAdd = win.DOMTokenList.prototype.add;
        this.tokenListRemove = win.DOMTokenList.prototype.remove;
        this.tokenListReplace = win.DOMTokenList.prototype.replace;
        this.tokenListSupports = win.DOMTokenList.prototype.supports;
        this.tokenListToggle = win.DOMTokenList.prototype.toggle;
        this.tokenListContains = win.DOMTokenList.prototype.contains;
        this.refreshClasses(win);
    };

    NativeMethods.prototype.refreshClasses = function refreshClasses(win) {
        this.windowClass = win.Window;
        this.documentClass = win.Document;
        this.locationClass = win.Location;
        this.elementClass = win.Element;
        this.svgElementClass = win.SVGElement;
        this.Worker = win.Worker;
        this.ArrayBuffer = win.ArrayBuffer;
        this.Uint8Array = win.Uint8Array;
        this.DataView = win.DataView;
        this.Blob = win.Blob;
        this.XMLHttpRequest = win.XMLHttpRequest;
        this.Image = win.Image;
        this.Function = win.Function;
        this.functionToString = win.Function.toString;
        this.FontFace = win.FontFace;
        this.StorageEvent = win.StorageEvent;
        this.MutationObserver = win.MutationObserver;
        this.EventSource = win.EventSource;
        this.Proxy = win.Proxy;
        this.WebSocket = win.WebSocket;
        this.HTMLCollection = win.HTMLCollection;
        this.NodeList = win.NodeList;
        this.Node = win.Node;
        if (win.Proxy) this.Proxy = win.Proxy;
        if (win.DataTransfer) this.DataTransfer = win.DataTransfer;
        if (win.DataTransferItemList) this.DataTransferItemList = win.DataTransferItemList;
        if (win.DataTransferItem) this.DataTransferItem = win.DataTransferItem;
        if (win.FileList) this.FileList = win.FileList;
    };

    NativeMethods.prototype.refreshElectronMeths = function refreshElectronMeths(vmModule) {
        if (this.createScript && this.createScript.toString() !== vmModule.createScript.toString()) return false;
        this.createScript = vmModule.createScript;
        this.runInDebugContext = vmModule.runInDebugContext;
        this.runInContext = vmModule.runInContext;
        this.runInNewContext = vmModule.runInNewContext;
        this.runInThisContext = vmModule.runInThisContext;
        return true;
    };

    NativeMethods._ensureDocumentMethodRestore = function _ensureDocumentMethodRestore(document, prototype, methodName, savedNativeMethod) {
        prototype[methodName] = savedNativeMethod;
        if (document[methodName] !== prototype[methodName]) document[methodName] = savedNativeMethod;
    };

    NativeMethods.prototype.restoreDocumentMeths = function restoreDocumentMeths(window, document) {
        var docPrototype = window.Document.prototype;
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createDocumentFragment', this.createDocumentFragment);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createElement', this.createElement);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createElementNS', this.createElementNS);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'elementFromPoint', this.elementFromPoint);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'caretRangeFromPoint', this.caretRangeFromPoint);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'caretPositionFromPoint', this.caretPositionFromPoint);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'getElementById', this.getElementById);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'getElementsByClassName', this.getElementsByClassName);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'getElementsByName', this.getElementsByName);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'getElementsByTagName', this.getElementsByTagName);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'querySelector', this.querySelector);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'querySelectorAll', this.querySelectorAll);
        // Event
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'addEventListener', this.documentAddEventListener);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'removeEventListener', this.documentRemoveEventListener);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createEvent', this.documentCreateEvent);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createTouch', this.documentCreateTouch);
        NativeMethods._ensureDocumentMethodRestore(document, docPrototype, 'createTouchList', this.documentCreateTouchList);
        NativeMethods._ensureDocumentMethodRestore(document, window[this.documentOpenPropOwnerName].prototype, 'open', this.documentOpen);
        NativeMethods._ensureDocumentMethodRestore(document, window[this.documentClosePropOwnerName].prototype, 'close', this.documentClose);
        NativeMethods._ensureDocumentMethodRestore(document, window[this.documentWritePropOwnerName].prototype, 'write', this.documentWrite);
        NativeMethods._ensureDocumentMethodRestore(document, window[this.documentWriteLnPropOwnerName].prototype, 'writeln', this.documentWriteLn);
    };

    NativeMethods.prototype.refreshIfNecessary = function refreshIfNecessary(doc, win) {
        var _this2 = this;

        var tryToExecuteCode = function tryToExecuteCode(func) {
            try {
                return func();
            } catch (e) {
                return true;
            }
        };
        var needToRefreshDocumentMethods = tryToExecuteCode(function () {
            return !doc.createElement || _this2.createElement.toString() === document.createElement.toString();
        });
        var needToRefreshElementMethods = tryToExecuteCode(function () {
            var nativeElement = _this2.createElement.call(doc, 'div');
            return nativeElement.getAttribute.toString() === _this2.getAttribute.toString();
        });
        var needToRefreshWindowMethods = tryToExecuteCode(function () {
            _this2.setTimeout.call(win, function () {
                return void 0;
            }, 0);
            return win.XMLHttpRequest.prototype.open.toString() === _this2.xhrOpen.toString();
        });
        // NOTE: T173709
        if (needToRefreshDocumentMethods) this.refreshDocumentMeths(doc, win);
        if (needToRefreshElementMethods) this.refreshElementMeths(doc, win);
        // NOTE: T239109
        if (needToRefreshWindowMethods) this.refreshWindowMeths(win);
    };

    return NativeMethods;
}();

exports.default = new NativeMethods();
module.exports = exports['default'];