'use strict';

exports.__esModule = true;
exports.assignAttributes = assignAttributes;
exports.getAttributes = getAttributes;
exports.refreshAttributesWrapper = refreshAttributesWrapper;

var _wrapper = require('./wrapper');

var _wrapper2 = _interopRequireDefault(_wrapper);

var _dom = require('../../../utils/dom');

var _nativeMethods = require('../../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dom2 = require('../../../../processing/dom');

var _dom3 = _interopRequireDefault(_dom2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ATTRIBUTES_WRAPPER = 'hammerhead|element-attribute-wrappers';
function cleanAttributes(wrapper) {
    if (wrapper.length) {
        for (var i = 0; i < wrapper.length; i++) {
            delete wrapper[wrapper[i].name];
            delete wrapper[i];
        }
    }
}
function assignAttributes(wrapper, attributes) {
    var length = 0;
    var properties = {};

    for (var _i2 = 0; _i2 < attributes.length; _i2++) {
        var attr = attributes[_i2];
        if (!(0, _dom.isHammerheadAttr)(attr.name)) {
            var storedAttr = attributes[_dom3.default.getStoredAttrName(attr.name)];
            if (storedAttr) {
                // eslint-disable-next-line no-restricted-properties
                if (_dom3.default.isAddedAutocompleteAttr(attr.name, storedAttr.value)) continue;
                attr = _nativeMethods2.default.cloneNode.call(attr);
                // eslint-disable-next-line no-restricted-properties
                attr.value = storedAttr.value;
            }
            properties[attr.name] = { value: attr, configurable: true, enumerable: true };
            properties[length] = { value: attr, configurable: true };
            length++;
        }
    }

    properties['length'] = { value: length, configurable: true };
    _nativeMethods2.default.objectDefineProperties(wrapper, properties);
}
function getAttributes(el) {
    if (el[ATTRIBUTES_WRAPPER]) {
        refreshAttributesWrapper(el);
        return el[ATTRIBUTES_WRAPPER];
    }
    var attributes = _nativeMethods2.default.elementAttributesGetter.call(el);
    if (!attributes) return attributes;

    for (var _i4 = 0; _i4 < attributes.length; _i4++) {
        var attr = attributes[_i4];
        if ((0, _dom.isHammerheadAttr)(attr.name)) {
            _wrapper2.default.prototype = attributes;
            el[ATTRIBUTES_WRAPPER] = new _wrapper2.default(el, attributes);
            return el[ATTRIBUTES_WRAPPER];
        }
    }

    return attributes;
}
function refreshAttributesWrapper(el) {
    var attributesWrapper = el[ATTRIBUTES_WRAPPER];
    if (attributesWrapper) {
        cleanAttributes(attributesWrapper);
        assignAttributes(attributesWrapper, _nativeMethods2.default.elementAttributesGetter.call(el));
    }
}