'use strict';

exports.__esModule = true;

var _base = require('../../base');

var _base2 = _interopRequireDefault(_base);

var _iframe = require('../../iframe');

var _iframe2 = _interopRequireDefault(_iframe);

var _nativeMethods = require('../../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _domProcessor = require('../../../dom-processor');

var _domProcessor2 = _interopRequireDefault(_domProcessor);

var _url = require('../../../utils/url');

var urlUtils = _interopRequireWildcard(_url);

var _settings = require('../../../settings');

var _settings2 = _interopRequireDefault(_settings);

var _browser = require('../../../utils/browser');

var _dom = require('../../../utils/dom');

var _writer = require('./writer');

var _writer2 = _interopRequireDefault(_writer);

var _shadowUi = require('./../../shadow-ui');

var _shadowUi2 = _interopRequireDefault(_shadowUi);

var _internalProperties = require('../../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _location = require('../../code-instrumentation/location');

var _location2 = _interopRequireDefault(_location);

var _propertyOverriding = require('../../../utils/property-overriding');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var DocumentSandbox = function (_SandboxBase) {
    _inherits(DocumentSandbox, _SandboxBase);

    function DocumentSandbox(nodeSandbox, shadowUI, cookieSandbox) {
        _classCallCheck(this, DocumentSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.nodeSandbox = nodeSandbox;
        _this.documentWriter = null;
        _this.shadowUI = shadowUI;
        _this.cookieSandbox = cookieSandbox;
        return _this;
    }

    DocumentSandbox.forceProxySrcForImageIfNecessary = function forceProxySrcForImageIfNecessary(element) {
        if ((0, _dom.isImgElement)(element) && _settings2.default.get().forceProxySrcForImage) element[_internalProperties2.default.forceProxySrcForImage] = true;
    };

    DocumentSandbox._isDocumentInDesignMode = function _isDocumentInDesignMode(doc) {
        return doc.designMode === 'on';
    };

    DocumentSandbox.prototype._isUninitializedIframeWithoutSrc = function _isUninitializedIframeWithoutSrc(win) {
        var frameElement = (0, _dom.getFrameElement)(win);
        return win !== win.top && frameElement && (0, _dom.isIframeWithoutSrc)(frameElement) && !_iframe2.default.isIframeInitialized(frameElement);
    };

    DocumentSandbox.prototype._beforeDocumentCleaned = function _beforeDocumentCleaned() {
        this.nodeSandbox.mutation.onBeforeDocumentCleaned({ document: this.document });
    };

    DocumentSandbox.prototype._onDocumentClosed = function _onDocumentClosed() {
        this.nodeSandbox.mutation.onDocumentClosed({ document: this.document });
    };

    DocumentSandbox._shouldEmitDocumentCleanedEvents = function _shouldEmitDocumentCleanedEvents(doc) {
        if (_browser.isIE) {
            if (doc.readyState !== 'loading') return true;
            var window = doc.defaultView;
            if (window[_internalProperties2.default.documentWasCleaned]) return false;
            var iframe = window && (0, _dom.getFrameElement)(window);
            return iframe && (0, _dom.isIframeWithoutSrc)(iframe);
        }
        return doc.readyState !== 'loading' && doc.readyState !== 'uninitialized';
    };

    DocumentSandbox.prototype._performDocumentWrite = function _performDocumentWrite(args, ln) {
        var shouldEmitEvents = DocumentSandbox._shouldEmitDocumentCleanedEvents(this.document);
        if (shouldEmitEvents) this._beforeDocumentCleaned();
        var result = this.documentWriter.write(args, ln, shouldEmitEvents);
        // NOTE: B234357
        if (!shouldEmitEvents) this.nodeSandbox.processNodes(null, this.document);
        return result;
    };

    DocumentSandbox.prototype._needToUpdateDocumentWriter = function _needToUpdateDocumentWriter(window, document) {
        try {
            return !this.documentWriter || this.window !== window || this.document !== document;
        } catch (e) {
            return true;
        }
    };

    DocumentSandbox._definePropertyDescriptor = function _definePropertyDescriptor(owner, childOfOwner, prop, overriddenDescriptor) {
        // NOTE: The 'URL', 'domain' and 'referrer' properties are non configurable in IE and Edge
        if (!overriddenDescriptor.configurable) {
            // NOTE: property doesn't redefined yet
            if (!childOfOwner.hasOwnProperty(prop)) _nativeMethods2.default.objectDefineProperty(childOfOwner, prop, overriddenDescriptor);
        } else _nativeMethods2.default.objectDefineProperty(owner, prop, overriddenDescriptor);
    };

    DocumentSandbox.prototype.iframeDocumentOpen = function iframeDocumentOpen(window, document, args) {
        var iframe = window.frameElement;
        var result = _nativeMethods2.default.documentOpen.apply(document, args);
        _nativeMethods2.default.objectDefineProperty(window, _internalProperties2.default.documentWasCleaned, { value: true, configurable: true });
        this.nodeSandbox.iframeSandbox.onIframeBeganToRun(iframe);
        return result;
    };

    DocumentSandbox._ensureDocumentMethodOverride = function _ensureDocumentMethodOverride(document, overridenMethods, methodName) {
        if (document[methodName] !== overridenMethods[methodName]) document[methodName] = overridenMethods[methodName];
    };

    DocumentSandbox.prototype.attach = function attach(window, document) {
        var _this2 = this;

        if (this._needToUpdateDocumentWriter(window, document)) {
            this.documentWriter = new _writer2.default(window, document);
            this.nodeSandbox.mutation.on(this.nodeSandbox.mutation.BEFORE_DOCUMENT_CLEANED_EVENT, function () {
                _this2.documentWriter = new _writer2.default(window, document);
            });
        }
        _SandboxBase.prototype.attach.call(this, window, document);
        var documentSandbox = this;
        var docPrototype = window.Document.prototype;
        var overridenMethods = {
            open: function open() {
                var isUninitializedIframe = documentSandbox._isUninitializedIframeWithoutSrc(window);
                if (!isUninitializedIframe) documentSandbox._beforeDocumentCleaned();

                for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                    args[_key] = arguments[_key];
                }

                if (_browser.isIE) return window.parent[_internalProperties2.default.hammerhead].sandbox.node.doc.iframeDocumentOpen(window, this, args);
                var result = _nativeMethods2.default.documentOpen.apply(this, args);
                // NOTE: Chrome does not remove the "%hammerhead%" property from window
                // after document.open call
                var objectDefinePropertyFn = window[_internalProperties2.default.hammerhead] ? window[_internalProperties2.default.hammerhead].nativeMethods.objectDefineProperty : window.Object.defineProperty;
                objectDefinePropertyFn(window, _internalProperties2.default.documentWasCleaned, { value: true, configurable: true });
                if (!isUninitializedIframe) documentSandbox.nodeSandbox.mutation.onDocumentCleaned({ window: window, document: this });else
                    // NOTE: If iframe initialization is in progress, we need to override the document.write and document.open
                    // methods once again, because they were cleaned after the native document.open method call.
                    documentSandbox.attach(window, this);
                return result;
            },
            close: function close() {
                // NOTE: IE11 raise the "load" event only when the document.close method is called. We need to
                // restore the overriden document.open and document.write methods before Hammerhead injection, if the
                // window is not initialized.
                if (_browser.isIE && !_iframe2.default.isWindowInited(window)) _nativeMethods2.default.restoreDocumentMeths(window, this);
                // NOTE: IE doesn't run scripts in iframe if iframe.documentContent.designMode equals 'on' (GH-871)
                if (DocumentSandbox._isDocumentInDesignMode(this)) _shadowUi2.default.removeSelfRemovingScripts(this);

                for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                    args[_key2] = arguments[_key2];
                }

                var result = _nativeMethods2.default.documentClose.apply(this, args);
                if (!documentSandbox._isUninitializedIframeWithoutSrc(window)) documentSandbox._onDocumentClosed();
                var iframe = (0, _dom.getFrameElement)(window);
                // NOTE: Firefox misses the Hammerhead instance after the iframe.contentDocument.close function calling (GH-1821)
                if (iframe) documentSandbox.nodeSandbox.iframeSandbox.onIframeBeganToRun(iframe);
                return result;
            },
            write: function write() {
                return documentSandbox._performDocumentWrite(arguments);
            },
            writeln: function writeln() {
                return documentSandbox._performDocumentWrite(arguments, true);
            }
        };
        window[_nativeMethods2.default.documentOpenPropOwnerName].prototype.open = overridenMethods.open;
        window[_nativeMethods2.default.documentClosePropOwnerName].prototype.close = overridenMethods.close;
        window[_nativeMethods2.default.documentWritePropOwnerName].prototype.write = overridenMethods.write;
        window[_nativeMethods2.default.documentWriteLnPropOwnerName].prototype.writeln = overridenMethods.writeln;
        DocumentSandbox._ensureDocumentMethodOverride(document, overridenMethods, 'open');
        DocumentSandbox._ensureDocumentMethodOverride(document, overridenMethods, 'close');
        DocumentSandbox._ensureDocumentMethodOverride(document, overridenMethods, 'write');
        DocumentSandbox._ensureDocumentMethodOverride(document, overridenMethods, 'writeln');
        if (document.open !== overridenMethods.open) document.open = overridenMethods.open;
        docPrototype.createElement = function () {
            for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                args[_key3] = arguments[_key3];
            }

            var el = _nativeMethods2.default.createElement.apply(this, args);
            DocumentSandbox.forceProxySrcForImageIfNecessary(el);
            _domProcessor2.default.processElement(el, urlUtils.convertToProxyUrl);
            documentSandbox.nodeSandbox.processNodes(el);
            return el;
        };
        docPrototype.createElementNS = function () {
            for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
                args[_key4] = arguments[_key4];
            }

            var el = _nativeMethods2.default.createElementNS.apply(this, args);
            DocumentSandbox.forceProxySrcForImageIfNecessary(el);
            _domProcessor2.default.processElement(el, urlUtils.convertToProxyUrl);
            documentSandbox.nodeSandbox.processNodes(el);
            return el;
        };
        docPrototype.createDocumentFragment = function () {
            for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
                args[_key5] = arguments[_key5];
            }

            var fragment = _nativeMethods2.default.createDocumentFragment.apply(this, args);
            documentSandbox.nodeSandbox.processNodes(fragment);
            return fragment;
        };
        var htmlDocPrototype = window.HTMLDocument.prototype;
        var storedDomain = '';
        if (_nativeMethods2.default.documentDocumentURIGetter) {
            (0, _propertyOverriding.overrideDescriptor)(docPrototype, 'documentURI', {
                getter: function getter() {
                    var documentURI = _nativeMethods2.default.documentDocumentURIGetter.call(this);
                    var parsedProxyUrl = urlUtils.parseProxyUrl(documentURI);
                    return parsedProxyUrl ? parsedProxyUrl.destUrl : documentURI;
                }
            });
        }
        var referrerOverriddenDescriptor = (0, _propertyOverriding.createOverriddenDescriptor)(docPrototype, 'referrer', {
            getter: function getter() {
                var referrer = _nativeMethods2.default.documentReferrerGetter.call(this);
                var parsedProxyUrl = urlUtils.parseProxyUrl(referrer);
                return parsedProxyUrl ? parsedProxyUrl.destUrl : '';
            }
        });
        DocumentSandbox._definePropertyDescriptor(docPrototype, htmlDocPrototype, 'referrer', referrerOverriddenDescriptor);
        var urlOverriddenDescriptor = (0, _propertyOverriding.createOverriddenDescriptor)(docPrototype, 'URL', {
            getter: function getter() {
                // eslint-disable-next-line no-restricted-properties
                return _location2.default.getLocationWrapper(this).href;
            }
        });
        DocumentSandbox._definePropertyDescriptor(docPrototype, htmlDocPrototype, 'URL', urlOverriddenDescriptor);
        var domainPropertyOwner = _nativeMethods2.default.objectHasOwnProperty.call(docPrototype, 'domain') ? docPrototype : htmlDocPrototype;
        var domainOverriddenDescriptor = (0, _propertyOverriding.createOverriddenDescriptor)(domainPropertyOwner, 'domain', {
            getter: function getter() {
                // eslint-disable-next-line no-restricted-properties
                return storedDomain || _location2.default.getLocationWrapper(window).hostname;
            },
            setter: function setter(value) {
                storedDomain = value;
            }
        });
        DocumentSandbox._definePropertyDescriptor(domainPropertyOwner, htmlDocPrototype, 'domain', domainOverriddenDescriptor);
        (0, _propertyOverriding.overrideDescriptor)(docPrototype, 'styleSheets', {
            getter: function getter() {
                var styleSheets = _nativeMethods2.default.documentStyleSheetsGetter.call(this);
                return documentSandbox.shadowUI._filterStyleSheetList(styleSheets, styleSheets.length);
            }
        });
        var documentCookiePropOwnerPrototype = window[_nativeMethods2.default.documentCookiePropOwnerName].prototype;
        (0, _propertyOverriding.overrideDescriptor)(documentCookiePropOwnerPrototype, 'cookie', {
            getter: function getter() {
                return documentSandbox.cookieSandbox.getCookie();
            },
            setter: function setter(value) {
                documentSandbox.cookieSandbox.setCookie(this, String(value));
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(docPrototype, 'activeElement', {
            getter: function getter() {
                var activeElement = _nativeMethods2.default.documentActiveElementGetter.call(this);
                if (activeElement && (0, _dom.isShadowUIElement)(activeElement)) return documentSandbox.shadowUI.getLastActiveElement() || this.body;
                return activeElement;
            }
        });
        var documentScriptsPropOwnerPrototype = window[_nativeMethods2.default.documentScriptsPropOwnerName].prototype;
        (0, _propertyOverriding.overrideDescriptor)(documentScriptsPropOwnerPrototype, 'scripts', {
            getter: function getter() {
                var scripts = _nativeMethods2.default.documentScriptsGetter.call(this);
                var length = _nativeMethods2.default.htmlCollectionLengthGetter.call(scripts);
                return documentSandbox.shadowUI._filterNodeList(scripts, length);
            }
        });
    };

    return DocumentSandbox;
}(_base2.default);

exports.default = DocumentSandbox;
module.exports = exports['default'];