'use strict';

exports.__esModule = true;

var _nativeMethods = require('../../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _html = require('../../../utils/html');

var htmlUtils = _interopRequireWildcard(_html);

var _dom = require('../../../utils/dom');

var _browser = require('../../../utils/browser');

var _script = require('../../../../processing/script');

var _style = require('../../../../processing/style');

var _style2 = _interopRequireDefault(_style);

var _url = require('../../../utils/url');

var _internalProperties = require('../../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _createSelfRemovingScript = require('../../../../utils/create-self-removing-script');

var _createSelfRemovingScript2 = _interopRequireDefault(_createSelfRemovingScript);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var BEGIN_MARKER_TAG_NAME = 'hammerhead_write_marker_begin';
var END_MARKER_TAG_NAME = 'hammerhead_write_marker_end';
var BEGIN_MARKER_MARKUP = '<' + BEGIN_MARKER_TAG_NAME + '></' + BEGIN_MARKER_TAG_NAME + '>';
var END_MARKER_MARKUP = '<' + END_MARKER_TAG_NAME + '></' + END_MARKER_TAG_NAME + '>';
var BEGIN_REMOVE_RE = new RegExp('^[\\S\\s]*' + BEGIN_MARKER_MARKUP, 'g');
var END_REMOVE_RE = new RegExp(END_MARKER_MARKUP + '[\\S\\s]*$', 'g');
var REMOVE_OPENING_TAG_RE = /^<[^>]+>/g;
var REMOVE_CLOSING_TAG_RE = /<\/[^<>]+>$/g;
var PENDING_RE = /<\/?(?:[A-Za-z][^>]*)?$/g;
var UNCLOSED_ELEMENT_FLAG = 'hammerhead|unclosed-element-flag';
var ON_WINDOW_RECREATION_SCRIPT_TEMPLATE = (0, _createSelfRemovingScript2.default)('\n    var hammerhead = window["' + _internalProperties2.default.hammerhead + '"];\n    var sandbox    = hammerhead && hammerhead.sandbox;\n\n    if (!sandbox) {\n        try {\n            sandbox = window.parent["' + _internalProperties2.default.hammerhead + '"].get(\'./sandbox/backup\').get(window);\n        } catch(e) {}\n    }\n\n    if (sandbox) {\n        Object.defineProperty(window, "' + _internalProperties2.default.documentWasCleaned + '", { value: true, configurable: true });\n        \n        sandbox.node.mutation.onDocumentCleaned({\n            window: window,\n            document: document\n        });\n\n        /* NOTE: B234357 */\n        sandbox.node.processNodes(null, document);\n    }\n');

var DocumentWriter = function () {
    function DocumentWriter(window, document) {
        _classCallCheck(this, DocumentWriter);

        this.window = window;
        this.document = document;
        this.pending = '';
        this.parentTagChain = [];
        this.isBeginMarkerInDOM = false;
        this.isEndMarkerInDOM = false;
        this.isClosingContentEl = false;
        this.isNonClosedComment = false;
        this.isAddContentToEl = false;
        this.contentForProcessing = '';
        this.nonClosedEl = null;
        this.cachedStartsWithClosingTagRegExps = {};
    }

    DocumentWriter.prototype._cutPending = function _cutPending(htmlChunk) {
        var match = htmlChunk.match(PENDING_RE);
        this.pending = match ? match[0] : '';
        return this.pending ? htmlChunk.substring(0, htmlChunk.length - this.pending.length) : htmlChunk;
    };

    DocumentWriter.prototype._wrapHtmlChunk = function _wrapHtmlChunk(htmlChunk) {
        var parentTagChainMarkup = this.parentTagChain.length ? '<' + this.parentTagChain.join('><') + '>' : '';
        if (this.isNonClosedComment) parentTagChainMarkup += '<!--';
        return parentTagChainMarkup + BEGIN_MARKER_MARKUP + htmlChunk + END_MARKER_MARKUP;
    };

    DocumentWriter.prototype._unwrapHtmlChunk = function _unwrapHtmlChunk(htmlChunk) {
        if (!htmlChunk) return htmlChunk;
        htmlChunk = htmlChunk.replace(BEGIN_REMOVE_RE, '').replace(END_REMOVE_RE, '');
        if (!this.isBeginMarkerInDOM) htmlChunk = this.isNonClosedComment ? htmlChunk.slice(4) : htmlChunk.replace(REMOVE_OPENING_TAG_RE, '');
        if (!this.isEndMarkerInDOM) htmlChunk = this.isNonClosedComment ? htmlChunk.slice(0, -3) : htmlChunk.replace(REMOVE_CLOSING_TAG_RE, '');
        if (!this.isBeginMarkerInDOM && this.isEndMarkerInDOM) this.isNonClosedComment = false;
        return htmlChunk;
    };

    DocumentWriter._setUnclosedElementFlag = function _setUnclosedElementFlag(el) {
        if ((0, _dom.isScriptElement)(el) || (0, _dom.isStyleElement)(el)) el[UNCLOSED_ELEMENT_FLAG] = true;
    };

    DocumentWriter.hasUnclosedElementFlag = function hasUnclosedElementFlag(el) {
        return !!el[UNCLOSED_ELEMENT_FLAG];
    };

    DocumentWriter._searchBeginMarker = function _searchBeginMarker(container) {
        var beginMarker = _nativeMethods2.default.elementQuerySelector.call(container, BEGIN_MARKER_TAG_NAME);
        if (beginMarker) return beginMarker;
        beginMarker = container;
        while (_nativeMethods2.default.elementFirstElementChildGetter.call(beginMarker)) {
            beginMarker = _nativeMethods2.default.elementFirstElementChildGetter.call(beginMarker);
        }if (_nativeMethods2.default.nodeFirstChildGetter.call(beginMarker.parentNode) !== beginMarker) beginMarker = _nativeMethods2.default.nodeFirstChildGetter.call(beginMarker.parentNode);else if ((0, _dom.isCommentNode)(_nativeMethods2.default.nodeFirstChildGetter.call(beginMarker))) beginMarker = _nativeMethods2.default.nodeFirstChildGetter.call(beginMarker);
        return beginMarker;
    };

    DocumentWriter._searchEndMarker = function _searchEndMarker(container) {
        var endMarker = _nativeMethods2.default.elementQuerySelector.call(container, END_MARKER_TAG_NAME);
        if (endMarker) return endMarker;
        endMarker = container;
        while (_nativeMethods2.default.elementLastElementChildGetter.call(endMarker)) {
            endMarker = _nativeMethods2.default.elementLastElementChildGetter.call(endMarker);
        }if (_nativeMethods2.default.nodeLastChildGetter.call(endMarker.parentNode) !== endMarker) endMarker = _nativeMethods2.default.nodeLastChildGetter.call(endMarker.parentNode);else if ((0, _dom.isCommentNode)(_nativeMethods2.default.nodeLastChildGetter.call(endMarker))) endMarker = _nativeMethods2.default.nodeLastChildGetter.call(endMarker);
        return endMarker;
    };

    DocumentWriter.prototype._updateParentTagChain = function _updateParentTagChain(container, endMarker) {
        var endMarkerParent = (0, _dom.getTagName)(endMarker) !== END_MARKER_TAG_NAME ? endMarker : endMarker.parentNode;
        if ((0, _dom.isCommentNode)(endMarker)) {
            this.isNonClosedComment = true;
            endMarkerParent = endMarker.parentNode;
        }
        this.parentTagChain = [];
        while (endMarkerParent !== container) {
            this.parentTagChain.unshift((0, _dom.getTagName)(endMarkerParent));
            endMarkerParent = endMarkerParent.parentNode;
        }
    };

    DocumentWriter.prototype._processBeginMarkerInContent = function _processBeginMarkerInContent(beginMarker) {
        var elWithContent = beginMarker;
        DocumentWriter._setUnclosedElementFlag(elWithContent);
        if (this.isClosingContentEl && ((0, _dom.isScriptElement)(elWithContent) || (0, _dom.isStyleElement)(elWithContent))) {
            this.contentForProcessing = _nativeMethods2.default.nodeTextContentGetter.call(this.nonClosedEl) + _nativeMethods2.default.nodeTextContentGetter.call(elWithContent).replace(BEGIN_REMOVE_RE, '');
            _nativeMethods2.default.nodeTextContentSetter.call(elWithContent, '');
        } else {
            var textContent = _nativeMethods2.default.nodeTextContentGetter.call(elWithContent);
            _nativeMethods2.default.nodeTextContentSetter.call(elWithContent, textContent.replace(BEGIN_REMOVE_RE, ''));
        }
        beginMarker = _nativeMethods2.default.createElement.call(document, BEGIN_MARKER_TAG_NAME);
        _nativeMethods2.default.insertBefore.call(elWithContent.parentNode, beginMarker, elWithContent);
    };

    DocumentWriter._createStartsWithClosingTagRegExp = function _createStartsWithClosingTagRegExp(tagName) {
        var regExpStrParts = [tagName.charAt(tagName.length - 1), '?'];
        for (var i = tagName.length - 2; i > -1; i--) {
            regExpStrParts.unshift('(?:', tagName.charAt(i));
            regExpStrParts.push(')?');
        }
        regExpStrParts.unshift('^</');
        regExpStrParts.push('$');
        return new RegExp(regExpStrParts.join(''), 'i');
    };

    DocumentWriter.prototype._getStartsWithClosingTagRegExp = function _getStartsWithClosingTagRegExp(tagName) {
        tagName = tagName.toLowerCase();
        if (!this.cachedStartsWithClosingTagRegExps[tagName]) this.cachedStartsWithClosingTagRegExps[tagName] = DocumentWriter._createStartsWithClosingTagRegExp(tagName);
        return this.cachedStartsWithClosingTagRegExps[tagName];
    };

    DocumentWriter.prototype._processEndMarkerInContent = function _processEndMarkerInContent(endMarker) {
        var elWithContent = endMarker;
        var textContent = _nativeMethods2.default.nodeTextContentGetter.call(elWithContent);
        DocumentWriter._setUnclosedElementFlag(elWithContent);
        _nativeMethods2.default.nodeTextContentSetter.call(elWithContent, textContent.replace(END_REMOVE_RE, ''));
        endMarker = _nativeMethods2.default.createElement.call(document, END_MARKER_TAG_NAME);
        if (this.pending) {
            var startsWithClosingTagRegExp = this._getStartsWithClosingTagRegExp(elWithContent.tagName);
            var isPendingStartsWithClosingTagPart = startsWithClosingTagRegExp.test(this.pending);
            if (!isPendingStartsWithClosingTagPart) {
                var newContent = _nativeMethods2.default.nodeTextContentGetter.call(elWithContent) + this.pending;
                _nativeMethods2.default.nodeTextContentSetter.call(elWithContent, newContent);
                this.pending = '';
            }
        }
        _nativeMethods2.default.appendChild.call(elWithContent.parentNode, endMarker);
    };

    DocumentWriter._addOnDocumentRecreationScript = function _addOnDocumentRecreationScript(endMarker) {
        var span = _nativeMethods2.default.createElement.call(endMarker.ownerDocument, 'span');
        _nativeMethods2.default.insertBefore.call(endMarker.parentNode, span, endMarker);
        _nativeMethods2.default.elementOuterHTMLSetter.call(span, ON_WINDOW_RECREATION_SCRIPT_TEMPLATE);
    };

    DocumentWriter.prototype._prepareDom = function _prepareDom(container, isDocumentCleaned) {
        var beginMarker = DocumentWriter._searchBeginMarker(container);
        var endMarker = DocumentWriter._searchEndMarker(container);
        this.isBeginMarkerInDOM = (0, _dom.getTagName)(beginMarker) === BEGIN_MARKER_TAG_NAME;
        this.isEndMarkerInDOM = (0, _dom.getTagName)(endMarker) === END_MARKER_TAG_NAME;
        this.isAddContentToEl = beginMarker === endMarker;
        this.isClosingContentEl = !this.isBeginMarkerInDOM && !this.isAddContentToEl;
        if (!this.isAddContentToEl) {
            this._updateParentTagChain(container, endMarker);
            if (isDocumentCleaned) DocumentWriter._addOnDocumentRecreationScript(endMarker);
        }
        if (!this.isBeginMarkerInDOM && !this.isEndMarkerInDOM) {
            this._processBeginMarkerInContent(beginMarker);
            this._processEndMarkerInContent(endMarker);
        } else if (this.isBeginMarkerInDOM && !this.isEndMarkerInDOM) this._processEndMarkerInContent(endMarker);else if (!this.isBeginMarkerInDOM && this.isEndMarkerInDOM) this._processBeginMarkerInContent(beginMarker);
    };

    DocumentWriter.prototype._processHtmlChunk = function _processHtmlChunk(htmlChunk, isDocumentCleaned) {
        var _this = this;

        htmlChunk = this._cutPending(this.pending + htmlChunk);
        htmlChunk = this._wrapHtmlChunk(htmlChunk);
        htmlChunk = htmlUtils.processHtml(htmlChunk, {
            prepareDom: function prepareDom(container) {
                return _this._prepareDom(container, isDocumentCleaned);
            },
            processedContext: this.window
        });
        htmlChunk = this._unwrapHtmlChunk(htmlChunk);
        // NOTE: Firefox and IE recreate a window instance during the document.write function execution (T213930).
        if (htmlChunk && this.isBeginMarkerInDOM && (_browser.isFirefox || _browser.isIE) && !htmlUtils.isPageHtml(htmlChunk)) htmlChunk = htmlUtils.INIT_SCRIPT_FOR_IFRAME_TEMPLATE + htmlChunk;
        return htmlChunk;
    };

    DocumentWriter.prototype.write = function write(args, ln, isDocumentCleaned) {
        var htmlChunk = this._processHtmlChunk(_nativeMethods2.default.arrayJoin.call(args, ''), isDocumentCleaned);
        if (this.nonClosedEl && this.contentForProcessing) {
            var processedContent = this.contentForProcessing;
            if ((0, _dom.isScriptElement)(this.nonClosedEl)) processedContent = (0, _script.processScript)(this.contentForProcessing, true);else if ((0, _dom.isStyleElement)(this.nonClosedEl)) processedContent = _style2.default.process(this.contentForProcessing, _url.getProxyUrl, true);
            _nativeMethods2.default.nodeTextContentSetter.call(this.nonClosedEl, processedContent);
            this.contentForProcessing = '';
        }
        var nativeWriteMethod = ln ? _nativeMethods2.default.documentWriteLn : _nativeMethods2.default.documentWrite;
        var result = nativeWriteMethod.call(this.document, htmlChunk);
        if (isDocumentCleaned && _browser.isIE) return result;
        if (!this.isEndMarkerInDOM && !this.isAddContentToEl) {
            var el = this.document.documentElement;
            while (_nativeMethods2.default.elementLastElementChildGetter.call(el)) {
                el = _nativeMethods2.default.elementLastElementChildGetter.call(el);
            }this.nonClosedEl = el;
        }
        return result;
    };

    return DocumentWriter;
}();

exports.default = DocumentWriter;
module.exports = exports['default'];