'use strict';

exports.__esModule = true;

var _internalProperties = require('../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _index = require('../node/index');

var _index2 = _interopRequireDefault(_index);

var _dom = require('../../../processing/dom');

var _dom2 = _interopRequireDefault(_dom);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _domProcessor = require('../../dom-processor');

var _domProcessor2 = _interopRequireDefault(_domProcessor);

var _script = require('../../../processing/script');

var _style = require('../../../processing/style');

var _style2 = _interopRequireDefault(_style);

var _url = require('../../utils/url');

var urlUtils = _interopRequireWildcard(_url);

var _dom3 = require('../../utils/dom');

var domUtils = _interopRequireWildcard(_dom3);

var _hiddenInfo = require('../upload/hidden-info');

var hiddenInfo = _interopRequireWildcard(_hiddenInfo);

var _urlResolver = require('../../utils/url-resolver');

var urlResolver = _interopRequireWildcard(_urlResolver);

var _destinationLocation = require('../../utils/destination-location');

var _event = require('../../utils/event');

var _html = require('../../utils/html');

var _querySelector = require('../../utils/query-selector');

var _url2 = require('../../../utils/url');

var _stringTrim = require('../../../utils/string-trim');

var _stringTrim2 = _interopRequireDefault(_stringTrim);

var _windowsStorage = require('../windows-storage');

var windowsStorage = _interopRequireWildcard(_windowsStorage);

var _attributes = require('./attributes');

var _shadowUi = require('../shadow-ui');

var _shadowUi2 = _interopRequireDefault(_shadowUi);

var _domMutationTracker = require('./live-node-list/dom-mutation-tracker');

var _domMutationTracker2 = _interopRequireDefault(_domMutationTracker);

var _attributes2 = require('../../../processing/dom/attributes');

var _settings = require('../../settings');

var _settings2 = _interopRequireDefault(_settings);

var _propertyOverriding = require('../../utils/property-overriding');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var KEYWORD_TARGETS = ['_blank', '_self', '_parent', '_top'];
var RESTRICTED_META_HTTP_EQUIV_VALUES = ['refresh', 'content-security-policy'];

var ElementSandbox = function (_SandboxBase) {
    _inherits(ElementSandbox, _SandboxBase);

    function ElementSandbox(nodeSandbox, uploadSandbox, iframeSandbox, shadowUI, eventSandbox) {
        _classCallCheck(this, ElementSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.BEFORE_FORM_SUBMIT_EVENT = 'hammerhead|event|before-form-submit';
        _this.SCRIPT_ELEMENT_ADDED_EVENT = 'hammerhead|event|script-added';
        _this.nodeSandbox = nodeSandbox;
        _this.shadowUI = shadowUI;
        _this.uploadSandbox = uploadSandbox;
        _this.iframeSandbox = iframeSandbox;
        _this.eventSandbox = eventSandbox;
        _this.overriddenMethods = null;
        return _this;
    }

    ElementSandbox._isKeywordTarget = function _isKeywordTarget(value) {
        value = value.toLowerCase();
        return KEYWORD_TARGETS.indexOf(value) !== -1;
    };

    ElementSandbox._onTargetChanged = function _onTargetChanged(el) {
        var tagName = domUtils.getTagName(el);
        var targetAttr = _domProcessor2.default.getTargetAttr(el);
        if (!_dom2.default.isIframeFlagTag(tagName)) return;
        var urlAttr = void 0;
        if (targetAttr === 'target') urlAttr = tagName === 'form' ? 'action' : 'href';else if (targetAttr === 'formtarget') urlAttr = 'formaction';
        var storedUrlAttr = _dom2.default.getStoredAttrName(urlAttr);
        if (el.hasAttribute(storedUrlAttr)) {
            var url = el.getAttribute(storedUrlAttr);
            if (urlUtils.isSupportedProtocol(url)) el.setAttribute(urlAttr, url);
        }
    };

    ElementSandbox._setProxiedSrc = function _setProxiedSrc(img) {
        if (!img[_internalProperties2.default.forceProxySrcForImage]) {
            var imgSrc = _nativeMethods2.default.imageSrcGetter.call(img);
            var skipNextLoadEvent = !!imgSrc && img.complete;
            img[_internalProperties2.default.forceProxySrcForImage] = true;
            if (imgSrc) img.setAttribute('src', imgSrc);
            img[_internalProperties2.default.skipNextLoadEventForImage] = skipNextLoadEvent;
        }
    };

    ElementSandbox.prototype.getAttributeCore = function getAttributeCore(el, args, isNs) {
        var attr = String(args[isNs ? 1 : 0]);
        var loweredAttr = attr.toLowerCase();
        var ns = isNs ? args[0] : null;
        var getAttrMeth = isNs ? _nativeMethods2.default.getAttributeNS : _nativeMethods2.default.getAttribute;
        var tagName = domUtils.getTagName(el);
        // OPTIMIZATION: The hasAttribute method is very slow.
        if (_domProcessor2.default.isUrlAttr(el, loweredAttr, ns) || _domProcessor2.default.EVENTS.indexOf(loweredAttr) !== -1 || _attributes2.ATTRS_WITH_SPECIAL_PROXYING_LOGIC.indexOf(loweredAttr) !== -1) {
            var storedAttrName = _dom2.default.getStoredAttrName(attr);
            var storedAttrValue = getAttrMeth.apply(el, isNs ? [ns, storedAttrName] : [storedAttrName]);
            if (_dom2.default.isAddedAutocompleteAttr(loweredAttr, storedAttrValue)) return null;else if (el.hasAttribute(storedAttrName)) args[isNs ? 1 : 0] = storedAttrName;
        }
        // NOTE: We simply remove the 'integrity' attribute because its value will not be relevant after the script
        // content changes (http://www.w3.org/TR/SRI/). If this causes problems in the future, we will need to generate
        // the correct SHA for the changed script. (GH-235)
        else if (!isNs && loweredAttr === 'integrity' && _dom2.default.isTagWithIntegrityAttr(tagName)) {
                var storedIntegrityAttr = _dom2.default.getStoredAttrName(attr);
                if (_nativeMethods2.default.hasAttribute.call(el, storedIntegrityAttr)) args[0] = storedIntegrityAttr;
            }
            // NOTE: We simply remove the 'rel' attribute if rel='prefetch' and use stored 'rel' attribute, because the prefetch
            // resource type is unknown. https://github.com/DevExpress/testcafe/issues/2528
            else if (!isNs && loweredAttr === 'rel' && tagName === 'link') {
                    var storedRelAttr = _dom2.default.getStoredAttrName(attr);
                    if (_nativeMethods2.default.hasAttribute.call(el, storedRelAttr)) args[0] = storedRelAttr;
                } else if (!isNs && loweredAttr === 'required' && domUtils.isFileInput(el)) {
                    var storedRequiredAttr = _dom2.default.getStoredAttrName(attr);
                    if (_nativeMethods2.default.hasAttribute.call(el, storedRequiredAttr)) args[0] = storedRequiredAttr;
                }
        return getAttrMeth.apply(el, args);
    };

    ElementSandbox.prototype.setAttributeCore = function setAttributeCore(el, args, isNs) {
        var ns = isNs ? args[0] : null;
        var attr = String(args[isNs ? 1 : 0]);
        var loweredAttr = attr.toLowerCase();
        var valueIndex = isNs ? 2 : 1;
        var value = String(args[valueIndex]);
        var setAttrMeth = isNs ? _nativeMethods2.default.setAttributeNS : _nativeMethods2.default.setAttribute;
        var tagName = domUtils.getTagName(el);
        var isUrlAttr = _domProcessor2.default.isUrlAttr(el, attr, ns);
        var isEventAttr = _domProcessor2.default.EVENTS.indexOf(attr) !== -1;
        var needToCallTargetChanged = false;
        var isSpecialPage = urlUtils.isSpecialPage(value);
        var isSupportedProtocol = urlUtils.isSupportedProtocol(value);
        if (isUrlAttr && !isSupportedProtocol && !isSpecialPage || isEventAttr) {
            var isJsProtocol = _dom2.default.isJsProtocol(value);
            var storedJsAttr = _dom2.default.getStoredAttrName(attr);
            if (isUrlAttr && isJsProtocol || isEventAttr) args[valueIndex] = _dom2.default.processJsAttrValue(value, { isJsProtocol: isJsProtocol, isEventAttr: isEventAttr });
            setAttrMeth.apply(el, isNs ? [ns, storedJsAttr, value] : [storedJsAttr, value]);
        } else if (isUrlAttr && (isSupportedProtocol || isSpecialPage)) {
            var storedUrlAttr = _dom2.default.getStoredAttrName(attr);
            setAttrMeth.apply(el, isNs ? [ns, storedUrlAttr, value] : [storedUrlAttr, value]);
            if (tagName !== 'img' || el[_internalProperties2.default.forceProxySrcForImage]) {
                if (tagName === 'img') el[_internalProperties2.default.skipNextLoadEventForImage] = false;
                if (value !== '' && (!isSpecialPage || tagName === 'a')) {
                    var isIframe = tagName === 'iframe' || tagName === 'frame';
                    var isScript = tagName === 'script';
                    var isCrossDomainUrl = isSupportedProtocol && !(0, _destinationLocation.sameOriginCheck)(location.toString(), value);
                    var resourceType = _domProcessor2.default.getElementResourceType(el);
                    var elCharset = isScript && el.charset;
                    var currentDocument = el.ownerDocument || this.document;
                    if (loweredAttr === 'formaction' && !_nativeMethods2.default.hasAttribute.call(el, 'formtarget')) {
                        resourceType = 'f';
                        if (el.form && _nativeMethods2.default.hasAttribute.call(el.form, 'action')) {
                            var parsedFormAction = urlUtils.parseProxyUrl(_nativeMethods2.default.formActionGetter.call(el.form));
                            if (parsedFormAction) resourceType = parsedFormAction.resourceType;
                        }
                    }
                    if (ElementSandbox._isHrefAttrForBaseElement(el, attr) && domUtils.isElementInDocument(el, currentDocument))
                        // @ts-ignore
                        urlResolver.updateBase(value, currentDocument);
                    args[valueIndex] = isIframe && isCrossDomainUrl ? urlUtils.getCrossDomainIframeProxyUrl(value) : urlUtils.getProxyUrl(value, { resourceType: resourceType, charset: elCharset, doc: currentDocument });
                }
            } else if (value && !isSpecialPage && !urlUtils.parseProxyUrl(value)) {
                args[valueIndex] = el[_internalProperties2.default.forceProxySrcForImage] ? urlUtils.getProxyUrl(value) : urlUtils.resolveUrlAsDest(value);
            }
        } else if (loweredAttr === 'autocomplete') {
            var storedAutocompleteAttr = _dom2.default.getStoredAttrName(attr);
            setAttrMeth.apply(el, isNs ? [ns, storedAutocompleteAttr, value] : [storedAutocompleteAttr, value]);
            args[valueIndex] = 'off';
        } else if (loweredAttr === 'target' && _dom2.default.isTagWithTargetAttr(tagName) || loweredAttr === 'formtarget' && _dom2.default.isTagWithFormTargetAttr(tagName)) {
            var currentTarget = _nativeMethods2.default.getAttribute.call(el, loweredAttr);
            var newTarget = this.getTarget(el, value);
            if (newTarget !== currentTarget) {
                var storedTargetAttr = _dom2.default.getStoredAttrName(attr);
                setAttrMeth.apply(el, isNs ? [ns, storedTargetAttr, value] : [storedTargetAttr, value]);
                args[valueIndex] = newTarget;
                needToCallTargetChanged = true;
            } else return null;
        } else if (attr === 'sandbox') {
            var storedSandboxAttr = _dom2.default.getStoredAttrName(attr);
            var allowSameOrigin = value.indexOf('allow-same-origin') !== -1;
            var allowScripts = value.indexOf('allow-scripts') !== -1;
            setAttrMeth.apply(el, isNs ? [ns, storedSandboxAttr, value] : [storedSandboxAttr, value]);
            if (!allowSameOrigin || !allowScripts) {
                args[valueIndex] += !allowSameOrigin ? ' allow-same-origin' : '';
                args[valueIndex] += !allowScripts ? ' allow-scripts' : '';
            }
            if (el[this.nodeSandbox.win.SANDBOX_DOM_TOKEN_LIST_UPDATE_FN]) el[this.nodeSandbox.win.SANDBOX_DOM_TOKEN_LIST_UPDATE_FN](value);
        }
        // TODO: remove after https://github.com/DevExpress/testcafe-hammerhead/issues/244 implementation
        else if (tagName === 'meta' && attr === 'http-equiv') {
                var loweredValue = value.toLowerCase();
                if (RESTRICTED_META_HTTP_EQUIV_VALUES.indexOf(loweredValue) !== -1) return null;
            } else if (loweredAttr === 'xlink:href' && _domProcessor2.default.SVG_XLINK_HREF_TAGS.indexOf(tagName) !== -1 && domUtils.isSVGElement(el)) {
                var storedXLinkHrefAttr = _dom2.default.getStoredAttrName(attr);
                setAttrMeth.apply(el, isNs ? [ns, storedXLinkHrefAttr, value] : [storedXLinkHrefAttr, value]);
                if (!_url2.HASH_RE.test(value)) args[valueIndex] = urlUtils.getProxyUrl(value);
            } else if (loweredAttr === 'style') {
                var storedStyleAttr = _dom2.default.getStoredAttrName(attr);
                setAttrMeth.apply(el, isNs ? [ns, storedStyleAttr, value] : [storedStyleAttr, value]);
                args[valueIndex] = _style2.default.process(value, urlUtils.getProxyUrl);
            } else if (!isNs && loweredAttr === 'integrity' && _dom2.default.isTagWithIntegrityAttr(tagName)) {
                var storedIntegrityAttr = _dom2.default.getStoredAttrName(attr);
                return setAttrMeth.apply(el, [storedIntegrityAttr, value]);
            } else if (!isNs && loweredAttr === 'rel' && tagName === 'link') {
                var formatedValue = (0, _stringTrim2.default)(value.toLowerCase());
                var storedRelAttr = _dom2.default.getStoredAttrName(attr);
                if (formatedValue === 'prefetch') {
                    _nativeMethods2.default.removeAttribute.call(el, attr);
                    args[0] = storedRelAttr;
                } else _nativeMethods2.default.removeAttribute.call(el, storedRelAttr);
            } else if (!isNs && loweredAttr === 'required' && domUtils.isFileInput(el)) {
                var storedRequiredAttr = _dom2.default.getStoredAttrName(attr);
                _nativeMethods2.default.removeAttribute.call(el, attr);
                args[0] = storedRequiredAttr;
            } else if (!isNs && loweredAttr === 'type' && domUtils.isInputElement(el)) {
                var currentType = _nativeMethods2.default.getAttribute.call(el, loweredAttr);
                var newType = value.toLowerCase();
                var _storedRequiredAttr = _dom2.default.getStoredAttrName('required');
                var currentRequired = _nativeMethods2.default.hasAttribute.call(el, _storedRequiredAttr) ? _nativeMethods2.default.getAttribute.call(el, _storedRequiredAttr) : _nativeMethods2.default.getAttribute.call(el, 'required');
                var typeIsChanged = !currentType || newType !== currentType.toLowerCase();
                if (typeIsChanged && currentRequired !== null) {
                    if (newType === 'file') {
                        _nativeMethods2.default.setAttribute.call(el, _storedRequiredAttr, currentRequired);
                        _nativeMethods2.default.removeAttribute.call(el, 'required');
                    } else if (currentType === 'file') {
                        _nativeMethods2.default.setAttribute.call(el, 'required', currentRequired);
                        _nativeMethods2.default.removeAttribute.call(el, _storedRequiredAttr);
                    }
                }
            }
        var result = setAttrMeth.apply(el, args);
        if (needToCallTargetChanged) ElementSandbox._onTargetChanged(el);
        return result;
    };

    ElementSandbox.prototype._hasAttributeCore = function _hasAttributeCore(el, args, isNs) {
        var attributeNameArgIndex = isNs ? 1 : 0;
        var hasAttrMeth = isNs ? _nativeMethods2.default.hasAttributeNS : _nativeMethods2.default.hasAttribute;
        var storedAutocompleteAttrName = _dom2.default.getStoredAttrName('autocomplete');
        var storedAutocompleteAttrValue = _nativeMethods2.default.getAttribute.call(el, storedAutocompleteAttrName);
        var tagName = domUtils.getTagName(el);
        if (typeof args[attributeNameArgIndex] === 'string' && _dom2.default.isAddedAutocompleteAttr(args[attributeNameArgIndex], storedAutocompleteAttrValue)) return false;
        // NOTE: We simply remove the 'integrity' attribute because its value will not be relevant after the script
        // content changes (http://www.w3.org/TR/SRI/). If this causes problems in the future, we will need to generate
        // the correct SHA for the changed script.
        // _hasAttributeCore returns true for 'integrity' attribute if the stored attribute is exists. (GH-235)
        else if (!isNs && args[0] === 'integrity' && _dom2.default.isTagWithIntegrityAttr(tagName)) args[0] = _dom2.default.getStoredAttrName('integrity');
            // NOTE: We simply remove the 'rel' attribute if rel='prefetch' and use stored 'rel' attribute, because the prefetch
            // resource type is unknown.
            // _hasAttributeCore returns true for 'rel' attribute if the original 'rel' or stored attribute is exists.
            // https://github.com/DevExpress/testcafe/issues/2528
            else if (!isNs && args[0] === 'rel' && tagName === 'link') {
                    var storedRelAttr = _dom2.default.getStoredAttrName(args[0]);
                    return hasAttrMeth.apply(el, args) || hasAttrMeth.apply(el, [storedRelAttr]);
                } else if (!isNs && args[0] === 'required' && domUtils.isFileInput(el)) {
                    var storedRequiredAttr = _dom2.default.getStoredAttrName(args[0]);
                    return hasAttrMeth.apply(el, args) || hasAttrMeth.call(el, storedRequiredAttr);
                }
        return hasAttrMeth.apply(el, args);
    };

    ElementSandbox.prototype._removeAttributeCore = function _removeAttributeCore(el, args, isNs) {
        var attr = String(args[isNs ? 1 : 0]);
        var formatedAttr = attr.toLowerCase();
        var removeAttrFunc = isNs ? _nativeMethods2.default.removeAttributeNS : _nativeMethods2.default.removeAttribute;
        var tagName = domUtils.getTagName(el);
        var result = void 0;
        if (_domProcessor2.default.isUrlAttr(el, formatedAttr, isNs ? args[0] : null) || formatedAttr === 'sandbox' || formatedAttr === 'autocomplete' || _domProcessor2.default.EVENTS.indexOf(formatedAttr) !== -1 || formatedAttr === 'target' && _dom2.default.isTagWithTargetAttr(tagName) || formatedAttr === 'formtarget' && _dom2.default.isTagWithFormTargetAttr(tagName)) {
            var storedAttr = _dom2.default.getStoredAttrName(attr);
            if (formatedAttr === 'autocomplete') _nativeMethods2.default.setAttribute.call(el, storedAttr, _domProcessor2.default.AUTOCOMPLETE_ATTRIBUTE_ABSENCE_MARKER);else removeAttrFunc.apply(el, isNs ? [args[0], storedAttr] : [storedAttr]);
        } else if (!isNs && formatedAttr === 'rel' && tagName === 'link') {
            var storedRelAttr = _dom2.default.getStoredAttrName(attr);
            removeAttrFunc.apply(el, [storedRelAttr]);
        } else if (!isNs && formatedAttr === 'required' && domUtils.isFileInput(el)) {
            var storedRequiredAttr = _dom2.default.getStoredAttrName(attr);
            removeAttrFunc.call(el, storedRequiredAttr);
        } else if (!isNs && formatedAttr === 'type' && domUtils.isInputElement(el)) {
            var _storedRequiredAttr2 = _dom2.default.getStoredAttrName('required');
            if (_nativeMethods2.default.hasAttribute.call(el, _storedRequiredAttr2)) {
                var currentRequired = _nativeMethods2.default.getAttribute.call(el, _storedRequiredAttr2);
                _nativeMethods2.default.setAttribute.call(el, 'required', currentRequired);
                _nativeMethods2.default.removeAttribute.call(el, _storedRequiredAttr2);
            }
        }
        if (ElementSandbox._isHrefAttrForBaseElement(el, formatedAttr))
            // @ts-ignore
            urlResolver.updateBase((0, _destinationLocation.get)(), this.document);
        if (formatedAttr !== 'autocomplete') result = removeAttrFunc.apply(el, args);
        if (formatedAttr === 'target' && _dom2.default.isTagWithTargetAttr(tagName) || formatedAttr === 'formtarget' && _dom2.default.isTagWithFormTargetAttr(tagName)) ElementSandbox._onTargetChanged(el);
        return result;
    };

    ElementSandbox.prototype._addNodeCore = function _addNodeCore(_ref) {
        var parentNode = _ref.parentNode,
            args = _ref.args,
            nativeFn = _ref.nativeFn,
            checkBody = _ref.checkBody;

        var newNode = args[0];
        this._prepareNodeForInsertion(newNode, parentNode);
        var result = null;
        var childNodesArray = null;
        if (domUtils.isDocumentFragmentNode(newNode)) childNodesArray = domUtils.nodeListToArray(newNode.childNodes);
        // NOTE: Before the page's <body> is processed and added to DOM,
        // some javascript frameworks create their own body element, perform
        // certain manipulations and then remove it.
        // Therefore, we need to check if the body element is present in DOM
        if (checkBody && domUtils.isBodyElementWithChildren(parentNode) && domUtils.isElementInDocument(parentNode)) result = this.shadowUI.insertBeforeRoot(newNode);else result = nativeFn.apply(parentNode, args);
        if (childNodesArray) {
            for (var _i2 = 0, _childNodesArray2 = childNodesArray; _i2 < _childNodesArray2.length; _i2++) {
                var child = _childNodesArray2[_i2];
                this._onElementAdded(child);
            }
        } else this._onElementAdded(newNode);
        return result;
    };

    ElementSandbox.prototype._prepareNodeForInsertion = function _prepareNodeForInsertion(node, parentNode) {
        if (domUtils.isTextNode(node)) ElementSandbox._processTextNodeContent(node, parentNode);
        this.nodeSandbox.processNodes(node);
    };

    ElementSandbox.prototype._createOverridedMethods = function _createOverridedMethods() {
        // NOTE: We need the closure because a context of overridden methods is an html element
        var sandbox = this;
        this.overriddenMethods = {
            insertRow: function insertRow() {
                var nativeMeth = domUtils.isTableElement(this) ? _nativeMethods2.default.insertTableRow : _nativeMethods2.default.insertTBodyRow;
                var row = nativeMeth.apply(this, arguments);
                sandbox.nodeSandbox.processNodes(row);
                return row;
            },
            insertCell: function insertCell() {
                var cell = _nativeMethods2.default.insertCell.apply(this, arguments);
                sandbox.nodeSandbox.processNodes(cell);
                return cell;
            },
            insertAdjacentHTML: function insertAdjacentHTML() {
                for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                    args[_key] = arguments[_key];
                }

                var position = args[0];
                var html = args[1];
                var el = this;
                var parentEl = el.parentNode;
                if (args.length > 1 && html !== null) {
                    args[1] = (0, _html.processHtml)(String(html), {
                        parentTag: parentEl && parentEl.tagName,
                        processedContext: el[_internalProperties2.default.processedContext]
                    });
                }
                _nativeMethods2.default.insertAdjacentHTML.apply(el, args);
                sandbox.nodeSandbox.processNodes(parentEl || el);
                if (position === 'afterbegin' || position === 'beforeend') _domMutationTracker2.default.onChildrenChanged(el);else if (parentEl) _domMutationTracker2.default.onChildrenChanged(parentEl);
            },
            formSubmit: function formSubmit() {
                sandbox._ensureTargetContainsExistingBrowsingContext(this);
                sandbox.emit(sandbox.BEFORE_FORM_SUBMIT_EVENT, { form: this });
                return _nativeMethods2.default.formSubmit.apply(this, arguments);
            },
            insertBefore: function insertBefore() {
                for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                    args[_key2] = arguments[_key2];
                }

                return sandbox._addNodeCore({
                    parentNode: this,
                    nativeFn: _nativeMethods2.default.insertBefore,
                    checkBody: !args[1],
                    args: args
                });
            },
            appendChild: function appendChild() {
                for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                    args[_key3] = arguments[_key3];
                }

                return sandbox._addNodeCore({
                    parentNode: this,
                    nativeFn: _nativeMethods2.default.appendChild,
                    checkBody: true,
                    args: args
                });
            },
            removeChild: function removeChild() {
                var child = arguments[0];
                sandbox._onRemoveFileInputInfo(child);
                sandbox._onRemoveIframe(child);
                var result = _nativeMethods2.default.removeChild.apply(this, arguments);
                sandbox._onElementRemoved(child);
                return result;
            },
            replaceChild: function replaceChild() {
                var newChild = arguments[0];
                var oldChild = arguments[1];
                if (domUtils.isTextNode(newChild)) ElementSandbox._processTextNodeContent(newChild, this);
                sandbox._onRemoveFileInputInfo(oldChild);
                var result = _nativeMethods2.default.replaceChild.apply(this, arguments);
                sandbox._onAddFileInputInfo(newChild);
                _domMutationTracker2.default.onElementChanged(newChild);
                _domMutationTracker2.default.onElementChanged(oldChild);
                return result;
            },
            cloneNode: function cloneNode() {
                var clone = _nativeMethods2.default.cloneNode.apply(this, arguments);
                sandbox.nodeSandbox.processNodes(clone);
                return clone;
            },
            getAttribute: function getAttribute() {
                return sandbox.getAttributeCore(this, arguments);
            },
            getAttributeNS: function getAttributeNS() {
                return sandbox.getAttributeCore(this, arguments, true);
            },
            setAttribute: function setAttribute() {
                var result = sandbox.setAttributeCore(this, arguments);
                (0, _attributes.refreshAttributesWrapper)(this);
                return result;
            },
            setAttributeNS: function setAttributeNS() {
                var result = sandbox.setAttributeCore(this, arguments, true);
                (0, _attributes.refreshAttributesWrapper)(this);
                return result;
            },
            removeAttribute: function removeAttribute() {
                var result = sandbox._removeAttributeCore(this, arguments);
                (0, _attributes.refreshAttributesWrapper)(this);
                return result;
            },
            removeAttributeNS: function removeAttributeNS() {
                var result = sandbox._removeAttributeCore(this, arguments, true);
                (0, _attributes.refreshAttributesWrapper)(this);
                return result;
            },
            querySelector: function querySelector() {
                if (typeof arguments[0] === 'string') arguments[0] = _index2.default.processSelector(arguments[0]);
                return (0, _querySelector.getNativeQuerySelector)(this).apply(this, arguments);
            },
            querySelectorAll: function querySelectorAll() {
                if (typeof arguments[0] === 'string') arguments[0] = _index2.default.processSelector(arguments[0]);
                return (0, _querySelector.getNativeQuerySelectorAll)(this).apply(this, arguments);
            },
            hasAttribute: function hasAttribute() {
                return sandbox._hasAttributeCore(this, arguments, false);
            },
            hasAttributeNS: function hasAttributeNS() {
                return sandbox._hasAttributeCore(this, arguments, true);
            },
            hasAttributes: function hasAttributes() {
                if (_nativeMethods2.default.elementAttributesGetter.call(this).length === 2 && _nativeMethods2.default.elementAttributesGetter.call(this).getNamedItem('autocomplete') && _nativeMethods2.default.elementAttributesGetter.call(this).getNamedItem(_dom2.default.getStoredAttrName('autocomplete'))) return sandbox._hasAttributeCore(this, ['autocomplete'], false);
                return _nativeMethods2.default.hasAttributes.apply(this, arguments);
            },
            anchorToString: function anchorToString() {
                var href = _nativeMethods2.default.anchorToString.call(this);
                var parsedProxyHref = urlUtils.parseProxyUrl(href);
                return parsedProxyHref ? parsedProxyHref.destUrl : href;
            },
            registerElement: function registerElement() {
                for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
                    args[_key4] = arguments[_key4];
                }

                var opts = args[1];
                if (opts && opts.prototype && opts.prototype.createdCallback) {
                    var storedCreatedCallback = opts.prototype.createdCallback;
                    opts.prototype.createdCallback = function () {
                        if (!(0, _html.isInternalHtmlParserElement)(this)) storedCreatedCallback.call(this);
                    };
                }
                return _nativeMethods2.default.registerElement.apply(this, args);
            }
        };
    };

    ElementSandbox._processTextNodeContent = function _processTextNodeContent(node, parentNode) {
        if (!parentNode.tagName) return;
        if (domUtils.isScriptElement(parentNode)) node.data = (0, _script.processScript)(node.data, true);else if (domUtils.isStyleElement(parentNode)) node.data = _style2.default.process(node.data, urlUtils.getProxyUrl);
    };

    ElementSandbox._isHrefAttrForBaseElement = function _isHrefAttrForBaseElement(el, attr) {
        return domUtils.isBaseElement(el) && attr === 'href';
    };

    ElementSandbox._removeFileInputInfo = function _removeFileInputInfo(el) {
        hiddenInfo.removeInputInfo(el);
    };

    ElementSandbox._hasShadowUIParentOrContainsShadowUIClassPostfix = function _hasShadowUIParentOrContainsShadowUIClassPostfix(el) {
        return el.parentNode && domUtils.isShadowUIElement(el.parentNode) || _shadowUi2.default.containsShadowUIClassPostfix(el);
    };

    ElementSandbox.prototype._isFirstBaseTagOnPage = function _isFirstBaseTagOnPage(el) {
        var doc = el.ownerDocument || this.document;
        return _nativeMethods2.default.querySelector.call(doc, 'base') === el;
    };

    ElementSandbox.prototype._onAddFileInputInfo = function _onAddFileInputInfo(el) {
        if (!domUtils.isDomElement(el)) return;
        var fileInputs = domUtils.getFileInputs(el);

        for (var _i4 = 0; _i4 < fileInputs.length; _i4++) {
            var fileInput = fileInputs[_i4];
            this.addFileInputInfo(fileInput);
        }
    };

    ElementSandbox.prototype._onRemoveFileInputInfo = function _onRemoveFileInputInfo(el) {
        if (!domUtils.isDomElement(el)) return;
        if (domUtils.isFileInput(el)) ElementSandbox._removeFileInputInfo(el);else domUtils.find(el, 'input[type=file]', ElementSandbox._removeFileInputInfo);
    };

    ElementSandbox.prototype._onRemoveIframe = function _onRemoveIframe(el) {
        if (domUtils.isDomElement(el) && domUtils.isIframeElement(el)) windowsStorage.remove(_nativeMethods2.default.contentWindowGetter.call(el));
    };

    ElementSandbox.prototype._onElementAdded = function _onElementAdded(el) {
        if (ElementSandbox._hasShadowUIParentOrContainsShadowUIClassPostfix(el)) _shadowUi2.default.markElementAndChildrenAsShadow(el);
        if ((domUtils.isDomElement(el) || domUtils.isDocument(el)) && domUtils.isElementInDocument(el)) {
            var iframes = domUtils.getIframes(el);

            for (var _i6 = 0; _i6 < iframes.length; _i6++) {
                var iframe = iframes[_i6];
                this.onIframeAddedToDOM(iframe);
                windowsStorage.add(_nativeMethods2.default.contentWindowGetter.call(iframe));
            }

            var scripts = domUtils.getScripts(el);

            for (var _i8 = 0; _i8 < scripts.length; _i8++) {
                var script = scripts[_i8];
                this.emit(this.SCRIPT_ELEMENT_ADDED_EVENT, { el: script });
            }

            _domMutationTracker2.default.onElementChanged(el);
        }
        // NOTE: recalculate `formaction` attribute value if it placed in the dom
        if ((domUtils.isInputElement(el) || domUtils.isButtonElement(el)) && el.form && _nativeMethods2.default.hasAttribute.call(el, 'formaction')) el.setAttribute('formaction', el.getAttribute('formaction'));
        if (domUtils.isBodyElement(el)) this.shadowUI.onBodyElementMutation();
        this._onAddFileInputInfo(el);
        if (domUtils.isBaseElement(el) && this._isFirstBaseTagOnPage(el)) {
            var storedHrefAttrName = _dom2.default.getStoredAttrName('href');
            var storedHrefAttrValue = el.getAttribute(storedHrefAttrName);
            if (storedHrefAttrValue !== null)
                // @ts-ignore
                urlResolver.updateBase(storedHrefAttrValue, this.document);
        }
    };

    ElementSandbox.prototype._onElementRemoved = function _onElementRemoved(el) {
        if (domUtils.isBodyElement(el)) this.shadowUI.onBodyElementMutation();else if (domUtils.isBaseElement(el)) {
            var firstBaseEl = _nativeMethods2.default.querySelector.call(this.document, 'base');
            var storedHrefAttr = firstBaseEl && firstBaseEl.getAttribute(_dom2.default.getStoredAttrName('href'));
            // @ts-ignore
            urlResolver.updateBase(storedHrefAttr || (0, _destinationLocation.get)(), this.document);
        }
        _domMutationTracker2.default.onElementChanged(el);
    };

    ElementSandbox.prototype.addFileInputInfo = function addFileInputInfo(el) {
        var infoManager = this.uploadSandbox.infoManager;
        hiddenInfo.addInputInfo(el, infoManager.getFiles(el), infoManager.getValue(el));
    };

    ElementSandbox.prototype.onIframeAddedToDOM = function onIframeAddedToDOM(iframe) {
        if (!domUtils.isCrossDomainIframe(iframe, true)) this.nodeSandbox.mutation.onIframeAddedToDOM({ iframe: iframe });
    };

    ElementSandbox.prototype.attach = function attach(window) {
        _SandboxBase.prototype.attach.call(this, window);
        this._createOverridedMethods();
        window.Element.prototype.insertBefore = this.overriddenMethods.insertBefore;
        window.Element.prototype.appendChild = this.overriddenMethods.appendChild;
        window.Element.prototype.replaceChild = this.overriddenMethods.replaceChild;
        window.Element.prototype.removeChild = this.overriddenMethods.removeChild;
        window.Element.prototype.setAttribute = this.overriddenMethods.setAttribute;
        window.Element.prototype.setAttributeNS = this.overriddenMethods.setAttributeNS;
        window.Element.prototype.getAttribute = this.overriddenMethods.getAttribute;
        window.Element.prototype.getAttributeNS = this.overriddenMethods.getAttributeNS;
        window.Element.prototype.removeAttribute = this.overriddenMethods.removeAttribute;
        window.Element.prototype.removeAttributeNS = this.overriddenMethods.removeAttributeNS;
        window.Element.prototype.cloneNode = this.overriddenMethods.cloneNode;
        window.Element.prototype.querySelector = this.overriddenMethods.querySelector;
        window.Element.prototype.querySelectorAll = this.overriddenMethods.querySelectorAll;
        window.Element.prototype.hasAttribute = this.overriddenMethods.hasAttribute;
        window.Element.prototype.hasAttributeNS = this.overriddenMethods.hasAttributeNS;
        window.Element.prototype.hasAttributes = this.overriddenMethods.hasAttributes;
        window.Node.prototype.cloneNode = this.overriddenMethods.cloneNode;
        window.Node.prototype.appendChild = this.overriddenMethods.appendChild;
        window.Node.prototype.removeChild = this.overriddenMethods.removeChild;
        window.Node.prototype.insertBefore = this.overriddenMethods.insertBefore;
        window.Node.prototype.replaceChild = this.overriddenMethods.replaceChild;
        window.DocumentFragment.prototype.querySelector = this.overriddenMethods.querySelector;
        window.DocumentFragment.prototype.querySelectorAll = this.overriddenMethods.querySelectorAll;
        window.HTMLTableElement.prototype.insertRow = this.overriddenMethods.insertRow;
        window.HTMLTableSectionElement.prototype.insertRow = this.overriddenMethods.insertRow;
        window.HTMLTableRowElement.prototype.insertCell = this.overriddenMethods.insertCell;
        window.HTMLFormElement.prototype.submit = this.overriddenMethods.formSubmit;
        window.HTMLAnchorElement.prototype.toString = this.overriddenMethods.anchorToString;
        if (window.Document.prototype.registerElement) window.Document.prototype.registerElement = this.overriddenMethods.registerElement;
        if (window.Element.prototype.insertAdjacentHTML) window.Element.prototype.insertAdjacentHTML = this.overriddenMethods.insertAdjacentHTML;else if (window.HTMLElement.prototype.insertAdjacentHTML) window.HTMLElement.prototype.insertAdjacentHTML = this.overriddenMethods.insertAdjacentHTML;
        this._setValidBrowsingContextOnElementClick(window);
        // NOTE: Cookie can be set up for the page by using the request initiated by img.
        // For example: img.src = '<url that responds with the Set-Cookie header>'
        // If img has the 'load' event handler, we redirect the request through proxy.
        // For details, see https://github.com/DevExpress/testcafe-hammerhead/issues/651
        this.eventSandbox.listeners.on(this.eventSandbox.listeners.EVENT_LISTENER_ATTACHED_EVENT, function (e) {
            if (e.eventType === 'load' && domUtils.isImgElement(e.el)) ElementSandbox._setProxiedSrc(e.el);
        });
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLElement.prototype, 'onload', {
            getter: null,
            setter: function setter(handler) {
                if (domUtils.isImgElement(this) && (0, _event.isValidEventListener)(handler)) ElementSandbox._setProxiedSrc(this);
                _nativeMethods2.default.htmlElementOnloadSetter.call(this, handler);
            }
        });
    };

    ElementSandbox.prototype._ensureTargetContainsExistingBrowsingContext = function _ensureTargetContainsExistingBrowsingContext(el) {
        if (!_nativeMethods2.default.hasAttribute.call(el, 'target')) return;
        var attr = _nativeMethods2.default.getAttribute.call(el, 'target');
        var storedAttr = _nativeMethods2.default.getAttribute.call(el, _dom2.default.getStoredAttrName('target'));
        el.setAttribute('target', storedAttr || attr);
    };

    ElementSandbox.prototype._setValidBrowsingContextOnElementClick = function _setValidBrowsingContextOnElementClick(window) {
        var _this2 = this;

        this.eventSandbox.listeners.initElementListening(window, ['click']);
        this.eventSandbox.listeners.addInternalEventListener(window, ['click'], function (e) {
            var el = e.target;
            if (domUtils.isInputElement(el) && el.form) el = el.form;
            var tagName = domUtils.getTagName(el);
            if (!_dom2.default.isTagWithTargetAttr(tagName)) return;
            _this2._ensureTargetContainsExistingBrowsingContext(el);
        });
    };

    ElementSandbox.prototype._setProxiedSrcUrlOnError = function _setProxiedSrcUrlOnError(img) {
        img.addEventListener('error', function (e) {
            var storedAttr = _nativeMethods2.default.getAttribute.call(img, _dom2.default.getStoredAttrName('src'));
            var imgSrc = _nativeMethods2.default.imageSrcGetter.call(img);
            if (storedAttr && !urlUtils.parseProxyUrl(imgSrc) && urlUtils.isSupportedProtocol(imgSrc) && !urlUtils.isSpecialPage(imgSrc)) {
                _nativeMethods2.default.setAttribute.call(img, 'src', urlUtils.getProxyUrl(storedAttr));
                (0, _event.stopPropagation)(e);
            }
        }, false);
    };

    ElementSandbox.prototype.getTarget = function getTarget(_el, newTarget) {
        var target = newTarget || '';
        if (target && !ElementSandbox._isKeywordTarget(target) && !windowsStorage.findByName(target) || /_blank/i.test(target)) return '_top';
        return target;
    };

    ElementSandbox.prototype.processElement = function processElement(el) {
        var tagName = domUtils.getTagName(el);
        switch (tagName) {
            case 'img':
                this.eventSandbox.listeners.initElementListening(el, ['load']);
                this.eventSandbox.listeners.addInternalEventListener(el, ['load'], function (_e, _dispatched, preventEvent, _cancelHandlers, stopEventPropagation) {
                    if (!el[_internalProperties2.default.skipNextLoadEventForImage]) return;
                    el[_internalProperties2.default.skipNextLoadEventForImage] = false;
                    preventEvent();
                    stopEventPropagation();
                });
                if (!el[_internalProperties2.default.forceProxySrcForImage] && !_settings2.default.get().forceProxySrcForImage) this._setProxiedSrcUrlOnError(el);
                break;
            case 'iframe':
            case 'frame':
                this.iframeSandbox.processIframe(el);
                break;
            case 'base':
                {
                    if (!this._isFirstBaseTagOnPage(el)) break;
                    var storedUrlAttr = _nativeMethods2.default.getAttribute.call(el, _dom2.default.getStoredAttrName('href'));
                    if (storedUrlAttr !== null)
                        // @ts-ignore
                        urlResolver.updateBase(storedUrlAttr, el.ownerDocument || this.document);
                    break;
                }
        }
        // NOTE: we need to reprocess a tag client-side if it wasn't processed on the server.
        // See the usage of Parse5DomAdapter.needToProcessUrl
        var targetAttr = _domProcessor2.default.getTargetAttr(el);
        if (_dom2.default.isIframeFlagTag(tagName) && _nativeMethods2.default.getAttribute.call(el, targetAttr) === '_parent') _domProcessor2.default.processElement(el, urlUtils.convertToProxyUrl);
    };

    return ElementSandbox;
}(_base2.default);

exports.default = ElementSandbox;
module.exports = exports['default'];