'use strict';

exports.__esModule = true;

var _internalProperties = require('../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _window = require('./window');

var _window2 = _interopRequireDefault(_window);

var _document = require('./document');

var _document2 = _interopRequireDefault(_document);

var _element = require('./element');

var _element2 = _interopRequireDefault(_element);

var _focusBlur = require('../event/focus-blur');

var _focusBlur2 = _interopRequireDefault(_focusBlur);

var _dom = require('../../../processing/dom');

var _dom2 = _interopRequireDefault(_dom);

var _domProcessor = require('../../dom-processor');

var _domProcessor2 = _interopRequireDefault(_domProcessor);

var _dom3 = require('../../utils/dom');

var domUtils = _interopRequireWildcard(_dom3);

var _querySelector = require('../../utils/query-selector');

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _attributes = require('../../../processing/dom/attributes');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ATTRIBUTE_SELECTOR_REG_EX = /\[([\w-]+)(\^?=.+?)]/g;
var ATTRIBUTE_OPERATOR_WITH_HASH_VALUE = /^\W+\s*#/;

var NodeSandbox = function (_SandboxBase) {
    _inherits(NodeSandbox, _SandboxBase);

    function NodeSandbox(nodeMutation, iframeSandbox, eventSandbox, uploadSandbox, shadowUI, cookieSandbox) {
        _classCallCheck(this, NodeSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.raiseBodyCreatedEvent = _this._onBodyCreated;
        // NOTE: We need to define the property with the 'writable' descriptor for testing purposes
        _nativeMethods2.default.objectDefineProperty(document, _internalProperties2.default.documentCharset, {
            value: domUtils.parseDocumentCharset(),
            writable: true
        });
        _this.eventSandbox = eventSandbox;
        _this.iframeSandbox = iframeSandbox;
        _this.shadowUI = shadowUI;
        _this.mutation = nodeMutation;
        _this.doc = new _document2.default(_this, shadowUI, cookieSandbox);
        _this.win = new _window2.default(_this, eventSandbox, uploadSandbox, nodeMutation);
        _this.element = new _element2.default(_this, uploadSandbox, iframeSandbox, shadowUI, eventSandbox);
        return _this;
    }

    NodeSandbox.prototype._onBodyCreated = function _onBodyCreated() {
        this.eventSandbox.listeners.initDocumentBodyListening(this.document);
        this.mutation.onBodyCreated({
            body: this.document.body
        });
    };

    NodeSandbox.prototype._processElement = function _processElement(el) {
        var processedContext = el[_internalProperties2.default.processedContext];
        if (domUtils.isShadowUIElement(el) || processedContext === this.window) return;
        var urlAttrName = null;
        if (processedContext) {
            urlAttrName = _domProcessor2.default.getUrlAttr(el);
            urlAttrName = urlAttrName && el.hasAttribute(urlAttrName) ? urlAttrName : null;
        }
        var canAddNewProp = _nativeMethods2.default.objectIsExtensible(el);
        var canUpdateExistingProp = processedContext && !_nativeMethods2.default.objectIsFrozen(el);
        if (canAddNewProp || canUpdateExistingProp) {
            _nativeMethods2.default.objectDefineProperty(el, _internalProperties2.default.processedContext, {
                value: this.window,
                writable: true
            });
        }
        // NOTE: We need to reprocess url attribute of element, if it's moved to different window (GH-564)
        if (urlAttrName) el.setAttribute(urlAttrName, el.getAttribute(urlAttrName));
        this.element.processElement(el);
    };

    NodeSandbox.prototype.processNodes = function processNodes(el, doc) {
        if (!el) {
            doc = doc || this.document;
            if (doc.documentElement) this.processNodes(doc.documentElement);
        } else if (el.querySelectorAll) {
            this._processElement(el);
            var children = (0, _querySelector.getNativeQuerySelectorAll)(el).call(el, '*');
            var length = _nativeMethods2.default.nodeListLengthGetter.call(children);
            for (var i = 0; i < length; i++) {
                this._processElement(children[i]);
            }
        }
    };
    // NOTE: DOM sandbox hides evidence of the content proxying from a page native script. Proxy replaces URLs for
    // resources. Our goal is to make the native script think that all resources are fetched from the destination
    // resource, not from proxy, and also provide proxying for dynamically created elements.


    NodeSandbox.prototype.attach = function attach(window) {
        var _this2 = this;

        var document = window.document;
        var domContentLoadedEventRaised = false;
        _SandboxBase.prototype.attach.call(this, window, document);
        this.iframeSandbox.on(this.iframeSandbox.IFRAME_DOCUMENT_CREATED_EVENT, function (_ref) {
            var iframe = _ref.iframe;

            var contentWindow = _nativeMethods2.default.contentWindowGetter.call(iframe);
            var contentDocument = _nativeMethods2.default.contentDocumentGetter.call(iframe);
            // NOTE: Before overriding the iframe, we must restore native document methods.
            // Therefore, we save them before they are overridden.
            var iframeNativeMethods = new _this2.nativeMethods.constructor(contentDocument, contentWindow);
            contentWindow[_internalProperties2.default.iframeNativeMethods] = iframeNativeMethods;
            // NOTE: Override only the document (in fact, we only need the 'write' and 'writeln' methods).
            _this2.doc.attach(contentWindow, contentDocument);
        });
        // NOTE: In Google Chrome, iframes whose src contains html code raise the 'load' event twice.
        // So, we need to define code instrumentation functions as 'configurable' so that they can be redefined.
        _nativeMethods2.default.objectDefineProperty(window, _internalProperties2.default.processDomMethodName, {
            value: function value(el, doc) {
                // NOTE: TestCafe creates a shadow-ui root before the DOMContentLoaded event (once document.body is
                // available). Sometimes for a very heavy DOM or a very slow loading the body doesn't contain all
                // elements at that moment and as a result after a full page loading our root element becomes not
                // the last child of the body. So we need to make the root last body child manually on every script
                // loading until the DOMContentLoaded event is raised.
                if (!domContentLoadedEventRaised) _this2.shadowUI.onBodyElementMutation();
                _this2.processNodes(el, doc);
            },
            configurable: true
        });
        // NOTE: In some browsers (for example Firefox), the 'window.document' object is different when iframe is
        // created and when the document’s ready event is raised. Therefore, we need to update the 'document' object
        // to override its methods (Q527555).
        document.addEventListener('DOMContentLoaded', function () {
            domContentLoadedEventRaised = true;
            _this2.processNodes(null, document);
        }, false);
        this.doc.attach(window, document);
        this.win.attach(window);
        this.element.attach(window);
    };

    NodeSandbox._processAttributeSelector = function _processAttributeSelector(selector) {
        if (!ATTRIBUTE_SELECTOR_REG_EX.test(selector)) return selector;
        return selector.replace(ATTRIBUTE_SELECTOR_REG_EX, function (str, name, operatorWithValue) {
            if (_attributes.URL_ATTRS.indexOf(name) !== -1 && !ATTRIBUTE_OPERATOR_WITH_HASH_VALUE.test(operatorWithValue)) {
                name = _dom2.default.getStoredAttrName(name);
                return '[' + name + operatorWithValue + ']';
            }
            return str;
        });
    };

    NodeSandbox.processSelector = function processSelector(selector) {
        if (selector) {
            selector = _focusBlur2.default._processFocusPseudoClassSelector(selector);
            selector = NodeSandbox._processAttributeSelector(selector);
        }
        return selector;
    };

    return NodeSandbox;
}(_base2.default);

exports.default = NodeSandbox;
module.exports = exports['default'];