'use strict';

exports.__esModule = true;

var _nativeMethods = require('../../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dom = require('../../../utils/dom');

var _querySelector = require('../../../utils/query-selector');

var _integerIdGenerator = require('../../../utils/integer-id-generator');

var _integerIdGenerator2 = _interopRequireDefault(_integerIdGenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DOMMutationTracker = function () {
    function DOMMutationTracker() {
        var _this = this;

        _classCallCheck(this, DOMMutationTracker);

        this._mutations = _nativeMethods2.default.objectCreate(null);
        this._isDomContentLoaded = false;
        _nativeMethods2.default.addEventListener.call(document, 'DOMContentLoaded', function () {
            _this._isDomContentLoaded = true;
        });
    }

    DOMMutationTracker.prototype._updateVersion = function _updateVersion(tagName) {
        if (tagName in this._mutations) this._mutations[tagName].increment();
    };

    DOMMutationTracker.prototype._processElement = function _processElement(el) {
        if (!el.tagName || (0, _dom.isShadowUIElement)(el)) return;
        var tagName = (0, _dom.getTagName)(el);
        this._updateVersion('*');
        this._updateVersion(tagName);
    };

    DOMMutationTracker.prototype._processChildren = function _processChildren(el) {
        if (!el.querySelectorAll) return;
        var children = (0, _querySelector.getNativeQuerySelectorAll)(el).call(el, '*');
        var length = _nativeMethods2.default.nodeListLengthGetter.call(children);
        for (var i = 0; i < length; i++) {
            this._processElement(children[i]);
        }
    };

    DOMMutationTracker.prototype.onElementChanged = function onElementChanged(el) {
        this._processElement(el);
        this._processChildren(el);
    };

    DOMMutationTracker.prototype.onChildrenChanged = function onChildrenChanged(el) {
        this._processChildren(el);
    };

    DOMMutationTracker.prototype.isOutdated = function isOutdated(tagName, version) {
        if (!this._isDomContentLoaded) return true;
        var isTagTracked = tagName in this._mutations;
        if (!isTagTracked) this._mutations[tagName] = (0, _integerIdGenerator2.default)();
        var lastVersion = this._mutations[tagName].value; // eslint-disable-line no-restricted-properties
        return version < lastVersion;
    };

    DOMMutationTracker.prototype.getVersion = function getVersion(tagName) {
        if (tagName in this._mutations) return this._mutations[tagName].value; // eslint-disable-line no-restricted-properties
        return -Infinity;
    };

    return DOMMutationTracker;
}();

exports.default = new DOMMutationTracker();
module.exports = exports['default'];