'use strict';

exports.__esModule = true;
exports.default = HTMLCollectionWrapper;

var _domMutationTracker = require('./dom-mutation-tracker');

var _domMutationTracker2 = _interopRequireDefault(_domMutationTracker);

var _nativeMethods = require('../../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dom = require('../../../utils/dom');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// NOTE: Tags from https://www.w3schools.com/tags/att_name.asp
var ELEMENTS_WITH_NAME_ATTRIBUTE = ['button', 'fieldset', 'form', 'iframe', 'input', 'map', 'meta', 'object', 'output', 'param', 'select', 'textarea'];
var COLLECTION_PROTO_GETTERS_RESERVE = 10;
var collectionProtoGettersCount = 0;
function HTMLCollectionWrapper(collection, tagName) {
    tagName = tagName.toLowerCase();
    _nativeMethods2.default.objectDefineProperties.call(Object, this, {
        _collection: { value: collection },
        _filteredCollection: { value: [] },
        _tagName: { value: tagName },
        _version: { value: -Infinity, writable: true },
        _namedProps: { value: ELEMENTS_WITH_NAME_ATTRIBUTE.indexOf(tagName) !== -1 ? [] : null }
    });
    this._refreshCollection();
}
HTMLCollectionWrapper.prototype = _nativeMethods2.default.objectCreate.call(Object, HTMLCollection.prototype);
HTMLCollectionWrapper.prototype.item = function (index) {
    this._refreshCollection();
    return this._filteredCollection[index];
};
if (HTMLCollection.prototype.namedItem) {
    HTMLCollectionWrapper.prototype.namedItem = function () {
        this._refreshCollection();

        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
        }

        var namedItem = this._collection.namedItem.apply(this._collection, args);
        return namedItem && (0, _dom.isShadowUIElement)(namedItem) ? null : namedItem;
    };
}
_nativeMethods2.default.objectDefineProperties.call(Object, HTMLCollectionWrapper.prototype, {
    length: {
        configurable: true,
        enumerable: true,
        get: function get() {
            this._refreshCollection();
            return this._filteredCollection.length;
        }
    },
    _refreshCollection: {
        value: function value() {
            if (!_domMutationTracker2.default.isOutdated(this._tagName, this._version)) return;
            var storedFilteredCollectionLength = this._filteredCollection.length;
            var currentNamedProps = filterCollection(this);
            this._version = _domMutationTracker2.default.getVersion(this._tagName);
            updateCollectionIndexGetters(this, storedFilteredCollectionLength, this._filteredCollection.length);
            updateNamedProps(this, this._namedProps, currentNamedProps);
        }
    }
});
addShadowGetters(COLLECTION_PROTO_GETTERS_RESERVE);
function addShadowGetters(count) {
    var _loop = function _loop(i) {
        var idx = collectionProtoGettersCount++;
        _nativeMethods2.default.objectDefineProperty(HTMLCollectionWrapper.prototype, idx, {
            get: function get() {
                this.item(idx);
            }
        });
    };

    for (var i = 0; i < count; i++) {
        _loop(i);
    }
}
function updateCollectionIndexGetters(wrapper, oldLength, currentLength) {
    if (oldLength === currentLength) return;

    var _loop2 = function _loop2() {
        var idx = oldLength++;
        _nativeMethods2.default.objectDefineProperty(wrapper, idx, {
            enumerable: true,
            configurable: true,
            get: function get() {
                return wrapper.item(idx);
            }
        });
    };

    while (oldLength < currentLength) {
        _loop2();
    }
    while (oldLength > currentLength) {
        delete wrapper[--oldLength];
    }var maxCollectionLength = collectionProtoGettersCount - COLLECTION_PROTO_GETTERS_RESERVE;
    if (currentLength > maxCollectionLength) addShadowGetters(currentLength - maxCollectionLength);
}
function updateNamedProps(wrapper, oldNamedProps, currentNamedProps) {
    if (!currentNamedProps) return;

    for (var _i2 = 0; _i2 < oldNamedProps.length; _i2++) {
        var oldProp = oldNamedProps[_i2];
        if (currentNamedProps.indexOf(oldProp) === -1) delete wrapper[oldProp];
    }

    var _loop3 = function _loop3(_i4) {
        var prop = currentNamedProps[_i4];
        if (!wrapper._collection[prop]) return 'continue';
        _nativeMethods2.default.objectDefineProperty(wrapper, prop, {
            configurable: true,
            get: function get() {
                this._refreshCollection();
                return wrapper._collection[prop];
            }
        });
    };

    for (var _i4 = 0; _i4 < currentNamedProps.length; _i4++) {
        var _ret3 = _loop3(_i4);

        if (_ret3 === 'continue') continue;
    }
}
function filterCollection(wrapper) {
    var nativeCollection = wrapper._collection;
    var nativeCollectionLength = _nativeMethods2.default.htmlCollectionLengthGetter.call(nativeCollection);
    var currentNamedProps = wrapper._namedProps ? [] : null;
    var filteredCollection = wrapper._filteredCollection;
    filteredCollection.length = 0;
    for (var i = 0; i < nativeCollectionLength; i++) {
        var el = nativeCollection[i];
        if ((0, _dom.isShadowUIElement)(el)) continue;
        filteredCollection.push(el);
        if (!currentNamedProps) continue;
        var nameAttr = _nativeMethods2.default.getAttribute.call(el, 'name');
        if (nameAttr !== null) currentNamedProps.push(nameAttr);
    }
    return currentNamedProps;
}
module.exports = exports['default'];