'use strict';

exports.__esModule = true;

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _shadowUi = require('../shadow-ui');

var _shadowUi2 = _interopRequireDefault(_shadowUi);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _script = require('../../../processing/script');

var _style = require('../../../processing/style');

var _style2 = _interopRequireDefault(_style);

var _destinationLocation = require('../../utils/destination-location');

var destLocation = _interopRequireWildcard(_destinationLocation);

var _html = require('../../utils/html');

var _url = require('../../utils/url');

var _browser = require('../../utils/browser');

var _dom = require('../../utils/dom');

var _types = require('../../utils/types');

var _internalAttributes = require('../../../processing/dom/internal-attributes');

var _internalAttributes2 = _interopRequireDefault(_internalAttributes);

var _internalProperties = require('../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _constructorIsCalledWithoutNewKeyword = require('../../utils/constructor-is-called-without-new-keyword');

var _constructorIsCalledWithoutNewKeyword2 = _interopRequireDefault(_constructorIsCalledWithoutNewKeyword);

var _instruction = require('../../../processing/script/instruction');

var _instruction2 = _interopRequireDefault(_instruction);

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _getMimeType = require('../../utils/get-mime-type');

var _getMimeType2 = _interopRequireDefault(_getMimeType);

var _propertyOverriding = require('../../utils/property-overriding');

var _featureDetection = require('../../utils/feature-detection');

var _url2 = require('../../../utils/url');

var _upload = require('../upload');

var _upload2 = _interopRequireDefault(_upload);

var _anchor = require('../code-instrumentation/properties/anchor');

var _namespaces = require('../../../processing/dom/namespaces');

var _urlResolver = require('../../utils/url-resolver');

var _urlResolver2 = _interopRequireDefault(_urlResolver);

var _header = require('../../../processing/script/header');

var _domMutationTracker = require('./live-node-list/dom-mutation-tracker');

var _domMutationTracker2 = _interopRequireDefault(_domMutationTracker);

var _attributes = require('./attributes');

var _replaceProxiedUrlsInStack = require('../../utils/replace-proxied-urls-in-stack');

var _replaceProxiedUrlsInStack2 = _interopRequireDefault(_replaceProxiedUrlsInStack);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /*global history, navigator*/


var nativeFunctionToString = _nativeMethods2.default.Function.toString();
var HTTP_PROTOCOL_RE = /^http/i;
var ALLOWED_SERVICE_WORKER_PROTOCOLS = ['https:', 'wss:', 'file:'];
var ALLOWED_SERVICE_WORKER_HOST_NAMES = ['localhost', '127.0.0.1'];
var JAVASCRIPT_MIME_TYPES = ['text/javascript', 'application/javascript', 'application/x-javascript'];
// NOTE: SVGAnimatedString prototype does not have a way to access the appropriate svg element.
// This is why we use this property to store svg element for which animVal and baseVal properties were set.
// It allows us to change the href-hammerhead-stored-value when it needs.
var CONTEXT_SVG_IMAGE_ELEMENT = 'hammerhead|context-svg-image-element';
var SANDBOX_DOM_TOKEN_LIST = 'hammerhead|sandbox-dom-token-list';
var SANDBOX_DOM_TOKEN_LIST_OWNER = 'hammerhead|sandbox-dom-token-list-owner';
var SANDBOX_DOM_TOKEN_LIST_UPDATE_FN = 'hammerhead|sandbox-dom-token-list-update';
var NO_STACK_TRACE_AVAILABLE_MESSAGE = 'No stack trace available';

var WindowSandbox = function (_SandboxBase) {
    _inherits(WindowSandbox, _SandboxBase);

    function WindowSandbox(nodeSandbox, eventSandbox, uploadSandbox, nodeMutation) {
        _classCallCheck(this, WindowSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.UNCAUGHT_JS_ERROR_EVENT = 'hammerhead|event|uncaught-js-error';
        _this.UNHANDLED_REJECTION_EVENT = 'hammerhead|event|unhandled-rejection';
        _this.nodeSandbox = nodeSandbox;
        _this.messageSandbox = eventSandbox.message;
        _this.listenersSandbox = eventSandbox.listeners;
        _this.elementEditingWatcher = eventSandbox.elementEditingWatcher;
        _this.uploadSandbox = uploadSandbox;
        _this.shadowUI = nodeSandbox.shadowUI;
        _this.nodeMutation = nodeMutation;
        _this.SANDBOX_DOM_TOKEN_LIST_UPDATE_FN = SANDBOX_DOM_TOKEN_LIST_UPDATE_FN;
        _this.isInternalGetter = false;
        return _this;
    }

    WindowSandbox._prepareStack = function _prepareStack(msg, stack) {
        // NOTE: Firefox does not include an error message in a stack trace (unlike other browsers)
        // It is possible to get a stack trace for unhandled Promise rejections only if Promise is rejected with the 'Error' instance value.
        // This is why we should convert the stack to a common format.
        if (!stack || stack.indexOf(msg) === -1) {
            stack = stack || '    ' + NO_STACK_TRACE_AVAILABLE_MESSAGE;
            return msg + ':\n' + stack;
        }
        return stack;
    };

    WindowSandbox.prototype._raiseUncaughtJsErrorEvent = function _raiseUncaughtJsErrorEvent(type, event, window) {
        if ((0, _dom.isCrossDomainWindows)(window, window.top)) return;
        var sendToTopWindow = window !== window.top;
        var pageUrl = destLocation.get();
        var msg = null;
        var stack = null;
        if (type === this.UNHANDLED_REJECTION_EVENT) {
            msg = WindowSandbox._formatUnhandledRejectionReason(event.reason);
            stack = event.reason && event.reason.stack;
        } else if (type === this.UNCAUGHT_JS_ERROR_EVENT) {
            msg = event.error ? event.error.message : event.message;
            stack = event.error && event.error.stack;
        }
        stack = WindowSandbox._prepareStack(msg, stack);
        stack = (0, _replaceProxiedUrlsInStack2.default)(stack);
        if (sendToTopWindow) {
            this.emit(type, { msg: msg, pageUrl: pageUrl, stack: stack, inIframe: true });
            this.messageSandbox.sendServiceMsg({ msg: msg, pageUrl: pageUrl, stack: stack, cmd: type }, window.top);
        } else this.emit(type, { msg: msg, pageUrl: pageUrl, stack: stack });
    };

    WindowSandbox.prototype._reattachHandler = function _reattachHandler(window, eventName) {
        _nativeMethods2.default.windowRemoveEventListener.call(window, eventName, this);
        _nativeMethods2.default.windowAddEventListener.call(window, eventName, this);
    };

    WindowSandbox._formatUnhandledRejectionReason = function _formatUnhandledRejectionReason(reason) {
        if (!(0, _types.isPrimitiveType)(reason)) {
            var reasonStr = _nativeMethods2.default.objectToString.call(reason);
            if (reasonStr === '[object Error]') return reason.message;
            return reasonStr;
        }
        return String(reason);
    };

    WindowSandbox._getUrlAttr = function _getUrlAttr(el, attr) {
        var attrValue = _nativeMethods2.default.getAttribute.call(el, attr);
        var currentDocument = el.ownerDocument || document;
        if (attrValue === '' || attrValue === null && attr === 'action' && _featureDetection.emptyActionAttrFallbacksToTheLocation) return _urlResolver2.default.resolve('', currentDocument);else if (attrValue === null) return '';else if (_url2.HASH_RE.test(attrValue)) return _urlResolver2.default.resolve(attrValue, currentDocument);else if (!(0, _url2.isValidUrl)(attrValue)) return _urlResolver2.default.resolve(attrValue, currentDocument);
        return (0, _url.resolveUrlAsDest)(attrValue);
    };

    WindowSandbox._removeProcessingInstructions = function _removeProcessingInstructions(text) {
        if (text) {
            text = (0, _header.remove)(text);
            return _style2.default.cleanUp(text, _url.parseProxyUrl);
        }
        return text;
    };

    WindowSandbox._processTextPropValue = function _processTextPropValue(el, text) {
        var processedText = text !== null && text !== void 0 ? String(text) : text;
        if (processedText) {
            if ((0, _dom.isScriptElement)(el)) return (0, _script.processScript)(processedText, true);else if ((0, _dom.isStyleElement)(el)) return _style2.default.process(processedText, _url.getProxyUrl, true);
        }
        return processedText;
    };

    WindowSandbox.prototype._overrideUrlAttrDescriptors = function _overrideUrlAttrDescriptors(attr, elementConstructors) {
        var windowSandbox = this;

        for (var _i2 = 0; _i2 < elementConstructors.length; _i2++) {
            var _constructor = elementConstructors[_i2];
            (0, _propertyOverriding.overrideDescriptor)(_constructor.prototype, attr, {
                getter: function getter() {
                    return WindowSandbox._getUrlAttr(this, attr);
                },
                setter: function setter(value) {
                    windowSandbox.nodeSandbox.element.setAttributeCore(this, [attr, value]);
                }
            });
        }
    };

    WindowSandbox.prototype._overrideAttrDescriptors = function _overrideAttrDescriptors(attr, elementConstructors) {
        var windowSandbox = this;

        for (var _i4 = 0; _i4 < elementConstructors.length; _i4++) {
            var _constructor2 = elementConstructors[_i4];
            (0, _propertyOverriding.overrideDescriptor)(_constructor2.prototype, attr, {
                getter: function getter() {
                    return windowSandbox.nodeSandbox.element.getAttributeCore(this, [attr]) || '';
                },
                setter: function setter(value) {
                    windowSandbox.nodeSandbox.element.setAttributeCore(this, [attr, value]);
                }
            });
        }
    };

    WindowSandbox.prototype._overrideUrlPropDescriptor = function _overrideUrlPropDescriptor(prop, nativePropGetter, nativePropSetter) {
        // @ts-ignore
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLAnchorElement.prototype, prop, {
            getter: function getter() {
                return (0, _anchor.getAnchorProperty)(this, nativePropGetter);
            },
            setter: function setter(value) {
                (0, _anchor.setAnchorProperty)(this, nativePropSetter, value);
            }
        });
    };

    WindowSandbox.prototype._overrideErrEventPropDescriptor = function _overrideErrEventPropDescriptor(window, eventName, nativePropSetter) {
        var _this2 = this;

        var eventPropsOwner = _nativeMethods2.default.isEventPropsLocatedInProto ? window.Window.prototype : window;
        (0, _propertyOverriding.overrideDescriptor)(eventPropsOwner, 'on' + eventName, {
            getter: null,
            setter: function setter(handler) {
                nativePropSetter.call(window, handler);
                _this2.listenersSandbox.emit(_this2.listenersSandbox.EVENT_LISTENER_ATTACHED_EVENT, {
                    el: window,
                    listener: handler,
                    eventType: eventName
                });
            }
        });
    };

    WindowSandbox.prototype._createOverriddenDOMTokenListMethod = function _createOverriddenDOMTokenListMethod(nativeMethod) {
        var windowSandbox = this;
        return function () {
            var executionResult = nativeMethod.apply(this, arguments);
            var tokenListOwner = this[SANDBOX_DOM_TOKEN_LIST_OWNER];
            if (tokenListOwner)
                // eslint-disable-next-line no-restricted-properties
                windowSandbox.nodeSandbox.element.setAttributeCore(tokenListOwner, ['sandbox', this.toString()]);
            return executionResult;
        };
    };

    WindowSandbox._isSecureOrigin = function _isSecureOrigin(url) {
        // NOTE: https://www.chromium.org/Home/chromium-security/prefer-secure-origins-for-powerful-new-features
        var parsedUrl = (0, _url.parseUrl)((0, _url.resolveUrlAsDest)(url));
        /*eslint-disable no-restricted-properties*/
        return ALLOWED_SERVICE_WORKER_PROTOCOLS.indexOf(parsedUrl.protocol) === -1 && ALLOWED_SERVICE_WORKER_HOST_NAMES.indexOf(parsedUrl.hostname) === -1;
        /*eslint-enable no-restricted-properties*/
    };

    WindowSandbox.prototype.handleEvent = function handleEvent(event) {
        if (event.defaultPrevented) return;
        if (event.type === 'unhandledrejection') this._raiseUncaughtJsErrorEvent(this.UNHANDLED_REJECTION_EVENT, event, this.window);else if (event.type === 'error') {
            if (event.message.indexOf('NS_ERROR_NOT_INITIALIZED') !== -1) event.preventDefault();else this._raiseUncaughtJsErrorEvent(this.UNCAUGHT_JS_ERROR_EVENT, event, window);
        }
    };

    WindowSandbox.prototype.attach = function attach(window) {
        var _this3 = this;

        _SandboxBase.prototype.attach.call(this, window);
        var messageSandbox = this.messageSandbox;
        var nodeSandbox = this.nodeSandbox;
        var windowSandbox = this;
        this._reattachHandler(window, 'unhandledrejection');
        this._reattachHandler(window, 'error');
        this.listenersSandbox.initElementListening(window, ['error', 'unhandledrejection']);
        this.listenersSandbox.on(this.listenersSandbox.EVENT_LISTENER_ATTACHED_EVENT, function (e) {
            if (e.el !== window) return;
            if (e.eventType === 'unhandledrejection') _this3._reattachHandler(window, 'unhandledrejection');else if (e.eventType === 'error') _this3._reattachHandler(window, 'error');
        });
        this._overrideErrEventPropDescriptor(window, 'error', _nativeMethods2.default.winOnErrorSetter);
        if (_nativeMethods2.default.winOnUnhandledRejectionSetter) this._overrideErrEventPropDescriptor(window, 'unhandledrejection', _nativeMethods2.default.winOnUnhandledRejectionSetter);
        messageSandbox.on(messageSandbox.SERVICE_MSG_RECEIVED_EVENT, function (e) {
            var _e$message = e.message,
                msg = _e$message.msg,
                pageUrl = _e$message.pageUrl,
                stack = _e$message.stack,
                cmd = _e$message.cmd;

            if (cmd === _this3.UNCAUGHT_JS_ERROR_EVENT || cmd === _this3.UNHANDLED_REJECTION_EVENT) windowSandbox.emit(cmd, { msg: msg, pageUrl: pageUrl, stack: stack });
        });
        window.CanvasRenderingContext2D.prototype.drawImage = function () {
            var _this4 = this;

            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
            }

            var image = args[0];
            if ((0, _dom.isImgElement)(image) && !image[_internalProperties2.default.forceProxySrcForImage]) {
                var src = _nativeMethods2.default.imageSrcGetter.call(image);
                if (destLocation.sameOriginCheck(location.toString(), src)) {
                    image = _nativeMethods2.default.createElement.call(window.document, 'img');
                    _nativeMethods2.default.imageSrcSetter.call(image, (0, _url.getProxyUrl)(src));
                    args[0] = image;
                    if (!image.complete) {
                        _nativeMethods2.default.addEventListener.call(image, 'load', function () {
                            return _nativeMethods2.default.canvasContextDrawImage.apply(_this4, args);
                        });
                    }
                }
            }
            return _nativeMethods2.default.canvasContextDrawImage.apply(this, args);
        };
        if (_nativeMethods2.default.objectAssign) {
            window.Object.assign = function (target) {
                var args = [];
                args.push(target);
                var targetType = typeof target;

                for (var _len2 = arguments.length, sources = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
                    sources[_key2 - 1] = arguments[_key2];
                }

                if (target && (targetType === 'object' || targetType === 'function') && sources.length) {
                    for (var _i6 = 0; _i6 < sources.length; _i6++) {
                        var source = sources[_i6];
                        var sourceType = typeof source;
                        if (!source || sourceType !== 'object' && sourceType !== 'function') {
                            _nativeMethods2.default.objectAssign.call(this, target, source);
                            continue;
                        }
                        var sourceKeys = _nativeMethods2.default.objectKeys.call(window.Object, source);

                        for (var _i8 = 0; _i8 < sourceKeys.length; _i8++) {
                            var key = sourceKeys[_i8];
                            window[_instruction2.default.setProperty](target, key, source[key]);
                        }
                    }
                } else args = _nativeMethods2.default.arrayConcat.call(args, sources);
                return _nativeMethods2.default.objectAssign.apply(this, args);
            };
        }
        window.open = function () {
            for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                args[_key3] = arguments[_key3];
            }

            args[0] = (0, _url.getProxyUrl)(args[0]);
            args[1] = args[1] ? nodeSandbox.element.getTarget(null, String(args[1])) : '_self';
            return _nativeMethods2.default.windowOpen.apply(window, args);
        };
        if (window.FontFace) {
            window.FontFace = function (family, source, descriptors) {
                source = _style2.default.process(source, _url.convertToProxyUrl);
                return new _nativeMethods2.default.FontFace(family, source, descriptors);
            };
            window.FontFace.prototype = _nativeMethods2.default.FontFace.prototype;
            window.FontFace.toString = function () {
                return _nativeMethods2.default.FontFace.toString();
            };
        }
        if (window.Worker) {
            window.Worker = function (scriptURL, options) {
                if ((0, _constructorIsCalledWithoutNewKeyword2.default)(this, window.Worker)) _nativeMethods2.default.Worker.apply(this, arguments);
                if (arguments.length === 0) return new _nativeMethods2.default.Worker();
                if (typeof scriptURL === 'string') scriptURL = (0, _url.getProxyUrl)(scriptURL, { resourceType: (0, _url.stringifyResourceType)({ isScript: true }) });
                return arguments.length === 1 ? new _nativeMethods2.default.Worker(scriptURL) : new _nativeMethods2.default.Worker(scriptURL, options);
            };
            window.Worker.prototype = _nativeMethods2.default.Worker.prototype;
            window.Worker.toString = function () {
                return _nativeMethods2.default.Worker.toString();
            };
        }
        if (window.Blob) {
            window.Blob = function (array, opts) {
                if (arguments.length === 0) return new _nativeMethods2.default.Blob();
                var type = opts && opts.type && opts.type.toString().toLowerCase() || (0, _getMimeType2.default)(array);
                // NOTE: If we cannot identify the content type of data, we're trying to process it as a script.
                // Unfortunately, we do not have the ability to exactly identify a script. That's why we make such
                // an assumption. We cannot solve this problem at the Worker level either, because the operation of
                // creating a new Blob instance is asynchronous. (GH-231)
                if (!type || JAVASCRIPT_MIME_TYPES.indexOf(type) !== -1) array = [(0, _script.processScript)(array.join(''), true)];
                // NOTE: IE11 throws an error when the second parameter of the Blob function is undefined (GH-44)
                // If the overridden function is called with one parameter, we need to call the original function
                // with one parameter as well.
                return arguments.length === 1 ? new _nativeMethods2.default.Blob(array) : new _nativeMethods2.default.Blob(array, opts);
            };
            window.Blob.prototype = _nativeMethods2.default.Blob.prototype;
            window.Blob.toString = function () {
                return _nativeMethods2.default.Blob.toString();
            };
        }
        if (window.EventSource) {
            window.EventSource = function (url, opts) {
                if (arguments.length) {
                    var proxyUrl = (0, _url.getProxyUrl)(url, { resourceType: (0, _url.stringifyResourceType)({ isEventSource: true }) });
                    if (arguments.length === 1) return new _nativeMethods2.default.EventSource(proxyUrl);
                    return new _nativeMethods2.default.EventSource(proxyUrl, opts);
                }
                return new _nativeMethods2.default.EventSource();
            };
            window.EventSource.prototype = _nativeMethods2.default.EventSource.prototype;
            window.EventSource.CONNECTING = _nativeMethods2.default.EventSource.CONNECTING;
            window.EventSource.OPEN = _nativeMethods2.default.EventSource.OPEN;
            window.EventSource.CLOSED = _nativeMethods2.default.EventSource.CLOSED;
            window.EventSource.toString = function () {
                return _nativeMethods2.default.EventSource.toString();
            };
        }
        if (window.MutationObserver) {
            window.MutationObserver = function (callback) {
                var wrapper = function wrapper(mutations) {
                    var result = [];

                    for (var _i10 = 0; _i10 < mutations.length; _i10++) {
                        var mutation = mutations[_i10];
                        if (!_shadowUi2.default.isShadowUIMutation(mutation)) result.push(mutation);
                    }

                    if (result.length) callback.call(this, result, this);
                };
                return new _nativeMethods2.default.MutationObserver(wrapper);
            };
            window.MutationObserver.prototype = _nativeMethods2.default.MutationObserver.prototype;
            window.MutationObserver.toString = function () {
                return _nativeMethods2.default.MutationObserver.toString();
            };
        }
        if (window.Proxy) {
            window.Proxy = function (target, handler) {
                if (handler.get) {
                    var storedGet = handler.get;
                    handler.get = function (getterTarget, name, receiver) {
                        if (windowSandbox.isInternalGetter) return getterTarget[name];
                        return storedGet.call(this, getterTarget, name, receiver);
                    };
                }
                return new _nativeMethods2.default.Proxy(target, handler);
            };
            window.Proxy.toString = function () {
                return _nativeMethods2.default.Proxy.toString();
            };
            window.Proxy.revocable = _nativeMethods2.default.Proxy.revocable;
        }
        if (_nativeMethods2.default.registerServiceWorker) {
            window.navigator.serviceWorker.register = function () {
                for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
                    args[_key4] = arguments[_key4];
                }

                var url = args[0];
                if (typeof url === 'string') {
                    if (WindowSandbox._isSecureOrigin(url)) {
                        // NOTE: We cannot create an instance of the DOMException in the Android 6.0 and in the Edge 17 browsers.
                        // The 'TypeError: Illegal constructor' error is raised if we try to call the constructor.
                        return _pinkie2.default.reject(_browser.isAndroid || _browser.isMSEdge && _browser.version >= 17 ? new Error('Only secure origins are allowed.') : new DOMException('Only secure origins are allowed.', 'SecurityError'));
                    }
                    args[0] = (0, _url.getProxyUrl)(url, { resourceType: (0, _url.stringifyResourceType)({ isScript: true }) });
                }
                if (args[1] && typeof args[1].scope === 'string') {
                    args[1].scope = (0, _url.getProxyUrl)(args[1].scope, {
                        resourceType: (0, _url.stringifyResourceType)({ isScript: true })
                    });
                }
                return _nativeMethods2.default.registerServiceWorker.apply(window.navigator.serviceWorker, args);
            };
        }
        if (_nativeMethods2.default.getRegistrationServiceWorker) {
            window.navigator.serviceWorker.getRegistration = function () {
                for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
                    args[_key5] = arguments[_key5];
                }

                var scopeUrl = args[0];
                if (typeof scopeUrl === 'string') {
                    args[0] = (0, _url.getProxyUrl)(scopeUrl, {
                        resourceType: (0, _url.stringifyResourceType)({ isScript: true })
                    });
                }
                return _nativeMethods2.default.getRegistrationServiceWorker.apply(window.navigator.serviceWorker, args);
            };
        }
        if (window.Range.prototype.createContextualFragment) {
            window.Range.prototype.createContextualFragment = function () {
                for (var _len6 = arguments.length, args = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
                    args[_key6] = arguments[_key6];
                }

                var tagString = args[0];
                if (typeof tagString === 'string') {
                    args[0] = (0, _html.processHtml)(tagString, {
                        processedContext: this.startContainer && this.startContainer[_internalProperties2.default.processedContext]
                    });
                }
                var fragment = _nativeMethods2.default.createContextualFragment.apply(this, args);
                nodeSandbox.processNodes(fragment);
                return fragment;
            };
        }
        window.Image = function () {
            var image = null;
            if (!arguments.length) image = new _nativeMethods2.default.Image();else if (arguments.length === 1) image = new _nativeMethods2.default.Image(arguments[0]);else image = new _nativeMethods2.default.Image(arguments[0], arguments[1]);
            image[_internalProperties2.default.forceProxySrcForImage] = true;
            nodeSandbox.processNodes(image);
            return image;
        };
        window.Image.prototype = _nativeMethods2.default.Image.prototype;
        window.Image.toString = function () {
            return _nativeMethods2.default.Image.toString();
        };
        var FunctionWrapper = function FunctionWrapper() {
            for (var _len7 = arguments.length, args = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
                args[_key7] = arguments[_key7];
            }

            var functionBodyArgIndex = args.length - 1;
            if (typeof args[functionBodyArgIndex] === 'string') args[functionBodyArgIndex] = (0, _script.processScript)(args[functionBodyArgIndex], false);
            return _nativeMethods2.default.Function.apply(this, args);
        };
        window.Function = FunctionWrapper;
        window.Function.prototype = _nativeMethods2.default.Function.prototype;
        window.Function.prototype.constructor = FunctionWrapper;
        // NOTE: We need to create function which returns string without calling toString every time
        // because if the Function.prototype.toString is overridden it can be the cause of recursion
        window.Function.toString = function () {
            return this === FunctionWrapper ? nativeFunctionToString : _nativeMethods2.default.functionToString.call(this);
        };
        if (typeof window.history.pushState === 'function' && typeof window.history.replaceState === 'function') {
            var createWrapperForHistoryStateManipulationFn = function createWrapperForHistoryStateManipulationFn(nativeFn) {
                return function () {
                    for (var _len8 = arguments.length, args = Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
                        args[_key8] = arguments[_key8];
                    }

                    var url = args[2];
                    if (args.length > 2 && url !== null && (_browser.isIE || url !== void 0)) args[2] = (0, _url.getProxyUrl)(String(url));
                    return nativeFn.apply(this, args);
                };
            };
            window.history.pushState = createWrapperForHistoryStateManipulationFn(_nativeMethods2.default.historyPushState);
            window.history.replaceState = createWrapperForHistoryStateManipulationFn(_nativeMethods2.default.historyReplaceState);
        }
        if (window.navigator.sendBeacon) {
            window.navigator.sendBeacon = function () {
                if (typeof arguments[0] === 'string') arguments[0] = (0, _url.getProxyUrl)(arguments[0]);
                return _nativeMethods2.default.sendBeacon.apply(this, arguments);
            };
        }
        if (window.navigator.registerProtocolHandler) {
            window.navigator.registerProtocolHandler = function () {
                var urlIndex = 1;

                for (var _len9 = arguments.length, args = Array(_len9), _key9 = 0; _key9 < _len9; _key9++) {
                    args[_key9] = arguments[_key9];
                }

                if (typeof args[urlIndex] === 'string') {
                    // eslint-disable-next-line no-restricted-properties
                    var destHostname = destLocation.getParsed().hostname;
                    var isDestUrl = false;
                    if (_browser.isFirefox) {
                        var parsedUrl = (0, _url.parseUrl)(args[urlIndex]);
                        // eslint-disable-next-line no-restricted-properties
                        isDestUrl = parsedUrl.hostname && destHostname === parsedUrl.hostname;
                    } else isDestUrl = destLocation.sameOriginCheck(destLocation.get(), args[urlIndex]);
                    if (isDestUrl) args[urlIndex] = (0, _url.getProxyUrl)(args[urlIndex]);
                }
                return _nativeMethods2.default.registerProtocolHandler.apply(navigator, args);
            };
        }
        if (window.FormData) {
            window.FormData.prototype.append = function (name, value) {
                // NOTE: We should not send our hidden input's value along with the file info,
                // because our input may have incorrect value if the input with the file has been removed from DOM.
                if (name === _internalAttributes2.default.uploadInfoHiddenInputName) return;
                // NOTE: If we append our file wrapper to FormData, we will lose the file name.
                // This happens because the file wrapper is an instance of Blob
                // and a browser thinks that Blob does not contain the "name" property.
                if (arguments.length === 2 && (0, _dom.isBlob)(value) && 'name' in value) _nativeMethods2.default.formDataAppend.call(this, name, value, value.name);else _nativeMethods2.default.formDataAppend.apply(this, arguments);
            };
        }
        if (window.WebSocket) {
            window.WebSocket = function (url, protocols) {
                if (arguments.length === 0) return new _nativeMethods2.default.WebSocket();
                var proxyUrl = (0, _url.getProxyUrl)(url, { resourceType: (0, _url.stringifyResourceType)({ isWebSocket: true }) });
                if (arguments.length === 1) return new _nativeMethods2.default.WebSocket(proxyUrl);else if (arguments.length === 2) return new _nativeMethods2.default.WebSocket(proxyUrl, protocols);
                return new _nativeMethods2.default.WebSocket(proxyUrl, protocols, arguments[2]);
            };
            window.WebSocket.prototype = _nativeMethods2.default.WebSocket.prototype;
            window.WebSocket.CONNECTING = _nativeMethods2.default.WebSocket.CONNECTING;
            window.WebSocket.OPEN = _nativeMethods2.default.WebSocket.OPEN;
            window.WebSocket.CLOSING = _nativeMethods2.default.WebSocket.CLOSING;
            window.WebSocket.CLOSED = _nativeMethods2.default.WebSocket.CLOSED;
            if (_nativeMethods2.default.webSocketUrlGetter) {
                (0, _propertyOverriding.overrideDescriptor)(window.WebSocket.prototype, 'url', {
                    getter: function getter() {
                        var url = _nativeMethods2.default.webSocketUrlGetter.call(this);
                        var parsedUrl = (0, _url.parseProxyUrl)(url);
                        if (parsedUrl && parsedUrl.destUrl) return parsedUrl.destUrl.replace(HTTP_PROTOCOL_RE, 'ws');
                        return url;
                    }
                });
            }
            window.WebSocket.toString = function () {
                return _nativeMethods2.default.WebSocket.toString();
            };
        }
        (0, _propertyOverriding.overrideDescriptor)(window.MessageEvent.prototype, 'origin', {
            getter: function getter() {
                var target = this.target;
                var origin = _nativeMethods2.default.messageEventOriginGetter.call(this);
                if ((0, _dom.isWebSocket)(target)) {
                    var parsedUrl = (0, _url.parseUrl)(target.url);
                    if (parsedUrl)
                        // eslint-disable-next-line no-restricted-properties
                        return parsedUrl.protocol + '//' + parsedUrl.host;
                } else if ((0, _dom.isWindow)(target)) {
                    var data = _nativeMethods2.default.messageEventDataGetter.call(this);
                    if (data) return data.originUrl;
                }
                return origin;
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLCollection.prototype, 'length', {
            getter: function getter() {
                var length = _nativeMethods2.default.htmlCollectionLengthGetter.call(this);
                if (_shadowUi2.default.isShadowContainerCollection(this)) return windowSandbox.shadowUI.getShadowUICollectionLength(this, length);
                // IE11 and Edge have a strange behavior: shadow container collection flag may be lost (GH-1763)
                else if (_browser.isIE && length && (0, _dom.isHeadOrBodyElement)(this[0].parentNode)) {
                        _shadowUi2.default.markAsShadowContainerCollection(this);
                        return windowSandbox.shadowUI.getShadowUICollectionLength(this, length);
                    }
                return length;
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.NodeList.prototype, 'length', {
            getter: function getter() {
                var length = _nativeMethods2.default.nodeListLengthGetter.call(this);
                if (_shadowUi2.default.isShadowContainerCollection(this)) return windowSandbox.shadowUI.getShadowUICollectionLength(this, length);
                return length;
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.Element.prototype, 'childElementCount', {
            getter: function getter() {
                if (_shadowUi2.default.isShadowContainer(this)) {
                    var childrenLength = _nativeMethods2.default.htmlCollectionLengthGetter.call(this.children);
                    return windowSandbox.shadowUI.getShadowUICollectionLength(this.children, childrenLength);
                }
                return _nativeMethods2.default.elementChildElementCountGetter.call(this);
            }
        });
        if (_nativeMethods2.default.performanceEntryNameGetter) {
            (0, _propertyOverriding.overrideDescriptor)(window.PerformanceEntry.prototype, 'name', {
                getter: function getter() {
                    var name = _nativeMethods2.default.performanceEntryNameGetter.call(this);
                    if ((0, _dom.isPerformanceNavigationTiming)(this)) {
                        var parsedProxyUrl = (0, _url.parseProxyUrl)(name);
                        if (parsedProxyUrl) return parsedProxyUrl.destUrl;
                    }
                    return name;
                }
            });
        }
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLInputElement.prototype, 'files', {
            getter: function getter() {
                if (this.type.toLowerCase() === 'file') return _upload2.default.getFiles(this);
                return _nativeMethods2.default.inputFilesGetter.call(this);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLInputElement.prototype, 'value', {
            getter: function getter() {
                if (this.type.toLowerCase() === 'file') return _upload2.default.getUploadElementValue(this);
                return _nativeMethods2.default.inputValueGetter.call(this);
            },
            setter: function setter(value) {
                if (this.type.toLowerCase() !== 'file') {
                    _nativeMethods2.default.inputValueSetter.call(this, value);
                    var valueChanged = value !== _nativeMethods2.default.inputValueGetter.call(this);
                    if (valueChanged && !(0, _dom.isShadowUIElement)(this) && (0, _dom.isTextEditableElementAndEditingAllowed)(this)) windowSandbox.elementEditingWatcher.restartWatchingElementEditing(this);
                } else windowSandbox.uploadSandbox.setUploadElementValue(this, value);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLInputElement.prototype, 'required', {
            getter: function getter() {
                return windowSandbox.nodeSandbox.element.getAttributeCore(this, ['required']) !== null;
            },
            setter: function setter(value) {
                if (this.type.toLowerCase() !== 'file') _nativeMethods2.default.inputRequiredSetter.call(this, value);else if (value) windowSandbox.nodeSandbox.element.setAttributeCore(this, ['required', '']);else windowSandbox.nodeSandbox.element._removeAttributeCore(this, ['required']);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLTextAreaElement.prototype, 'value', {
            getter: null,
            setter: function setter(value) {
                _nativeMethods2.default.textAreaValueSetter.call(this, value);
                if (!(0, _dom.isShadowUIElement)(this) && (0, _dom.isTextEditableElementAndEditingAllowed)(this)) windowSandbox.elementEditingWatcher.restartWatchingElementEditing(this);
            }
        });
        this._overrideUrlAttrDescriptors('data', [window.HTMLObjectElement]);
        this._overrideUrlAttrDescriptors('src', [window.HTMLImageElement, window.HTMLScriptElement, window.HTMLEmbedElement, window.HTMLSourceElement, window.HTMLMediaElement, window.HTMLInputElement, window.HTMLFrameElement, window.HTMLIFrameElement]);
        this._overrideUrlAttrDescriptors('action', [window.HTMLFormElement]);
        this._overrideUrlAttrDescriptors('formAction', [window.HTMLInputElement, window.HTMLButtonElement]);
        this._overrideUrlAttrDescriptors('href', [window.HTMLAnchorElement, window.HTMLLinkElement, window.HTMLAreaElement, window.HTMLBaseElement]);
        if (_nativeMethods2.default.htmlManifestGetter) this._overrideUrlAttrDescriptors('manifest', [window.HTMLHtmlElement]);
        this._overrideAttrDescriptors('target', [window.HTMLAnchorElement, window.HTMLFormElement, window.HTMLAreaElement, window.HTMLBaseElement]);
        this._overrideAttrDescriptors('formTarget', [window.HTMLInputElement, window.HTMLButtonElement]);
        this._overrideAttrDescriptors('autocomplete', [window.HTMLInputElement]);
        // NOTE: Some browsers (for example, Edge, Internet Explorer 11, Safari) don't support the 'integrity' property.
        if (_nativeMethods2.default.scriptIntegrityGetter && _nativeMethods2.default.linkIntegrityGetter) {
            this._overrideAttrDescriptors('integrity', [window.HTMLScriptElement]);
            this._overrideAttrDescriptors('integrity', [window.HTMLLinkElement]);
        }
        this._overrideAttrDescriptors('rel', [window.HTMLLinkElement]);
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLInputElement.prototype, 'type', {
            getter: null,
            setter: function setter(value) {
                windowSandbox.nodeSandbox.element.setAttributeCore(this, ['type', value]);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLIFrameElement.prototype, 'sandbox', {
            getter: function getter() {
                var domTokenList = this[SANDBOX_DOM_TOKEN_LIST];
                if (!domTokenList) {
                    var span = _nativeMethods2.default.createElement.call(document, 'span');
                    domTokenList = _nativeMethods2.default.elementClassListGetter.call(span);
                    span.className = windowSandbox.nodeSandbox.element.getAttributeCore(this, ['sandbox']) || '';
                    _nativeMethods2.default.objectDefineProperty(domTokenList, SANDBOX_DOM_TOKEN_LIST_OWNER, { value: this });
                    _nativeMethods2.default.objectDefineProperty(this, SANDBOX_DOM_TOKEN_LIST, { value: domTokenList });
                    _nativeMethods2.default.objectDefineProperty(this, SANDBOX_DOM_TOKEN_LIST_UPDATE_FN, {
                        value: function value(_value) {
                            span.className = _value;
                        }
                    });
                }
                return domTokenList;
            },
            setter: function setter(value) {
                windowSandbox.nodeSandbox.element.setAttributeCore(this, ['sandbox', value]);
                if (this[SANDBOX_DOM_TOKEN_LIST_UPDATE_FN]) this[SANDBOX_DOM_TOKEN_LIST_UPDATE_FN](windowSandbox.nodeSandbox.element.getAttributeCore(this, ['sandbox']) || '');
            }
        });
        this._overrideUrlPropDescriptor('port', _nativeMethods2.default.anchorPortGetter, _nativeMethods2.default.anchorPortSetter);
        this._overrideUrlPropDescriptor('host', _nativeMethods2.default.anchorHostGetter, _nativeMethods2.default.anchorHostSetter);
        this._overrideUrlPropDescriptor('hostname', _nativeMethods2.default.anchorHostnameGetter, _nativeMethods2.default.anchorHostnameSetter);
        this._overrideUrlPropDescriptor('pathname', _nativeMethods2.default.anchorPathnameGetter, _nativeMethods2.default.anchorPathnameSetter);
        this._overrideUrlPropDescriptor('protocol', _nativeMethods2.default.anchorProtocolGetter, _nativeMethods2.default.anchorProtocolSetter);
        this._overrideUrlPropDescriptor('search', _nativeMethods2.default.anchorSearchGetter, _nativeMethods2.default.anchorSearchSetter);
        (0, _propertyOverriding.overrideDescriptor)(window.SVGImageElement.prototype, 'href', {
            getter: function getter() {
                var imageHref = _nativeMethods2.default.svgImageHrefGetter.call(this);
                if (!imageHref[CONTEXT_SVG_IMAGE_ELEMENT]) {
                    _nativeMethods2.default.objectDefineProperty(imageHref, CONTEXT_SVG_IMAGE_ELEMENT, {
                        value: this,
                        configurable: true
                    });
                }
                return imageHref;
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.SVGAnimatedString.prototype, 'baseVal', {
            getter: function getter() {
                var baseVal = _nativeMethods2.default.svgAnimStrBaseValGetter.call(this);
                if (this[CONTEXT_SVG_IMAGE_ELEMENT]) {
                    var parsedHref = (0, _url.parseProxyUrl)(baseVal);
                    baseVal = parsedHref ? parsedHref.destUrl : baseVal;
                }
                return baseVal;
            },
            setter: function setter(value) {
                var contextSVGImageElement = this[CONTEXT_SVG_IMAGE_ELEMENT];
                if (contextSVGImageElement) {
                    var hasXlinkHrefAttr = _nativeMethods2.default.hasAttributeNS.call(contextSVGImageElement, _namespaces.XLINK_NAMESPACE, 'href');
                    windowSandbox.nodeSandbox.element.setAttributeCore(contextSVGImageElement, [hasXlinkHrefAttr ? 'xlink:href' : 'href', value]);
                    value = (0, _url.getProxyUrl)(value);
                }
                _nativeMethods2.default.svgAnimStrBaseValSetter.call(this, value);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.SVGAnimatedString.prototype, 'animVal', {
            getter: function getter() {
                var animVal = _nativeMethods2.default.svgAnimStrAnimValGetter.call(this);
                if (this[CONTEXT_SVG_IMAGE_ELEMENT]) {
                    var parsedAnimVal = (0, _url.parseProxyUrl)(animVal);
                    return parsedAnimVal ? parsedAnimVal.destUrl : animVal;
                }
                return animVal;
            }
        });
        if (_nativeMethods2.default.anchorOriginGetter) {
            (0, _propertyOverriding.overrideDescriptor)(window.HTMLAnchorElement.prototype, 'origin', {
                getter: function getter() {
                    return (0, _anchor.getAnchorProperty)(this, _nativeMethods2.default.anchorOriginGetter);
                }
            });
        }
        (0, _propertyOverriding.overrideDescriptor)(window.StyleSheet.prototype, 'href', {
            getter: function getter() {
                var href = _nativeMethods2.default.styleSheetHrefGetter.call(this);
                var parsedUrl = (0, _url.parseProxyUrl)(href);
                return parsedUrl ? parsedUrl.destUrl : href;
            }
        });
        if (_nativeMethods2.default.cssStyleSheetHrefGetter) {
            (0, _propertyOverriding.overrideDescriptor)(window.CSSStyleSheet.prototype, 'href', {
                getter: function getter() {
                    var href = _nativeMethods2.default.cssStyleSheetHrefGetter.call(this);
                    var parsedUrl = (0, _url.parseProxyUrl)(href);
                    return parsedUrl ? parsedUrl.destUrl : href;
                }
            });
        }
        if (_nativeMethods2.default.nodeBaseURIGetter) {
            (0, _propertyOverriding.overrideDescriptor)(window.Node.prototype, 'baseURI', {
                getter: function getter() {
                    var baseURI = _nativeMethods2.default.nodeBaseURIGetter.call(this);
                    var parsedUrl = (0, _url.parseProxyUrl)(baseURI);
                    return parsedUrl ? parsedUrl.destUrl : baseURI;
                }
            });
        }
        if (window.DOMParser) {
            window.DOMParser.prototype.parseFromString = function () {
                for (var _len10 = arguments.length, args = Array(_len10), _key10 = 0; _key10 < _len10; _key10++) {
                    args[_key10] = arguments[_key10];
                }

                var str = args[0];
                var type = args[1];
                var processedHtml = void 0;
                if (args.length > 1 && typeof str === 'string' && type === 'text/html') {
                    processedHtml = (0, _html.processHtml)(str);
                    args[0] = processedHtml;
                }
                var document = _nativeMethods2.default.DOMParserParseFromString.apply(this, args);
                if (processedHtml) _shadowUi2.default.removeSelfRemovingScripts(document);
                return document;
            };
        }
        (0, _propertyOverriding.overrideDescriptor)(window.Node.prototype, 'firstChild', {
            getter: function getter() {
                if (_shadowUi2.default.isShadowContainer(this)) return windowSandbox.shadowUI.getFirstChild(this);
                return _nativeMethods2.default.nodeFirstChildGetter.call(this);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.Element.prototype, 'firstElementChild', {
            getter: function getter() {
                if (_shadowUi2.default.isShadowContainer(this)) return windowSandbox.shadowUI.getFirstElementChild(this);
                return _nativeMethods2.default.elementFirstElementChildGetter.call(this);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.Node.prototype, 'lastChild', {
            getter: function getter() {
                if (_shadowUi2.default.isShadowContainer(this)) return windowSandbox.shadowUI.getLastChild(this);
                return _nativeMethods2.default.nodeLastChildGetter.call(this);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.Element.prototype, 'lastElementChild', {
            getter: function getter() {
                if (_shadowUi2.default.isShadowContainer(this)) return windowSandbox.shadowUI.getLastElementChild(this);
                return _nativeMethods2.default.elementLastElementChildGetter.call(this);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.Node.prototype, 'nextSibling', {
            getter: function getter() {
                return windowSandbox.shadowUI.getNextSibling(this);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.Node.prototype, 'previousSibling', {
            getter: function getter() {
                return windowSandbox.shadowUI.getPrevSibling(this);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.Element.prototype, 'nextElementSibling', {
            getter: function getter() {
                return windowSandbox.shadowUI.getNextElementSibling(this);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.Element.prototype, 'previousElementSibling', {
            getter: function getter() {
                return windowSandbox.shadowUI.getPrevElementSibling(this);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window[_nativeMethods2.default.elementHTMLPropOwnerName].prototype, 'innerHTML', {
            getter: function getter() {
                var innerHTML = _nativeMethods2.default.elementInnerHTMLGetter.call(this);
                if ((0, _dom.isScriptElement)(this)) return (0, _header.remove)(innerHTML);else if ((0, _dom.isStyleElement)(this)) return _style2.default.cleanUp(innerHTML, _url.parseProxyUrl);
                return (0, _html.cleanUpHtml)(innerHTML);
            },
            setter: function setter(value) {
                var el = this;
                var isStyleEl = (0, _dom.isStyleElement)(el);
                var isScriptEl = (0, _dom.isScriptElement)(el);
                var processedValue = value !== null && value !== void 0 ? String(value) : value;
                if (processedValue) {
                    if (isStyleEl) processedValue = _style2.default.process(processedValue, _url.getProxyUrl, true);else if (isScriptEl) processedValue = (0, _script.processScript)(processedValue, true);else {
                        processedValue = (0, _html.processHtml)(processedValue, {
                            parentTag: el.tagName,
                            processedContext: el[_internalProperties2.default.processedContext]
                        });
                    }
                }
                if (!isStyleEl && !isScriptEl) _domMutationTracker2.default.onChildrenChanged(el);
                _nativeMethods2.default.elementInnerHTMLSetter.call(el, processedValue);
                if (isStyleEl || isScriptEl) return;
                _domMutationTracker2.default.onChildrenChanged(el);
                if (windowSandbox.document.body === el) {
                    var shadowUIRoot = windowSandbox.shadowUI.getRoot();
                    windowSandbox.shadowUI.markShadowUIContainers(windowSandbox.document.head, el);
                    _shadowUi2.default.markElementAndChildrenAsShadow(shadowUIRoot);
                } else if ((0, _dom.isShadowUIElement)(el)) _shadowUi2.default.markElementAndChildrenAsShadow(el);
                var parentDocument = (0, _dom.findDocument)(el);
                var parentWindow = parentDocument ? parentDocument.defaultView : null;
                // NOTE: For the iframe with an empty src.
                if (parentWindow && parentWindow !== window && parentWindow[_internalProperties2.default.processDomMethodName]) parentWindow[_internalProperties2.default.processDomMethodName](el, parentDocument);else if (window[_internalProperties2.default.processDomMethodName]) window[_internalProperties2.default.processDomMethodName](el);
                // NOTE: Fix for B239138 - unroll.me 'Cannot read property 'document' of null' error raised
                // during recording. There was an issue when the document.body was replaced, so we need to
                // reattach UI to a new body manually.
                // NOTE: This check is required because jQuery calls the set innerHTML method for an element
                // in an unavailable window.
                if (window.self) {
                    // NOTE: Use timeout, so that changes take effect.
                    if ((0, _dom.isHtmlElement)(el) || (0, _dom.isBodyElement)(el)) _nativeMethods2.default.setTimeout.call(window, function () {
                        return windowSandbox.nodeMutation.onBodyContentChanged(el);
                    }, 0);
                }
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window[_nativeMethods2.default.elementHTMLPropOwnerName].prototype, 'outerHTML', {
            getter: function getter() {
                var outerHTML = _nativeMethods2.default.elementOuterHTMLGetter.call(this);
                return (0, _html.cleanUpHtml)(outerHTML);
            },
            setter: function setter(value) {
                var el = this;
                var parentEl = el.parentNode;
                _domMutationTracker2.default.onElementChanged(el);
                if (parentEl && value !== null && value !== void 0) {
                    var parentDocument = (0, _dom.findDocument)(parentEl);
                    var parentWindow = parentDocument ? parentDocument.defaultView : null;
                    _nativeMethods2.default.elementOuterHTMLSetter.call(el, (0, _html.processHtml)(String(value), {
                        parentTag: parentEl && parentEl.tagName,
                        processedContext: el[_internalProperties2.default.processedContext]
                    }));
                    _domMutationTracker2.default.onChildrenChanged(parentEl);
                    // NOTE: For the iframe with an empty src.
                    if (parentWindow && parentWindow !== window && parentWindow[_internalProperties2.default.processDomMethodName]) parentWindow[_internalProperties2.default.processDomMethodName](parentEl, parentDocument);else if (window[_internalProperties2.default.processDomMethodName]) window[_internalProperties2.default.processDomMethodName](parentEl);
                    // NOTE: This check is required for an element in an unavailable window.
                    // NOTE: Use timeout, so that changes take effect.
                    if (window.self && (0, _dom.isBodyElement)(el)) _nativeMethods2.default.setTimeout.call(window, function () {
                        return windowSandbox.shadowUI.onBodyElementMutation();
                    }, 0);
                } else _nativeMethods2.default.elementOuterHTMLSetter.call(el, value);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLElement.prototype, 'innerText', {
            getter: function getter() {
                var textContent = _nativeMethods2.default.htmlElementInnerTextGetter.call(this);
                return WindowSandbox._removeProcessingInstructions(textContent);
            },
            setter: function setter(value) {
                var processedValue = WindowSandbox._processTextPropValue(this, value);
                _domMutationTracker2.default.onChildrenChanged(this);
                _nativeMethods2.default.htmlElementInnerTextSetter.call(this, processedValue);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLScriptElement.prototype, 'text', {
            getter: function getter() {
                var text = _nativeMethods2.default.scriptTextGetter.call(this);
                return (0, _header.remove)(text);
            },
            setter: function setter(value) {
                var processedValue = value ? (0, _script.processScript)(String(value), true) : value;
                _nativeMethods2.default.scriptTextSetter.call(this, processedValue);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.HTMLAnchorElement.prototype, 'text', {
            getter: function getter() {
                var textContent = _nativeMethods2.default.anchorTextGetter.call(this);
                return WindowSandbox._removeProcessingInstructions(textContent);
            },
            setter: function setter(value) {
                var processedValue = WindowSandbox._processTextPropValue(this, value);
                _domMutationTracker2.default.onChildrenChanged(this);
                _nativeMethods2.default.anchorTextSetter.call(this, processedValue);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.Node.prototype, 'textContent', {
            getter: function getter() {
                var textContent = _nativeMethods2.default.nodeTextContentGetter.call(this);
                return WindowSandbox._removeProcessingInstructions(textContent);
            },
            setter: function setter(value) {
                var processedValue = WindowSandbox._processTextPropValue(this, value);
                _domMutationTracker2.default.onChildrenChanged(this);
                _nativeMethods2.default.nodeTextContentSetter.call(this, processedValue);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window[_nativeMethods2.default.elementAttributesPropOwnerName].prototype, 'attributes', {
            getter: function getter() {
                return (0, _attributes.getAttributes)(this);
            }
        });
        window.DOMTokenList.prototype.add = this._createOverriddenDOMTokenListMethod(_nativeMethods2.default.tokenListAdd);
        window.DOMTokenList.prototype.remove = this._createOverriddenDOMTokenListMethod(_nativeMethods2.default.tokenListRemove);
        window.DOMTokenList.prototype.toggle = this._createOverriddenDOMTokenListMethod(_nativeMethods2.default.tokenListToggle);
        if (_nativeMethods2.default.tokenListReplace) window.DOMTokenList.prototype.replace = this._createOverriddenDOMTokenListMethod(_nativeMethods2.default.tokenListReplace);
        if (_nativeMethods2.default.tokenListSupports) {
            window.DOMTokenList.prototype.supports = function () {
                if (this[SANDBOX_DOM_TOKEN_LIST_OWNER]) {
                    var nativeTokenList = _nativeMethods2.default.iframeSandboxGetter.call(this[SANDBOX_DOM_TOKEN_LIST_OWNER]);
                    return _nativeMethods2.default.tokenListSupports.apply(nativeTokenList, arguments);
                }
                return _nativeMethods2.default.tokenListSupports.apply(this, arguments);
            };
        }
        window.DOMImplementation.prototype.createHTMLDocument = function () {
            for (var _len11 = arguments.length, args = Array(_len11), _key11 = 0; _key11 < _len11; _key11++) {
                args[_key11] = arguments[_key11];
            }

            var doc = _nativeMethods2.default.createHTMLDocument.apply(this, args);
            _urlResolver2.default.init(doc);
            return doc;
        };
        (0, _propertyOverriding.overrideDescriptor)(window.MutationRecord.prototype, 'nextSibling', {
            getter: function getter() {
                var originNextSibling = _nativeMethods2.default.mutationRecordNextSiblingGetter.call(this);
                return windowSandbox.shadowUI.getNextSibling(originNextSibling);
            }
        });
        (0, _propertyOverriding.overrideDescriptor)(window.MutationRecord.prototype, 'previousSibling', {
            getter: function getter() {
                var originPrevSibling = _nativeMethods2.default.mutationRecordPrevSiblingGetter.call(this);
                return windowSandbox.shadowUI.getPrevSibling(originPrevSibling);
            }
        });
    };

    return WindowSandbox;
}(_base2.default);

exports.default = WindowSandbox;
module.exports = exports['default'];