'use strict';

exports.__esModule = true;

var _internalProperties = require('../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _base = require('./base');

var _base2 = _interopRequireDefault(_base);

var _index = require('./node/index');

var _index2 = _interopRequireDefault(_index);

var _nativeMethods = require('./native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dom = require('../utils/dom');

var domUtils = _interopRequireWildcard(_dom);

var _domProcessor = require('../dom-processor');

var _domProcessor2 = _interopRequireDefault(_domProcessor);

var _position = require('../utils/position');

var _className = require('../../shadow-ui/class-name');

var _className2 = _interopRequireDefault(_className);

var _style = require('../utils/style');

var _event = require('../utils/event');

var _querySelector = require('../utils/query-selector');

var _htmlCollectionWrapper = require('./node/live-node-list/html-collection-wrapper');

var _htmlCollectionWrapper2 = _interopRequireDefault(_htmlCollectionWrapper);

var _featureDetection = require('../utils/feature-detection');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var IS_NON_STATIC_POSITION_RE = /fixed|relative|absolute/;
var CLASSNAME_RE = /\.((?:\\.|[-\w]|[^\x00-\xa0])+)/g;
var IS_SHADOW_CONTAINER_FLAG = 'hammerhead|shadow-ui|container-flag';
var IS_SHADOW_CONTAINER_COLLECTION_FLAG = 'hammerhead|shadow-ui|container-collection-flag';
var HTML_COLLECTION_WRAPPER = 'hammerhead|shadow-ui|html-collection-wrapper';

var ShadowUI = function (_SandboxBase) {
    _inherits(ShadowUI, _SandboxBase);

    function ShadowUI(nodeMutation, messageSandbox, iframeSandbox, ieDebugSandbox) {
        _classCallCheck(this, ShadowUI);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.BODY_CONTENT_CHANGED_COMMAND = 'hammerhead|command|body-content-changed';
        _this.ROOT_CLASS = 'root';
        _this.ROOT_ID = 'root';
        _this.HIDDEN_CLASS = 'hidden';
        _this.BLIND_CLASS = 'blind';
        _this.nodeMutation = nodeMutation;
        _this.messageSandbox = messageSandbox;
        _this.iframeSandbox = iframeSandbox;
        _this.ieDebugSandbox = ieDebugSandbox;
        _this.root = null;
        _this.lastActiveElement = null;
        _this.uiStyleSheetsHtmlBackup = null;
        _this.wrapperCreators = _this._createWrapperCreators();
        _this._initEventCallbacks();
        return _this;
    }

    ShadowUI.prototype._initEventCallbacks = function _initEventCallbacks() {
        var _this2 = this;

        this.runTaskScriptEventCallback = function (e) {
            var contentDocument = _nativeMethods2.default.contentDocumentGetter.call(e.iframe);
            var iframeHead = contentDocument.head;
            var iframeBody = contentDocument.body;
            _this2._restoreUIStyleSheets(iframeHead, _this2._getUIStyleSheetsHtml());
            _this2.markShadowUIContainers(iframeHead, iframeBody);
        };
        this.beforeDocumentCleanedEventCallback = function () {
            _this2.uiStyleSheetsHtmlBackup = _this2._getUIStyleSheetsHtml();
        };
        this.documentCleanedEventCallback = function (e) {
            _this2._restoreUIStyleSheets(e.document.head, _this2.uiStyleSheetsHtmlBackup);
            _this2.uiStyleSheetsHtmlBackup = null;
            _this2.markShadowUIContainers(_this2.document.head, _this2.document.body);
        };
        this.documentClosedEventCallback = function (e) {
            _this2._restoreUIStyleSheets(e.document.head, _this2.uiStyleSheetsHtmlBackup);
            _this2.uiStyleSheetsHtmlBackup = null;
            _this2.markShadowUIContainers(e.document.head, e.document.body);
        };
        this.bodyContentChangedEventCallback = function (el) {
            var elContextWindow = el[_internalProperties2.default.processedContext];
            if (elContextWindow !== window) {
                _this2.messageSandbox.sendServiceMsg({
                    cmd: _this2.BODY_CONTENT_CHANGED_COMMAND
                }, elContextWindow);
            } else _this2.onBodyElementMutation();
        };
        this.serviceMsgReceivedEventCallback = function (e) {
            if (e.message.cmd === _this2.BODY_CONTENT_CHANGED_COMMAND) _this2.onBodyElementMutation();
        };
        this.bodyCreatedEventCallback = function (_ref) {
            var body = _ref.body;
            return _this2.markShadowUIContainers(_this2.document.head, body);
        };
    };

    ShadowUI._filterElement = function _filterElement(el) {
        return el && domUtils.isShadowUIElement(el) ? null : el;
    };

    ShadowUI.prototype._filterList = function _filterList(list, listLength, predicate) {
        var filteredList = [];
        for (var i = 0; i < listLength; i++) {
            var el = predicate(list[i]);
            if (el) filteredList.push(list[i]);
        }
        _nativeMethods2.default.objectDefineProperty(filteredList, 'item', {
            value: function value(index) {
                return index >= filteredList.length ? null : filteredList[index];
            }
        });
        if (list.namedItem) {
            _nativeMethods2.default.objectDefineProperty(filteredList, 'namedItem', {
                value: function value(name) {
                    return list.namedItem(name);
                }
            });
        }
        return filteredList.length === listLength ? list : filteredList;
    };

    ShadowUI.prototype._filterNodeList = function _filterNodeList(nodeList, originLength) {
        return this._filterList(nodeList, originLength, function (item) {
            return ShadowUI._filterElement(item);
        });
    };

    ShadowUI.prototype._filterStyleSheetList = function _filterStyleSheetList(styleSheetList, originLength) {
        return this._filterList(styleSheetList, originLength, function (item) {
            return ShadowUI._filterElement(item.ownerNode);
        });
    };

    ShadowUI._getFirstNonShadowElement = function _getFirstNonShadowElement(nodeList) {
        var length = _nativeMethods2.default.nodeListLengthGetter.call(nodeList);
        for (var i = 0; i < length; i++) {
            if (ShadowUI._filterElement(nodeList[i])) return nodeList[i];
        }
        return null;
    };

    ShadowUI.prototype._createWrapperCreators = function _createWrapperCreators() {
        var sandbox = this;
        return {
            getElementsByClassName: function getElementsByClassName(nativeGetElementsByClassNameFnName) {
                return function () {
                    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                        args[_key] = arguments[_key];
                    }

                    var elements = _nativeMethods2.default[nativeGetElementsByClassNameFnName].apply(this, args);
                    var length = _nativeMethods2.default.htmlCollectionLengthGetter.call(elements);
                    return sandbox._filterNodeList(elements, length);
                };
            },
            getElementsByTagName: function getElementsByTagName(nativeGetElementsByTagNameFnName) {
                return function () {
                    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                        args[_key2] = arguments[_key2];
                    }

                    var nativeCollection = _nativeMethods2.default[nativeGetElementsByTagNameFnName].apply(this, args);
                    var tagName = args[0];
                    if (typeof tagName !== 'string') return nativeCollection;
                    if (!nativeCollection[HTML_COLLECTION_WRAPPER]) nativeCollection[HTML_COLLECTION_WRAPPER] = new _htmlCollectionWrapper2.default(nativeCollection, tagName);else nativeCollection[HTML_COLLECTION_WRAPPER]._refreshCollection();
                    return nativeCollection[HTML_COLLECTION_WRAPPER];
                };
            },
            querySelector: function querySelector(nativeQuerySelectorFnName, nativeQuerySelectorAllFnName) {
                return function () {
                    for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                        args[_key3] = arguments[_key3];
                    }

                    if (typeof args[0] === 'string') args[0] = _index2.default.processSelector(args[0]);
                    var element = _nativeMethods2.default[nativeQuerySelectorFnName].apply(this, args);
                    var filteredElement = ShadowUI._filterElement(element);
                    if (!element || filteredElement) return filteredElement;
                    return ShadowUI._getFirstNonShadowElement(_nativeMethods2.default[nativeQuerySelectorAllFnName].apply(this, args));
                };
            },
            querySelectorAll: function querySelectorAll(nativeQuerySelectorAllFnName) {
                return function () {
                    for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
                        args[_key4] = arguments[_key4];
                    }

                    if (typeof args[0] === 'string') args[0] = _index2.default.processSelector(args[0]);
                    var list = _nativeMethods2.default[nativeQuerySelectorAllFnName].apply(this, args);
                    var length = _nativeMethods2.default.nodeListLengthGetter.call(list);
                    return sandbox._filterNodeList(list, length);
                };
            }
        };
    };

    ShadowUI.prototype._markShadowUIContainerAndCollections = function _markShadowUIContainerAndCollections(containerEl) {
        ShadowUI._markAsShadowContainer(containerEl);
        ShadowUI.markAsShadowContainerCollection(containerEl.children);
        ShadowUI.markAsShadowContainerCollection(containerEl.childNodes);
    };

    ShadowUI.prototype.markShadowUIContainers = function markShadowUIContainers(head, body) {
        if (head) this._markShadowUIContainerAndCollections(head);
        if (body) this._markShadowUIContainerAndCollections(body);
    };

    ShadowUI.prototype._bringRootToWindowTopLeft = function _bringRootToWindowTopLeft() {
        var rootHasParentWithNonStaticPosition = false;
        var parent = this.root.parentNode;
        while (parent) {
            var elementPosition = (0, _style.get)(parent, 'position');
            if (IS_NON_STATIC_POSITION_RE.test(elementPosition)) rootHasParentWithNonStaticPosition = true;
            parent = parent.parentNode;
        }
        if (rootHasParentWithNonStaticPosition) {
            var rootOffset = (0, _position.getOffsetPosition)(this.root);
            if (rootOffset.left !== 0 || rootOffset.top !== 0) {
                var currentRootLeft = parseFloat((0, _style.get)(this.root, 'left')) || 0;
                var currentRootTop = parseFloat((0, _style.get)(this.root, 'top')) || 0;
                var newRootLeft = currentRootLeft - rootOffset.left + 'px';
                var newRootTop = currentRootTop - rootOffset.top + 'px';
                (0, _style.set)(this.root, 'left', newRootLeft);
                (0, _style.set)(this.root, 'top', newRootTop);
            }
        }
    };

    ShadowUI.prototype._overrideDocumentMethods = function _overrideDocumentMethods(window, document) {
        var shadowUI = this;
        var docProto = window.Document.prototype;
        docProto.elementFromPoint = function () {
            // NOTE: T212974
            shadowUI.addClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);

            for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
                args[_key5] = arguments[_key5];
            }

            var res = ShadowUI._filterElement(_nativeMethods2.default.elementFromPoint.apply(this, args));
            shadowUI.removeClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);
            return res;
        };
        if (document.caretRangeFromPoint) {
            docProto.caretRangeFromPoint = function () {
                shadowUI.addClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);

                for (var _len6 = arguments.length, args = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
                    args[_key6] = arguments[_key6];
                }

                var res = _nativeMethods2.default.caretRangeFromPoint.apply(this, args);
                if (res && res.startContainer && !ShadowUI._filterElement(res.startContainer)) res = null;
                shadowUI.removeClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);
                return res;
            };
        }
        if (document.caretPositionFromPoint) {
            docProto.caretPositionFromPoint = function () {
                shadowUI.addClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);

                for (var _len7 = arguments.length, args = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
                    args[_key7] = arguments[_key7];
                }

                var res = _nativeMethods2.default.caretPositionFromPoint.apply(this, args);
                if (res && res.offsetNode && !ShadowUI._filterElement(res.offsetNode)) res = null;
                shadowUI.removeClass(shadowUI.getRoot(), shadowUI.HIDDEN_CLASS);
                return res;
            };
        }
        docProto.getElementById = function () {
            for (var _len8 = arguments.length, args = Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
                args[_key8] = arguments[_key8];
            }

            return ShadowUI._filterElement(_nativeMethods2.default.getElementById.apply(this, args));
        };
        docProto.getElementsByName = function () {
            for (var _len9 = arguments.length, args = Array(_len9), _key9 = 0; _key9 < _len9; _key9++) {
                args[_key9] = arguments[_key9];
            }

            var elements = _nativeMethods2.default.getElementsByName.apply(this, args);
            var length = _featureDetection.getElementsByNameReturnsHTMLCollection ? _nativeMethods2.default.htmlCollectionLengthGetter.call(elements) : _nativeMethods2.default.nodeListLengthGetter.call(elements);
            return shadowUI._filterNodeList(elements, length);
        };
        docProto.getElementsByClassName = this.wrapperCreators.getElementsByClassName('getElementsByClassName');
        docProto.getElementsByTagName = this.wrapperCreators.getElementsByTagName('getElementsByTagName');
        docProto.querySelector = this.wrapperCreators.querySelector('querySelector', 'querySelectorAll');
        docProto.querySelectorAll = this.wrapperCreators.querySelectorAll('querySelectorAll');
        // NOTE: T195358
        docProto.querySelectorAll.toString = function () {
            return _nativeMethods2.default.querySelectorAll.toString();
        };
        docProto.getElementsByClassName.toString = function () {
            return _nativeMethods2.default.getElementsByClassName.toString();
        };
    };

    ShadowUI.prototype._overrideElementMethods = function _overrideElementMethods(window) {
        var bodyProto = window.HTMLBodyElement.prototype;
        var headProto = window.HTMLHeadElement.prototype;
        bodyProto.getElementsByClassName = this.wrapperCreators.getElementsByClassName('elementGetElementsByClassName');
        bodyProto.getElementsByTagName = this.wrapperCreators.getElementsByTagName('elementGetElementsByTagName');
        bodyProto.querySelector = this.wrapperCreators.querySelector('elementQuerySelector', 'elementQuerySelectorAll');
        bodyProto.querySelectorAll = this.wrapperCreators.querySelectorAll('elementQuerySelectorAll');
        headProto.getElementsByClassName = bodyProto.getElementsByClassName;
        headProto.getElementsByTagName = bodyProto.getElementsByTagName;
        headProto.querySelector = bodyProto.querySelector;
        headProto.querySelectorAll = bodyProto.querySelectorAll;
    };

    ShadowUI.prototype._getUIStyleSheetsHtml = function _getUIStyleSheetsHtml() {
        var stylesSelector = 'link.' + _className2.default.uiStylesheet;
        var stylesheets = this.nativeMethods.querySelectorAll.call(this.document, stylesSelector);
        var length = this.nativeMethods.nodeListLengthGetter.call(stylesheets);
        var result = '';
        for (var i = 0; i < length; i++) {
            result += _nativeMethods2.default.elementOuterHTMLGetter.call(stylesheets[i]);
        }return result;
    };

    ShadowUI.prototype._restoreUIStyleSheets = function _restoreUIStyleSheets(head, uiStyleSheetsHtml) {
        if (!head || !uiStyleSheetsHtml) return;
        var parser = this.nativeMethods.createElement.call(this.document, 'div');
        _nativeMethods2.default.elementInnerHTMLSetter.call(parser, uiStyleSheetsHtml);
        var length = _nativeMethods2.default.htmlCollectionLengthGetter.call(parser.children);
        for (var i = 0; i < length; i++) {
            var refNode = head.children[i] || null;
            var newNode = _nativeMethods2.default.cloneNode.call(parser.children[i]);
            ShadowUI._markElementAsShadow(newNode);
            this.nativeMethods.insertBefore.call(head, newNode, refNode);
        }
    };

    ShadowUI.prototype._markScriptsAndStylesAsShadowInHead = function _markScriptsAndStylesAsShadowInHead(head) {
        // NOTE: document.head equals null after call 'document.open' function
        if (!head) return;
        var length = _nativeMethods2.default.htmlCollectionLengthGetter.call(head.children);
        for (var i = 0; i < length; i++) {
            var headChild = head.children[i];
            if (ShadowUI.containsShadowUIClassPostfix(headChild)) ShadowUI._markElementAsShadow(headChild);
        }
    };

    ShadowUI.prototype.getRoot = function getRoot() {
        var _this3 = this;

        if (!this.root || /* NOTE: T225944 */!this.document.body.contains(this.root)) {
            if (!this.root) {
                // NOTE: B254893
                this.root = _nativeMethods2.default.createElement.call(this.document, 'div');
                _nativeMethods2.default.setAttribute.call(this.root, 'id', ShadowUI.patchId(this.ROOT_ID));
                _nativeMethods2.default.setAttribute.call(this.root, 'contenteditable', 'false');
                this.addClass(this.root, this.ROOT_CLASS);
                ShadowUI._markElementAsShadow(this.root);
                _nativeMethods2.default.appendChild.call(this.document.body, this.root);

                for (var _i2 = 0, _DomProcessor$EVENTS2 = _domProcessor2.default.EVENTS; _i2 < _DomProcessor$EVENTS2.length; _i2++) {
                    var event = _DomProcessor$EVENTS2[_i2];
                    this.root.addEventListener(event, _event.stopPropagation);
                }

                this._bringRootToWindowTopLeft();
                _nativeMethods2.default.documentAddEventListener.call(this.document, 'DOMContentLoaded', function () {
                    _this3.onBodyElementMutation();
                    _this3._bringRootToWindowTopLeft();
                });
            } else _nativeMethods2.default.appendChild.call(this.document.body, this.root);
        }
        return this.root;
    };

    ShadowUI.prototype.attach = function attach(window) {
        _SandboxBase.prototype.attach.call(this, window, window.document);
        this.markShadowUIContainers(this.document.head, this.document.body);
        this._overrideDocumentMethods(window, window.document);
        this._overrideElementMethods(window);
        this._markScriptsAndStylesAsShadowInHead(window.document.head);
        this._initEvents();
    };

    ShadowUI.prototype._initEvents = function _initEvents() {
        this.iframeSandbox.on(this.iframeSandbox.RUN_TASK_SCRIPT_EVENT, this.runTaskScriptEventCallback);
        this.nodeMutation.on(this.nodeMutation.BEFORE_DOCUMENT_CLEANED_EVENT, this.beforeDocumentCleanedEventCallback);
        this.nodeMutation.on(this.nodeMutation.DOCUMENT_CLEANED_EVENT, this.documentCleanedEventCallback);
        this.nodeMutation.on(this.nodeMutation.DOCUMENT_CLOSED_EVENT, this.documentClosedEventCallback);
        this.nodeMutation.on(this.nodeMutation.BODY_CONTENT_CHANGED_EVENT, this.bodyContentChangedEventCallback);
        this.messageSandbox.on(this.messageSandbox.SERVICE_MSG_RECEIVED_EVENT, this.serviceMsgReceivedEventCallback);
        this.nodeMutation.on(this.nodeMutation.BODY_CREATED_EVENT, this.bodyCreatedEventCallback);
    };

    ShadowUI.prototype.onBodyElementMutation = function onBodyElementMutation() {
        if (!this.root || !this.document.body) return;
        var isRootInDom = domUtils.closest(this.root, 'html');
        var isRootLastChild = !this.nativeMethods.elementNextElementSiblingGetter.call(this.root);
        // NOTE: Fix for B239138 - The 'Cannot read property 'document' of null' error
        // is thrown on recording on the unroll.me site. There was an issue when
        // document.body was replaced, so we need to reattach a UI to a new body manually.
        var isRootInBody = this.root.parentNode === this.document.body;
        if (!(isRootInDom && isRootLastChild && isRootInBody)) this.nativeMethods.appendChild.call(this.document.body, this.root);
        this.markShadowUIContainers(this.document.head, this.document.body);
    };
    // Accessors


    ShadowUI.prototype.getFirstChild = function getFirstChild(el) {
        var length = _nativeMethods2.default.nodeListLengthGetter.call(el.childNodes);
        var filteredNodes = this._filterNodeList(el.childNodes, length);
        return filteredNodes[0] || null;
    };

    ShadowUI.prototype.getFirstElementChild = function getFirstElementChild(el) {
        var length = _nativeMethods2.default.htmlCollectionLengthGetter.call(el.children);
        var filteredNodes = this._filterNodeList(el.children, length);
        return filteredNodes[0] || null;
    };

    ShadowUI.prototype.getLastChild = function getLastChild(el) {
        var length = _nativeMethods2.default.nodeListLengthGetter.call(el.childNodes);
        var filteredNodes = this._filterNodeList(el.childNodes, length);
        var index = el.childNodes === filteredNodes ? length - 1 : filteredNodes.length - 1;
        return index >= 0 ? filteredNodes[index] : null;
    };

    ShadowUI.prototype.getLastElementChild = function getLastElementChild(el) {
        var length = _nativeMethods2.default.htmlCollectionLengthGetter.call(el.children);
        var filteredNodes = this._filterNodeList(el.children, length);
        var index = el.children === filteredNodes ? length - 1 : filteredNodes.length - 1;
        return index >= 0 ? filteredNodes[index] : null;
    };

    ShadowUI.prototype.getNextSibling = function getNextSibling(el) {
        if (!el) return el;
        do {
            el = _nativeMethods2.default.nodeNextSiblingGetter.call(el);
        } while (el && domUtils.isShadowUIElement(el));
        return el;
    };

    ShadowUI.prototype.getPrevSibling = function getPrevSibling(el) {
        if (!el) return el;
        do {
            el = _nativeMethods2.default.nodePrevSiblingGetter.call(el);
        } while (el && domUtils.isShadowUIElement(el));
        return el;
    };

    ShadowUI.prototype.getNextElementSibling = function getNextElementSibling(el) {
        do {
            el = _nativeMethods2.default.elementNextElementSiblingGetter.call(el);
        } while (el && domUtils.isShadowUIElement(el));
        return el;
    };

    ShadowUI.prototype.getPrevElementSibling = function getPrevElementSibling(el) {
        do {
            el = _nativeMethods2.default.elementPrevElementSiblingGetter.call(el);
        } while (el && domUtils.isShadowUIElement(el));
        return el;
    };
    // Utils


    ShadowUI._checkElementsPosition = function _checkElementsPosition(collection, length) {
        if (!length) return;
        var shadowUIElements = [];
        for (var i = 0; i < length; i++) {
            var item = collection[i];
            if (domUtils.isShadowUIElement(item)) shadowUIElements.push(item);
        }
        var collectionOwner = shadowUIElements.length && shadowUIElements[0].parentNode;

        for (var _i4 = 0; _i4 < shadowUIElements.length; _i4++) {
            var shadowUIElement = shadowUIElements[_i4];
            _nativeMethods2.default.appendChild.call(collectionOwner, shadowUIElement);
        }
    };

    ShadowUI._hasFlag = function _hasFlag(obj, flag) {
        try {
            return !!obj[flag];
        } catch (e) {
            return false;
        }
    };

    ShadowUI.isShadowContainer = function isShadowContainer(el) {
        return ShadowUI._hasFlag(el, IS_SHADOW_CONTAINER_FLAG);
    };

    ShadowUI.isShadowContainerCollection = function isShadowContainerCollection(collection) {
        return ShadowUI._hasFlag(collection, IS_SHADOW_CONTAINER_COLLECTION_FLAG);
    };

    ShadowUI._isShadowUIChildListMutation = function _isShadowUIChildListMutation(mutation) {
        if (domUtils.isShadowUIElement(mutation.target)) return true;
        var removedNodesLength = _nativeMethods2.default.nodeListLengthGetter.call(mutation.removedNodes);
        for (var i = 0; i < removedNodesLength; i++) {
            if (domUtils.isShadowUIElement(mutation.removedNodes[i])) return true;
        }
        var addedNodesLength = _nativeMethods2.default.nodeListLengthGetter.call(mutation.addedNodes);
        for (var _i5 = 0; _i5 < addedNodesLength; _i5++) {
            if (domUtils.isShadowUIElement(mutation.addedNodes[_i5])) return true;
        }
        return false;
    };

    ShadowUI._isShadowUIAttributeMutation = function _isShadowUIAttributeMutation(mutation) {
        return domUtils.isShadowUIElement(mutation.target) || domUtils.isHammerheadAttr(mutation.attributeName);
    };

    ShadowUI._isShadowUICharacterDataMutation = function _isShadowUICharacterDataMutation(mutation) {
        return domUtils.isShadowUIElement(mutation.target);
    };

    ShadowUI.isShadowUIMutation = function isShadowUIMutation(mutation) {
        switch (mutation.type) {
            case 'childList':
                return ShadowUI._isShadowUIChildListMutation(mutation);
            case 'attributes':
                return ShadowUI._isShadowUIAttributeMutation(mutation);
            case 'characterData':
                return ShadowUI._isShadowUICharacterDataMutation(mutation);
            default:
                return false;
        }
    };

    ShadowUI.removeSelfRemovingScripts = function removeSelfRemovingScripts(document) {
        var selfRemovingScripts = _nativeMethods2.default.querySelectorAll.call(document, '.' + _className2.default.selfRemovingScript);
        var length = _nativeMethods2.default.nodeListLengthGetter.call(selfRemovingScripts);
        for (var i = 0; i < length; i++) {
            _nativeMethods2.default.removeChild.call(selfRemovingScripts[i].parentNode, selfRemovingScripts[i]);
        }
    };
    // API


    ShadowUI.prototype.getShadowUICollectionLength = function getShadowUICollectionLength(collection, length) {
        var shadowUIElementCount = 0;
        for (var i = 0; i < length; i++) {
            if (domUtils.isShadowUIElement(collection[i])) shadowUIElementCount++;
        }
        if (shadowUIElementCount && !this.ieDebugSandbox.isDebuggerInitiator()) ShadowUI._checkElementsPosition(collection, length);
        return length - shadowUIElementCount;
    };
    // NOTE: this method cannot be static because it is a part of the public API


    ShadowUI.prototype.addClass = function addClass(el, value) {
        var patchedClass = ShadowUI.patchClassNames(value);
        domUtils.addClass(el, patchedClass);
    };
    // NOTE: this method cannot be static because it is a part of the public API


    ShadowUI.prototype.removeClass = function removeClass(elem, value) {
        var patchedClass = ShadowUI.patchClassNames(value);
        domUtils.removeClass(elem, patchedClass);
    };

    ShadowUI.hasClass = function hasClass(el, value) {
        var patchedClass = ShadowUI.patchClassNames(value);
        return domUtils.hasClass(el, patchedClass);
    };

    ShadowUI.patchId = function patchId(value) {
        return value + _className2.default.postfix;
    };

    ShadowUI.patchClassNames = function patchClassNames(value) {
        var names = value.split(/\s+/);
        for (var i = 0; i < names.length; i++) {
            names[i] += _className2.default.postfix;
        }return names.join(' ');
    };

    ShadowUI.prototype.select = function select(selector, context) {
        var patchedSelector = selector.replace(CLASSNAME_RE, function (className) {
            return className + _className2.default.postfix;
        });
        return context ? _nativeMethods2.default.elementQuerySelectorAll.call(context, patchedSelector) : _nativeMethods2.default.querySelectorAll.call(this.document, patchedSelector);
    };

    ShadowUI.prototype.setBlind = function setBlind(value) {
        if (value) this.addClass(this.getRoot(), this.BLIND_CLASS);else this.removeClass(this.getRoot(), this.BLIND_CLASS);
    };

    ShadowUI.prototype.getLastActiveElement = function getLastActiveElement() {
        return this.lastActiveElement;
    };

    ShadowUI.prototype.setLastActiveElement = function setLastActiveElement(el) {
        this.lastActiveElement = el;
    };

    ShadowUI.prototype.insertBeforeRoot = function insertBeforeRoot(el) {
        var rootParent = this.getRoot().parentNode;
        var lastParentChild = this.nativeMethods.nodeLastChildGetter.call(rootParent);
        return _nativeMethods2.default.insertBefore.call(rootParent, el, lastParentChild);
    };

    ShadowUI._markElementAsShadow = function _markElementAsShadow(el) {
        el[_internalProperties2.default.shadowUIElement] = true;
    };

    ShadowUI.markElementAndChildrenAsShadow = function markElementAndChildrenAsShadow(el) {
        ShadowUI._markElementAsShadow(el);
        // NOTE: For Text, Comment and ProcessingInstruction nodes
        if (!el.querySelectorAll) return;
        var childElements = (0, _querySelector.getNativeQuerySelectorAll)(el).call(el, '*');
        var length = _nativeMethods2.default.nodeListLengthGetter.call(childElements);
        for (var i = 0; i < length; i++) {
            ShadowUI._markElementAsShadow(childElements[i]);
        }
    };

    ShadowUI._markAsShadowContainer = function _markAsShadowContainer(container) {
        _nativeMethods2.default.objectDefineProperty(container, IS_SHADOW_CONTAINER_FLAG, { value: true });
    };

    ShadowUI.markAsShadowContainerCollection = function markAsShadowContainerCollection(collection) {
        _nativeMethods2.default.objectDefineProperty(collection, IS_SHADOW_CONTAINER_COLLECTION_FLAG, { value: true });
    };

    ShadowUI.containsShadowUIClassPostfix = function containsShadowUIClassPostfix(element) {
        return typeof element.className === 'string' && element.className.indexOf(_className2.default.postfix) !== -1;
    };

    return ShadowUI;
}(_base2.default);

exports.default = ShadowUI;
module.exports = exports['default'];