'use strict';

exports.__esModule = true;

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _wrapper = require('./wrapper');

var _wrapper2 = _interopRequireDefault(_wrapper);

var _settings = require('../../settings');

var _settings2 = _interopRequireDefault(_settings);

var _destinationLocation = require('../../utils/destination-location');

var destLocation = _interopRequireWildcard(_destinationLocation);

var _dom = require('../../utils/dom');

var _getStorageKey = require('../../../utils/get-storage-key');

var _getStorageKey2 = _interopRequireDefault(_getStorageKey);

var _internalProperties = require('../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _jsonHammerhead = require('json-hammerhead');

var JSON = _interopRequireWildcard(_jsonHammerhead);

var _propertyOverriding = require('../../utils/property-overriding');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var StorageSandbox = function (_SandboxBase) {
    _inherits(StorageSandbox, _SandboxBase);

    function StorageSandbox(listeners, unloadSandbox, eventSimulator) {
        _classCallCheck(this, StorageSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.localStorageWrapper = null;
        _this.sessionStorageWrapper = null;
        _this.listeners = listeners;
        _this.unloadSandbox = unloadSandbox;
        _this.eventSimulator = eventSimulator;
        _this.storages = {};
        _this.isLocked = false;
        return _this;
    }

    StorageSandbox.prototype._simulateStorageEventIfNecessary = function _simulateStorageEventIfNecessary(event, storageArea) {
        if (this.isDeactivated()) return;
        if (storageArea && storageArea.getContext() !== this.window) {
            event.storageArea = storageArea;
            this.eventSimulator.storage(this.window, event);
        }
    };
    // NOTE: We are using a single storage wrapper instance for all same-domain windows.
    // This wrapper is saved in the top same-domain window's sandbox.


    StorageSandbox.prototype._createStorageWrappers = function _createStorageWrappers() {
        var _this2 = this;

        // eslint-disable-next-line no-restricted-properties
        var host = destLocation.getParsed().host;
        var sessionId = _settings2.default.get().sessionId;
        var storageKey = (0, _getStorageKey2.default)(sessionId, host);
        var topSameDomainWindow = (0, _dom.getTopSameDomainWindow)(this.window);
        var topSameDomainStorages = topSameDomainWindow[_internalProperties2.default.hammerhead].sandbox.storageSandbox.storages;
        // NOTE: Use the already created wrappers.
        if (topSameDomainStorages[storageKey]) {
            this.localStorageWrapper = topSameDomainStorages[storageKey].localStorageWrapper;
            this.sessionStorageWrapper = topSameDomainStorages[storageKey].sessionStorageWrapper;
        }
        // NOTE: Or create new.
        else {
                // @ts-ignore
                this.localStorageWrapper = new _wrapper2.default(this.window, this.nativeMethods.winLocalStorageGetter.call(this.window), storageKey);
                // @ts-ignore
                this.sessionStorageWrapper = new _wrapper2.default(this.window, this.nativeMethods.winSessionStorageGetter.call(this.window), storageKey);
                this.unloadSandbox.on(this.unloadSandbox.BEFORE_UNLOAD_EVENT, function () {
                    if (!_this2.isLocked) {
                        _this2.localStorageWrapper.saveToNativeStorage();
                        _this2.sessionStorageWrapper.saveToNativeStorage();
                    }
                });
                // NOTE: Push to the top same-domain sandbox.
                topSameDomainStorages[storageKey] = {
                    localStorageWrapper: this.localStorageWrapper,
                    sessionStorageWrapper: this.sessionStorageWrapper
                };
            }
    };

    StorageSandbox.prototype._overrideStorageEvent = function _overrideStorageEvent() {
        var _this3 = this;

        // @ts-ignore
        this.window.StorageEvent = function (type, opts) {
            if (arguments.length === 0) throw new TypeError();
            var storedArea = opts.storageArea || null;
            if (storedArea) delete opts.storageArea;
            var event = new this.nativeMethods.StorageEvent(type, opts);
            if (storedArea) {
                this.nativeMethods.objectDefineProperty(event, 'storageArea', {
                    get: function get() {
                        return storedArea;
                    },
                    set: function set() {
                        return void 0;
                    }
                });
            }
            return event;
        };
        // @ts-ignore
        window.StorageEvent.toString = function () {
            return _this3.nativeMethods.StorageEvent.toString();
        };
    };

    StorageSandbox.prototype.clear = function clear() {
        this.nativeMethods.winLocalStorageGetter.call(this.window).removeItem(this.localStorageWrapper.nativeStorageKey);
        this.nativeMethods.winSessionStorageGetter.call(this.window).removeItem(this.sessionStorageWrapper.nativeStorageKey);
    };

    StorageSandbox.prototype.lock = function lock() {
        this.isLocked = true;
    };

    StorageSandbox.prototype.backup = function backup() {
        return {
            localStorage: JSON.stringify(this.localStorageWrapper.getCurrentState()),
            sessionStorage: JSON.stringify(this.sessionStorageWrapper.getCurrentState())
        };
    };

    StorageSandbox.prototype.restore = function restore(_ref) {
        var localStorage = _ref.localStorage,
            sessionStorage = _ref.sessionStorage;

        this.localStorageWrapper.restore(localStorage);
        this.sessionStorageWrapper.restore(sessionStorage);
    };

    StorageSandbox.prototype.attach = function attach(window) {
        var _this4 = this;

        _SandboxBase.prototype.attach.call(this, window);
        this._createStorageWrappers();
        this.onLocalStorageChangeListener = this.localStorageWrapper.on(this.localStorageWrapper.STORAGE_CHANGED_EVENT, function (e) {
            return _this4._simulateStorageEventIfNecessary(e, _this4.localStorageWrapper);
        });
        this.onSessionStorageListener = this.sessionStorageWrapper.on(this.sessionStorageWrapper.STORAGE_CHANGED_EVENT, function (e) {
            return _this4._simulateStorageEventIfNecessary(e, _this4.sessionStorageWrapper);
        });
        this.listeners.initElementListening(window, ['storage']);
        this.listeners.addInternalEventListener(window, ['storage'], function (_e, dispatched, preventEvent) {
            if (!dispatched) preventEvent();
        });
        this._overrideStorageEvent();
        var storagesPropsOwner = this.nativeMethods.getStoragesPropsOwner(window);
        // NOTE: Storage properties is located in Window.prototype in the IE11 and these are non configurable.
        // We define descriptors from a prototype with an overridden getter on a window instance.
        // We don't need define descriptors again if these was overridden.
        var shouldDefineStorageProps = !this.nativeMethods.isStoragePropsLocatedInProto || !this.nativeMethods.objectHasOwnProperty.call(window, 'localStorage');
        if (shouldDefineStorageProps) {
            this.nativeMethods.objectDefineProperties(window, {
                'localStorage': (0, _propertyOverriding.createOverriddenDescriptor)(storagesPropsOwner, 'localStorage', {
                    getter: function getter() {
                        _this4.localStorageWrapper.setContext(window);
                        return _this4.localStorageWrapper;
                    }
                }),
                'sessionStorage': (0, _propertyOverriding.createOverriddenDescriptor)(storagesPropsOwner, 'sessionStorage', {
                    getter: function getter() {
                        _this4.sessionStorageWrapper.setContext(window);
                        return _this4.sessionStorageWrapper;
                    }
                })
            });
        }
    };

    StorageSandbox.prototype.dispose = function dispose() {
        this.localStorageWrapper.off(this.localStorageWrapper.STORAGE_CHANGED_EVENT, this.onLocalStorageChangeListener);
        this.sessionStorageWrapper.off(this.sessionStorageWrapper.STORAGE_CHANGED_EVENT, this.onSessionStorageListener);
        var topSameDomainWindow = (0, _dom.getTopSameDomainWindow)(this.window);
        // NOTE: For removed iframe without src in IE11 window.top equals iframe's window
        if (this.window === topSameDomainWindow && !topSameDomainWindow.frameElement) {
            this.localStorageWrapper.dispose();
            this.sessionStorageWrapper.dispose();
        }
    };

    return StorageSandbox;
}(_base2.default);

exports.default = StorageSandbox;
module.exports = exports['default'];