'use strict';

exports.__esModule = true;

var _eventEmitter = require('../../utils/event-emitter');

var _eventEmitter2 = _interopRequireDefault(_eventEmitter);

var _browser = require('../../utils/browser');

var _url = require('../../utils/url');

var _destinationLocation = require('../../utils/destination-location');

var destLocation = _interopRequireWildcard(_destinationLocation);

var _jsonHammerhead = require('json-hammerhead');

var JSON = _interopRequireWildcard(_jsonHammerhead);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var STORAGES_SANDBOX_TEMP = 'hammerhead|storages-sandbox-temp';
var API_KEY_PREFIX = 'hammerhead|api-key-prefix|';
var KEY = 0;
var VALUE = 1;
function getWrapperMethods() {
    var methods = [];
    for (var key in StorageWrapper.prototype) {
        methods.push(key);
    }return methods;
}

var StorageWrapper = function StorageWrapper(window, nativeStorage, nativeStorageKey) {
    var _this = this,
        _arguments = arguments;

    _classCallCheck(this, StorageWrapper);

    this.STORAGE_CHANGED_EVENT = 'hammerhead|event|storage-changed';
    this.eventEmitter = new _eventEmitter2.default();
    this.on = function (ev, handler) {
        return _this.eventEmitter.on(ev, handler);
    };
    this.off = function (ev, handler) {
        return _this.eventEmitter.off(ev, handler);
    };
    this.emit = function (ev, e) {
        return _this.eventEmitter.emit(ev, e);
    };
    this.nativeStorage = nativeStorage;
    this.nativeStorageKey = nativeStorageKey;
    this.lastState = null;
    this.window = window;
    this.initialProperties = [];
    this.wrapperMethods = [];
    this.context = window;
    this.intervalId = null;
    this.EMPTY_OLD_VALUE_ARG = _browser.isIE ? '' : null;
    var getAddedProperties = function getAddedProperties() {
        // NOTE: The standard doesn't regulate the order in which properties are enumerated.
        // But we rely on the fact that they are enumerated in the order they were created in all the supported browsers.
        // In this case we cannot use Object.getOwnPropertyNames
        // because the enumeration order in Android 6.0 is different from all other browsers.
        var properties = [];
        for (var property in _this) {
            if (_nativeMethods2.default.objectHasOwnProperty.call(_this, property) && _this.initialProperties.indexOf(property) === -1) properties.push(property);
        }
        return properties;
    };
    _nativeMethods2.default.objectDefineProperty(this, 'length', {
        get: function get() {
            return getAddedProperties().length;
        },
        set: function set() {
            return void 0;
        }
    });
    var loadStorage = function loadStorage(storage) {
        if (!storage) storage = _this.nativeStorage[_this.nativeStorageKey];
        storage = JSON.parse(storage || '[[],[]]');
        for (var i = 0; i < storage[KEY].length; i++) {
            _this[storage[KEY][i]] = storage[VALUE][i];
        }
    };
    var raiseStorageChanged = function raiseStorageChanged(key, oldValue, newValue) {
        var url = null;
        try {
            var parsedContextUrl = (0, _url.parseProxyUrl)(_this.context.location.toString());
            url = parsedContextUrl ? parsedContextUrl.destUrl : destLocation.get();
        } catch (e) {
            _this.context = _this.window;
            url = destLocation.get();
        }
        _this.emit(_this.STORAGE_CHANGED_EVENT, { key: key, oldValue: oldValue, newValue: newValue, url: url });
    };
    var checkStorageChanged = function checkStorageChanged() {
        var currentState = _this.getCurrentState();
        for (var i = 0; i < _this.lastState[KEY].length; i++) {
            var lastStateKey = _this.lastState[KEY][i];
            var lastStateValue = _this.lastState[VALUE][i];
            var keyIndex = currentState[KEY].indexOf(lastStateKey);
            if (keyIndex !== -1) {
                if (currentState[VALUE][keyIndex] !== lastStateValue) raiseStorageChanged(currentState[KEY][keyIndex], lastStateValue, currentState[VALUE][keyIndex]);
                currentState[KEY].splice(keyIndex, 1);
                currentState[VALUE].splice(keyIndex, 1);
            } else raiseStorageChanged(lastStateKey, lastStateValue, null);
        }
        for (var j = 0; j < currentState[KEY].length; j++) {
            raiseStorageChanged(currentState[KEY][j], _this.EMPTY_OLD_VALUE_ARG, currentState[VALUE][j]);
        }_this.lastState = _this.getCurrentState();
    };
    var clearStorage = function clearStorage() {
        var addedProperties = getAddedProperties();
        var changed = false;

        for (var _i2 = 0; _i2 < addedProperties.length; _i2++) {
            var addedProperty = addedProperties[_i2];
            delete _this[addedProperty];
            changed = true;
        }

        return changed;
    };
    var init = function init() {
        loadStorage();
        _this.lastState = _this.getCurrentState();
        _this.intervalId = _nativeMethods2.default.setInterval.call(_this.window, function () {
            return checkStorageChanged();
        }, 10);
    };
    this.setContext = function (context) {
        _this.context = context;
    };
    this.getContext = function () {
        return _this.context;
    };
    this.saveToNativeStorage = function () {
        var state = JSON.stringify(_this.getCurrentState());
        if (_this.nativeStorage[_this.nativeStorageKey] !== state) _this.nativeStorage[_this.nativeStorageKey] = state;
    };
    this.getCurrentState = function () {
        var addedProperties = getAddedProperties();
        var state = [[], []];

        for (var _i4 = 0; _i4 < addedProperties.length; _i4++) {
            var addedProperty = addedProperties[_i4];
            state[KEY].push(addedProperty);
            state[VALUE].push(_this[addedProperty]);
        }

        return state;
    };
    this.restore = function (storage) {
        clearStorage();
        loadStorage(storage);
        _this.lastState = _this.getCurrentState();
    };
    this.dispose = function () {
        _nativeMethods2.default.clearInterval.call(_this.window, _this.intervalId);
    };
    var castToString = function castToString(value) {
        // NOTE: The browser automatically translates the key and the value to a string. To repeat this behavior,
        // we use native storage:
        // localStorage.setItem(null, null) equivalently to localStorage.setItem('null', 'null')
        _this.nativeStorage[STORAGES_SANDBOX_TEMP] = value;
        return _this.nativeStorage[STORAGES_SANDBOX_TEMP];
    };
    var getValidKey = function getValidKey(key) {
        var isWrapperMember = _this.wrapperMethods.indexOf(key) !== -1 || _this.initialProperties.indexOf(key) !== -1;
        key = isWrapperMember ? API_KEY_PREFIX + key : key;
        return castToString(key);
    };
    // API
    this.clear = function () {
        if (clearStorage()) {
            raiseStorageChanged(null, null, null);
            _this.lastState = _this.getCurrentState();
        }
    };
    this.getItem = function (key) {
        if (_arguments.length === 0) throw new TypeError();
        key = getValidKey(key);
        return _nativeMethods2.default.objectHasOwnProperty.call(_this, key) ? _this[key] : null;
    };
    this.key = function (keyNum) {
        if (keyNum === void 0) throw new TypeError();
        // NOTE: http://w3c-test.org/webstorage/storage_key.html
        keyNum %= 0x100000000;
        var addedProperties = getAddedProperties();
        return keyNum >= 0 && keyNum < addedProperties.length ? addedProperties[keyNum] : null;
    };
    this.removeItem = function (key) {
        if (_arguments.length === 0) throw new TypeError();
        key = getValidKey(key);
        delete _this[key];
        checkStorageChanged();
    };
    this.setItem = function (key, value) {
        if (_arguments.length < 2) throw new TypeError();
        key = getValidKey(key);
        value = castToString(value);
        _this[key] = value;
        checkStorageChanged();
    };
    // NOTE: Save wrapper properties and methods to be able to distinguish them from
    // properties that will be created from the outside.
    this.initialProperties = _nativeMethods2.default.objectGetOwnPropertyNames.call(window.Object, this);
    this.wrapperMethods = getWrapperMethods();
    init();
};
// @ts-ignore


exports.default = StorageWrapper;
StorageWrapper.prototype = Storage.prototype;
module.exports = exports['default'];