'use strict';

exports.__esModule = true;

var _base = require('./base');

var _base2 = _interopRequireDefault(_base);

var _propertyOverriding = require('./../utils/property-overriding');

var _style = require('./../../processing/style');

var _style2 = _interopRequireDefault(_style);

var _url = require('./../utils/url');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CSS_STYLE_IS_PROCESSED = 'hammerhead|style|is-processed';
var CSS_STYLE_PROXY_OBJECT = 'hammerhead|style|proxy-object';
var CSS_STYLE_PROXY_TARGET = 'hammerhead|style|proxy-target';

var StyleSandbox = function (_SandboxBase) {
    _inherits(StyleSandbox, _SandboxBase);

    function StyleSandbox() {
        _classCallCheck(this, StyleSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.URL_PROPS = ['background', 'backgroundImage', 'borderImage', 'borderImageSource', 'listStyle', 'listStyleImage', 'cursor'];
        _this.DASHED_URL_PROPS = StyleSandbox._generateDashedProps(_this.URL_PROPS);
        _this.FEATURES = _this._detectBrowserFeatures();
        return _this;
    }

    StyleSandbox._convertToDashed = function _convertToDashed(prop) {
        return prop.replace(/[A-Z]/g, '-$&').toLowerCase();
    };

    StyleSandbox._generateDashedProps = function _generateDashedProps(props) {
        var dashedProps = [];

        for (var _i2 = 0; _i2 < props.length; _i2++) {
            var prop = props[_i2];
            var dashedProp = StyleSandbox._convertToDashed(prop);
            if (prop !== dashedProp) dashedProps.push(dashedProp);
        }

        return dashedProps;
    };

    StyleSandbox.prototype._detectBrowserFeatures = function _detectBrowserFeatures() {
        var features = {};
        // NOTE: The CSS2Properties class is supported only in the Firefox
        // and its prototype contains all property descriptors
        // @ts-ignore
        features.protoContainsAllProps = !!window.CSS2Properties;
        // NOTE: The CSSStyleDeclaration class contains not dashed url properties only in the IE
        features.protoContainsUrlProps = this.nativeMethods.objectHasOwnProperty
        // @ts-ignore
        .call(window.CSSStyleDeclaration.prototype, 'background');
        if (!features.protoContainsAllProps && !features.protoContainsUrlProps) {
            var testDiv = this.nativeMethods.createElement.call(document, 'div');
            var propertySetterIsCalled = false;
            var testDivDescriptor = this.nativeMethods.objectGetOwnPropertyDescriptor
            // @ts-ignore
            .call(window.Object, testDiv.style, 'background');
            if (testDivDescriptor.configurable) {
                // eslint-disable-next-line no-restricted-properties
                delete testDivDescriptor.value;
                delete testDivDescriptor.writable;
                testDivDescriptor.set = function () {
                    propertySetterIsCalled = true;
                };
                this.nativeMethods.objectDefineProperty(testDiv.style, 'background', testDivDescriptor);
                testDiv.style.background = 'url';
            }
            // NOTE: A style instance contains all url properties.
            // They are non-configurable in Safari less than 11.1.
            // Their setter cannot be called in Safari 11.1.
            features.propsCannotBeOverridden = !testDivDescriptor.configurable || !propertySetterIsCalled;
        }
        return features;
    };

    StyleSandbox.prototype._overrideStyleProp = function _overrideStyleProp(proto, prop) {
        var nativeMethods = this.nativeMethods;
        var dashedProp = StyleSandbox._convertToDashed(prop);
        (0, _propertyOverriding.overrideDescriptor)(proto, prop, {
            getter: function getter() {
                var value = nativeMethods.styleGetPropertyValue.call(this, dashedProp);
                return _style2.default.cleanUp(value, _url.parseProxyUrl);
            },
            setter: function setter(value) {
                if (typeof value === 'string') value = _style2.default.process(value, _url.getProxyUrl);
                nativeMethods.styleSetProperty.call(this, dashedProp, value);
            }
        });
    };

    StyleSandbox.prototype._overrideStyleInstanceProp = function _overrideStyleInstanceProp(style, prop) {
        var nativeMethods = this.nativeMethods;
        var dashedProp = StyleSandbox._convertToDashed(prop);
        (0, _propertyOverriding.overrideDescriptor)(style, prop, {
            getter: function getter() {
                var value = nativeMethods.styleGetPropertyValue.call(this, dashedProp);
                return _style2.default.cleanUp(value, _url.parseProxyUrl);
            },
            setter: function setter(value) {
                if (typeof value === 'string') value = _style2.default.process(value, _url.getProxyUrl);
                nativeMethods.styleSetProperty.call(this, dashedProp, value);
            }
        });
    };

    StyleSandbox.prototype._processStyleInstance = function _processStyleInstance(style) {
        var isProcessed = style[CSS_STYLE_IS_PROCESSED];
        if (!isProcessed) {
            for (var _i4 = 0, _DASHED_URL_PROPS2 = this.DASHED_URL_PROPS; _i4 < _DASHED_URL_PROPS2.length; _i4++) {
                var prop = _DASHED_URL_PROPS2[_i4];
                this._overrideStyleInstanceProp(style, prop);
            }

            if (!this.FEATURES.protoContainsUrlProps) {
                for (var _i6 = 0, _URL_PROPS2 = this.URL_PROPS; _i6 < _URL_PROPS2.length; _i6++) {
                    var _prop = _URL_PROPS2[_i6];
                    this._overrideStyleInstanceProp(style, _prop);
                }
            }
            this.nativeMethods.objectDefineProperty(style, CSS_STYLE_IS_PROCESSED, { value: true });
        }
        return style;
    };

    StyleSandbox.prototype._getStyleProxy = function _getStyleProxy(style) {
        var _this2 = this;

        var proxyObject = style[CSS_STYLE_PROXY_OBJECT];
        if (!proxyObject) {
            proxyObject = new this.nativeMethods.Proxy(style, {
                get: function get(target, prop) {
                    if (_this2.URL_PROPS.indexOf(prop) !== -1 || _this2.DASHED_URL_PROPS.indexOf(prop) !== -1) return _style2.default.cleanUp(target[prop], _url.parseProxyUrl);
                    if (prop === CSS_STYLE_PROXY_TARGET) return target;
                    return target[prop];
                },
                set: function set(target, prop, value) {
                    if (_this2.URL_PROPS.indexOf(prop) !== -1 || _this2.DASHED_URL_PROPS.indexOf(prop) !== -1) {
                        if (typeof value === 'string') value = _style2.default.process(value, _url.getProxyUrl);
                    }
                    target[prop] = value;
                    return true;
                }
            });
            this.nativeMethods.objectDefineProperty(style, CSS_STYLE_PROXY_OBJECT, { value: proxyObject });
        }
        return proxyObject;
    };

    StyleSandbox.prototype._overrideCSSStyleDeclarationFunctionsCtx = function _overrideCSSStyleDeclarationFunctionsCtx(window) {
        var _this3 = this;

        // @ts-ignore
        var styleDeclarationProto = window.CSSStyleDeclaration.prototype;

        var _loop = function _loop(prop) {
            // @ts-ignore
            var nativeFn = _this3.nativeMethods.objectGetOwnPropertyDescriptor.call(window.Object, styleDeclarationProto, prop).value; // eslint-disable-line no-restricted-properties
            if (_this3.nativeMethods.objectHasOwnProperty.call(styleDeclarationProto, prop) && typeof nativeFn === 'function') {
                styleDeclarationProto[prop] = function () {
                    return nativeFn.apply(this[CSS_STYLE_PROXY_TARGET] || this, arguments);
                };
            }
        };

        for (var prop in styleDeclarationProto) {
            _loop(prop);
        }
    };

    StyleSandbox.prototype.attach = function attach(window) {
        _SandboxBase.prototype.attach.call(this, window);
        var nativeMethods = this.nativeMethods;
        var styleSandbox = this;
        (0, _propertyOverriding.overrideDescriptor)(window[nativeMethods.htmlElementStylePropOwnerName].prototype, 'style', {
            getter: this.FEATURES.protoContainsAllProps ? null : function () {
                var style = nativeMethods.htmlElementStyleGetter.call(this);
                if (styleSandbox.FEATURES.propsCannotBeOverridden) return styleSandbox._getStyleProxy(style);
                return styleSandbox._processStyleInstance(style);
            },
            setter: nativeMethods.htmlElementStyleSetter ? function (value) {
                var processedCss = _style2.default.process(value, _url.getProxyUrl);
                nativeMethods.htmlElementStyleSetter.call(this, processedCss);
            } : null
        });
        if (this.FEATURES.protoContainsAllProps) {
            for (var _i8 = 0, _URL_PROPS4 = this.URL_PROPS; _i8 < _URL_PROPS4.length; _i8++) {
                var prop = _URL_PROPS4[_i8];
                // @ts-ignore
                this._overrideStyleProp(window.CSS2Properties.prototype, prop);
            }

            for (var _i10 = 0, _DASHED_URL_PROPS4 = this.DASHED_URL_PROPS; _i10 < _DASHED_URL_PROPS4.length; _i10++) {
                var _prop2 = _DASHED_URL_PROPS4[_i10];
                // @ts-ignore
                this._overrideStyleProp(window.CSS2Properties.prototype, _prop2);
            }
        } else if (this.FEATURES.protoContainsUrlProps) {
            for (var _i12 = 0, _URL_PROPS6 = this.URL_PROPS; _i12 < _URL_PROPS6.length; _i12++) {
                var _prop3 = _URL_PROPS6[_i12];
                // @ts-ignore
                this._overrideStyleProp(window.CSSStyleDeclaration.prototype, _prop3);
            }
        }
        // @ts-ignore
        (0, _propertyOverriding.overrideDescriptor)(window.CSSStyleDeclaration.prototype, 'cssText', {
            getter: function getter() {
                var cssText = nativeMethods.styleCssTextGetter.call(this);
                return _style2.default.cleanUp(cssText, _url.parseProxyUrl);
            },
            setter: function setter(value) {
                if (typeof value === 'string') value = _style2.default.process(value, _url.getProxyUrl);
                nativeMethods.styleCssTextSetter.call(this, value);
            }
        });
        // @ts-ignore
        window.CSSStyleSheet.prototype.insertRule = function (rule, index) {
            var newRule = _style2.default.process(rule, _url.getProxyUrl);
            return nativeMethods.styleInsertRule.call(this, newRule, index);
        };
        // @ts-ignore
        window.CSSStyleDeclaration.prototype.getPropertyValue = function () {
            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
            }

            var value = nativeMethods.styleGetPropertyValue.apply(this, args);
            return _style2.default.cleanUp(value, _url.parseProxyUrl);
        };
        // @ts-ignore
        window.CSSStyleDeclaration.prototype.setProperty = function () {
            for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                args[_key2] = arguments[_key2];
            }

            var value = args[1];
            if (typeof value === 'string') args[1] = _style2.default.process(value, _url.getProxyUrl);
            return nativeMethods.styleSetProperty.apply(this, args);
        };
        // @ts-ignore
        window.CSSStyleDeclaration.prototype.removeProperty = function () {
            for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                args[_key3] = arguments[_key3];
            }

            var oldValue = nativeMethods.styleRemoveProperty.apply(this, args);
            return _style2.default.cleanUp(oldValue, _url.parseProxyUrl);
        };
        // NOTE: We need to override context of all functions from the CSSStyleDeclaration prototype if we use the Proxy feature.
        // Can only call CSSStyleDeclaration.<function name> on instances of CSSStyleDeclaration
        // The error above occurs if functions will be called on a proxy instance.
        if (this.FEATURES.propsCannotBeOverridden) this._overrideCSSStyleDeclarationFunctionsCtx(window);
    };

    return StyleSandbox;
}(_base2.default);

exports.default = StyleSandbox;
module.exports = exports['default'];