'use strict';

exports.__esModule = true;

var _base = require('./base');

var _base2 = _interopRequireDefault(_base);

var _nativeMethods = require('./native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _script = require('../../processing/script');

var _browser = require('../utils/browser');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// NOTE: When you call the focus and blur function for some elements in IE, the event handlers  must be raised
// asynchronously, but before executing functions that are called by using the window.setTimeout function. So,
// we need to raise the handlers with a timeout, but do it before calling other asynchronous functions.
var TimersSandbox = function (_SandboxBase) {
    _inherits(TimersSandbox, _SandboxBase);

    function TimersSandbox() {
        _classCallCheck(this, TimersSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.timeouts = [];
        _this.deferredFunctions = [];
        _this.setTimeout = _nativeMethods2.default.setTimeout;
        return _this;
    }

    TimersSandbox.prototype._wrapTimeoutFunctionsArguments = function _wrapTimeoutFunctionsArguments(args) {
        var _this2 = this;

        var isScriptFirstArg = typeof args[0] === 'string';
        var func = !isScriptFirstArg ? args[0] : null;
        var script = isScriptFirstArg ? (0, _script.processScript)(args[0], false) : null;
        if (_browser.isIE && _browser.version < 12) {
            var timersSandbox = this;
            var fnToRun = isScriptFirstArg ? function () {
                // NOTE: We are switching eval to the global context with this assignment.
                // Unlike eval, the setTimeout/setInterval functions always work in the global context.
                // @ts-ignore
                var ev = _this2.window.eval;
                return ev(script);
            } : func;
            args[0] = function () {
                return timersSandbox._callDeferredFunction(fnToRun, arguments);
            };
        } else if (isScriptFirstArg) args[0] = script;
        return args;
    };

    TimersSandbox.prototype._callDeferredFunction = function _callDeferredFunction(fn, args) {
        if (this.timeouts.length) {
            var curTimeouts = [];
            var curHandlers = [];
            for (var i = 0; i < this.timeouts.length; i++) {
                curTimeouts.push(this.timeouts[i]);
                curHandlers.push(this.deferredFunctions[i]);
            }
            this.timeouts = [];
            this.deferredFunctions = [];
            for (var j = 0; j < curTimeouts.length; j++) {
                _nativeMethods2.default.clearInterval.call(this.window, curTimeouts[j]);
                curHandlers[j]();
            }
            // NOTE: Handlers can create new deferred functions.
            return this._callDeferredFunction(fn, args);
        }
        return fn.apply(this.window, args);
    };

    TimersSandbox.prototype.attach = function attach(window) {
        _SandboxBase.prototype.attach.call(this, window);
        var timersSandbox = this;
        window.setTimeout = function () {
            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
            }

            return _nativeMethods2.default.setTimeout.apply(window, timersSandbox._wrapTimeoutFunctionsArguments(args));
        };
        window.setInterval = function () {
            for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                args[_key2] = arguments[_key2];
            }

            return _nativeMethods2.default.setInterval.apply(window, timersSandbox._wrapTimeoutFunctionsArguments(args));
        };
        // NOTE: We are saving the setTimeout wrapper for internal use in case the page-script replaces
        // it with an invalid value.
        this.setTimeout = window.setTimeout;
    };

    TimersSandbox.prototype.deferFunction = function deferFunction(fn) {
        var _this3 = this;

        var deferredFunction = function deferredFunction() {
            fn();
            for (var i = 0; i < _this3.deferredFunctions.length; i++) {
                if (_this3.deferredFunctions[i] === deferredFunction) {
                    _this3.deferredFunctions.splice(i, 1);
                    _this3.timeouts.splice(i, 1);
                    break;
                }
            }
        };
        this.deferredFunctions.push(deferredFunction);
        this.timeouts.push(_nativeMethods2.default.setTimeout.call(window, deferredFunction, 0));
    };

    return TimersSandbox;
}(_base2.default);

exports.default = TimersSandbox;
module.exports = exports['default'];