'use strict';

exports.__esModule = true;
exports.addInputInfo = addInputInfo;
exports.getFormInfo = getFormInfo;
exports.setFormInfo = setFormInfo;
exports.removeInputInfo = removeInputInfo;

var _internalAttributes = require('../../../processing/dom/internal-attributes');

var _internalAttributes2 = _interopRequireDefault(_internalAttributes);

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _jsonHammerhead = require('json-hammerhead');

var JSON = _interopRequireWildcard(_jsonHammerhead);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createInput(form) {
    var hiddenInput = _nativeMethods2.default.createElement.call(document, 'input');
    hiddenInput.type = 'hidden';
    hiddenInput.name = _internalAttributes2.default.uploadInfoHiddenInputName;
    _nativeMethods2.default.inputValueSetter.call(hiddenInput, '[]');
    _nativeMethods2.default.appendChild.call(form, hiddenInput);
    return hiddenInput;
}
function getInput(form) {
    var inputSelector = '[name="' + _internalAttributes2.default.uploadInfoHiddenInputName + '"]';
    return _nativeMethods2.default.elementQuerySelector.call(form, inputSelector) || createInput(form);
}
function indexOf(info, input) {
    for (var index = 0; index < info.length; index++) {
        if (info[index].id === input.id || info[index].name === input.name) return index;
    }
    return -1;
}
function addInputInfo(input, fileList, value) {
    var formInfo = getFormInfo(input);
    if (formInfo) {
        var files = [];
        fileList = _nativeMethods2.default.arraySlice.call(fileList);
        for (var i = 0, len = fileList.length; i < len; i++) {
            var file = fileList[i];
            files.push({
                name: file.name,
                type: file.type,
                data: file.base64
            });
        }
        var inputInfoIndex = indexOf(formInfo, input);
        var inputInfo = {
            id: input.id,
            name: input.name,
            files: files,
            value: value
        };
        if (inputInfoIndex === -1) formInfo.push(inputInfo);else formInfo[inputInfoIndex] = inputInfo;
        setFormInfo(input, formInfo);
    }
}
function getFormInfo(input) {
    return input.form ? JSON.parse(_nativeMethods2.default.inputValueGetter.call(getInput(input.form))) : null;
}
function setFormInfo(input, info) {
    if (input.form) {
        var hiddenInput = getInput(input.form);
        _nativeMethods2.default.inputValueSetter.call(hiddenInput, JSON.stringify(info));
    }
}
function removeInputInfo(input) {
    var uploadInfo = getFormInfo(input);
    if (uploadInfo) {
        var inputInfoIndex = indexOf(uploadInfo, input);
        if (inputInfoIndex !== -1) {
            uploadInfo.splice(inputInfoIndex, 1);
            setFormInfo(input, uploadInfo);
            return true;
        }
    }
    return false;
}