'use strict';

exports.__esModule = true;

var _internalProperties = require('../../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _infoManager = require('./info-manager');

var _infoManager2 = _interopRequireDefault(_infoManager);

var _dom = require('../../utils/dom');

var _browser = require('../../utils/browser');

var _event = require('../../utils/event');

var _backup = require('../backup');

var _nativeMethods = require('../native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var UploadSandbox = function (_SandboxBase) {
    _inherits(UploadSandbox, _SandboxBase);

    function UploadSandbox(listeners, eventSimulator, shadowUI) {
        _classCallCheck(this, UploadSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.START_FILE_UPLOADING_EVENT = 'hammerhead|event|start-file-uploading';
        _this.END_FILE_UPLOADING_EVENT = 'hammerhead|event|end-file-uploading';
        _this.infoManager = new _infoManager2.default(shadowUI);
        _this.listeners = listeners;
        _this.eventSimulator = eventSimulator;
        return _this;
    }

    UploadSandbox.prototype._riseChangeEvent = function _riseChangeEvent(input) {
        this.eventSimulator.change(input);
    };

    UploadSandbox._getCurrentInfoManager = function _getCurrentInfoManager(input) {
        // @ts-ignore
        var contextWindow = input[_internalProperties2.default.processedContext];
        return (0, _backup.get)(contextWindow).upload.infoManager;
    };
    /*eslint-disable max-nested-callbacks */


    UploadSandbox.prototype.attach = function attach(window) {
        var _this2 = this;

        _SandboxBase.prototype.attach.call(this, window);
        this.listeners.addInternalEventListener(window, ['change'], function (e, dispatched) {
            var input = e.target;
            var currentInfoManager = UploadSandbox._getCurrentInfoManager(input);
            if ((0, _dom.isFileInput)(input) && !dispatched) {
                (0, _event.stopPropagation)(e);
                (0, _event.preventDefault)(e);
                var value = _nativeMethods2.default.inputValueGetter.call(input);
                if (!!value || !!currentInfoManager.getValue(input)) {
                    var files = _nativeMethods2.default.inputFilesGetter.call(input);
                    var fileNames = _infoManager2.default.getFileNames(files, value);
                    _this2.emit(_this2.START_FILE_UPLOADING_EVENT, fileNames, input);
                    currentInfoManager.loadFileListData(input, files).then(function (fileList) {
                        currentInfoManager.setUploadInfo(input, fileList, value);
                        return _infoManager2.default.sendFilesInfoToServer(fileList, fileNames);
                    }).then(function (uploadInfo) {
                        _this2._riseChangeEvent(input);
                        _this2.emit(_this2.END_FILE_UPLOADING_EVENT, uploadInfo);
                    });
                }
            }
        });
        if (_browser.isIE) {
            // NOTE: Prevent the browser's open file dialog.
            this.listeners.addInternalEventListener(window, ['click'], function (e, dispatched) {
                var input = e.target || e.srcElement;
                if ((0, _dom.isFileInput)(input) && dispatched) (0, _event.preventDefault)(e, true);
            });
        }
    };
    /*eslint-enable max-nested-callbacks */


    UploadSandbox.getFiles = function getFiles(input) {
        var files = _nativeMethods2.default.inputFilesGetter.call(input);
        return files !== void 0 ? UploadSandbox._getCurrentInfoManager(input).getFiles(input) : void 0;
    };

    UploadSandbox.getUploadElementValue = function getUploadElementValue(input) {
        return UploadSandbox._getCurrentInfoManager(input).getValue(input);
    };

    UploadSandbox.prototype.setUploadElementValue = function setUploadElementValue(input, value) {
        if (value === '') {
            if (UploadSandbox._getCurrentInfoManager(input).clearUploadInfo(input) && _browser.isIE && _browser.version > 10) this._riseChangeEvent(input);
        }
        return value;
    };

    UploadSandbox.prototype.doUpload = function doUpload(input, filePaths) {
        var _this3 = this;

        var currentInfoManager = UploadSandbox._getCurrentInfoManager(input);
        filePaths = filePaths || [];
        return _infoManager2.default.loadFilesInfoFromServer(filePaths).then(function (filesInfo) {
            return _infoManager2.default.prepareFileListWrapper(filesInfo);
        }).then(function (data) {
            if (!data.errs.length) {
                var value = _infoManager2.default.formatValue(filePaths);
                currentInfoManager.setUploadInfo(input, data.fileList, value);
                _this3._riseChangeEvent(input);
            }
            return data.errs;
        });
    };

    return UploadSandbox;
}(_base2.default);

exports.default = UploadSandbox;
module.exports = exports['default'];