'use strict';

exports.__esModule = true;

var _command = require('../../../session/command');

var _command2 = _interopRequireDefault(_command);

var _fileListWrapper = require('./file-list-wrapper');

var _fileListWrapper2 = _interopRequireDefault(_fileListWrapper);

var _transport = require('../../transport');

var _transport2 = _interopRequireDefault(_transport);

var _browser = require('../../utils/browser');

var Browser = _interopRequireWildcard(_browser);

var _hiddenInfo = require('./hidden-info');

var HiddenInfo = _interopRequireWildcard(_hiddenInfo);

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// NOTE: https://html.spec.whatwg.org/multipage/forms.html#fakepath-srsly.
var FAKE_PATH_STRING = 'C:\\fakepath\\';

var UploadInfoManager = function () {
    function UploadInfoManager(shadowUI) {
        _classCallCheck(this, UploadInfoManager);

        this.shadowUI = shadowUI;
        this.uploadInfo = [];
    }

    UploadInfoManager._getFileListData = function _getFileListData(fileList) {
        var data = [];

        for (var _i2 = 0; _i2 < fileList.length; _i2++) {
            var file = fileList[_i2];
            data.push(file.base64);
        }

        return data;
    };

    UploadInfoManager.formatValue = function formatValue(fileNames) {
        var value = '';
        fileNames = typeof fileNames === 'string' ? [fileNames] : fileNames;
        if (fileNames && fileNames.length) {
            if (Browser.isWebKit) value = FAKE_PATH_STRING + fileNames[0].split('/').pop();else return fileNames[0].split('/').pop();
        }
        return value;
    };

    UploadInfoManager.getFileNames = function getFileNames(fileList, value) {
        var result = [];
        if (fileList) {
            for (var _i4 = 0; _i4 < fileList.length; _i4++) {
                var file = fileList[_i4];
                result.push(file.name);
            }
        } else if (value.lastIndexOf('\\') !== -1) result.push(value.substr(value.lastIndexOf('\\') + 1));
        return result;
    };

    UploadInfoManager.loadFilesInfoFromServer = function loadFilesInfoFromServer(filePaths) {
        return _transport2.default.asyncServiceMsg({
            cmd: _command2.default.getUploadedFiles,
            filePaths: typeof filePaths === 'string' ? [filePaths] : filePaths
        });
    };

    UploadInfoManager.prepareFileListWrapper = function prepareFileListWrapper(filesInfo) {
        var errs = [];
        var validFilesInfo = [];

        for (var _i6 = 0; _i6 < filesInfo.length; _i6++) {
            var fileInfo = filesInfo[_i6];
            if (fileInfo.err) errs.push(fileInfo);else validFilesInfo.push(fileInfo);
        }

        return {
            errs: errs,
            fileList: new _fileListWrapper2.default(validFilesInfo)
        };
    };

    UploadInfoManager.sendFilesInfoToServer = function sendFilesInfoToServer(fileList, fileNames) {
        return _transport2.default.asyncServiceMsg({
            cmd: _command2.default.uploadFiles,
            data: UploadInfoManager._getFileListData(fileList),
            fileNames: fileNames
        });
    };

    UploadInfoManager.prototype.clearUploadInfo = function clearUploadInfo(input) {
        var inputInfo = this.getUploadInfo(input);
        if (inputInfo) {
            /*eslint-disable no-restricted-properties*/
            inputInfo.files = new _fileListWrapper2.default([]);
            inputInfo.value = '';
            /*eslint-enable no-restricted-properties*/
            return HiddenInfo.removeInputInfo(input);
        }
        return null;
    };

    UploadInfoManager.prototype.getFiles = function getFiles(input) {
        var inputInfo = this.getUploadInfo(input);
        // eslint-disable-next-line no-restricted-properties
        return inputInfo ? inputInfo.files : new _fileListWrapper2.default([]);
    };

    UploadInfoManager.prototype.getUploadInfo = function getUploadInfo(input) {
        for (var _i8 = 0, _uploadInfo2 = this.uploadInfo; _i8 < _uploadInfo2.length; _i8++) {
            var uploadInfoItem = _uploadInfo2[_i8];
            if (uploadInfoItem.input === input) return uploadInfoItem;
        }

        return null;
    };

    UploadInfoManager.prototype.getValue = function getValue(input) {
        var inputInfo = this.getUploadInfo(input);
        // eslint-disable-next-line no-restricted-properties
        return inputInfo ? inputInfo.value : '';
    };

    UploadInfoManager.prototype.loadFileListData = function loadFileListData(_input, fileList) {
        if (!fileList.length) return _pinkie2.default.resolve(new _fileListWrapper2.default([]));
        return new _pinkie2.default(function (resolve) {
            var fileReader = new FileReader();
            var readedFiles = [];
            var index = 0;
            var file = fileList[index];
            fileReader.addEventListener('load', function (e) {
                readedFiles.push({
                    data: e.target.result.substr(e.target.result.indexOf(',') + 1),
                    blob: file.slice(0, file.size),
                    info: {
                        type: file.type,
                        name: file.name,
                        lastModifiedDate: file.lastModifiedDate
                    }
                });
                if (fileList[++index]) {
                    file = fileList[index];
                    fileReader.readAsDataURL(file);
                } else resolve(new _fileListWrapper2.default(readedFiles));
            });
            fileReader.readAsDataURL(file);
        });
    };

    UploadInfoManager.prototype.setUploadInfo = function setUploadInfo(input, fileList, value) {
        var inputInfo = this.getUploadInfo(input);
        if (!inputInfo) {
            inputInfo = { input: input };
            this.uploadInfo.push(inputInfo);
        }
        /*eslint-disable no-restricted-properties*/
        inputInfo.files = fileList;
        inputInfo.value = value;
        /*eslint-enable no-restricted-properties*/
        HiddenInfo.addInputInfo(input, fileList, value);
    };

    return UploadInfoManager;
}();

exports.default = UploadInfoManager;
module.exports = exports['default'];