'use strict';

exports.__esModule = true;
exports.add = add;
exports.remove = remove;
exports.findByName = findByName;

var _dom = require('../utils/dom');

var _nativeMethods = require('./native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var WINDOWS_STORAGE = 'hammerhead|windows-storage';
function getStorage() {
    var topSameDomainWindow = (0, _dom.getTopSameDomainWindow)(window);
    var storage = topSameDomainWindow[WINDOWS_STORAGE];
    if (!storage) {
        storage = [];
        _nativeMethods2.default.objectDefineProperty(topSameDomainWindow, WINDOWS_STORAGE, { value: storage });
    }
    return storage;
}
function add(wnd) {
    var storage = getStorage();
    for (var i = storage.length - 1; i >= 0; i--) {
        try {
            if (storage[i] === wnd) return;
        } catch (e) {
            storage.splice(i, 1);
        }
    }
    storage.push(wnd);
}
function remove(wnd) {
    var storage = getStorage();
    var index = storage.indexOf(wnd);
    if (index !== -1) storage.splice(index, 1);
}
function findByName(name) {
    var storage = getStorage();
    for (var i = 0; i < storage.length; i++) {
        try {
            if (storage[i].name === name) return storage[i];
        } catch (e) {
            // NOTE: During loading, an iframe can be changed from same-domain to cross-domain.
            // Iframe's window is reinitialized, and we add 2 windows to the window storages:
            // one to the same-domain storage and another one to the cross-domain storage.
            // We remove the cross-domain window from this storage
            // because it is already added to the cross-domain window storage.
            storage.splice(i, 1);
            i--;
        }
    }
    return null;
}