'use strict';

exports.__esModule = true;

var _base = require('./base');

var _base2 = _interopRequireDefault(_base);

var _nativeMethods = require('./native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _url = require('../utils/url');

var _headers = require('../../request-pipeline/xhr/headers');

var _headers2 = _interopRequireDefault(_headers);

var _authorization = require('../../request-pipeline/xhr/authorization');

var _authorization2 = _interopRequireDefault(_authorization);

var _destinationLocation = require('../utils/destination-location');

var _propertyOverriding = require('../utils/property-overriding');

var _sameOriginCheckFailedStatusCode = require('../../request-pipeline/xhr/same-origin-check-failed-status-code');

var _sameOriginCheckFailedStatusCode2 = _interopRequireDefault(_sameOriginCheckFailedStatusCode);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var XHR_READY_STATES = ['UNSENT', 'OPENED', 'HEADERS_RECEIVED', 'LOADING', 'DONE'];

var XhrSandbox = function (_SandboxBase) {
    _inherits(XhrSandbox, _SandboxBase);

    function XhrSandbox(cookieSandbox) {
        _classCallCheck(this, XhrSandbox);

        var _this = _possibleConstructorReturn(this, _SandboxBase.call(this));

        _this.XHR_COMPLETED_EVENT = 'hammerhead|event|xhr-completed';
        _this.XHR_ERROR_EVENT = 'hammerhead|event|xhr-error';
        _this.BEFORE_XHR_SEND_EVENT = 'hammerhead|event|before-xhr-send';
        _this.XHR_COMPLETED_EVENT = 'hammerhead|event|xhr-completed';
        _this.XHR_ERROR_EVENT = 'hammerhead|event|xhr-error';
        _this.BEFORE_XHR_SEND_EVENT = 'hammerhead|event|before-xhr-send';
        _this.cookieSandbox = cookieSandbox;
        return _this;
    }

    XhrSandbox.createNativeXHR = function createNativeXHR() {
        var xhr = new _nativeMethods2.default.XMLHttpRequest();
        xhr.open = _nativeMethods2.default.xhrOpen;
        xhr.abort = _nativeMethods2.default.xhrAbort;
        xhr.send = _nativeMethods2.default.xhrSend;
        xhr.addEventListener = _nativeMethods2.default.xhrAddEventListener;
        xhr.removeEventListener = _nativeMethods2.default.xhrRemoveEventListener;
        xhr.setRequestHeader = _nativeMethods2.default.xhrSetRequestHeader;
        xhr.getResponseHeader = _nativeMethods2.default.xhrGetResponseHeader;
        xhr.getAllResponseHeaders = _nativeMethods2.default.xhrGetAllResponseHeaders;
        xhr.overrideMimeType = _nativeMethods2.default.xhrOverrideMimeType;
        xhr.dispatchEvent = _nativeMethods2.default.xhrDispatchEvent;
        return xhr;
    };

    XhrSandbox.openNativeXhr = function openNativeXhr(xhr, url, isAsync) {
        xhr.open('POST', url, isAsync);
        xhr.setRequestHeader('Cache-Control', 'no-cache, no-store, must-revalidate');
    };

    XhrSandbox.prototype.attach = function attach(window) {
        _SandboxBase.prototype.attach.call(this, window);
        var xhrSandbox = this;
        var xmlHttpRequestProto = window.XMLHttpRequest.prototype;
        var xmlHttpRequestToString = _nativeMethods2.default.XMLHttpRequest.toString();
        var emitXhrCompletedEventIfNecessary = function emitXhrCompletedEventIfNecessary() {
            if (this.readyState === this.DONE) {
                xhrSandbox.emit(xhrSandbox.XHR_COMPLETED_EVENT, { xhr: this });
                _nativeMethods2.default.xhrRemoveEventListener.call(this, 'readystatechange', emitXhrCompletedEventIfNecessary);
            }
        };
        var syncCookieWithClientIfNecessary = function syncCookieWithClientIfNecessary() {
            if (this.readyState < this.HEADERS_RECEIVED) return;
            xhrSandbox.cookieSandbox.syncCookie();
            _nativeMethods2.default.xhrRemoveEventListener.call(this, 'readystatechange', syncCookieWithClientIfNecessary);
        };
        var xmlHttpRequestWrapper = function xmlHttpRequestWrapper() {
            var xhr = new _nativeMethods2.default.XMLHttpRequest();
            _nativeMethods2.default.xhrAddEventListener.call(xhr, 'readystatechange', emitXhrCompletedEventIfNecessary);
            _nativeMethods2.default.xhrAddEventListener.call(xhr, 'readystatechange', syncCookieWithClientIfNecessary);
            return xhr;
        };

        for (var _i2 = 0; _i2 < XHR_READY_STATES.length; _i2++) {
            var readyState = XHR_READY_STATES[_i2];
            _nativeMethods2.default.objectDefineProperty(xmlHttpRequestWrapper, readyState, {
                value: XMLHttpRequest[readyState],
                enumerable: true
            });
        }

        window.XMLHttpRequest = xmlHttpRequestWrapper;
        xmlHttpRequestWrapper.prototype = xmlHttpRequestProto;
        xmlHttpRequestWrapper.toString = function () {
            return xmlHttpRequestToString;
        };
        // NOTE: We cannot just assign constructor property in OS X 10.11 safari 9.0
        _nativeMethods2.default.objectDefineProperty(xmlHttpRequestProto, 'constructor', {
            value: xmlHttpRequestWrapper
        });
        xmlHttpRequestProto.abort = function () {
            _nativeMethods2.default.xhrAbort.apply(this, arguments);
            xhrSandbox.emit(xhrSandbox.XHR_ERROR_EVENT, {
                err: new Error('XHR aborted'),
                xhr: this
            });
        };
        // NOTE: Redirect all requests to the Hammerhead proxy and ensure that requests don't
        // violate Same Origin Policy.
        xmlHttpRequestProto.open = function () {
            var url = arguments[1];
            var urlIsString = typeof url === 'string';
            arguments[1] = (0, _url.getProxyUrl)(urlIsString ? url : String(url));
            _nativeMethods2.default.xhrOpen.apply(this, arguments);
        };
        xmlHttpRequestProto.send = function () {
            xhrSandbox.emit(xhrSandbox.BEFORE_XHR_SEND_EVENT, { xhr: this });
            // NOTE: Add the XHR request mark, so that a proxy can recognize a request as a XHR request. As all
            // requests are passed to the proxy, we need to perform Same Origin Policy compliance checks on the
            // server side. So, we pass the CORS support flag to inform the proxy that it can analyze the
            // Access-Control_Allow_Origin flag and skip "preflight" requests.
            _nativeMethods2.default.xhrSetRequestHeader.call(this, _headers2.default.requestMarker, 'true');
            // eslint-disable-next-line no-restricted-properties
            _nativeMethods2.default.xhrSetRequestHeader.call(this, _headers2.default.origin, (0, _destinationLocation.getOriginHeader)());
            if (this.withCredentials) _nativeMethods2.default.xhrSetRequestHeader.call(this, _headers2.default.withCredentials, 'true');
            _nativeMethods2.default.xhrSend.apply(this, arguments);
            // NOTE: For xhr with the sync mode
            emitXhrCompletedEventIfNecessary.call(this);
            syncCookieWithClientIfNecessary.call(this);
        };
        xmlHttpRequestProto.setRequestHeader = function (header, value) {
            if (typeof header === 'string' && _authorization2.default.headers.indexOf(header.toLowerCase()) !== -1) value = _authorization2.default.valuePrefix + value;
            return _nativeMethods2.default.xhrSetRequestHeader.call(this, header, value);
        };
        (0, _propertyOverriding.overrideDescriptor)(window.XMLHttpRequest.prototype, 'status', {
            getter: function getter() {
                var status = _nativeMethods2.default.xhrStatusGetter.call(this);
                return status === _sameOriginCheckFailedStatusCode2.default ? 0 : status;
            }
        });
        if (_nativeMethods2.default.xhrResponseURLGetter) {
            (0, _propertyOverriding.overrideDescriptor)(window.XMLHttpRequest.prototype, 'responseURL', {
                getter: function getter() {
                    var responseUrl = _nativeMethods2.default.xhrResponseURLGetter.call(this);
                    var parsedProxyUrl = responseUrl && (0, _url.parseProxyUrl)(responseUrl);
                    return parsedProxyUrl ? parsedProxyUrl.destUrl : responseUrl;
                }
            });
        }
    };

    return XhrSandbox;
}(_base2.default);

exports.default = XhrSandbox;
module.exports = exports['default'];