'use strict';

exports.__esModule = true;

var _nativeMethods = require('./sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _settings = require('./settings');

var _settings2 = _interopRequireDefault(_settings);

var _xhr = require('./sandbox/xhr');

var _xhr2 = _interopRequireDefault(_xhr);

var _jsonHammerhead = require('json-hammerhead');

var _browser = require('./utils/browser');

var _createUnresolvablePromise = require('./utils/create-unresolvable-promise');

var _createUnresolvablePromise2 = _interopRequireDefault(_createUnresolvablePromise);

var _noop = require('./utils/noop');

var _noop2 = _interopRequireDefault(_noop);

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _dom = require('./utils/dom');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var SERVICE_MESSAGES_WAITING_INTERVAL = 50;

var Transport = function () {
    function Transport() {
        _classCallCheck(this, Transport);

        this.msgQueue = {};
        this.activeServiceMessagesCounter = 0;
        this.shouldAddRefferer = Transport._shouldAddReferrer();
    }

    Transport._shouldAddReferrer = function _shouldAddReferrer() {
        var frameElement = (0, _dom.getFrameElement)(window);
        return frameElement && (0, _dom.isIframeWithoutSrc)(frameElement);
    };

    Transport._storeMessage = function _storeMessage(msg) {
        var storedMessages = Transport._getStoredMessages();
        storedMessages.push(msg);
        _nativeMethods2.default.winLocalStorageGetter.call(window).setItem(_settings2.default.get().sessionId, (0, _jsonHammerhead.stringify)(storedMessages));
    };

    Transport._getStoredMessages = function _getStoredMessages() {
        var storedMessagesStr = _nativeMethods2.default.winLocalStorageGetter.call(window).getItem(_settings2.default.get().sessionId);
        return storedMessagesStr ? (0, _jsonHammerhead.parse)(storedMessagesStr) : [];
    };

    Transport._removeMessageFromStore = function _removeMessageFromStore(cmd) {
        var messages = Transport._getStoredMessages();
        for (var i = 0; i < messages.length; i++) {
            if (messages[i].cmd === cmd) {
                messages.splice(i, 1);
                break;
            }
        }
        _nativeMethods2.default.winLocalStorageGetter.call(window).setItem(_settings2.default.get().sessionId, (0, _jsonHammerhead.stringify)(messages));
    };
    // TODO: Rewrite this using Promise after getting rid of syncServiceMsg.


    Transport.prototype._performRequest = function _performRequest(msg, callback) {
        var _this = this;

        msg.sessionId = _settings2.default.get().sessionId;
        if (this.shouldAddRefferer) msg.referer = _settings2.default.get().referer;
        var sendMsg = function sendMsg(forced) {
            _this.activeServiceMessagesCounter++;
            var isAsyncRequest = !forced;
            var transport = _this;
            var request = _xhr2.default.createNativeXHR();
            var msgCallback = function msgCallback() {
                // NOTE: The 500 status code is returned by server when an error occurred into service message handler
                if (_nativeMethods2.default.xhrStatusGetter.call(this) === 500 && this.responseText) {
                    msg.disableResending = true;
                    errorHandler.call(this); // eslint-disable-line no-use-before-define
                    return;
                }
                transport.activeServiceMessagesCounter--;
                var response = this.responseText && (0, _jsonHammerhead.parse)(this.responseText);
                request = null;
                callback(null, response);
            };
            var errorHandler = function errorHandler() {
                if (msg.disableResending) {
                    transport.activeServiceMessagesCounter--;
                    var errorMsg = 'XHR request failed with ' + request.status + ' status code.';
                    if (this.responseText) errorMsg += '\nError message: ' + this.responseText;
                    callback(new Error(errorMsg));
                    return;
                }
                if (_browser.isWebKit || _browser.isFirefox) {
                    Transport._storeMessage(msg);
                    msgCallback.call(this);
                } else sendMsg(true);
            };
            _xhr2.default.openNativeXhr(request, _settings2.default.get().serviceMsgUrl, isAsyncRequest);
            if (forced) {
                request.addEventListener('readystatechange', function () {
                    if (this.readyState !== 4) return;
                    msgCallback.call(this);
                });
            } else {
                request.addEventListener('load', msgCallback);
                request.addEventListener('abort', errorHandler);
                request.addEventListener('error', errorHandler);
                request.addEventListener('timeout', errorHandler);
            }
            request.send((0, _jsonHammerhead.stringify)(msg));
        };
        Transport._removeMessageFromStore(msg.cmd);
        sendMsg();
    };

    Transport.prototype.waitForServiceMessagesCompleted = function waitForServiceMessagesCompleted(timeout) {
        var _this2 = this;

        return new _pinkie2.default(function (resolve) {
            if (!_this2.activeServiceMessagesCounter) {
                resolve();
                return;
            }
            var intervalId = null;
            var timeoutId = window.setTimeout(function () {
                _nativeMethods2.default.clearInterval.call(window, intervalId);
                resolve();
            }, timeout);
            intervalId = window.setInterval(function () {
                if (!_this2.activeServiceMessagesCounter) {
                    _nativeMethods2.default.clearInterval.call(window, intervalId);
                    _nativeMethods2.default.clearTimeout.call(window, timeoutId);
                    resolve();
                }
            }, SERVICE_MESSAGES_WAITING_INTERVAL);
        });
    };

    Transport.prototype.asyncServiceMsg = function asyncServiceMsg(msg) {
        var _this3 = this;

        return new _pinkie2.default(function (resolve, reject) {
            _this3._performRequest(msg, function (err, data) {
                if (!err) resolve(data);else if (msg.allowRejecting) reject(err);
            });
        });
    };

    Transport.prototype.batchUpdate = function batchUpdate() {
        var storedMessages = Transport._getStoredMessages();
        if (storedMessages.length) {
            var tasks = [];
            _nativeMethods2.default.winLocalStorageGetter.call(window).removeItem(_settings2.default.get().sessionId);

            for (var _i2 = 0; _i2 < storedMessages.length; _i2++) {
                var storedMessage = storedMessages[_i2];
                tasks.push(this.queuedAsyncServiceMsg(storedMessage));
            }

            return _pinkie2.default.all(tasks);
        }
        return _pinkie2.default.resolve();
    };

    Transport.prototype.queuedAsyncServiceMsg = function queuedAsyncServiceMsg(msg) {
        var _this4 = this;

        if (!this.msgQueue[msg.cmd]) this.msgQueue[msg.cmd] = _pinkie2.default.resolve();
        var isRejectingAllowed = msg.allowRejecting;
        msg.allowRejecting = true;
        this.msgQueue[msg.cmd] = this.msgQueue[msg.cmd].catch(_noop2.default).then(function () {
            return _this4.asyncServiceMsg(msg);
        });
        return this.msgQueue[msg.cmd].catch(function (err) {
            if (isRejectingAllowed) return _pinkie2.default.reject(err);
            return (0, _createUnresolvablePromise2.default)();
        });
    };

    return Transport;
}();

exports.default = new Transport();
module.exports = exports['default'];