'use strict';

exports.__esModule = true;
exports.parse = parse;
exports.formatClientString = formatClientString;
exports.setDefaultValues = setDefaultValues;
exports.domainMatch = domainMatch;
exports.pathMatch = pathMatch;
exports.getUTCDate = getUTCDate;

var _stringTrim = require('../../utils/string-trim');

var _stringTrim2 = _interopRequireDefault(_stringTrim);

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var COOKIE_PAIR_REGEX = /^((?:=)?([^=;]*)\s*=\s*)?([^\n\r\0]*)/;
var TRAILING_SEMICOLON_REGEX = /;+$/;
var FIX_COOKIE_DATE = /((?:\s|,)[0-9]{1,2})(?:\s|-)([A-Za-z]{3})(?:\s|-)([0-9]{4}\s)/;
function parse(str) {
    str = (0, _stringTrim2.default)(str);
    var trailingSemicolonCheck = TRAILING_SEMICOLON_REGEX.exec(str);
    if (trailingSemicolonCheck) str = str.slice(0, trailingSemicolonCheck.index);
    var firstSemicolonIdx = str.indexOf(';');
    var keyValueString = firstSemicolonIdx > -1 ? str.substr(0, firstSemicolonIdx) : str;
    var keyValueParsingResult = COOKIE_PAIR_REGEX.exec(keyValueString);
    if (!keyValueParsingResult) return null;
    var parsedCookie = {
        key: keyValueParsingResult[1] ? (0, _stringTrim2.default)(keyValueParsingResult[2]) : '',
        value: (0, _stringTrim2.default)(keyValueParsingResult[3])
    };
    if (firstSemicolonIdx === -1) return parsedCookie;
    var attributesString = (0, _stringTrim2.default)(str.slice(firstSemicolonIdx).replace(/^\s*;\s*/, ''));
    if (attributesString.length === 0) return parsedCookie;
    var attrValStrings = attributesString.split(/\s*;\s*/);
    while (attrValStrings.length) {
        var attrValueStr = attrValStrings.shift();
        var separatorIdx = attrValueStr.indexOf('=');
        var key = null;
        var value = null;
        var date = null;
        if (separatorIdx === -1) key = attrValueStr;else {
            key = attrValueStr.substr(0, separatorIdx);
            value = (0, _stringTrim2.default)(attrValueStr.substr(separatorIdx + 1));
        }
        key = (0, _stringTrim2.default)(key.toLowerCase());
        switch (key) {
            case 'expires':
                value = value.replace(FIX_COOKIE_DATE, '$1 $2 $3');
                date = getUTCDate(Date.parse(value));
                if (date) parsedCookie.expires = date;
                break;
            case 'max-age':
                parsedCookie.maxAge = value;
                break;
            case 'path':
                parsedCookie.path = value;
                break;
            case 'secure':
                parsedCookie.secure = true;
                break;
            case 'httponly':
                parsedCookie.httpOnly = true;
                break;
            case 'domain':
                // NOTE: Remove leading '.'.
                parsedCookie.domain = (0, _stringTrim2.default)(value.replace(/^\./, ''));
                break;
            default:
                break;
        }
    }
    return parsedCookie;
}
function formatClientString(parsedCookie) {
    // eslint-disable-next-line no-restricted-properties
    var cookieStr = parsedCookie.value || '';
    if (parsedCookie.key !== '') cookieStr = parsedCookie.key + '=' + cookieStr;
    return cookieStr;
}
function setDefaultValues(parsedCookie, _ref) {
    var hostname = _ref.hostname,
        pathname = _ref.pathname;

    if (!parsedCookie.domain) parsedCookie.domain = hostname; // eslint-disable-line no-restricted-properties
    if (!parsedCookie.path || parsedCookie.path.charAt(0) !== '/') {
        var path = pathname; // eslint-disable-line no-restricted-properties
        var defaultPath = path.slice(0, path.lastIndexOf('/'));
        parsedCookie.path = defaultPath || '/';
    }
    if (!parsedCookie.expires) parsedCookie.expires = 'Infinity';
}
function domainMatch(currentDomain, cookieDomain) {
    if (!cookieDomain) return true;
    currentDomain = currentDomain.toLowerCase();
    cookieDomain = cookieDomain.toLowerCase();
    if (currentDomain === cookieDomain) return true;
    var cookieDomainIdx = currentDomain.indexOf(cookieDomain);
    return cookieDomainIdx > 0 && currentDomain.length === cookieDomain.length + cookieDomainIdx && currentDomain.charAt(cookieDomainIdx - 1) === '.';
}
function pathMatch(currentPath, cookiePath) {
    if (!cookiePath || cookiePath.charAt(0) !== '/' || currentPath === cookiePath) return true;
    return currentPath.length > cookiePath.length && currentPath.indexOf(cookiePath) === 0 && (cookiePath.charAt(cookiePath.length - 1) === '/' || currentPath.charAt(cookiePath.length) === '/');
}
function getUTCDate(timestamp) {
    if (!arguments.length) timestamp = _nativeMethods2.default.dateNow();else if (isNaN(timestamp)) return null;
    // NOTE: remove milliseconds
    timestamp = Math.floor(timestamp / 1000) * 1000;
    return new _nativeMethods2.default.date(timestamp); // eslint-disable-line new-cap
}