'use strict';

exports.__esModule = true;
exports.getLocation = getLocation;
exports.forceLocation = forceLocation;
exports.sameOriginCheck = sameOriginCheck;
exports.resolveUrl = resolveUrl;
exports.get = get;
exports.withHash = withHash;
exports.getParsed = getParsed;
exports.getOriginHeader = getOriginHeader;

var _url = require('../../utils/url');

var sharedUrlUtils = _interopRequireWildcard(_url);

var _dom = require('./dom');

var domUtils = _interopRequireWildcard(_dom);

var _urlResolver = require('./url-resolver');

var urlResolver = _interopRequireWildcard(_urlResolver);

var _settings = require('../settings');

var _settings2 = _interopRequireDefault(_settings);

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var forcedLocation = null;
// NOTE: exposed only for tests
function getLocation() {
    // NOTE: Used for testing. Unfortunately, we cannot override the 'getLocation' method in a test.
    if (forcedLocation) return forcedLocation;
    var frameElement = domUtils.getFrameElement(window);
    // NOTE: Fallback to the owner page's URL if we are in an iframe without src.
    if (frameElement && domUtils.isIframeWithoutSrc(frameElement)) return _settings2.default.get().referer;
    return window.location.toString();
}
// NOTE: We need to be able to force the page location. During the test, Hammerhead should think that it is on the
// proxied page, not in the test environment. Unfortunately, we cannot do it in any other way.
function forceLocation(url) {
    forcedLocation = url;
}
function sameOriginCheck(location, checkedUrl) {
    if (checkedUrl) checkedUrl = resolveUrl(checkedUrl);
    return sharedUrlUtils.sameOriginCheck(location, checkedUrl);
}
function resolveUrl(url, doc) {
    url = sharedUrlUtils.getURLString(url);
    if (url && url.indexOf('//') === 0) {
        // eslint-disable-next-line no-restricted-properties
        var pageProtocol = getParsed().protocol;
        url = pageProtocol + sharedUrlUtils.correctMultipleSlashes(url, pageProtocol);
    } else url = sharedUrlUtils.correctMultipleSlashes(url);
    // @ts-ignore
    return urlResolver.resolve(url, doc || document);
}
function get() {
    var location = getLocation();
    var parsedProxyUrl = sharedUrlUtils.parseProxyUrl(location);
    return parsedProxyUrl ? parsedProxyUrl.destUrl : location;
}
function withHash(hash) {
    var location = get();
    // NOTE: Remove the previous hash if there is any.
    location = location.replace(/(#.*)$/, '');
    return location + hash;
}
function getParsed() {
    // @ts-ignore
    var resolver = urlResolver.getResolverElement(document);
    var dest = get();
    // eslint-disable-next-line no-restricted-properties
    var destPort = sharedUrlUtils.parseUrl(dest).port;
    // NOTE: IE browser adds the default port for the https protocol while resolving.
    _nativeMethods2.default.anchorHrefSetter.call(resolver, get());
    var hostname = _nativeMethods2.default.anchorHostnameGetter.call(resolver);
    var pathname = _nativeMethods2.default.anchorPathnameGetter.call(resolver);
    // NOTE: IE ignores the first '/' symbol in the pathname.
    if (pathname.charAt(0) !== '/') pathname = '/' + pathname;
    // TODO: Describe default ports logic.
    return {
        protocol: _nativeMethods2.default.anchorProtocolGetter.call(resolver),
        // NOTE: Remove the default port.
        port: destPort ? _nativeMethods2.default.anchorPortGetter.call(resolver) : '',
        hostname: hostname,
        // NOTE: Remove the default port from the host.
        host: destPort ? _nativeMethods2.default.anchorHostGetter.call(resolver) : hostname,
        pathname: pathname,
        hash: resolver.hash,
        search: _nativeMethods2.default.anchorSearchGetter.call(resolver)
    };
}
function getOriginHeader() {
    var parsedDest = getParsed();
    // eslint-disable-next-line no-restricted-properties
    return parsedDest.protocol === 'file:' ? get() : sharedUrlUtils.getDomain(parsedDest);
}