'use strict';

exports.__esModule = true;
exports.instanceToString = instanceToString;
exports.getActiveElement = getActiveElement;
exports.getChildVisibleIndex = getChildVisibleIndex;
exports.getIframeByElement = getIframeByElement;
exports.getIframeLocation = getIframeLocation;
exports.getFrameElement = getFrameElement;
exports.getMapContainer = getMapContainer;
exports.getParentWindowWithSrc = getParentWindowWithSrc;
exports.getScrollbarSize = getScrollbarSize;
exports.getSelectParent = getSelectParent;
exports.getSelectVisibleChildren = getSelectVisibleChildren;
exports.getTopSameDomainWindow = getTopSameDomainWindow;
exports.find = find;
exports.findDocument = findDocument;
exports.isContentEditableElement = isContentEditableElement;
exports.isCrossDomainIframe = isCrossDomainIframe;
exports.isCrossDomainWindows = isCrossDomainWindows;
exports.isDomElement = isDomElement;
exports.getTagName = getTagName;
exports.isElementInDocument = isElementInDocument;
exports.isElementInIframe = isElementInIframe;
exports.isHammerheadAttr = isHammerheadAttr;
exports.isIframeElement = isIframeElement;
exports.isIframeWithoutSrc = isIframeWithoutSrc;
exports.isImgElement = isImgElement;
exports.isInputElement = isInputElement;
exports.isButtonElement = isButtonElement;
exports.isHtmlElement = isHtmlElement;
exports.isBodyElement = isBodyElement;
exports.isHeadElement = isHeadElement;
exports.isHeadOrBodyElement = isHeadOrBodyElement;
exports.isBaseElement = isBaseElement;
exports.isScriptElement = isScriptElement;
exports.isStyleElement = isStyleElement;
exports.isLabelElement = isLabelElement;
exports.isTextAreaElement = isTextAreaElement;
exports.isOptionElement = isOptionElement;
exports.isSelectElement = isSelectElement;
exports.isFormElement = isFormElement;
exports.isFileInput = isFileInput;
exports.isBodyElementWithChildren = isBodyElementWithChildren;
exports.isMapElement = isMapElement;
exports.isRenderedNode = isRenderedNode;
exports.getTabIndex = getTabIndex;
exports.isElementFocusable = isElementFocusable;
exports.isShadowUIElement = isShadowUIElement;
exports.isWindow = isWindow;
exports.isDocument = isDocument;
exports.isBlob = isBlob;
exports.isLocation = isLocation;
exports.isSVGElement = isSVGElement;
exports.isSVGElementOrChild = isSVGElementOrChild;
exports.isFetchHeaders = isFetchHeaders;
exports.isFetchRequest = isFetchRequest;
exports.isElementReadOnly = isElementReadOnly;
exports.isTextEditableInput = isTextEditableInput;
exports.isTextEditableElement = isTextEditableElement;
exports.isTextEditableElementAndEditingAllowed = isTextEditableElementAndEditingAllowed;
exports.isElementNode = isElementNode;
exports.isTextNode = isTextNode;
exports.isProcessingInstructionNode = isProcessingInstructionNode;
exports.isCommentNode = isCommentNode;
exports.isDocumentFragmentNode = isDocumentFragmentNode;
exports.isShadowRoot = isShadowRoot;
exports.isAnchorElement = isAnchorElement;
exports.isTableElement = isTableElement;
exports.isTableDataCellElement = isTableDataCellElement;
exports.isWebSocket = isWebSocket;
exports.isMessageEvent = isMessageEvent;
exports.isPerformanceNavigationTiming = isPerformanceNavigationTiming;
exports.isArrayBuffer = isArrayBuffer;
exports.isArrayBufferView = isArrayBufferView;
exports.isDataView = isDataView;
exports.matches = matches;
exports.closest = closest;
exports.addClass = addClass;
exports.removeClass = removeClass;
exports.hasClass = hasClass;
exports.parseDocumentCharset = parseDocumentCharset;
exports.getParents = getParents;
exports.findParent = findParent;
exports.nodeListToArray = nodeListToArray;
exports.getFileInputs = getFileInputs;
exports.getIframes = getIframes;
exports.getScripts = getScripts;

var _internalAttributes = require('../../processing/dom/internal-attributes');

var _internalAttributes2 = _interopRequireDefault(_internalAttributes);

var _internalProperties = require('../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _className = require('../../shadow-ui/class-name');

var _className2 = _interopRequireDefault(_className);

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _url = require('./url');

var urlUtils = _interopRequireWildcard(_url);

var _style = require('./style');

var _destinationLocation = require('./destination-location');

var _browser = require('./browser');

var _querySelector = require('./query-selector');

var _featureDetection = require('../utils/feature-detection');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var scrollbarSize = null;
var NATIVE_MAP_ELEMENT_STRINGS = ['[object HTMLMapElement]', '[object HTMLAreaElement]'];
var NATIVE_WINDOW_STR = instanceToString(window);
var IS_DOCUMENT_RE = /^\[object .*?Document]$/i;
var IS_PROCESSING_INSTRUCTION_RE = /^\[object .*?ProcessingInstruction]$/i;
var IS_SVG_ELEMENT_RE = /^\[object SVG\w+?Element]$/i;
var IS_HTML_ELEMENT_RE = /^\[object HTML.*?Element]$/i;
var IS_ARRAY_BUFFER_RE = /^\[object ArrayBuffer]$/i;
var IS_DATA_VIEW_RE = /^\[object DataView]$/i;
var NATIVE_TABLE_CELL_STR = instanceToString(_nativeMethods2.default.createElement.call(document, 'td'));
var ELEMENT_NODE_TYPE = Node.ELEMENT_NODE;
var NOT_CONTENT_EDITABLE_ELEMENTS_RE = /^(select|option|applet|area|audio|canvas|datalist|keygen|map|meter|object|progress|source|track|video|img)$/;
var INPUT_ELEMENTS_RE = /^(input|textarea|button)$/;
var SCRIPT_OR_STYLE_RE = /^(script|style)$/i;
var EDITABLE_INPUT_TYPES_RE = /^(email|number|password|search|tel|text|url)$/;
function getFocusableSelector() {
    // NOTE: We don't take into account the case of embedded contentEditable elements, and we
    // specify the contentEditable attribute for focusable elements.
    return 'input, select, textarea, button, body, iframe, [contenteditable="true"], [contenteditable=""], [tabIndex]';
}
function isHidden(el) {
    return el.offsetWidth <= 0 && el.offsetHeight <= 0;
}
function isAlwaysNotEditableElement(el) {
    var tagName = getTagName(el);
    return tagName && (NOT_CONTENT_EDITABLE_ELEMENTS_RE.test(tagName) || INPUT_ELEMENTS_RE.test(tagName));
}
function closestFallback(el, selector) {
    while (el) {
        if (matches(el, selector)) return el;
        el = el.parentNode;
    }
    return null;
}
function instanceToString(instance) {
    if (!_featureDetection.instanceAndPrototypeToStringAreEqual) return _nativeMethods2.default.objectToString.call(instance);
    return instance && typeof instance === 'object' ? _nativeMethods2.default.objectToString.call(_nativeMethods2.default.objectGetPrototypeOf(instance)) : '';
}
function getActiveElement(currentDocument) {
    // NOTE: Sometimes document.activeElement returns an empty object or null (IE11).
    // https://github.com/DevExpress/testcafe-hammerhead/issues/768
    var doc = currentDocument || document;
    var activeElement = _nativeMethods2.default.documentActiveElementGetter.call(doc);
    var el = isDomElement(activeElement) ? activeElement : doc.body;
    while (el && el.shadowRoot) {
        // eslint-disable-next-line no-restricted-properties
        var shadowEl = el.shadowRoot.activeElement;
        if (!shadowEl) break;
        el = shadowEl;
    }
    return el;
}
function getChildVisibleIndex(select, child) {
    var childrenArray = getSelectVisibleChildren(select);
    return childrenArray.indexOf(child);
}
function getIframeByElement(el) {
    var elWindow = el[_internalProperties2.default.processedContext];
    return getFrameElement(elWindow);
}
function getIframeLocation(iframe) {
    var documentLocation = null;
    try {
        // eslint-disable-next-line no-restricted-properties
        documentLocation = _nativeMethods2.default.contentDocumentGetter.call(iframe).location.href;
    } catch (e) {
        documentLocation = null;
    }
    var srcLocation = _nativeMethods2.default.getAttribute.call(iframe, 'src' + _internalAttributes2.default.storedAttrPostfix) || _nativeMethods2.default.getAttribute.call(iframe, 'src') || _nativeMethods2.default.iframeSrcGetter.call(iframe);
    var parsedProxyDocumentLocation = documentLocation && urlUtils.isSupportedProtocol(documentLocation) && urlUtils.parseProxyUrl(documentLocation);
    var parsedProxySrcLocation = srcLocation && urlUtils.isSupportedProtocol(srcLocation) && urlUtils.parseProxyUrl(srcLocation);
    return {
        documentLocation: parsedProxyDocumentLocation ? parsedProxyDocumentLocation.destUrl : documentLocation,
        srcLocation: parsedProxySrcLocation ? parsedProxySrcLocation.destUrl : srcLocation
    };
}
function getFrameElement(win) {
    try {
        return win.frameElement;
    } catch (e) {
        return null;
    }
}
function getMapContainer(el) {
    var closestMap = closest(el, 'map');
    var closestMapName = _nativeMethods2.default.getAttribute.call(closestMap, 'name');
    var containerSelector = '[usemap="#' + closestMapName + '"]';
    return _nativeMethods2.default.querySelector.call(findDocument(el), containerSelector);
}
function getParentWindowWithSrc(window) {
    var parent = window.parent;
    var parentFrameElement = null;
    if (window === window.top) return window;
    if (parent === window.top || isCrossDomainWindows(window, parent)) return parent;
    try {
        parentFrameElement = parent.frameElement;
    } catch (e) {
        parentFrameElement = null;
    }
    if (parentFrameElement === null || !isIframeWithoutSrc(parentFrameElement)) return parent;
    return getParentWindowWithSrc(parent);
}
function getScrollbarSize() {
    if (!scrollbarSize) {
        var scrollDiv = _nativeMethods2.default.createElement.call(document, 'div');
        scrollDiv.style.height = '100px';
        scrollDiv.style.overflow = 'scroll';
        scrollDiv.style.position = 'absolute';
        scrollDiv.style.top = '-9999px';
        scrollDiv.style.width = '100px';
        _nativeMethods2.default.appendChild.call(document.body, scrollDiv);
        var scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth;
        scrollbarSize = scrollbarWidth;
        scrollDiv.parentNode.removeChild(scrollDiv);
    }
    return scrollbarSize;
}
function getSelectParent(child) {
    return closest(child.parentNode, 'select');
}
function getSelectVisibleChildren(select) {
    var children = _nativeMethods2.default.elementQuerySelectorAll.call(select, 'optgroup, option');
    var result = [];
    var length = _nativeMethods2.default.nodeListLengthGetter.call(children);
    for (var i = 0; i < length; i++) {
        var child = children[i];
        // NOTE: Firefox does not display groups without a label and with an empty label.
        var shouldAdd = _browser.isFirefox ? getTagName(child) !== 'optgroup' || child.label : true;
        if (shouldAdd) result.push(child);
    }
    return result;
}
function getTopSameDomainWindow(window) {
    var result = window;
    var currentWindow = window.parent;
    if (result === window.top) return result;
    while (currentWindow) {
        if (!isCrossDomainWindows(window, currentWindow)) {
            var frameElement = getFrameElement(currentWindow);
            if (!frameElement || !isIframeWithoutSrc(frameElement)) result = currentWindow;
        }
        currentWindow = currentWindow !== window.top ? currentWindow.parent : null;
    }
    return result;
}
function find(parent, selector, handler) {
    var nodeList = (0, _querySelector.getNativeQuerySelectorAll)(parent).call(parent, selector);
    if (handler) {
        var length = _nativeMethods2.default.nodeListLengthGetter.call(nodeList);
        for (var i = 0; i < length; i++) {
            handler(nodeList[i]);
        }
    }
    return nodeList;
}
function findDocument(el) {
    if (el.documentElement) return el;
    if (el.ownerDocument && el.ownerDocument.defaultView) return el.ownerDocument;
    return el.parentNode ? findDocument(el.parentNode) : document;
}
function isContentEditableElement(el) {
    var isContentEditable = false;
    var element = null;
    if (isTextNode(el)) element = el.parentElement || el.parentNode;else element = el;
    if (element) {
        isContentEditable = element.isContentEditable && !isAlwaysNotEditableElement(element);
        return isRenderedNode(element) && (isContentEditable || findDocument(el).designMode === 'on');
    }
    return false;
}
function isCrossDomainIframe(iframe, bySrc) {
    var iframeLocation = getIframeLocation(iframe);
    if (!bySrc && iframeLocation.documentLocation === null) return true;
    var currentLocation = bySrc ? iframeLocation.srcLocation : iframeLocation.documentLocation;
    if (currentLocation && urlUtils.isSupportedProtocol(currentLocation)) return !(0, _destinationLocation.sameOriginCheck)(location.toString(), currentLocation);
    return false;
}
function isCrossDomainWindows(window1, window2) {
    try {
        if (window1 === window2) return false;
        var window1Location = window1.location.toString();
        var window2Location = window2.location.toString();
        if (!urlUtils.isSupportedProtocol(window1Location) || !urlUtils.isSupportedProtocol(window2Location)) return false;
        return !(0, _destinationLocation.sameOriginCheck)(window1Location, window2Location);
    } catch (e) {
        return true;
    }
}
function isDomElement(el) {
    if (el instanceof _nativeMethods2.default.elementClass) return true;
    return el && IS_HTML_ELEMENT_RE.test(instanceToString(el)) && isElementNode(el) && el.tagName;
}
function getTagName(el) {
    // NOTE: Check for tagName being a string, because it may be a function in an Angular app (T175340).
    return el && typeof el.tagName === 'string' ? el.tagName.toLowerCase() : '';
}
function isElementInDocument(el, currentDocument) {
    var doc = currentDocument || document;
    return doc.documentElement.contains(el);
}
function isElementInIframe(el, currentDocument) {
    var doc = currentDocument || findDocument(el);
    return window.document !== doc;
}
function isHammerheadAttr(attr) {
    return attr === _internalAttributes2.default.focusPseudoClass || attr === _internalAttributes2.default.hoverPseudoClass || attr.indexOf(_internalAttributes2.default.storedAttrPostfix) !== -1;
}
function isIframeElement(el) {
    return instanceToString(el) === '[object HTMLIFrameElement]';
}
function isIframeWithoutSrc(iframe) {
    var iframeLocation = getIframeLocation(iframe);
    var iframeSrcLocation = iframeLocation.srcLocation;
    var iframeDocumentLocation = iframeLocation.documentLocation;
    // NOTE: is a cross-domain iframe
    if (iframeDocumentLocation === null) return false;
    // NOTE: after 'document.write' or 'document.open' call for iframe with/without src
    // we will process it as iframe without src
    if (_nativeMethods2.default.contentWindowGetter.call(iframe)[_internalProperties2.default.documentWasCleaned]) return true;
    var iframeDocumentLocationHaveSupportedProtocol = urlUtils.isSupportedProtocol(iframeDocumentLocation);
    // NOTE: When an iframe has an empty src attribute (<iframe src></iframe>) or has no src attribute (<iframe></iframe>),
    // the iframe.src property is not empty but has different values in different browsers.
    // Its document location is 'about:blank'. Therefore, we should check the src attribute.
    if (!iframeDocumentLocationHaveSupportedProtocol && !_nativeMethods2.default.getAttribute.call(iframe, 'src')) return true;
    // In Chrome, when an iframe with the src attribute is added to DOM,
    // its documentLocation is set to "about:blank" until the iframe has been loaded.
    // So, we should check srcLocation in this case.
    if (iframeSrcLocation && urlUtils.isSupportedProtocol(iframeSrcLocation)) return false;
    return !iframeDocumentLocationHaveSupportedProtocol;
}
function isImgElement(el) {
    return instanceToString(el) === '[object HTMLImageElement]';
}
function isInputElement(el) {
    return instanceToString(el) === '[object HTMLInputElement]';
}
function isButtonElement(el) {
    return instanceToString(el) === '[object HTMLButtonElement]';
}
function isHtmlElement(el) {
    return instanceToString(el) === '[object HTMLHtmlElement]';
}
function isBodyElement(el) {
    return instanceToString(el) === '[object HTMLBodyElement]';
}
function isHeadElement(el) {
    return instanceToString(el) === '[object HTMLHeadElement]';
}
function isHeadOrBodyElement(el) {
    var elString = instanceToString(el);
    return elString === '[object HTMLHeadElement]' || elString === '[object HTMLBodyElement]';
}
function isBaseElement(el) {
    return instanceToString(el) === '[object HTMLBaseElement]';
}
function isScriptElement(el) {
    return instanceToString(el) === '[object HTMLScriptElement]';
}
function isStyleElement(el) {
    return instanceToString(el) === '[object HTMLStyleElement]';
}
function isLabelElement(el) {
    return instanceToString(el) === '[object HTMLLabelElement]';
}
function isTextAreaElement(el) {
    return instanceToString(el) === '[object HTMLTextAreaElement]';
}
function isOptionElement(el) {
    return instanceToString(el) === '[object HTMLOptionElement]';
}
function isSelectElement(el) {
    return instanceToString(el) === '[object HTMLSelectElement]';
}
function isFormElement(el) {
    return instanceToString(el) === '[object HTMLFormElement]';
}
function isFileInput(el) {
    return isInputElement(el) && el.type.toLowerCase() === 'file';
}
function isBodyElementWithChildren(el) {
    return isBodyElement(el) && _nativeMethods2.default.htmlCollectionLengthGetter.call(el.children);
}
function isMapElement(el) {
    return NATIVE_MAP_ELEMENT_STRINGS.indexOf(instanceToString(el)) !== -1;
}
function isRenderedNode(node) {
    return !(isProcessingInstructionNode(node) || isCommentNode(node) || SCRIPT_OR_STYLE_RE.test(node.nodeName));
}
function getTabIndex(el) {
    // NOTE: we obtain the tabIndex value from an attribute because the el.tabIndex
    // property returns -1 for some elements (e.g. for body) with no tabIndex assigned
    var tabIndex = _nativeMethods2.default.getAttribute.call(el, 'tabIndex');
    tabIndex = parseInt(tabIndex, 10);
    return isNaN(tabIndex) ? null : tabIndex;
}
function isElementFocusable(el) {
    if (!el) return false;
    var tabIndex = getTabIndex(el);
    var isDisabledElement = matches(el, ':disabled');
    var isInvisibleElement = (0, _style.get)(el, 'visibility') === 'hidden';
    var isNotDisplayedElement = (0, _style.get)(el, 'display') === 'none';
    var isHiddenElement = _browser.isWebKit ? isHidden(el) && !isOptionElement(el) : isHidden(el);
    if (isDisabledElement || isInvisibleElement || isNotDisplayedElement || isHiddenElement) return false;
    if (isOptionElement(el) && _browser.isIE) return false;
    if (isAnchorElement(el)) {
        if (tabIndex !== null) return true;
        return matches(el, 'a[href]');
    }
    if (isTableDataCellElement(el) && _browser.isIE) return true;
    return matches(el, getFocusableSelector()) || tabIndex !== null;
}
function isShadowUIElement(element) {
    return !!element[_internalProperties2.default.shadowUIElement];
}
function isWindow(instance) {
    if (instance instanceof _nativeMethods2.default.windowClass) return true;
    try {
        // NOTE: The instanceToString call result has a strange values for the MessageEvent.target property:
        // * [object DispHTMLWindow2] for IE11
        // * [object Object] for MSEdge.
        if ((_browser.isIE || _browser.isMSEdge) && instance && instance === instance.window) instance = instance.window;
        return instance && instance.toString && NATIVE_WINDOW_STR === instanceToString(instance);
    } catch (e) {
        try {
            // NOTE: If a cross-domain object has the 'top' field, this object is a window
            // (not a document or location).
            return !!instance.top;
        } catch (x) {
            return false;
        }
    }
}
function isDocument(instance) {
    if (instance instanceof _nativeMethods2.default.documentClass) return true;
    try {
        return instance && IS_DOCUMENT_RE.test(instanceToString(instance));
    } catch (e) {
        // NOTE: For cross-domain objects (windows, documents or locations), we return false because
        // it's impossible to work with them in any case.
        return false;
    }
}
function isBlob(instance) {
    return instance && instanceToString(instance) === '[object Blob]';
}
function isLocation(instance) {
    if (instance instanceof _nativeMethods2.default.locationClass || _nativeMethods2.default.objectToString.call(instance) === '[object Location]') return true;
    try {
        // eslint-disable-next-line no-restricted-properties
        return instance && typeof instance === 'object' && instance.href !== void 0 && instance.assign !== void 0;
    } catch (e) {
        // NOTE: Try to detect cross-domain window location.
        // A cross-domain location has no the "assign" function in Safari.
        return instance.replace && (_browser.isSafari || !!instance.assign);
    }
}
function isSVGElement(instance) {
    if (instance instanceof _nativeMethods2.default.svgElementClass) return true;
    return instance && IS_SVG_ELEMENT_RE.test(instanceToString(instance));
}
function isSVGElementOrChild(el) {
    return !!closest(el, 'svg');
}
function isFetchHeaders(instance) {
    if (_nativeMethods2.default.Headers && instance instanceof _nativeMethods2.default.Headers) return true;
    return instance && instanceToString(instance) === '[object Headers]';
}
function isFetchRequest(instance) {
    if (_nativeMethods2.default.Request && instance instanceof _nativeMethods2.default.Request) return true;
    return instance && instanceToString(instance) === '[object Request]';
}
function isElementReadOnly(el) {
    return el.readOnly || el.getAttribute('readonly') === 'readonly';
}
function isTextEditableInput(el) {
    var attrType = el.getAttribute('type');
    return isInputElement(el) && attrType ? EDITABLE_INPUT_TYPES_RE.test(attrType) : EDITABLE_INPUT_TYPES_RE.test(el.type);
}
function isTextEditableElement(el) {
    return isTextEditableInput(el) || isTextAreaElement(el);
}
function isTextEditableElementAndEditingAllowed(el) {
    return isTextEditableElement(el) && !isElementReadOnly(el);
}
function isElementNode(node) {
    return node && node.nodeType === ELEMENT_NODE_TYPE;
}
function isTextNode(node) {
    return instanceToString(node) === '[object Text]';
}
function isProcessingInstructionNode(node) {
    return IS_PROCESSING_INSTRUCTION_RE.test(instanceToString(node));
}
function isCommentNode(node) {
    return instanceToString(node) === '[object Comment]';
}
function isDocumentFragmentNode(node) {
    return instanceToString(node) === '[object DocumentFragment]';
}
function isShadowRoot(root) {
    return instanceToString(root) === '[object ShadowRoot]';
}
function isAnchorElement(el) {
    return instanceToString(el) === '[object HTMLAnchorElement]';
}
function isTableElement(el) {
    return instanceToString(el) === '[object HTMLTableElement]';
}
function isTableDataCellElement(el) {
    return instanceToString(el) === NATIVE_TABLE_CELL_STR;
}
function isWebSocket(ws) {
    return instanceToString(ws) === '[object WebSocket]';
}
function isMessageEvent(e) {
    return instanceToString(e) === '[object MessageEvent]';
}
function isPerformanceNavigationTiming(entry) {
    return instanceToString(entry) === '[object PerformanceNavigationTiming]';
}
function isArrayBuffer(data) {
    if (data instanceof _nativeMethods2.default.ArrayBuffer) return true;
    return data && IS_ARRAY_BUFFER_RE.test(instanceToString(data));
}
function isArrayBufferView(data) {
    return data && _nativeMethods2.default.arrayBufferIsView(data);
}
function isDataView(data) {
    if (data instanceof _nativeMethods2.default.DataView) return true;
    return data && IS_DATA_VIEW_RE.test(instanceToString(data));
}
function matches(el, selector) {
    if (!isElementNode(el)) return false;
    return _nativeMethods2.default.matches.call(el, selector);
}
function closest(el, selector) {
    if (!isElementNode(el)) return null;
    if (_nativeMethods2.default.closest) return _nativeMethods2.default.closest.call(el, selector);
    return closestFallback(el, selector);
}
function addClass(el, className) {
    if (!el) return;
    var classNames = className.split(/\s+/);

    for (var _i2 = 0; _i2 < classNames.length; _i2++) {
        var currentClassName = classNames[_i2];
        _nativeMethods2.default.tokenListAdd.call(_nativeMethods2.default.elementClassListGetter.call(el), currentClassName);
    }
}
function removeClass(el, className) {
    if (!el) return;
    var classNames = className.split(/\s+/);

    for (var _i4 = 0; _i4 < classNames.length; _i4++) {
        var currentClassName = classNames[_i4];
        _nativeMethods2.default.tokenListRemove.call(_nativeMethods2.default.elementClassListGetter.call(el), currentClassName);
    }
}
function hasClass(el, className) {
    if (!el) return false;
    return _nativeMethods2.default.tokenListContains.call(_nativeMethods2.default.elementClassListGetter.call(el), className);
}
function parseDocumentCharset() {
    var metaCharset = _nativeMethods2.default.querySelector.call(document, '.' + _className2.default.charset);
    return metaCharset && metaCharset.getAttribute('charset');
}
function getParents(el, selector) {
    // eslint-disable-next-line no-restricted-properties
    var parent = el.parentNode || el.host;
    var parents = [];
    while (parent) {
        if (!selector && isElementNode(parent) || selector && matches(parent, selector)) parents.push(parent);
        // eslint-disable-next-line no-restricted-properties
        parent = parent.parentNode || parent.host;
    }
    return parents;
}
function findParent(node) {
    var includeSelf = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    var predicate = arguments[2];

    if (!includeSelf) node = node.parentNode;
    while (node) {
        if (typeof predicate !== 'function' || predicate(node)) return node;
        node = node.parentNode;
    }
    return null;
}
function nodeListToArray(nodeList) {
    var result = [];
    var length = _nativeMethods2.default.nodeListLengthGetter.call(nodeList);
    for (var i = 0; i < length; i++) {
        result.push(nodeList[i]);
    }return result;
}
function getFileInputs(el) {
    return isFileInput(el) ? [el] : nodeListToArray((0, _querySelector.getNativeQuerySelectorAll)(el).call(el, 'input[type=file]'));
}
function getIframes(el) {
    return isIframeElement(el) ? [el] : nodeListToArray((0, _querySelector.getNativeQuerySelectorAll)(el).call(el, 'iframe,frame'));
}
function getScripts(el) {
    return isScriptElement(el) ? [el] : nodeListToArray((0, _querySelector.getNativeQuerySelectorAll)(el).call(el, 'script'));
}