'use strict';

exports.__esModule = true;

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _browser = require('./browser');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var EventEmitter = function () {
    function EventEmitter() {
        _classCallCheck(this, EventEmitter);

        this.eventsListeners = _nativeMethods2.default.objectCreate(null);
    }

    EventEmitter.prototype.emit = function emit(evt) {
        var listeners = this.eventsListeners[evt];
        if (!listeners) return;
        var index = 0;

        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
        }

        while (listeners[index]) {
            // HACK: For IE: after calling document.write, the IFrameSandbox event handler throws the
            // 'Can't execute code from a freed script' exception because the document has been
            // recreated.
            if (_browser.isIE) {
                try {
                    listeners[index].toString();
                } catch (e) {
                    _nativeMethods2.default.arraySplice.call(listeners, index, 1);
                    continue;
                }
            }
            listeners[index++].apply(this, args);
        }
    };

    EventEmitter.prototype.off = function off(evt, listener) {
        var listeners = this.eventsListeners[evt];
        if (!listeners) return;
        this.eventsListeners[evt] = _nativeMethods2.default.arrayFilter.call(listeners, function (currentListener) {
            return currentListener !== listener;
        });
    };

    EventEmitter.prototype.on = function on(evt, listener) {
        this.eventsListeners[evt] = this.eventsListeners[evt] || [];
        if (this.eventsListeners[evt].indexOf(listener) === -1) this.eventsListeners[evt].push(listener);
        return listener;
    };

    return EventEmitter;
}();

exports.default = EventEmitter;
module.exports = exports['default'];