'use strict';

exports.__esModule = true;
exports.INIT_SCRIPT_FOR_IFRAME_TEMPLATE = undefined;
exports.isPageHtml = isPageHtml;
exports.cleanUpHtml = cleanUpHtml;
exports.processHtml = processHtml;
exports.dispose = dispose;
exports.isInternalHtmlParserElement = isInternalHtmlParserElement;

var _internalAttributes = require('../../processing/dom/internal-attributes');

var _internalAttributes2 = _interopRequireDefault(_internalAttributes);

var _className = require('../../shadow-ui/class-name');

var _className2 = _interopRequireDefault(_className);

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _dom = require('../../processing/dom');

var _dom2 = _interopRequireDefault(_dom);

var _domProcessor = require('../dom-processor');

var _domProcessor2 = _interopRequireDefault(_domProcessor);

var _header = require('../../processing/script/header');

var _style = require('../../processing/style');

var _style2 = _interopRequireDefault(_style);

var _dom3 = require('./dom');

var _url = require('./url');

var _browser = require('./browser');

var _urlResolver = require('./url-resolver');

var urlResolver = _interopRequireWildcard(_urlResolver);

var _internalProperties = require('../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _attributes = require('../../processing/dom/attributes');

var _createSelfRemovingScript = require('../../utils/create-self-removing-script');

var _createSelfRemovingScript2 = _interopRequireDefault(_createSelfRemovingScript);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var FAKE_TAG_NAME_PREFIX = 'hh_fake_tag_name_';
var FAKE_DOCTYPE_TAG_NAME = 'hh_fake_doctype';
var FAKE_HEAD_TAG_NAME = FAKE_TAG_NAME_PREFIX + 'head';
var FAKE_BODY_TAG_NAME = FAKE_TAG_NAME_PREFIX + 'body';
var FAKE_ATTR_WITH_TAG_NAME = 'hh_fake_attr';
var FAKE_TAG_NAME_RE = new RegExp('(<\\/?)' + FAKE_TAG_NAME_PREFIX, 'ig');
var WRAP_TAGS_RE = /(<\/?)(html|head|body|table|tbody|tfoot|thead|tr|td|th|caption|colgroup)((?:\s[^>]*)?>)/ig;
var WRAP_TAGS_TEMPLATE = '$1' + FAKE_TAG_NAME_PREFIX + '$2$3';
var WRAP_COL_NOSCRIPT_TAGS_RE = /<(\/?(?:col|noscript))(\s[^>]*?)?(\s?\/)?>/ig;
var WRAP_COL_NOSCRIPT_TAGS_TEMPLATE = '<br ' + FAKE_ATTR_WITH_TAG_NAME + '="$1|$3"$2>';
var UNWRAP_COL_NOSCRIPT_TAGS_RE = new RegExp('<br([^>]*?) ' + FAKE_ATTR_WITH_TAG_NAME + '="([^|]+)\\|([^"]*)"([^>]*)', 'ig');
var WRAP_DOCTYPE_RE = /<!doctype([^>]*)>/ig;
var WRAP_DOCTYPE_TEMPLATE = '<' + FAKE_DOCTYPE_TAG_NAME + '>$1</' + FAKE_DOCTYPE_TAG_NAME + '>';
var UNWRAP_DOCTYPE_RE = new RegExp('<' + FAKE_DOCTYPE_TAG_NAME + '>([\\S\\s]*?)</' + FAKE_DOCTYPE_TAG_NAME + '>', 'ig');
var FIND_SVG_RE = /<svg\s?[^>]*>/ig;
var FIND_NS_ATTRS_RE = /\s(?:NS[0-9]+:[^"']+('|")[\S\s]*?\1|[^:]+:NS[0-9]+=(?:""|''))/g;
var STORED_ATTRS_SELECTOR = function () {
    var storedAttrs = [];

    for (var _i2 = 0; _i2 < _attributes.URL_ATTRS.length; _i2++) {
        var attr = _attributes.URL_ATTRS[_i2];
        storedAttrs.push(_dom2.default.getStoredAttrName(attr));
    }

    for (var _i4 = 0; _i4 < _attributes.ATTRS_WITH_SPECIAL_PROXYING_LOGIC.length; _i4++) {
        var _attr = _attributes.ATTRS_WITH_SPECIAL_PROXYING_LOGIC[_i4];
        storedAttrs.push(_dom2.default.getStoredAttrName(_attr));
    }

    return '[' + storedAttrs.join('],[') + ']';
}();
var SHADOW_UI_ELEMENTS_SELECTOR = '[class*="' + _className2.default.postfix + '"]';
var HOVER_AND_FOCUS_PSEUDO_CLASS_ELEMENTS_SELECTOR = '[' + _internalAttributes2.default.hoverPseudoClass + '],[' + _internalAttributes2.default.focusPseudoClass + ']';
var FAKE_ELEMENTS_SELECTOR = FAKE_HEAD_TAG_NAME + ', ' + FAKE_BODY_TAG_NAME;
var HTML_PARSER_ELEMENT_FLAG = 'hammerhead|html-parser-element-flag';
var INIT_SCRIPT_FOR_IFRAME_TEMPLATE = exports.INIT_SCRIPT_FOR_IFRAME_TEMPLATE = (0, _createSelfRemovingScript2.default)('\n    var parentHammerhead = null;\n    \n    if (!window["' + _internalProperties2.default.hammerhead + '"])\n        Object.defineProperty(window, "' + _internalProperties2.default.documentWasCleaned + '", { value: true, configurable: true });\n\n    try {\n        parentHammerhead = window.parent["' + _internalProperties2.default.hammerhead + '"];\n    } catch(e) {}\n\n    if (parentHammerhead)\n        parentHammerhead.sandbox.onIframeDocumentRecreated(window.frameElement);\n');
var htmlDocument = _nativeMethods2.default.createHTMLDocument.call(document.implementation, 'title');
var htmlParser = _nativeMethods2.default.createDocumentFragment.call(htmlDocument);
htmlParser[HTML_PARSER_ELEMENT_FLAG] = true;
function getHtmlDocument() {
    try {
        // NOTE: IE bug: access denied.
        if (htmlDocument.location) htmlDocument.location.toString();
    } catch (e) {
        htmlDocument = _nativeMethods2.default.createHTMLDocument.call(document.implementation, 'title');
        htmlParser = _nativeMethods2.default.createDocumentFragment.call(htmlDocument);
        htmlParser[HTML_PARSER_ELEMENT_FLAG] = true;
    }
    return htmlDocument;
}
function wrapHtmlText(html) {
    return html.replace(WRAP_DOCTYPE_RE, WRAP_DOCTYPE_TEMPLATE).replace(WRAP_COL_NOSCRIPT_TAGS_RE, WRAP_COL_NOSCRIPT_TAGS_TEMPLATE).replace(WRAP_TAGS_RE, WRAP_TAGS_TEMPLATE);
}
function unwrapHtmlText(html) {
    return html.replace(UNWRAP_DOCTYPE_RE, '<!doctype$1>').replace(UNWRAP_COL_NOSCRIPT_TAGS_RE, '<$2$1$4$3').replace(FAKE_TAG_NAME_RE, '$1');
}
function isPageHtml(html) {
    return (/^\s*(<\s*(!doctype|html|head|body)[^>]*>)/i.test(html)
    );
}
function processHtmlInternal(html, process) {
    var container = _nativeMethods2.default.createElement.call(getHtmlDocument(), 'div');
    html = wrapHtmlText(html);
    _nativeMethods2.default.appendChild.call(htmlParser, container);
    _nativeMethods2.default.elementInnerHTMLSetter.call(container, html);
    var processedHtml = process(container) ? _nativeMethods2.default.elementInnerHTMLGetter.call(container) : html;
    _nativeMethods2.default.removeChild.call(container.parentNode, container);
    processedHtml = unwrapHtmlText(processedHtml);
    // NOTE: hack for IE (GH-1083)
    if (_browser.isIE && !_browser.isMSEdge && html !== processedHtml) processedHtml = removeExtraSvgNamespaces(html, processedHtml);
    return processedHtml;
}
function cleanUpUrlAttr(el) {
    var urlAttr = _domProcessor2.default.getUrlAttr(el);
    if (!urlAttr || !_nativeMethods2.default.hasAttribute.call(el, urlAttr)) return;
    var storedAttr = _dom2.default.getStoredAttrName(urlAttr);
    if (_nativeMethods2.default.hasAttribute.call(el, storedAttr)) {
        _nativeMethods2.default.setAttribute.call(el, urlAttr, _nativeMethods2.default.getAttribute.call(el, storedAttr));
        _nativeMethods2.default.removeAttribute.call(el, storedAttr);
    }
}
function cleanUpAutocompleteAttr(el) {
    if (!_nativeMethods2.default.hasAttribute.call(el, 'autocomplete')) return;
    var storedAttr = _dom2.default.getStoredAttrName('autocomplete');
    if (_nativeMethods2.default.hasAttribute.call(el, storedAttr)) {
        var storedAttrValue = _nativeMethods2.default.getAttribute.call(el, storedAttr);
        if (_dom2.default.isAddedAutocompleteAttr('autocomplete', storedAttrValue)) _nativeMethods2.default.removeAttribute.call(el, 'autocomplete');else _nativeMethods2.default.setAttribute.call(el, 'autocomplete', storedAttrValue);
        _nativeMethods2.default.removeAttribute.call(el, storedAttr);
    }
}
function cleanUpTargetAttr(el) {
    var targetAttr = _domProcessor2.default.getTargetAttr(el);
    if (!targetAttr || !_nativeMethods2.default.hasAttribute.call(el, targetAttr)) return;
    var storedAttr = _dom2.default.getStoredAttrName(targetAttr);
    if (_nativeMethods2.default.hasAttribute.call(el, storedAttr)) {
        _nativeMethods2.default.setAttribute.call(el, targetAttr, _nativeMethods2.default.getAttribute.call(el, storedAttr));
        _nativeMethods2.default.removeAttribute.call(el, storedAttr);
    }
}
function cleanUpSandboxAttr(el) {
    if (_domProcessor2.default.adapter.getTagName(el) !== 'iframe' || !_nativeMethods2.default.hasAttribute.call(el, 'sandbox')) return;
    var storedAttr = _dom2.default.getStoredAttrName('sandbox');
    if (_nativeMethods2.default.hasAttribute.call(el, storedAttr)) {
        _nativeMethods2.default.setAttribute.call(el, 'sandbox', _nativeMethods2.default.getAttribute.call(el, storedAttr));
        _nativeMethods2.default.removeAttribute.call(el, storedAttr);
    }
}
function cleanUpStyleAttr(el) {
    if (!_nativeMethods2.default.hasAttribute.call(el, 'style')) return;
    var storedAttr = _dom2.default.getStoredAttrName('style');
    if (_nativeMethods2.default.hasAttribute.call(el, storedAttr)) {
        _nativeMethods2.default.setAttribute.call(el, 'style', _nativeMethods2.default.getAttribute.call(el, storedAttr));
        _nativeMethods2.default.removeAttribute.call(el, storedAttr);
    }
}
function cleanUpHtml(html) {
    return processHtmlInternal(html, function (container) {
        var changed = false;
        (0, _dom3.find)(container, STORED_ATTRS_SELECTOR, function (el) {
            cleanUpUrlAttr(el);
            cleanUpAutocompleteAttr(el);
            cleanUpTargetAttr(el);
            cleanUpSandboxAttr(el);
            cleanUpStyleAttr(el);
            changed = true;
        });
        (0, _dom3.find)(container, SHADOW_UI_ELEMENTS_SELECTOR, function (el) {
            if (el.parentNode) {
                _nativeMethods2.default.removeChild.call(el.parentNode, el);
                changed = true;
            }
        });
        (0, _dom3.find)(container, 'script', function (el) {
            var textContent = _nativeMethods2.default.nodeTextContentGetter.call(el);
            var cleanedTextContent = (0, _header.remove)(textContent);
            if (textContent !== cleanedTextContent) {
                _nativeMethods2.default.nodeTextContentSetter.call(el, cleanedTextContent);
                changed = true;
            }
        });
        (0, _dom3.find)(container, 'style', function (el) {
            var textContent = _nativeMethods2.default.nodeTextContentGetter.call(el);
            var cleanedTextContent = _style2.default.cleanUp(textContent, _url.parseProxyUrl);
            if (textContent !== cleanedTextContent) {
                _nativeMethods2.default.nodeTextContentSetter.call(el, cleanedTextContent);
                changed = true;
            }
        });
        (0, _dom3.find)(container, HOVER_AND_FOCUS_PSEUDO_CLASS_ELEMENTS_SELECTOR, function (el) {
            _nativeMethods2.default.removeAttribute.call(el, _internalAttributes2.default.hoverPseudoClass);
            _nativeMethods2.default.removeAttribute.call(el, _internalAttributes2.default.focusPseudoClass);
            changed = true;
        });
        (0, _dom3.find)(container, FAKE_ELEMENTS_SELECTOR, function (el) {
            var innerHtml = _nativeMethods2.default.elementInnerHTMLGetter.call(el);
            if (innerHtml.indexOf(INIT_SCRIPT_FOR_IFRAME_TEMPLATE) !== -1) {
                _nativeMethods2.default.elementInnerHTMLSetter.call(el, innerHtml.replace(INIT_SCRIPT_FOR_IFRAME_TEMPLATE, ''));
                changed = true;
            }
        });
        return changed;
    });
}
function processHtml(html) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var parentTag = options.parentTag,
        prepareDom = options.prepareDom,
        processedContext = options.processedContext;

    return processHtmlInternal(html, function (container) {
        var doctypeElement = null;
        var htmlElements = [];
        var children = [];
        var length = 0;
        // @ts-ignore
        var storedBaseUrl = urlResolver.getBaseUrl(document);
        if (prepareDom) prepareDom(container);
        if (_nativeMethods2.default.htmlCollectionLengthGetter.call(container.children)) {
            children = _nativeMethods2.default.elementQuerySelectorAll.call(container, '*');
            length = _nativeMethods2.default.nodeListLengthGetter.call(children);
        }
        var base = _nativeMethods2.default.elementQuerySelector.call(container, 'base');
        if (base)
            // @ts-ignore
            urlResolver.updateBase(_nativeMethods2.default.getAttribute.call(base, 'href'), document);
        for (var i = 0; i < length; i++) {
            var child = children[i];
            if ((0, _dom3.isScriptElement)(child)) {
                var scriptContent = _nativeMethods2.default.nodeTextContentGetter.call(child);
                _nativeMethods2.default.nodeTextContentSetter.call(child, unwrapHtmlText(scriptContent));
            }
            child[_internalProperties2.default.processedContext] = processedContext;
            _domProcessor2.default.processElement(child, _url.convertToProxyUrl);
            var elTagName = (0, _dom3.getTagName)(child);
            if (elTagName === FAKE_HEAD_TAG_NAME || elTagName === FAKE_BODY_TAG_NAME) htmlElements.push(child);else if (elTagName === FAKE_DOCTYPE_TAG_NAME) doctypeElement = child;
        }
        if (!parentTag) {
            if (htmlElements.length) {
                for (var _i6 = 0; _i6 < htmlElements.length; _i6++) {
                    var htmlElement = htmlElements[_i6];
                    _nativeMethods2.default.insertAdjacentHTML.call(htmlElement, 'afterbegin', INIT_SCRIPT_FOR_IFRAME_TEMPLATE);
                }
            } else if (doctypeElement && _browser.isIE) _nativeMethods2.default.insertAdjacentHTML.call(doctypeElement, 'afterend', INIT_SCRIPT_FOR_IFRAME_TEMPLATE);
        }
        // @ts-ignore
        urlResolver.updateBase(storedBaseUrl, document);
        return true;
    });
}
function dispose() {
    htmlParser = null;
    htmlDocument = null;
}
function isInternalHtmlParserElement(el) {
    while (el.parentNode) {
        el = el.parentNode;
    }return !!el[HTML_PARSER_ELEMENT_FLAG];
}
function removeExtraSvgNamespaces(html, processedHtml) {
    var initialSvgStrs = html.match(FIND_SVG_RE);
    var index = 0;
    if (!initialSvgStrs) return processedHtml;
    return processedHtml.replace(FIND_SVG_RE, function (svgStr) {
        var initialSvgStr = initialSvgStrs[index];
        var initialNSAttrs = initialSvgStr ? initialSvgStr.match(FIND_NS_ATTRS_RE) : null;
        if (initialSvgStr) index++;
        return initialSvgStr ? svgStr.replace(FIND_NS_ATTRS_RE, function () {
            var replacement = initialNSAttrs ? initialNSAttrs.join('') : '';
            if (initialNSAttrs) initialNSAttrs = null;
            return replacement;
        }) : svgStr;
    });
}