'use strict';

exports.__esModule = true;
exports.createOverriddenDescriptor = createOverriddenDescriptor;
exports.overrideDescriptor = overrideDescriptor;

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function replaceNativeAccessor(descriptor, accessorName, newAccessor) {
    if (newAccessor && descriptor[accessorName]) {
        var stringifiedNativeAccessor = descriptor[accessorName].toString();
        newAccessor.toString = function () {
            return stringifiedNativeAccessor;
        };
    }
    descriptor[accessorName] = newAccessor;
}
function createOverriddenDescriptor(obj, prop, _ref) {
    var getter = _ref.getter,
        setter = _ref.setter,
        value = _ref.value;

    var descriptor = _nativeMethods2.default.objectGetOwnPropertyDescriptor(obj, prop);
    if ((getter || setter) && value) throw new Error('Cannot both specify accessors and a value or writable attribute.');
    if (value) {
        if (!_nativeMethods2.default.objectHasOwnProperty.call(descriptor, 'writable')) {
            descriptor.writable = !!descriptor.set;
            delete descriptor.get;
            delete descriptor.set;
        }
        descriptor.value = value; // eslint-disable-line no-restricted-properties
    } else {
        if (_nativeMethods2.default.objectHasOwnProperty.call(descriptor, 'writable')) {
            delete descriptor.value; // eslint-disable-line no-restricted-properties
            delete descriptor.writable;
        }
        if (getter !== null) replaceNativeAccessor(descriptor, 'get', getter);
        if (setter !== null) replaceNativeAccessor(descriptor, 'set', setter);
    }
    return descriptor;
}
function overrideDescriptor(obj, prop, propertyAccessors) {
    var descriptor = createOverriddenDescriptor(obj, prop, propertyAccessors);
    _nativeMethods2.default.objectDefineProperty(obj, prop, descriptor);
}