'use strict';

exports.__esModule = true;
exports.default = replaceProxiedUrlsInStack;

var _url = require('./url');

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var STACK_FRAME_REG_EXPS = [/^\s*at .*\((\S+)\)/, /^\s*at (\S+)/, /^.*@(\S+)/, /(.+)/ // Any string
];
var ROW_COLUMN_NUMBER_REG_EX = /:\d+:\d+$/;
function replaceUrlWithProxied(str, source) {
    var proxiedUrl = null;
    var rowColumnSuffix = '';
    if (ROW_COLUMN_NUMBER_REG_EX.test(source)) {
        var sourceMatch = source.match(ROW_COLUMN_NUMBER_REG_EX);
        proxiedUrl = source.substring(0, sourceMatch.index);
        rowColumnSuffix = sourceMatch[0];
    } else proxiedUrl = source;
    var parsedProxiedUrl = (0, _url.parseProxyUrl)(proxiedUrl);
    var destUrl = parsedProxiedUrl && parsedProxiedUrl.destUrl;
    if (!destUrl) return str;
    var newSource = destUrl + rowColumnSuffix;
    return str.replace(source, newSource);
}
function replaceProxiedUrlsInStack(stack) {
    if (!stack) return stack;
    var stackFrames = stack.split('\n');

    var _loop = function _loop(i) {
        var stackFrame = stackFrames[i];
        var targetStackFrameRegExp = _nativeMethods2.default.arrayFilter.call(STACK_FRAME_REG_EXPS, function (stackFrameRegExp) {
            return stackFrameRegExp.test(stackFrame);
        })[0];
        stackFrames[i] = stackFrame.replace(targetStackFrameRegExp, replaceUrlWithProxied);
    };

    for (var i = 0; i < stackFrames.length; i++) {
        _loop(i);
    }
    return stackFrames.join('\n');
}
module.exports = exports['default'];