'use strict';

exports.__esModule = true;
exports.get = get;
exports.set = set;
exports.getBordersWidth = getBordersWidth;
exports.getComputedStyle = getComputedStyle;
exports.getElementMargin = getElementMargin;
exports.getElementPadding = getElementPadding;
exports.getElementScroll = getElementScroll;
exports.getWidth = getWidth;
exports.getHeight = getHeight;
exports.getInnerWidth = getInnerWidth;
exports.getInnerHeight = getInnerHeight;
exports.getOptionHeight = getOptionHeight;
exports.getSelectElementSize = getSelectElementSize;
exports.isVisibleChild = isVisibleChild;
exports.getScrollLeft = getScrollLeft;
exports.getScrollTop = getScrollTop;
exports.setScrollLeft = setScrollLeft;
exports.setScrollTop = setScrollTop;
exports.getOffsetParent = getOffsetParent;
exports.getOffset = getOffset;
exports.isElementVisible = isElementVisible;
exports.isElementInInvisibleIframe = isElementInInvisibleIframe;

var _dom = require('./dom');

var domUtils = _interopRequireWildcard(_dom);

var _browser = require('./browser');

var browserUtils = _interopRequireWildcard(_browser);

var _featureDetection = require('./feature-detection');

var featureDetection = _interopRequireWildcard(_featureDetection);

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

// NOTE: For Chrome.
var MIN_SELECT_SIZE_VALUE = 4;
function getIntValue(value) {
    value = value || '';
    var parsedValue = parseInt(value.replace('px', ''), 10);
    return isNaN(parsedValue) ? 0 : parsedValue;
}
function get(el, property, doc, win) {
    el = el.documentElement || el;
    var computedStyle = getComputedStyle(el, doc, win);
    return computedStyle && computedStyle[property];
}
function set(el, property, value) {
    el = el.documentElement || el;
    el.style[property] = value;
}
function getBordersWidth(el) {
    return {
        bottom: getIntValue(get(el, 'borderBottomWidth')),
        left: getIntValue(get(el, 'borderLeftWidth')),
        right: getIntValue(get(el, 'borderRightWidth')),
        top: getIntValue(get(el, 'borderTopWidth'))
    };
}
function getComputedStyle(el, doc, win) {
    // NOTE: In Firefox, after calling the 'document.write' function for nested iframes with html src value
    // document.defaultView equals 'null'. But 'window.document' equals 'document'.
    // This is why, we are forced to calculate the targetWindow instead of use document.defaultView.
    doc = doc || document;
    win = win || window;
    var targetWin = doc.defaultView || win;
    return targetWin.getComputedStyle(el, null);
}
function getElementMargin(el) {
    return {
        bottom: getIntValue(get(el, 'marginBottom')),
        left: getIntValue(get(el, 'marginLeft')),
        right: getIntValue(get(el, 'marginRight')),
        top: getIntValue(get(el, 'marginTop'))
    };
}
function getElementPadding(el) {
    return {
        bottom: getIntValue(get(el, 'paddingBottom')),
        left: getIntValue(get(el, 'paddingLeft')),
        right: getIntValue(get(el, 'paddingRight')),
        top: getIntValue(get(el, 'paddingTop'))
    };
}
function getElementScroll(el) {
    var isHtmlElement = domUtils.isHtmlElement(el);
    var currentWindow = window;
    if (isHtmlElement && domUtils.isElementInIframe(el)) {
        var currentIframe = domUtils.getIframeByElement(el);
        if (currentIframe) currentWindow = _nativeMethods2.default.contentWindowGetter.call(currentIframe);
    }
    var targetEl = isHtmlElement ? currentWindow : el;
    return {
        left: getScrollLeft(targetEl),
        top: getScrollTop(targetEl)
    };
}
function getWidth(el) {
    if (!el) return null;
    if (domUtils.isWindow(el)) return el.document.documentElement.clientWidth;
    if (domUtils.isDocument(el)) {
        var doc = el.documentElement;
        var clientProp = 'clientWidth';
        var scrollProp = 'scrollWidth';
        var offsetProp = 'offsetWidth';
        if (doc[clientProp] >= doc[scrollProp]) return doc[clientProp];
        return Math.max(el.body[scrollProp], doc[scrollProp], el.body[offsetProp], doc[offsetProp]);
    }
    var value = el.offsetWidth;
    value -= getIntValue(get(el, 'paddingLeft'));
    value -= getIntValue(get(el, 'paddingRight'));
    value -= getIntValue(get(el, 'borderLeftWidth'));
    value -= getIntValue(get(el, 'borderRightWidth'));
    return value;
}
function getHeight(el) {
    if (!el) return null;
    if (domUtils.isWindow(el)) return el.document.documentElement.clientHeight;
    if (domUtils.isDocument(el)) {
        var doc = el.documentElement;
        var clientProp = 'clientHeight';
        var scrollProp = 'scrollHeight';
        var offsetProp = 'offsetHeight';
        if (doc[clientProp] >= doc[scrollProp]) return doc[clientProp];
        return Math.max(el.body[scrollProp], doc[scrollProp], el.body[offsetProp], doc[offsetProp]);
    }
    var value = el.offsetHeight;
    value -= getIntValue(get(el, 'paddingTop'));
    value -= getIntValue(get(el, 'paddingBottom'));
    value -= getIntValue(get(el, 'borderTopWidth'));
    value -= getIntValue(get(el, 'borderBottomWidth'));
    return value;
}
function getInnerWidth(el) {
    if (!el) return null;
    if (domUtils.isWindow(el)) return el.document.documentElement.clientWidth;
    if (domUtils.isDocument(el)) return el.documentElement.clientWidth;
    var value = el.offsetWidth;
    value -= getIntValue(get(el, 'borderLeftWidth'));
    value -= getIntValue(get(el, 'borderRightWidth'));
    return value;
}
function getInnerHeight(el) {
    if (!el) return null;
    if (domUtils.isWindow(el)) return el.document.documentElement.clientHeight;
    if (domUtils.isDocument(el)) return el.documentElement.clientHeight;
    var value = el.offsetHeight;
    value -= getIntValue(get(el, 'borderTopWidth'));
    value -= getIntValue(get(el, 'borderBottomWidth'));
    return value;
}
function getOptionHeight(select) {
    var realSizeValue = getSelectElementSize(select);
    var selectPadding = getElementPadding(select);
    var selectScrollHeight = select.scrollHeight - (selectPadding.top + selectPadding.bottom);
    var childrenCount = domUtils.getSelectVisibleChildren(select).length;
    if (realSizeValue === 1) return getHeight(select);
    return browserUtils.isIE && realSizeValue > childrenCount ? Math.round(selectScrollHeight / childrenCount) : Math.round(selectScrollHeight / Math.max(childrenCount, realSizeValue));
}
function getSelectElementSize(select) {
    // NOTE: iOS and Android ignore 'size' and 'multiple' attributes,
    // all select elements behave like a select with size=1.
    if (browserUtils.isSafari && featureDetection.hasTouchEvents || browserUtils.isAndroid) return 1;
    var sizeAttr = _nativeMethods2.default.getAttribute.call(select, 'size');
    var multipleAttr = _nativeMethods2.default.getAttribute.call(select, 'multiple');
    var size = !sizeAttr ? 1 : parseInt(sizeAttr, 10);
    if (multipleAttr && (!sizeAttr || size < 1)) size = MIN_SELECT_SIZE_VALUE;
    return size;
}
function isVisibleChild(el) {
    var select = domUtils.getSelectParent(el);
    var tagName = domUtils.getTagName(el);
    return domUtils.isSelectElement(select) && getSelectElementSize(select) > 1 && (tagName === 'option' || tagName === 'optgroup') && (
    // NOTE: Firefox does not display groups without a label or with an empty label.
    !browserUtils.isFirefox || el.label);
}
function getScrollLeft(el) {
    if (!el) return null;
    if (domUtils.isWindow(el)) return el.pageXOffset;
    if (domUtils.isDocument(el)) return el.defaultView.pageXOffset;
    return el.scrollLeft;
}
function getScrollTop(el) {
    if (!el) return null;
    if (domUtils.isWindow(el)) return el.pageYOffset;
    if (domUtils.isDocument(el)) return el.defaultView.pageYOffset;
    return el.scrollTop;
}
function setScrollLeft(el, value) {
    if (!el) return;
    if (domUtils.isWindow(el) || domUtils.isDocument(el)) {
        var win = domUtils.findDocument(el).defaultView;
        var scrollTop = getScrollTop(el);
        win.scrollTo(value, scrollTop);
    } else el.scrollLeft = value;
}
function setScrollTop(el, value) {
    if (!el) return;
    if (domUtils.isWindow(el) || domUtils.isDocument(el)) {
        var win = domUtils.findDocument(el).defaultView;
        var scrollLeft = getScrollLeft(el);
        win.scrollTo(scrollLeft, value);
    } else el.scrollTop = value;
}
function getOffsetParent(el) {
    if (el) {
        var offsetParent = el.offsetParent || document.body;
        while (offsetParent && !/^(?:body|html)$/i.test(offsetParent.nodeName) && get(offsetParent, 'position') === 'static') {
            offsetParent = offsetParent.offsetParent;
        }return offsetParent;
    }
    return void 0;
}
function getOffset(el) {
    if (!el || domUtils.isWindow(el) || domUtils.isDocument(el)) return null;
    var clientRect = el.getBoundingClientRect();
    // NOTE: A detached node or documentElement.
    var doc = el.ownerDocument;
    var docElement = doc.documentElement;
    if (!docElement.contains(el) || el === docElement) {
        return {
            top: clientRect.top,
            left: clientRect.left
        };
    }
    var win = doc.defaultView;
    var clientTop = docElement.clientTop || doc.body.clientTop || 0;
    var clientLeft = docElement.clientLeft || doc.body.clientLeft || 0;
    var scrollTop = win.pageYOffset || docElement.scrollTop || doc.body.scrollTop;
    var scrollLeft = win.pageXOffset || docElement.scrollLeft || doc.body.scrollLeft;
    clientRect = el.getBoundingClientRect();
    return {
        top: clientRect.top + scrollTop - clientTop,
        left: clientRect.left + scrollLeft - clientLeft
    };
}
function isElementVisible(el, doc) {
    if (!domUtils.isElementInDocument(el, doc)) return false;
    while (el) {
        if (get(el, 'display', doc) === 'none' || get(el, 'visibility', doc) === 'hidden') return false;
        el = el.parentNode;
    }
    return true;
}
function isElementInInvisibleIframe(el) {
    var frameElement = domUtils.getIframeByElement(el);
    return frameElement && !isElementVisible(frameElement, domUtils.findDocument(frameElement));
}