'use strict';

exports.__esModule = true;

var _nativeMethods = require('../sandbox/native-methods');

var _nativeMethods2 = _interopRequireDefault(_nativeMethods);

var _destinationLocation = require('../utils/destination-location');

var destLocation = _interopRequireWildcard(_destinationLocation);

var _url = require('../../utils/url');

var _dom = require('../utils/dom');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DOCUMENT_URL_RESOLVER = 'hammerhead|document-url-resolver';
exports.default = {
    _createResolver: function _createResolver(doc) {
        var htmlDocument = _nativeMethods2.default.createHTMLDocument.call(doc.implementation, 'title');
        var a = _nativeMethods2.default.createElement.call(htmlDocument, 'a');
        var base = _nativeMethods2.default.createElement.call(htmlDocument, 'base');
        _nativeMethods2.default.appendChild.call(htmlDocument.body, a);
        _nativeMethods2.default.appendChild.call(htmlDocument.head, base);
        return htmlDocument;
    },
    _getResolver: function _getResolver(doc) {
        // NOTE: Once a document is recreated (document.open, document.write is called), nativeMethods will be refreshed.
        // If we call urlResolve.updateBase after this,
        // we will use native methods from an actual document.
        // However, a document that contains an element for url resolving is created using a previous version of nativeMethods.
        if (!doc[DOCUMENT_URL_RESOLVER]) {
            _nativeMethods2.default.objectDefineProperty(doc, DOCUMENT_URL_RESOLVER, {
                value: this._createResolver(doc),
                writable: true
            });
        }
        return doc[DOCUMENT_URL_RESOLVER];
    },
    _isNestedIframeWithoutSrc: function _isNestedIframeWithoutSrc(win) {
        if (!win || !win.parent || win.parent === win || win.parent.parent === win.parent) return false;
        var iframeElement = (0, _dom.getFrameElement)(window);
        return !!iframeElement && (0, _dom.isIframeWithoutSrc)(iframeElement);
    },
    init: function init(doc) {
        this.updateBase(destLocation.get(), doc);
    },
    getResolverElement: function getResolverElement(doc) {
        return _nativeMethods2.default.nodeFirstChildGetter.call(this._getResolver(doc).body);
    },
    resolve: function resolve(url, doc) {
        var resolver = this.getResolverElement(doc);
        var href = null;
        if (url === null) _nativeMethods2.default.removeAttribute.call(resolver, 'href');else {
            _nativeMethods2.default.anchorHrefSetter.call(resolver, url);
            href = _nativeMethods2.default.anchorHrefGetter.call(resolver);
            // NOTE: It looks like a Chrome bug: in a nested iframe without src (when an iframe is placed into another
            // iframe) you cannot set a relative link href while the iframe loading is not completed. So, we'll do it with
            // the parent's urlResolver Safari demonstrates similar behavior, but urlResolver.href has a relative URL value.
            var needUseParentResolver = url && (!href || href.charAt(0) === '/') && this._isNestedIframeWithoutSrc(doc.defaultView);
            if (needUseParentResolver) return this.resolve(url, window.parent.document);
        }
        return (0, _url.ensureTrailingSlash)(url, href);
    },
    updateBase: function updateBase(url, doc) {
        var resolverDocument = this._getResolver(doc);
        var baseElement = _nativeMethods2.default.elementGetElementsByTagName.call(resolverDocument.head, 'base')[0];
        url = url || destLocation.get();
        /*eslint-disable no-restricted-properties*/
        var parsedUrl = (0, _url.parseUrl)(url);
        var isRelativeUrl = parsedUrl.protocol !== 'file:' && !parsedUrl.host;
        var isProtocolRelativeUrl = /^\/\//.test(url) && !!parsedUrl.host;
        /*eslint-enable no-restricted-properties*/
        if (isRelativeUrl || isProtocolRelativeUrl) {
            var destinationLocation = destLocation.get();
            this.updateBase(destinationLocation, doc);
            url = this.resolve(url, doc);
        }
        _nativeMethods2.default.setAttribute.call(baseElement, 'href', url);
    },
    getBaseUrl: function getBaseUrl(doc) {
        var baseElement = _nativeMethods2.default.elementGetElementsByTagName.call(this._getResolver(doc).head, 'base')[0];
        return _nativeMethods2.default.getAttribute.call(baseElement, 'href');
    },
    changeUrlPart: function changeUrlPart(url, nativePropSetter, value, doc) {
        var resolver = this.getResolverElement(doc);
        _nativeMethods2.default.anchorHrefSetter.call(resolver, url);
        nativePropSetter.call(resolver, value);
        return _nativeMethods2.default.anchorHrefGetter.call(resolver);
    },
    dispose: function dispose(doc) {
        doc[DOCUMENT_URL_RESOLVER] = null;
    }
};
module.exports = exports['default'];