'use strict';

exports.__esModule = true;
exports.REQUEST_DESCRIPTOR_VALUES_SEPARATOR = undefined;
exports.getProxyUrl = getProxyUrl;
exports.getNavigationUrl = getNavigationUrl;
exports.getCrossDomainIframeProxyUrl = getCrossDomainIframeProxyUrl;
exports.getCrossDomainProxyPort = getCrossDomainProxyPort;
exports.getCrossDomainProxyUrl = getCrossDomainProxyUrl;
exports.resolveUrlAsDest = resolveUrlAsDest;
exports.formatUrl = formatUrl;
exports.parseProxyUrl = parseProxyUrl;
exports.parseUrl = parseUrl;
exports.convertToProxyUrl = convertToProxyUrl;
exports.changeDestUrlPart = changeDestUrlPart;
exports.isValidWebSocketUrl = isValidWebSocketUrl;
exports.isSubDomain = isSubDomain;
exports.isSupportedProtocol = isSupportedProtocol;
exports.isSpecialPage = isSpecialPage;
exports.parseResourceType = parseResourceType;
exports.stringifyResourceType = stringifyResourceType;
exports.isChangedOnlyHash = isChangedOnlyHash;

var _internalProperties = require('../../processing/dom/internal-properties');

var _internalProperties2 = _interopRequireDefault(_internalProperties);

var _url = require('../../utils/url');

var sharedUrlUtils = _interopRequireWildcard(_url);

var _destinationLocation = require('./destination-location');

var destLocation = _interopRequireWildcard(_destinationLocation);

var _urlResolver = require('./url-resolver');

var urlResolver = _interopRequireWildcard(_urlResolver);

var _settings = require('../settings');

var _settings2 = _interopRequireDefault(_settings);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var HASH_RE = /#[\S\s]*$/;
var SUPPORTED_WEB_SOCKET_PROTOCOL_RE = /^wss?:/i;
// NOTE: The window.location equals 'about:blank' in iframes without src
// therefore we need to find a window with src to get the proxy settings
var DEFAULT_PROXY_SETTINGS = function () {
    /*eslint-disable no-restricted-properties*/
    var locationWindow = window;
    var proxyLocation = locationWindow.location;
    while (!proxyLocation.hostname) {
        locationWindow = locationWindow.parent;
        proxyLocation = locationWindow.location;
    }
    return {
        hostname: proxyLocation.hostname,
        port: proxyLocation.port.toString(),
        protocol: proxyLocation.protocol
    };
    /*eslint-enable no-restricted-properties*/
}();
var REQUEST_DESCRIPTOR_VALUES_SEPARATOR = exports.REQUEST_DESCRIPTOR_VALUES_SEPARATOR = sharedUrlUtils.REQUEST_DESCRIPTOR_VALUES_SEPARATOR;
function getProxyUrl(url, opts) {
    var resourceType = opts && opts.resourceType;
    var parsedResourceType = sharedUrlUtils.parseResourceType(resourceType);
    if (!parsedResourceType.isWebSocket && !isSupportedProtocol(url) && !isSpecialPage(url)) return url;
    // NOTE: Resolves relative URLs.
    var resolvedUrl = destLocation.resolveUrl(url, opts && opts.doc);
    if (parsedResourceType.isWebSocket && !isValidWebSocketUrl(resolvedUrl) || !sharedUrlUtils.isValidUrl(resolvedUrl)) return url;
    /*eslint-disable no-restricted-properties*/
    var proxyHostname = opts && opts.proxyHostname || DEFAULT_PROXY_SETTINGS.hostname;
    var proxyPort = opts && opts.proxyPort || DEFAULT_PROXY_SETTINGS.port;
    var proxyServerProtocol = opts && opts.proxyProtocol || DEFAULT_PROXY_SETTINGS.protocol;
    /*eslint-enable no-restricted-properties*/
    var proxyProtocol = parsedResourceType.isWebSocket ? proxyServerProtocol.replace('http', 'ws') : proxyServerProtocol;
    var sessionId = opts && opts.sessionId || _settings2.default.get().sessionId;
    var charset = opts && opts.charset;
    var reqOrigin = opts && opts.reqOrigin;
    var crossDomainPort = getCrossDomainProxyPort(proxyPort);
    // NOTE: If the relative URL contains no slash (e.g. 'img123'), the resolver will keep
    // the original proxy information, so that we can return such URL as is.
    // TODO: Implement the isProxyURL function.
    var parsedProxyUrl = sharedUrlUtils.parseProxyUrl(resolvedUrl);
    /*eslint-disable no-restricted-properties*/
    var isValidProxyUrl = !!parsedProxyUrl && parsedProxyUrl.proxy.hostname === proxyHostname && (parsedProxyUrl.proxy.port === proxyPort || parsedProxyUrl.proxy.port === crossDomainPort);
    /*eslint-enable no-restricted-properties*/
    if (isValidProxyUrl) {
        if (resourceType && parsedProxyUrl.resourceType === resourceType) return resolvedUrl;
        // NOTE: Need to change the proxy URL resource type.
        var destUrl = sharedUrlUtils.formatUrl(parsedProxyUrl.destResourceInfo);
        return getProxyUrl(destUrl, {
            proxyProtocol: proxyProtocol,
            proxyHostname: proxyHostname,
            proxyPort: proxyPort,
            sessionId: sessionId,
            resourceType: resourceType,
            charset: charset,
            reqOrigin: reqOrigin
        });
    }
    var parsedUrl = sharedUrlUtils.parseUrl(resolvedUrl);
    charset = charset || parsedResourceType.isScript && document[_internalProperties2.default.documentCharset];
    // NOTE: It seems that the relative URL had the leading slash or dots, so that the proxy info path part was
    // removed by the resolver and we have an origin URL with the incorrect host and protocol.
    /*eslint-disable no-restricted-properties*/
    if (parsedUrl.protocol === proxyServerProtocol && parsedUrl.hostname === proxyHostname && parsedUrl.port === proxyPort) {
        var parsedDestLocation = destLocation.getParsed();
        parsedUrl.protocol = parsedDestLocation.protocol;
        parsedUrl.host = parsedDestLocation.host;
        parsedUrl.hostname = parsedDestLocation.hostname;
        parsedUrl.port = parsedDestLocation.port || '';
        resolvedUrl = sharedUrlUtils.formatUrl(parsedUrl);
    }
    /*eslint-enable no-restricted-properties*/
    if (parsedResourceType.isWebSocket) {
        // eslint-disable-next-line no-restricted-properties
        parsedUrl.protocol = parsedUrl.protocol.replace('ws', 'http');
        resolvedUrl = sharedUrlUtils.formatUrl(parsedUrl);
        reqOrigin = reqOrigin || encodeURIComponent(destLocation.getOriginHeader());
    }
    return sharedUrlUtils.getProxyUrl(resolvedUrl, {
        proxyProtocol: proxyProtocol,
        proxyHostname: proxyHostname,
        proxyPort: proxyPort,
        sessionId: sessionId,
        resourceType: resourceType,
        charset: charset,
        reqOrigin: reqOrigin
    });
}
function getNavigationUrl(url, win) {
    // NOTE: For the 'about:blank' page, we perform url proxing only for the top window, 'location' object and links.
    // For images and iframes, we keep urls as they were.
    // See details in https://github.com/DevExpress/testcafe-hammerhead/issues/339
    var destinationLocation = null;
    var isIframe = win.top !== win;
    var winLocation = win.location.toString();
    if (isIframe) destinationLocation = winLocation;else {
        var parsedProxyUrl = parseProxyUrl(winLocation);
        destinationLocation = parsedProxyUrl && parsedProxyUrl.destUrl;
    }
    if (isSpecialPage(destinationLocation) && sharedUrlUtils.isRelativeUrl(url)) return '';
    url = sharedUrlUtils.prepareUrl(url);
    return getProxyUrl(url);
}
function getCrossDomainIframeProxyUrl(url) {
    return getProxyUrl(url, {
        proxyPort: _settings2.default.get().crossDomainProxyPort,
        resourceType: sharedUrlUtils.getResourceTypeString({ isIframe: true })
    });
}
function getCrossDomainProxyPort(proxyPort) {
    return _settings2.default.get().crossDomainProxyPort === proxyPort
    // eslint-disable-next-line no-restricted-properties
    ? location.port.toString() : _settings2.default.get().crossDomainProxyPort;
}
function getCrossDomainProxyUrl() {
    // eslint-disable-next-line no-restricted-properties
    return location.protocol + '//' + location.hostname + ':' + _settings2.default.get().crossDomainProxyPort + '/';
}
function resolveUrlAsDest(url) {
    return sharedUrlUtils.resolveUrlAsDest(url, getProxyUrl);
}
function formatUrl(parsedUrl) {
    return sharedUrlUtils.formatUrl(parsedUrl);
}
function parseProxyUrl(proxyUrl) {
    return sharedUrlUtils.parseProxyUrl(proxyUrl);
}
function parseUrl(url) {
    return sharedUrlUtils.parseUrl(url);
}
function convertToProxyUrl(url, resourceType, charset) {
    return getProxyUrl(url, { resourceType: resourceType, charset: charset });
}
function changeDestUrlPart(proxyUrl, nativePropSetter, value, resourceType) {
    var parsed = sharedUrlUtils.parseProxyUrl(proxyUrl);
    if (parsed) {
        var sessionId = parsed.sessionId;
        var proxy = parsed.proxy;
        // @ts-ignore
        var destUrl = urlResolver.changeUrlPart(parsed.destUrl, nativePropSetter, value, document);
        return getProxyUrl(destUrl, {
            /*eslint-disable no-restricted-properties*/
            proxyHostname: proxy.hostname,
            proxyPort: proxy.port,
            /*eslint-enable no-restricted-properties*/
            sessionId: sessionId,
            resourceType: resourceType
        });
    }
    return proxyUrl;
}
function isValidWebSocketUrl(url) {
    var resolvedUrl = resolveUrlAsDest(url);
    return SUPPORTED_WEB_SOCKET_PROTOCOL_RE.test(resolvedUrl);
}
function isSubDomain(domain, subDomain) {
    return sharedUrlUtils.isSubDomain(domain, subDomain);
}
function isSupportedProtocol(url) {
    return sharedUrlUtils.isSupportedProtocol(url);
}
function isSpecialPage(url) {
    return sharedUrlUtils.isSpecialPage(url);
}
function parseResourceType(resourceType) {
    return sharedUrlUtils.parseResourceType(resourceType);
}
function stringifyResourceType(resourceType) {
    return sharedUrlUtils.getResourceTypeString(resourceType);
}
function isChangedOnlyHash(currentUrl, newUrl) {
    // NOTE: we compare proxied urls because urls passed into the function may be proxied, non-proxied
    // or relative. The getProxyUrl function solves all the corresponding problems.
    return getProxyUrl(currentUrl).replace(HASH_RE, '') === getProxyUrl(newUrl).replace(HASH_RE, '');
}